<?php /* ==========================================================================
                                                                                  |
    Скрипт: Хелпер шаблона (в виде библиотеки ShortCodes).                        |
    ----------------------------------------------------------------------------  |
                                                                                  |
    Служит цели снабдить шаблон какими-либо дополнительными шаблонизационными     |
    функциями или переопределить дефолтные.                                       |
                                                                                  |
    ----------------------------------------------------------------------------  |
    Copyright 2017 Impera CMS                                                     |
                                                                                  |
================================================================================ */

    /* ============================================================================
    |                                                                             |
    |    Загрузка расширения хелпера (настройки), если есть.                      |
    |                                                                             |
    ============================================================================ */

    $file = dirname(__FILE__) . '/helper-settings.php';
    if (file_exists($file)) {
        require_once($file);
    } else {
        class TemplateEmulatorSettings {

            /* ====================================================================
            |                                                                     |
            |    Имена конфигурационных параметров шаблона, которые разрешаем     |
            |    перезаписывать в конфиге сайта при получении POST-формы от       |
            |    админа, редактирующего настройки шаблона (смотрите ниже методы   |
            |    prepare и saveTemplateSetupParams).                              |
            |    ---------------------------------------------------------------  |
            |                                                                     |
            |    Эти имена скорее всего уникальны для каждого шаблона. Если вы    |
            |    создадите принципиально другой шаблон, в котором захотите        |
            |    использовать этот хелпер, но с другими настройками, тогда вам    |
            |    понадобится создать в том шаблоне файл helper-settings.php       |
            |    такого же содержания, как сейчас представлен данный класс, и     |
            |    отредактировать эту секцию в созданном файле настроек.           |
            |                                                                     |
            ==================================================================== */

            protected $tplSetupParams = array(
                'exampleParam1',             // как строка (по умолчанию пустая)
                'exampleParam2' => 'def',    // как строка (по умолчанию def)
                'exampleParam3' => FALSE,    // как флаг (по умолчанию FALSE)
                'exampleParam4' => TRUE,     // как флаг (по умолчанию TRUE)
                'exampleParam...' => 2,      // как целое число (по умолчанию 2)
                'exampleParamN' => 1.5       // как действительное число (по умолчанию 1.5)
            );

            /* ====================================================================
            |                                                                     |
            |    Для каких переменных просим отменить упреждающее наполнение, то  |
            |    есть не используем их в шаблоне или, например, хотим наполнять   |
            |    их сами в нужной точке шаблона.                                  |
            |    ---------------------------------------------------------------  |
            |                                                                     |
            |    Этот список скорее всего уникален для каждого шаблона. Если вы   |
            |    создадите принципиально другой шаблон, в котором захотите        |
            |    использовать этот хелпер, но с другими настройками, тогда вам    |
            |    понадобится создать в том шаблоне файл helper-settings.php       |
            |    такого же содержания, как сейчас представлен данный класс, и     |
            |    отредактировать эту секцию в созданном файле настроек.           |
            |                                                                     |
            ==================================================================== */

            public $not_preassignable = array(
                // '*',                 // вообще отклонить наполнение

                // 'categories',        // отклонить наполнение категорий
                // 'categoriesTree',    // отклонить создание дерева категорий
                // 'all_brands',        // отклонить наполнение брендов
                // 'brandsTree',        // отклонить создание дерева брендов
                // 'filtered',          // отклонить наполнение переменных статистики фильтра (переменные вида filtered_...)
                // 'properties',        // отклонить наполнение фильтра свойств
                'brands',               // отклонить наполнение фильтра брендов
                'variants',             // отклонить наполнение фильтра имен вариантов
                'menus',                // отклонить наполнение менюшек
                // 'news',              // отклонить наполнение анонсового списка новостей
                // 'articles',          // отклонить наполнение анонсового списка статей
                // 'cart_products',     // отклонить наполнение товаров корзины
                // 'defer_products',    // отклонить наполнение отложенных товаров
                'banners',              // отклонить наполнение баннеров
                // 'recent_articles',   // отклонить наполнение недавно посещенных статей
                // 'recent_news',       // отклонить наполнение недавно посещенных новостей
                // 'recent_categories', // отклонить наполнение недавно посещенных категорий
                // 'recent_brands',     // отклонить наполнение недавно посещенных брендов
                // 'recent_products',   // отклонить наполнение недавно посещенных товаров
                'search_history',       // отклонить наполнение истории поиска

                'dummy'                 // фиктивная строка (для безопасного комментирования предыдущих)
            );
        }
    }

    /* ============================================================================
    |                                                                             |
    |    Загрузка расширения хелпера (коммерческие модули), если есть.            |
    |                                                                             |
    ============================================================================ */

    $file = dirname(__FILE__) . '/mod-helper.php';
    if (file_exists($file)) {
        require_once($file);
    } else {
        class TemplateEmulatorMods extends TemplateEmulatorSettings {
            protected function setSmartyPlugins ( & $cms ) {
            }
        }
    }

    /* ============================================================================
    |                                                                             |
    |    Непосредственно хелпер.                                                  |
    |                                                                             |
    ============================================================================ */

    class TemplateEmulator extends TemplateEmulatorMods {
        protected $shortCodesVersion = '2019.05.20';

        protected $sortByFieldName;
            protected $sortByDescending;
        protected $demoCategoryImages;
            protected $demoBrandImages;
            protected $demoProductImages;
            protected $demoNewsImages;
            protected $demoArticleImages;
        protected $enumStack = array();
            protected $enumStackNums = array();
        protected $pushStack = array();
        protected $moduleSuffix = array('');
        protected $preassignChecker = array();
        protected $templateFunctions;
        protected $cms = null;

        /* ========================================================================
        |                                                                         |
        |    Регистрация наших шаблонизационных функций.                          |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Таким образом в шаблоне, кроме дефолтных шаблонизационных функций    |
        |    и его объекта $helper, станут доступны перечисленные здесь функции   |
        |    и объект $emulator.                                                  |
        |                                                                         |
        ======================================================================== */

        protected function setSmartyPlugins ( & $cms ) {

            /* дополнительные инструкции php, которые разрешаем в шаблоне */
            $this->expandPhp(array(
                'json_decode',
                'json_encode',
                'hash_hmac'
            ));

            /* собираем теги из хелперного расширения */
            parent::setSmartyPlugins($cms);
            $subList = $this->isActualArray($this->templateFunctions) ? $this->templateFunctions : FALSE;
            $this->templateFunctions = FALSE;

            /* регистрируем однострочные теги библиотеки ShortCodes */
            $this->expandSmarty(array(
                'shortCodesVersion'                   => 'вывести номер версии этой библиотеки ShortCodes',
                'version'                             => 'вывести номер версии движка',
                    'versionYMD'                      => 'вывести ДАТА-номер версии',
                'header200'                           => 'отправить серверный заголовок 200 OK',
                    'headerLastModified'              => 'отправить серверный заголовок Last-Modified',
                    'headerExpires'                   => 'отправить серверные заголовки Expires',
                    'headerJSON'                      => 'отправить серверные заголовки JSON-данных',
                        'headerTEXT'                  => 'отправить серверные заголовки текстового контента',
                            'printTextFile'           => 'вывести контент текстового файла',
                        'headerHTML'                  => 'отправить серверные заголовки HTML-контента',
                        'headerJS'                    => 'отправить серверные заголовки JS-контента',
                        'headerCSS'                   => 'отправить серверные заголовки CSS-контента',
                        'headerXML'                   => 'отправить серверные заголовки XML-данных',
                        'headerCSV'                   => 'отправить серверные заголовки CSV-данных',
                        'headerEXCEL'                 => 'отправить серверные заголовки EXCEL-данных',
                        'headerANY'                   => 'отправить серверные заголовки данных конкретного типа',
                        'headerFile'                  => 'отправить серверные заголовки "передача файла"',
                'searchTitle'                         => 'вывести заголовок страницы поиска',
                    'searchKeyword'                   => 'вывести искомую фразу поиска',
                        'searchTag'                   => 'вывести искомый тег поиска',
                    'titleSearch'                     => '....... синоним функции searchTitle',
                'titleFeatured'                       => 'вывести заголовок страницы особых товаров',
                    'titleContent'                    => '....... синоним функции titleFeatured',
                'metaDescription'                     => 'вывести мета описание страницы',
                    'metaKeywords'                    => 'вывести мета ключевые слова страницы',
                        'keys'                        => 'получить мета ключи + теги записи как массив уникальных строк',
                            'keywords'                => 'получить мета ключи записи как массив уникальных строк',
                            'tags'                    => 'получить теги записи как массив уникальных строк',
                        'keysCatalog'                 => 'получить мета ключи + теги товаров как массив уникальных строк',
                    'echoBody'                        => 'вывести финальное тело страницы',
                    'htmlClass'                       => 'вывести класс тега HTML',
                        'childClass'                  => 'вывести название дочернего класса',
                        'htmlLang'                    => 'вывести язык тега HTML',
                            'htmlLangMarker'          => 'вывести URL-параметр языка',
                            'langDefault'             => 'вывести язык по умолчанию',
                        'htmlUrl'                     => 'вывести URL для некоторого HTML',
                        'openGraphImage'              => 'вывести URL скриншота для OpenGraph разметки',
                        'demoClass'                   => 'вывести класс demo если мы в деморежиме',
                    'otherMeta'                       => 'вывести остальные мета',
                    'otherJS'                         => 'вывести остальные скрипты',
                    'seoText'                         => 'вывести SEO-текст страницы',
                        'seoBody'                     => 'вывести SEO-текст записи',
                    'endTag'                          => 'вывести завершающий тег блока',
                    'asideHeader'                     => 'вывести разметку боковой панели Шапка',
                        'asideTop'                    => 'вывести разметку боковой панели Верхняя',
                        'asideRight'                  => 'вывести разметку боковой панели Правая',
                        'asideBottom'                 => 'вывести разметку боковой панели Нижняя',
                        'asideLeft'                   => 'вывести разметку боковой панели Левая',
                        'asideCenter'                 => 'вывести разметку боковой панели Центральная',
                        'asideBanner'                 => 'вывести разметку боковой панели Баннер',
                        'asideSlider'                 => 'вывести разметку боковой панели Слайдер',
                        'asideGallery'                => 'вывести разметку боковой панели Галерея',
                        'asideMenu'                   => 'вывести разметку боковой панели Меню',
                        'asideFooter'                 => 'вывести разметку боковой панели Подвал',
                    'subdomainHtml'                   => 'вывести контент субдомена записи',
                    'canonicalUri'                    => 'вывести относительный канонический адрес страницы',
                    'canonicalUrl'                    => 'вывести абсолютный канонический адрес страницы',
                'siteName'                            => 'вывести название сайта',
                    'company'                         => 'вывести название компании',
                        'companyName'                 => '....... синоним функции company',
                    'themeName'                       => 'вывести название текущего дизайна',
                    'siteDomain'                      => 'вывести домен сайта',
                'setConfigParam'                      => 'установить конфигурационный параметр',
                    'setSettingsParam'                => 'установить параметр настройки сайта',
                    'setSessionParam'                 => 'установить параметр сеанса',
                'phone'                               => 'вывести телефон, указанный в конфиге',
                    'phone2'                          => '....... синоним функции phone num=2',
                    'phone3'                          => '....... синоним функции phone num=3',
                    'phone4'                          => '....... синоним функции phone num=4',
                    'phone5'                          => '....... синоним функции phone num=5',
                    'email'                           => 'вывести емейл, указанный в конфиге',
                        'email2'                      => '....... синоним функции email num=2',
                        'email3'                      => '....... синоним функции email num=3',
                        'email4'                      => '....... синоним функции email num=4',
                        'email5'                      => '....... синоним функции email num=5',
                    'telegram'                        => 'вывести Telegram-номер, указанный в конфиге',
                        'telegram2'                   => '....... синоним функции telegram num=2',
                        'telegram3'                   => '....... синоним функции telegram num=3',
                        'telegram4'                   => '....... синоним функции telegram num=4',
                        'telegram5'                   => '....... синоним функции telegram num=5',
                    'skype'                           => 'вывести скайп, указанный в конфиге',
                        'skype2'                      => '....... синоним функции skype num=2',
                        'skype3'                      => '....... синоним функции skype num=3',
                        'skype4'                      => '....... синоним функции skype num=4',
                        'skype5'                      => '....... синоним функции skype num=5',
                    'address'                         => 'вывести адрес, указанный в конфиге сайта',
                        'address2'                    => '....... синоним функции address num=2',
                        'address3'                    => '....... синоним функции address num=3',
                        'address4'                    => '....... синоним функции address num=4',
                        'address5'                    => '....... синоним функции address num=5',
                'facebook'                            => 'вывести адрес страницы Facebook из конфига',
                    'vkontakte'                       => 'вывести адрес страницы ВКонтакте из конфига',
                    'twitter'                         => 'вывести адрес страницы Twitter из конфига',
                    'google'                          => 'вывести адрес страницы Google+ из конфига',
                    'youtube'                         => 'вывести адрес страницы YouTube из конфига',
                    'instagram'                       => 'вывести адрес страницы Instagram из конфига',
                    'linkedin'                        => 'вывести адрес страницы LinkedIn из конфига',
                    'pinterest'                       => 'вывести адрес страницы Pinterest из конфига',
                    'dribbble'                        => 'вывести адрес страницы Dribbble из конфига',
                    'behance'                         => 'вывести адрес страницы Behance из конфига',
                'developer'                           => 'вывести название разработчика из конфига',
                    'developer2'                      => '....... синоним функции developer num=2',
                    'developer3'                      => '....... синоним функции developer num=3',
                    'developer4'                      => '....... синоним функции developer num=4',
                    'developer5'                      => '....... синоним функции developer num=5',
                    'developerSite'                   => 'вывести сайт разработчика из конфига',
                        'developerSite2'              => '....... синоним функции developerSite num=2',
                        'developerSite3'              => '....... синоним функции developerSite num=3',
                        'developerSite4'              => '....... синоним функции developerSite num=4',
                        'developerSite5'              => '....... синоним функции developerSite num=5',
                'error'                               => 'вывести сообщение об ошибке',
                    'message'                         => 'вывести сообщение об успехе',
                'year'                                => 'вывести текущий год',
                'counters'                            => 'вывести коды счетчиков',
                'pagination'                          => 'вывести html-контент пагинации страниц',
                    'urlAllOnPage'                    => 'вывести URL пагинационной ссылки Все',
                    'urlPrevPage'                     => 'вывести URL пагинационной ссылки Назад',
                    'urlNextPage'                     => 'вывести URL пагинационной ссылки Вперед',
                    'pageNum'                         => 'вывести номер текущей страницы',
                        'pageMeter'                   => 'вывести номер первой позиции на текущей странице',
                        'pageSize'                    => 'вывести количество позиций на листаемой странице',
                    'pageCount'                       => 'вывести количество листаемых страниц',
                'lastTemplate'                        => 'признать файл шаблона самостоятельной страницей',
                'centralTemplate'                     => 'признать файл шаблона не самостоятельной страницей',
                'cacheClear'                          => 'очистить кеш',
                    'staticClear'                     => 'удалить статику текущей страницы',
                    'staticClearMainpage'             => 'удалить статику главной страницы',
                    'staticSave'                      => 'сохранить статику текущей страницы',
                    'staticSaveMainpage'              => 'сохранить статику главной страницы',
                'date'                                => 'вывести дату записи',
                'id'                                  => 'вывести ИД записи',
                    'idSection'                       => 'вывести ИД записи специальной страницы',
                    'idCategory'                      => 'вывести ИД записи категории',
                    'idBrand'                         => 'вывести ИД записи бренда',
                    'idProduct'                       => 'вывести ИД записи товара',
                    'idVariant'                       => 'вывести ИД записи варианта товара',
                    'idProperty'                      => 'вывести ИД записи свойства',
                    'idNews'                          => 'вывести ИД записи новости',
                    'idArticle'                       => 'вывести ИД записи статьи',
                    'idStock'                         => 'вывести ИД записи склада',
                    'idCountry'                       => 'вывести ИД записи страны',
                        'idRegion'                    => 'вывести ИД записи области',
                        'idTown'                      => 'вывести ИД записи города',
                    'idCurrency'                      => 'вывести ИД записи валюты',
                    'idComment'                       => 'вывести ИД записи комментария',
                        'hashComment'                 => 'вывести ИД-хеш записи комментария',
                        'urlComment'                  => 'вывести URL комментируемой записи',
                        'typeComment'                 => 'вывести тип записи комментария как атрибут',
                    'idOrder'                         => 'вывести ИД записи заказа',
                    'idDelivery'                      => 'вывести ИД записи способа доставки',
                    'idPayment'                       => 'вывести ИД записи способа оплаты',
                    'typeId'                          => 'вывести тип ИД записи',
                        'typePath'                    => 'вывести префиксный сегмент составного URL записи',
                'activeSection'                       => 'вывести атрибут активной специальной страницы',
                    'activeCategory'                  => 'вывести атрибут активной категории',
                    'activeBrand'                     => 'вывести атрибут активного бренда',
                    'activeProduct'                   => 'вывести атрибут активного товара',
                    'activeNews'                      => 'вывести атрибут активной новости',
                    'activeArticle'                   => 'вывести атрибут активной статьи',
                    'activeStock'                     => 'вывести атрибут активного склада',
                    'activeCountry'                   => 'вывести атрибут активной страны',
                        'activeRegion'                => 'вывести атрибут активной области',
                        'activeTown'                  => 'вывести атрибут активного города',
                    'activeSort'                      => 'вывести атрибут активной сортировки',
                        'activeSortDesc'              => 'вывести атрибут активной сортировки наоборот',
                    'activeOption'                    => 'вывести атрибут активного значения свойства',
                        'actualOption'                => 'вывести атрибут действующего значения свойства',
                    'activeSize'                      => 'вывести атрибут активного размера страницы',
                        'activePage'                  => 'вывести атрибут активного номера страницы',
                            'ellipsisPage'            => 'вывести атрибут номера страницы под многоточием',
                    'activeCurrency'                  => 'вывести атрибут активной валюты',
                    'activeDelivery'                  => 'вывести атрибут активного способа доставки',
                    'activePayment'                   => 'вывести атрибут активного способа оплаты',
                    'activeComment'                   => 'вывести атрибут активного комментария',
                    'activeLanguage'                  => 'вывести атрибут активного языка',
                    'activeFirst'                     => 'вывести атрибут активного первого элемента',
                'highlighted'                         => 'вывести атрибут флага Выделен',
                    'hit'                             => 'вывести атрибут флага Хит',
                    'newest'                          => 'вывести атрибут флага Новинка',
                    'actional'                        => 'вывести атрибут флага Акционный',
                    'awaited'                         => 'вывести атрибут флага Скоро в продаже',
                    'commented'                       => 'вывести атрибут флага Обсуждаемый',
                    'withhold'                        => 'вывести атрибут флага Не может быть продан',
                        'exhibit'                     => 'вывести атрибут флага Экспонат',
                'itemTitle'                           => 'вывести мета тайтл записи',
                    'itemMeta'                        => 'вывести мета описание записи',
                    'itemKeywords'                    => 'вывести мета ключевые слова записи',
                    'itemTags'                        => 'вывести теги записи',
                    'itemName'                        => 'вывести название записи',
                    'itemUrl'                         => 'вывести относительный URL записи',
                    'itemCategory'                    => 'вывести категорию записи',
                        'itemCategoryUrl'             => 'вывести URL категории записи',
                    'itemBrand'                       => 'вывести бренд записи',
                        'itemBrandUrl'                => 'вывести URL бренда записи',
                        'itemBrandOrCategory'         => 'вывести бренд или категорию записи',
                    'itemAnnotation'                  => 'вывести краткий текст записи',
                    'itemBody'                        => 'вывести полный текст записи',
                    'itemSeo'                         => 'вывести SEO-текст записи',
                        'seo'                         => '....... синоним функции itemSeo',
                    'itemSubdomain'                   => 'вывести текст субдомена записи',
                        'subdomain'                   => '....... синоним функции itemSubdomain',
                    'itemUser'                        => 'вывести составное имя пользователя',
                        'user'                        => '....... синоним функции itemUser',
                            'userNick'                => 'вывести ник пользователя',
                            'userCash'                => 'вывести баланс внутреннего счета пользователя',
                            'userUnits'               => 'вывести количество юнитов внутреннего счета пользователя',
                            'userName'                => 'вывести составное имя пользователя',
                            'userEmail'               => 'вывести емейл 1 или 2 пользователя',
                                'userPhone'           => 'вывести телефон 1 или 2 пользователя',
                        'commentator'                 => 'вывести имя автора комментария',
                            'comment'                 => 'вывести текст комментария',
                            'annotation'              => 'вывести краткий текст',
                        'visitorIp'                   => 'вывести IP-адрес посетителя',
                            'visitorHost'             => 'вывести хост посетителя',
                            'visitorReferer'          => 'вывести откуда пришел посетитель',
                            'visitorRefererDomain'    => 'вывести домен откуда пришел посетитель',
                            'visitorRefererQuery'     => 'вывести GET-параметры из данных откуда пришел посетитель',
                            'visitorQueryRequest'     => 'вывести REQUEST-параметры из запроса страницы',
                            'visitorQueryGet'         => 'вывести GET-параметры из запроса страницы',
                            'visitorQueryPost'        => 'вывести POST-параметры из запроса страницы',
                            'visitorQueryFiles'       => 'вывести FILES-параметры из запроса страницы',
                            'visitorCookies'          => 'вывести COOKIE-параметры из запроса страницы',
                    'itemEmail'                       => 'вывести емейл 1 или 2 записи',
                        'itemPhone'                   => 'вывести телефон 1 или 2 записи',
                    'itemPCode'                       => 'вывести код товара',
                        'pcode'                       => '....... синоним функции itemPCode',
                    'itemSku'                         => 'вывести артикул варианта товара',
                        'sku'                         => '....... синоним функции itemSku',
                    'itemStock'                       => 'вывести количество варианта товара на складе',
                    'itemVariant'                     => 'вывести вариант товара',
                        'itemProps'                   => 'вывести отобранные свойства товара',
                    'itemCount'                       => 'вывести количество товаров в записи',
                        'browsed'                     => 'вывести количество просмотров записи',
                    'itemCommentsCount'               => 'вывести количество комментариев в записи',
                'nameSection'                         => 'вывести название специальной страницы',
                    'nameCategory'                    => 'вывести название категории',
                        'category'                    => '....... синоним функции nameCategory',
                    'nameBrand'                       => 'вывести название бренда',
                        'brand'                       => '....... синоним функции nameBrand',
                    'nameProduct'                     => 'вывести название товара',
                        'product'                     => '....... синоним функции nameProduct',
                        'nameVariant'                 => 'вывести название варианта товара',
                            'variant'                 => '....... синоним функции nameVariant',
                    'nameNews'                        => 'вывести название новости',
                    'nameArticle'                     => 'вывести название статьи',
                        'article'                     => '....... синоним функции nameArticle',
                    'nameStock'                       => 'вывести название склада',
                    'nameCountry'                     => 'вывести название страны',
                        'nameRegion'                  => 'вывести название области',
                        'nameTown'                    => 'вывести название города',
                    'specification'                   => 'вывести название свойства товара',
                        'specie'                      => 'вывести значение свойства товара',
                        'option'                      => 'вывести значение свойства фильтра',
                    'item'                            => 'вывести текст переменной item',
                        'value'                       => 'вывести текст переменной value',
                        'type'                        => 'вывести текст переменной type',
                        'hash'                        => 'вывести текст переменной hash',
                        'language'                    => 'вывести текст переменной language',
                'bodySection'                         => 'вывести описание специальной страницы',
                    'bodyCategory'                    => 'вывести описание категории',
                    'bodyBrand'                       => 'вывести описание бренда',
                    'bodyProduct'                     => 'вывести описание товара',
                    'bodyNews'                        => 'вывести описание новости',
                    'bodyArticle'                     => 'вывести описание статьи',
                    'bodyStock'                       => 'вывести описание склада',
                    'bodyCountry'                     => 'вывести описание страны',
                        'bodyRegion'                  => 'вывести описание области',
                        'bodyTown'                    => 'вывести описание города',
                'thumb'                               => 'вывести URL миниатюры записи',
                    'thumbSection'                    => 'вывести URL миниатюры специальной страницы',
                    'thumbCategory'                   => 'вывести URL миниатюры категории',
                    'thumbBrand'                      => 'вывести URL миниатюры бренда',
                    'thumbProduct'                    => 'вывести URL миниатюры товара',
                    'thumbNews'                       => 'вывести URL миниатюры новости',
                    'thumbArticle'                    => 'вывести URL миниатюры статьи',
                    'thumbStock'                      => 'вывести URL миниатюры склада',
                    'thumbCountry'                    => 'вывести URL миниатюры страны',
                        'thumbRegion'                 => 'вывести URL миниатюры области',
                        'thumbTown'                   => 'вывести URL миниатюры города',
                'photo'                               => 'вывести URL фотографии записи',
                    'photoSection'                    => 'вывести URL фотографии специальной страницы',
                    'photoCategory'                   => 'вывести URL фотографии категории',
                    'photoBrand'                      => 'вывести URL фотографии бренда',
                    'photoProduct'                    => 'вывести URL фотографии товара',
                    'photoNews'                       => 'вывести URL фотографии новости',
                    'photoArticle'                    => 'вывести URL фотографии статьи',
                    'photoStock'                      => 'вывести URL фотографии склада',
                    'photoCountry'                    => 'вывести URL фотографии страны',
                        'photoRegion'                 => 'вывести URL фотографии области',
                        'photoTown'                   => 'вывести URL фотографии города',
                'price'                               => 'вывести цену текущего варианта товара',
                    'amount'                          => 'вывести количество товарной позиции в корзине / заказе',
                        'quantity'                    => 'вывести ПОЛОЖИТЕЛЬНОЕ количество товарной позиции в корзине / заказе',
                    'sum'                             => 'вывести стоимость товарной позиции в корзине / заказе',
                    'total'                           => 'вывести итоговую стоимость заказа',
                        'totalDelivery'               => 'вывести итоговую стоимость доставки в заказе',
                        'totalDiscount'               => 'вывести итоговую стоимость скидки в заказе',
                'found'                               => 'вывести количество найденных записей',
                'idx'                                 => 'вывести текущий индекс enum-функции',
                    'idxAsTag'                        => 'вывести текущий индекс enum-функции как тег-ссылку',
                    'idxAsHashtag'                    => 'вывести текущий индекс enum-функции как хештег',
                    'num'                             => 'вывести номер итерации enum-функции',
                        'numPrev'                     => 'вывести номер предыдущей итерации enum-функции',
                        'numNext'                     => 'вывести номер следующей итерации enum-функции',
                    'out'                             => 'прокрутить за конец итераций enum-функции',
                'sortMethod'                          => 'вывести ИД текущего способа сортировки',
                'cartTotal'                           => 'вывести итоговую сумму корзины',
                    'cartDiscount'                    => 'вывести итоговую скидку корзины',
                    'cartQuantity'                    => 'вывести количество товаров в корзине',
                'valueSurname'                        => 'вывести значение поля ввода ФАМИЛИЯ',
                    'valueName'                       => 'вывести значение поля ввода ИМЯ',
                    'valuePatronymic'                 => 'вывести значение поля ввода ОТЧЕСТВО',
                    'valueNick'                       => 'вывести значение поля ввода НИК',
                    'valueEmail'                      => 'вывести значение поля ввода ЕМЕЙЛ',
                        'valueEmail2'                 => 'вывести значение поля ввода ЕМЕЙЛ 2',
                        'valueLogin'                  => '....... синоним функции valueEmail',
                    'valuePhone'                      => 'вывести значение поля ввода ТЕЛЕФОН',
                        'valuePhone2'                 => 'вывести значение поля ввода ТЕЛЕФОН 2',
                    'valueAddress'                    => 'вывести значение поля ввода АДРЕС',
                        'valueCountry'                => 'вывести значение поля ввода АДРЕС-СТРАНА',
                        'valueRegion'                 => 'вывести значение поля ввода АДРЕС-ОБЛАСТЬ',
                        'valueTown'                   => 'вывести значение поля ввода АДРЕС-ГОРОД',
                        'valueStreet'                 => 'вывести значение поля ввода АДРЕС-УЛИЦА',
                        'valueHouse'                  => 'вывести значение поля ввода АДРЕС-ДОМ',
                        'valueRoom'                   => 'вывести значение поля ввода АДРЕС-КВАРТИРА',
                        'valueZipcode'                => 'вывести значение поля ввода АДРЕС-ИНДЕКС',
                    'valueDepartment'                 => 'вывести значение поля ввода ОТДЕЛ',
                    'valueSubject'                    => 'вывести значение поля ввода ТЕМА',
                    'valueMessage'                    => 'вывести значение поля ввода СООБЩЕНИЕ',
                    'valueComment'                    => 'вывести значение поля ввода КОММЕНТАРИЙ',
                    'valueParentId'                   => 'вывести значение поля ввода ИД РОДИТЕЛЯ',
                    'valueCopystop'                   => 'вывести значение поля ввода КОНТРОЛЬ РЕПОСТИНГА',
                'count'                               => 'вывести количество этих записей',
                    'countItems'                      => 'вывести количество записей в items',
                    'countSections'                   => 'вывести количество специальных страниц',
                    'countCategories'                 => 'вывести количество ветвей дерева категорий',
                    'countBrands'                     => 'вывести количество ветвей дерева брендов',
                    'countProducts'                   => 'вывести количество товаров',
                    'countNews'                       => 'вывести количество новостей',
                    'countArticles'                   => 'вывести количество статей',
                'undoModSuffix'                       => 'вернуть прежний суффикс модуля',
                'noItem'                              => 'забыть запись по умолчанию',
                    'noItems'                         => 'забыть список записей по умолчанию',
                    'noCategories'                    => 'забыть дерево категорий',
                    'noBrands'                        => 'забыть дерево брендов',
                    'noProducts'                      => 'забыть список товаров',
                    'noNews'                          => 'забыть список новостей',
                    'noArticles'                      => 'забыть список статей',
                    'noAnnounces'                     => 'забыть анонсовый список публикаций',
                    'noContent'                       => 'забыть контент страницы',
                    'noMetaDescription'               => 'забыть мета описание страницы',
                    'noMetaKeywords'                  => 'забыть мета ключевые слова страницы',
                    'noHtmlClass'                     => 'забыть класс тега HTML',
                        'noChildClass'                => 'забыть название дочернего класса',
                        'noHtmlLang'                  => 'забыть язык тега HTML',
                            'noHtmlLangMarker'        => 'забыть URL-параметр языка',
                            'noLangDefault'           => 'забыть язык по умолчанию',
                        'noHtmlUrl'                   => 'забыть URL для некоторого HTML',
                        'noOpenGraphImage'            => 'забыть URL скриншота для OpenGraph разметки',
                    'noOtherMeta'                     => 'забыть остальные мета',
                    'noOtherJS'                       => 'забыть остальные скрипты',
                    'noSeoText'                       => 'забыть SEO-текст страницы',
                    'noEndTag'                        => 'забыть завершающий тег блока',
                    'noPagination'                    => 'забыть html-контент пагинации страниц',
                'counter'                             => 'вывести значение произвольного счетчика',
                    'resetCounter'                    => 'сбросить произвольный счетчик',
                    'incrementCounter'                => 'увеличить произвольный счетчик',
                'useSection'                          => 'по умолчанию работаем со специальной страницей',
                    'useCategories'                   => 'по умолчанию работаем со списком категорий',
                        'useCategory'                 => 'по умолчанию работаем с категорией',
                    'useBrands'                       => 'по умолчанию работаем со списком брендов',
                        'useBrand'                    => 'по умолчанию работаем с брендом',
                        'useSectionOrBrandOrCategory' => 'по умолчанию работаем со страницей, брендом или категорией',
                        'useBrandOrCategory'          => 'по умолчанию работаем с брендом или категорией',
                    'useProducts'                     => 'по умолчанию работаем со списком товаров',
                        'useProduct'                  => 'по умолчанию работаем с товаром',
                    'useNews'                         => 'по умолчанию работаем со списком новостей',
                        'useNewsItem'                 => 'по умолчанию работаем с новостью',
                    'useArticles'                     => 'по умолчанию работаем со списком статей',
                        'useArticle'                  => 'по умолчанию работаем со статьей',
                    'useStocks'                       => 'по умолчанию работаем со списком складов',
                        'useStock'                    => 'по умолчанию работаем со складом',
                    'useCountries'                    => 'по умолчанию работаем со списком стран',
                        'useCountry'                  => 'по умолчанию работаем со страной',
                        'useRegions'                  => 'по умолчанию работаем со списком областей',
                            'useRegion'               => 'по умолчанию работаем с областью',
                        'useTowns'                    => 'по умолчанию работаем со списком городов',
                            'useTown'                 => 'по умолчанию работаем с городом',
                    'useOrders'                       => 'по умолчанию работаем со списком заказов',
                        'useOrder'                    => 'по умолчанию работаем с заказом',
                    'usePost'                         => 'по умолчанию работаем с переменной post',
                    'useAnything'                     => 'по умолчанию работаем с доступной записью',
                'pushItems'                           => 'поместить умалчиваемый список в стек',
                    'pushItem'                        => 'поместить умалчиваемую переменную в стек',
                'popItems'                            => 'извлечь умалчиваемый список из стека',
                    'popItem'                         => 'извлечь умалчиваемую переменную из стека',
                'globalItem'                          => 'глобализировать умалчиваемую переменную',
                    'parentItem'                      => 'глобализировать умалчиваемую переменную до надстоящего шаблона',
                'getRecordField'                      => 'получить поле некоторой записи из формы',
                'getRequest'                          => 'получить параметр запроса',
                    'getRequestPage'                  => 'получить параметр page запроса',
                        'valueRequestPage'            => 'вывести значение параметра page запроса',
                    'getRequestMode'                  => 'получить параметр mode запроса',
                        'valueRequestMode'            => 'вывести значение параметра mode запроса',
                    'getRequestLang'                  => 'получить параметр lang запроса',
                        'valueRequestLang'            => 'вывести значение параметра lang запроса',
                    'getRequestSign'                  => 'получить параметр sign запроса',
                        'valueRequestSign'            => 'вывести значение параметра sign запроса',
                    'getRequestData'                  => 'получить параметр data запроса',
                        'valueRequestData'            => 'вывести значение параметра data запроса',
                    'getRequestId'                    => 'получить параметр id запроса',
                        'valueRequestId'              => 'вывести значение параметра id запроса',
                    'getRequestTitle'                 => 'получить параметр title запроса',
                        'valueRequestTitle'           => 'вывести значение параметра title запроса',
                    'getRequestMetaKeywords'          => 'получить параметр meta_keywords запроса',
                        'valueRequestMetaKeywords'    => 'вывести значение параметра meta_keywords запроса',
                    'getRequestMetaDescription'       => 'получить параметр meta_description запроса',
                        'valueRequestMetaDescription' => 'вывести значение параметра meta_description запроса',
                    'getRequestName'                  => 'получить параметр name запроса',
                        'valueRequestName'            => 'вывести значение параметра name запроса',
                    'getRequestCard'                  => 'получить параметр card запроса',
                        'valueRequestCard'            => 'вывести значение параметра card запроса',
                    'getRequestEmail'                 => 'получить параметр email запроса',
                        'valueRequestEmail'           => 'вывести значение параметра email запроса',
                    'getRequestPhone'                 => 'получить параметр phone запроса',
                        'valueRequestPhone'           => 'вывести значение параметра phone запроса',
                    'getRequestMessage'               => 'получить параметр message запроса',
                        'valueRequestMessage'         => 'вывести значение параметра message запроса',
                    'getRequestAnnotation'            => 'получить параметр annotation запроса',
                        'valueRequestAnnotation'      => 'вывести значение параметра annotation запроса',
                    'getRequestBody'                  => 'получить параметр body запроса',
                        'valueRequestBody'            => 'вывести значение параметра body запроса',
                    'getRequestDomain'                => 'получить параметр domain запроса',
                        'valueRequestDomain'          => 'вывести значение параметра domain запроса',
                    'activeRequestAgree'              => 'вывести атрибут флага agree запроса',
                        'activeRequestSetting'        => 'вывести атрибут флага setting-... запроса',
                'getTopMenu'                          => 'получить массив специальных страниц верхнего меню',
                    'getSection'                      => 'получить запись о статической странице',
                    'getSearchHistory'                => 'получить массив истории поиска',
                    'getCategories'                   => 'получить дерево категорий',
                        'getBrands'                   => 'получить дерево брендов',
                    'getProducts'                     => 'получить список товаров',
                        'getFeatureds'                => 'получить массив особых товаров',
                        'getRandoms'                  => 'получить массив случайных товаров',
                        'getLasts'                    => 'получить массив последних поступивших товаров',
                        'getFreebies'                 => 'получить массив бесплатных товаров',
                        'getExhibits'                 => 'получить массив товаров экспонатов',
                    'getNews'                         => 'получить список новостей',
                    'getArticles'                     => 'получить список статей',
                    'getStocks'                       => 'получить список складов',
                    'getCountries'                    => 'получить список стран',
                        'getRegions'                  => 'получить список областей',
                        'getTowns'                    => 'получить список городов',
                    'getSlides'                       => 'получить адреса картинок папки шаблона',
                    'getComments'                     => 'получить массив последних комментариев',
                    'getOrders'                       => 'получить массив последних заказов',
                    'getUser'                         => 'получить запись о пользователе',
                        'writeOffUnits'               => 'списать юниты с внутреннего счета пользователе',
                'redirectToFirst'                     => 'редирект на начало списка, если мы не там',
                    'redirectToAll'                   => 'редирект на начало полного списка, если мы не там',
                'generateCaptcha'                     => 'сгенерировать код капчи',
                'loginWith'                           => 'авторизоваться под таким клиентским логином',
                'modCheck404'                         => 'подключить модуль Check404',
                    'modError404'                     => 'подключить модуль Error404',
                        'modError'                    => 'подключить модуль СообщениеОбОшибке',
                            'modError2'               => '....... синоним функции modError num=2',
                            'modError3'               => '....... синоним функции modError num=3',
                            'modError4'               => '....... синоним функции modError num=4',
                            'modError5'               => '....... синоним функции modError num=5',
                        'modSuccess'                  => 'подключить модуль СообщениеОбУспехе',
                            'modSuccess2'             => '....... синоним функции modSuccess num=2',
                            'modSuccess3'             => '....... синоним функции modSuccess num=3',
                            'modSuccess4'             => '....... синоним функции modSuccess num=4',
                            'modSuccess5'             => '....... синоним функции modSuccess num=5',
                    'modCanonizator'                  => 'подключить модуль Canonizator',
                    'modTransit'                      => 'подключить модуль ПоисковыеТранзиты',
                    'modExportOrderToCsv'             => 'подключить модуль ExportOrderToCsv',
                    'modAnalytics'                    => 'подключить модуль СкриптАналитики',
                        'modMetrics'                  => 'подключить модуль СкриптМетрики',
                        'modGTM'                      => 'подключить модуль СкриптGoogleTagManager',
                        'modGoogle'                   => 'подключить модуль СкриптGoogle',
                        'modYandex'                   => 'подключить модуль СкриптYandex',
                        'modFacebook'                 => 'подключить модуль СкриптFacebook',
                        'modVkontakte'                => 'подключить модуль СкриптВКонтакте',
                        'modTwitter'                  => 'подключить модуль СкриптTwitter',
                        'modOpenGraph'                => 'подключить модуль РазметкаOpenGraph',
                        'modSchemaOrg'                => 'подключить модуль РазметкаSchemaOrg',
                    'modCurrencies'                   => 'подключить модуль ФормаВыбораВалюты',
                    'modHead'                         => 'подключить модуль ЗаголовкиСтраницы',
                        'modHead2'                    => '....... синоним функции modHead num=2',
                        'modHead3'                    => '....... синоним функции modHead num=3',
                        'modHead4'                    => '....... синоним функции modHead num=4',
                        'modHead5'                    => '....... синоним функции modHead num=5',
                    'modHeader'                       => 'подключить модуль Шапка',
                        'modHeader2'                  => '....... синоним функции modHeader num=2',
                        'modHeader3'                  => '....... синоним функции modHeader num=3',
                        'modHeader4'                  => '....... синоним функции modHeader num=4',
                        'modHeader5'                  => '....... синоним функции modHeader num=5',
                    'modBanner'                       => 'подключить модуль Баннер',
                        'modBanner2'                  => '....... синоним функции modBanner num=2',
                        'modBanner3'                  => '....... синоним функции modBanner num=3',
                        'modBanner4'                  => '....... синоним функции modBanner num=4',
                        'modBanner5'                  => '....... синоним функции modBanner num=5',
                    'modSlider'                       => 'подключить модуль Слайдер',
                        'modSlider2'                  => '....... синоним функции modSlider num=2',
                        'modSlider3'                  => '....... синоним функции modSlider num=3',
                        'modSlider4'                  => '....... синоним функции modSlider num=4',
                        'modSlider5'                  => '....... синоним функции modSlider num=5',
                    'modGallery'                      => 'подключить модуль Фотогалерея',
                        'modGallery2'                 => '....... синоним функции modGallery num=2',
                        'modGallery3'                 => '....... синоним функции modGallery num=3',
                        'modGallery4'                 => '....... синоним функции modGallery num=4',
                        'modGallery5'                 => '....... синоним функции modGallery num=5',
                    'modHome'                         => 'подключить модуль ДомашняяСтраница',
                    'modMain'                         => 'подключить модуль Основное',
                    'modWelcome'                      => 'подключить модуль Приветствие',
                    'modServices'                     => 'подключить модуль Услуги',
                    'modMap'                          => 'подключить модуль Карта',
                    'modAbout'                        => 'подключить модуль ОНас',
                        'modAbout2'                   => '....... синоним функции modAbout num=2',
                        'modAbout3'                   => '....... синоним функции modAbout num=3',
                        'modAbout4'                   => '....... синоним функции modAbout num=4',
                        'modAbout5'                   => '....... синоним функции modAbout num=5',
                    'modContacts'                     => 'подключить модуль Контакты',
                        'modContacts2'                => '....... синоним функции modContacts num=2',
                        'modContacts3'                => '....... синоним функции modContacts num=3',
                        'modContacts4'                => '....... синоним функции modContacts num=4',
                        'modContacts5'                => '....... синоним функции modContacts num=5',
                    'modFooter'                       => 'подключить модуль Подвал',
                        'modFooter2'                  => '....... синоним функции modFooter num=2',
                        'modFooter3'                  => '....... синоним функции modFooter num=3',
                        'modFooter4'                  => '....... синоним функции modFooter num=4',
                        'modFooter5'                  => '....... синоним функции modFooter num=5',
                    'modAMP'                          => 'подключить модуль AcceleratedMobilePages',
                        'modAMP2'                     => '....... синоним функции modAMP num=2',
                        'modAMP3'                     => '....... синоним функции modAMP num=3',
                        'modAMP4'                     => '....... синоним функции modAMP num=4',
                        'modAMP5'                     => '....... синоним функции modAMP num=5',
                    'modSearch'                       => 'подключить модуль ФормаПоиска',
                        'modSearch2'                  => '....... синоним функции modSearch num=2',
                        'modSearch3'                  => '....... синоним функции modSearch num=3',
                        'modSearch4'                  => '....... синоним функции modSearch num=4',
                        'modSearch5'                  => '....... синоним функции modSearch num=5',
                        'modSearchHistory'            => 'подключить модуль ИсторияПоиска',
                    'modRegistration'                 => 'подключить модуль ФормаРегистрации',
                        'modRegistration2'            => '....... синоним функции modRegistration num=2',
                        'modRegistration3'            => '....... синоним функции modRegistration num=3',
                        'modRegistration4'            => '....... синоним функции modRegistration num=4',
                        'modRegistration5'            => '....... синоним функции modRegistration num=5',
                    'modLogin'                        => 'подключить модуль ФормаАвторизации',
                        'modLogin2'                   => '....... синоним функции modLogin num=2',
                        'modLogin3'                   => '....... синоним функции modLogin num=3',
                        'modLogin4'                   => '....... синоним функции modLogin num=4',
                        'modLogin5'                   => '....... синоним функции modLogin num=5',
                        'modRemind'                   => 'подключить модуль ФормаВосстановленияПароля',
                    'modFeedback'                     => 'подключить модуль ФормаОбратнойСвязи',
                        'modFeedback2'                => '....... синоним функции modFeedback num=2',
                        'modFeedback3'                => '....... синоним функции modFeedback num=3',
                        'modFeedback4'                => '....... синоним функции modFeedback num=4',
                        'modFeedback5'                => '....... синоним функции modFeedback num=5',
                    'modCallme'                       => 'подключить модуль ФормаОбратногоЗвонка',
                        'modCallme2'                  => '....... синоним функции modCallme num=2',
                        'modCallme3'                  => '....... синоним функции modCallme num=3',
                        'modCallme4'                  => '....... синоним функции modCallme num=4',
                        'modCallme5'                  => '....... синоним функции modCallme num=5',
                    'modChat'                         => 'подключить модуль ФормаЧата',
                        'modChat2'                    => '....... синоним функции modChat num=2',
                        'modChat3'                    => '....... синоним функции modChat num=3',
                        'modChat4'                    => '....... синоним функции modChat num=4',
                        'modChat5'                    => '....... синоним функции modChat num=5',
                    'modCart'                         => 'подключить модуль МиниКорзина',
                        'modCart2'                    => '....... синоним функции modCart num=2',
                        'modCart3'                    => '....... синоним функции modCart num=3',
                        'modCart4'                    => '....... синоним функции modCart num=4',
                        'modCart5'                    => '....... синоним функции modCart num=5',
                        'modDefer'                    => 'подключить модуль МиниОтложенные',
                            'modDefer2'               => '....... синоним функции modDefer num=2',
                            'modDefer3'               => '....... синоним функции modDefer num=3',
                            'modDefer4'               => '....... синоним функции modDefer num=4',
                            'modDefer5'               => '....... синоним функции modDefer num=5',
                        'modCompare'                  => 'подключить модуль МиниСравнение',
                    'modBreadcrumbs'                  => 'подключить модуль ХлебныеКрошки',
                        'modBreadcrumbs2'             => '....... синоним функции modBreadcrumbs num=2',
                        'modBreadcrumbs3'             => '....... синоним функции modBreadcrumbs num=3',
                        'modBreadcrumbs4'             => '....... синоним функции modBreadcrumbs num=4',
                        'modBreadcrumbs5'             => '....... синоним функции modBreadcrumbs num=5',
                    'modSort'                         => 'подключить модуль Сортировка',
                    'modFilter'                       => 'подключить модуль Фильтр',
                        'modFilter2'                  => '....... синоним функции modFilter num=2',
                        'modFilter3'                  => '....... синоним функции modFilter num=3',
                        'modFilter4'                  => '....... синоним функции modFilter num=4',
                        'modFilter5'                  => '....... синоним функции modFilter num=5',
                    'modPagination'                   => 'подключить модуль ПагинацияСтраниц',
                    'modPhotos'                       => 'подключить модуль Фотографии',
                        'modFotos'                    => 'подключить модуль Фотографии',
                        'modImages'                   => 'подключить модуль Фотографии',
                    'modVideos'                       => 'подключить модуль Видео',
                    'modFiles'                        => 'подключить модуль ПрикрепленныеФайлы',
                    'modPrice'                        => 'подключить модуль ЦенаТовара',
                    'modProperties'                   => 'подключить модуль СвойстваТовара',
                    'modTags'                         => 'подключить модуль Теги',
                        'modTags2'                    => '....... синоним функции modTags num=2',
                        'modTags3'                    => '....... синоним функции modTags num=3',
                        'modTags4'                    => '....... синоним функции modTags num=4',
                        'modTags5'                    => '....... синоним функции modTags num=5',
                    'modComments'                     => 'подключить модуль Комментарии',
                        'modComment'                  => 'подключить модуль МиниКарточкаКомментария',
                            'modComment2'             => '....... синоним функции modComment num=2',
                            'modComment3'             => '....... синоним функции modComment num=3',
                            'modComment4'             => '....... синоним функции modComment num=4',
                            'modComment5'             => '....... синоним функции modComment num=5',
                    'modSitemap'                      => 'подключить модуль КартаСайта',
                        'modSitemap2'                 => '....... синоним функции modSitemap num=2',
                        'modSitemap3'                 => '....... синоним функции modSitemap num=3',
                        'modSitemap4'                 => '....... синоним функции modSitemap num=4',
                        'modSitemap5'                 => '....... синоним функции modSitemap num=5',
                    'modPosts'                        => 'подключить модуль Посты',
                        'modPosts2'                   => '....... синоним функции modPosts num=2',
                        'modPosts3'                   => '....... синоним функции modPosts num=3',
                        'modPosts4'                   => '....... синоним функции modPosts num=4',
                        'modPosts5'                   => '....... синоним функции modPosts num=5',
                    'modSections'                     => 'подключить модуль СпециальныеСтраницы',
                        'modSections2'                => '....... синоним функции modSections num=2',
                        'modSections3'                => '....... синоним функции modSections num=3',
                        'modSections4'                => '....... синоним функции modSections num=4',
                        'modSections5'                => '....... синоним функции modSections num=5',
                    'modCategories'                   => 'подключить модуль Категории',
                        'modCategories2'              => '....... синоним функции modCategories num=2',
                        'modCategories3'              => '....... синоним функции modCategories num=3',
                        'modCategories4'              => '....... синоним функции modCategories num=4',
                        'modCategories5'              => '....... синоним функции modCategories num=5',
                    'modBrands'                       => 'подключить модуль Бренды',
                        'modBrands2'                  => '....... синоним функции modBrands num=2',
                        'modBrands3'                  => '....... синоним функции modBrands num=3',
                        'modBrands4'                  => '....... синоним функции modBrands num=4',
                        'modBrands5'                  => '....... синоним функции modBrands num=5',
                    'modProducts'                     => 'подключить модуль Товары',
                        'modProducts2'                => '....... синоним функции modProducts num=2',
                        'modProducts3'                => '....... синоним функции modProducts num=3',
                        'modProducts4'                => '....... синоним функции modProducts num=4',
                        'modProducts5'                => '....... синоним функции modProducts num=5',
                    'modBlog'                         => 'подключить модуль Блог',
                        'modBlog2'                    => '....... синоним функции modBlog num=2',
                        'modBlog3'                    => '....... синоним функции modBlog num=3',
                        'modBlog4'                    => '....... синоним функции modBlog num=4',
                        'modBlog5'                    => '....... синоним функции modBlog num=5',
                    'modNews'                         => 'подключить модуль Новости',
                        'modNews2'                    => '....... синоним функции modNews num=2',
                        'modNews3'                    => '....... синоним функции modNews num=3',
                        'modNews4'                    => '....... синоним функции modNews num=4',
                        'modNews5'                    => '....... синоним функции modNews num=5',
                    'modReviews'                      => 'подключить модуль Обзоры',
                        'modReviews2'                 => '....... синоним функции modReviews num=2',
                        'modReviews3'                 => '....... синоним функции modReviews num=3',
                        'modReviews4'                 => '....... синоним функции modReviews num=4',
                        'modReviews5'                 => '....... синоним функции modReviews num=5',
                    'modArticles'                     => 'подключить модуль Статьи',
                        'modArticles2'                => '....... синоним функции modArticles num=2',
                        'modArticles3'                => '....... синоним функции modArticles num=3',
                        'modArticles4'                => '....... синоним функции modArticles num=4',
                        'modArticles5'                => '....... синоним функции modArticles num=5',
                    'modStocks'                       => 'подключить модуль Склады',
                        'modStocks2'                  => '....... синоним функции modStocks num=2',
                        'modStocks3'                  => '....... синоним функции modStocks num=3',
                        'modStocks4'                  => '....... синоним функции modStocks num=4',
                        'modStocks5'                  => '....... синоним функции modStocks num=5',
                    'modCountries'                    => 'подключить модуль Страны',
                        'modCountries2'               => '....... синоним функции modCountries num=2',
                        'modCountries3'               => '....... синоним функции modCountries num=3',
                        'modCountries4'               => '....... синоним функции modCountries num=4',
                        'modCountries5'               => '....... синоним функции modCountries num=5',
                    'modRegions'                      => 'подключить модуль Области',
                        'modRegions2'                 => '....... синоним функции modRegions num=2',
                        'modRegions3'                 => '....... синоним функции modRegions num=3',
                        'modRegions4'                 => '....... синоним функции modRegions num=4',
                        'modRegions5'                 => '....... синоним функции modRegions num=5',
                    'modTowns'                        => 'подключить модуль Города',
                        'modTowns2'                   => '....... синоним функции modTowns num=2',
                        'modTowns3'                   => '....... синоним функции modTowns num=3',
                        'modTowns4'                   => '....... синоним функции modTowns num=4',
                        'modTowns5'                   => '....... синоним функции modTowns num=5',
                    'modCategoriesMap'                => 'подключить модуль КартаКатегорий',
                    'modCatalog'                      => 'подключить модуль КаталогТоваров',
                    'modAnnounces'                    => 'подключить модуль АнонсовыйСписокПубликаций',
                    'modFeatureds'                    => 'подключить модуль ОсобыеТовары',
                        'modNewests'                  => 'подключить модуль Новинки',
                        'modHits'                     => 'подключить модуль ХитыПродаж',
                        'modActions'                  => 'подключить модуль Акционные',
                        'modCommenteds'               => 'подключить модуль НедавноОбсуждали',
                        'modOrdereds'                 => 'подключить модуль НедавноПокупали',
                        'modAwaiteds'                 => 'подключить модуль СкороВпродаже',
                        'modHighlighteds'             => 'подключить модуль Выделены',
                        'modRecents'                  => 'подключить модуль НедавноПросмотренные',
                    'modLinks'                        => 'подключить модуль Ссылки',
                    'modMenu'                         => 'подключить модуль Меню',
                        'modMenu2'                    => '....... синоним функции modMenu num=2',
                        'modMenu3'                    => '....... синоним функции modMenu num=3',
                        'modMenu4'                    => '....... синоним функции modMenu num=4',
                        'modMenu5'                    => '....... синоним функции modMenu num=5',
                        'modMenuLanguages'            => 'подключить модуль МенюЯзыковСайта',
                    'modAside'                        => 'подключить модуль БоковаяПанель',
                        'modAside2'                   => '....... синоним функции modAside num=2',
                        'modAside3'                   => '....... синоним функции modAside num=3',
                        'modAside4'                   => '....... синоним функции modAside num=4',
                        'modAside5'                   => '....... синоним функции modAside num=5',
                    'modAsideTop'                     => 'подключить модуль БоковаяПанельВерхняя',
                        'modAsideRight'               => 'подключить модуль БоковаяПанельПравая',
                        'modAsideBottom'              => 'подключить модуль БоковаяПанельНижняя',
                        'modAsideLeft'                => 'подключить модуль БоковаяПанельЛевая',
                        'modAsideCenter'              => 'подключить модуль БоковаяПанельЦентральная',
                    'modSidebar'                      => 'подключить модуль БоковаяПанель',
                        'modSidebar2'                 => '....... синоним функции modSidebar num=2',
                        'modSidebar3'                 => '....... синоним функции modSidebar num=3',
                        'modSidebar4'                 => '....... синоним функции modSidebar num=4',
                        'modSidebar5'                 => '....... синоним функции modSidebar num=5',
                    'modPost'                         => 'подключить модуль МиниКарточкаПоста',
                        'modPost2'                    => '....... синоним функции modPost num=2',
                        'modPost3'                    => '....... синоним функции modPost num=3',
                        'modPost4'                    => '....... синоним функции modPost num=4',
                        'modPost5'                    => '....... синоним функции modPost num=5',
                    'modCategory'                     => 'подключить модуль МиниКарточкаКатегории',
                        'modCategory2'                => '....... синоним функции modCategory num=2',
                        'modCategory3'                => '....... синоним функции modCategory num=3',
                        'modCategory4'                => '....... синоним функции modCategory num=4',
                        'modCategory5'                => '....... синоним функции modCategory num=5',
                    'modBrand'                        => 'подключить модуль МиниКарточкаБренда',
                        'modBrand2'                   => '....... синоним функции modBrand num=2',
                        'modBrand3'                   => '....... синоним функции modBrand num=3',
                        'modBrand4'                   => '....... синоним функции modBrand num=4',
                        'modBrand5'                   => '....... синоним функции modBrand num=5',
                    'modProduct'                      => 'подключить модуль МиниКарточкаТовара',
                        'modProduct2'                 => '....... синоним функции modProduct num=2',
                        'modProduct3'                 => '....... синоним функции modProduct num=3',
                        'modProduct4'                 => '....... синоним функции modProduct num=4',
                        'modProduct5'                 => '....... синоним функции modProduct num=5',
                    'modNewsItem'                     => 'подключить модуль МиниКарточкаНовости',
                        'modNewsItem2'                => '....... синоним функции modNewsItem num=2',
                        'modNewsItem3'                => '....... синоним функции modNewsItem num=3',
                        'modNewsItem4'                => '....... синоним функции modNewsItem num=4',
                        'modNewsItem5'                => '....... синоним функции modNewsItem num=5',
                    'modArticle'                      => 'подключить модуль МиниКарточкаСтатьи',
                        'modArticle2'                 => '....... синоним функции modArticle num=2',
                        'modArticle3'                 => '....... синоним функции modArticle num=3',
                        'modArticle4'                 => '....... синоним функции modArticle num=4',
                        'modArticle5'                 => '....... синоним функции modArticle num=5',
                    'modStock'                        => 'подключить модуль МиниКарточкаСклада',
                        'modStock2'                   => '....... синоним функции modStock num=2',
                        'modStock3'                   => '....... синоним функции modStock num=3',
                        'modStock4'                   => '....... синоним функции modStock num=4',
                        'modStock5'                   => '....... синоним функции modStock num=5',
                    'modCountry'                      => 'подключить модуль МиниКарточкаСтраны',
                        'modCountry2'                 => '....... синоним функции modCountry num=2',
                        'modCountry3'                 => '....... синоним функции modCountry num=3',
                        'modCountry4'                 => '....... синоним функции modCountry num=4',
                        'modCountry5'                 => '....... синоним функции modCountry num=5',
                    'modRegion'                       => 'подключить модуль МиниКарточкаОбласти',
                        'modRegion2'                  => '....... синоним функции modRegion num=2',
                        'modRegion3'                  => '....... синоним функции modRegion num=3',
                        'modRegion4'                  => '....... синоним функции modRegion num=4',
                        'modRegion5'                  => '....... синоним функции modRegion num=5',
                    'modTown'                         => 'подключить модуль МиниКарточкаГорода',
                        'modTown2'                    => '....... синоним функции modTown num=2',
                        'modTown3'                    => '....... синоним функции modTown num=3',
                        'modTown4'                    => '....... синоним функции modTown num=4',
                        'modTown5'                    => '....... синоним функции modTown num=5',
                    'modAccount'                      => 'подключить модуль ЛичныйКабинет',
                        'modAccountInfo'              => 'подключить модуль ЛичныйКабинетИнформация',
                        'modAccountMenu'              => 'подключить модуль ЛичныйКабинетМеню',
                        'modAccountProfile'           => 'подключить модуль ЛичныйКабинетПрофиль',
                        'modAccountOrders'            => 'подключить модуль ЛичныйКабинетЗаказы',
                        'modAccountFavorites'         => 'подключить модуль ЛичныйКабинетИзбранное',
                        'modAccountSections'          => 'подключить модуль ЛичныйКабинетСтраницы',
                        'modAccountCategories'        => 'подключить модуль ЛичныйКабинетКатегории',
                        'modAccountBrands'            => 'подключить модуль ЛичныйКабинетБренды',
                        'modAccountProducts'          => 'подключить модуль ЛичныйКабинетТовары',
                        'modAccountNews'              => 'подключить модуль ЛичныйКабинетНовости',
                        'modAccountArticles'          => 'подключить модуль ЛичныйКабинетСтатьи',
                        'modAccountPosts'             => 'подключить модуль ЛичныйКабинетПубликации',
                        'modAccountComments'          => 'подключить модуль ЛичныйКабинетКомментарии',
                        'modAccountFeedback'          => 'подключить модуль ЛичныйКабинетПереписка',
                        'modAccountBalance'           => 'подключить модуль ЛичныйКабинетБалансе',
                    'modDashboard'                    => 'подключить модуль ОбщийМакетАдминпанели',
                        'modDashboard2'               => '....... синоним функции modDashboard num=2',
                        'modDashboard3'               => '....... синоним функции modDashboard num=3',
                        'modDashboard4'               => '....... синоним функции modDashboard num=4',
                        'modDashboard5'               => '....... синоним функции modDashboard num=5',
                    'modAbstract'                     => 'подключить модуль АбстрактнаяСтраница',
                    'modBox'                          => 'подключить модуль НекийБлок',
                        'modBox2'                     => '....... синоним функции modBox num=2',
                        'modBox3'                     => '....... синоним функции modBox num=3',
                        'modBox4'                     => '....... синоним функции modBox num=4',
                        'modBox5'                     => '....... синоним функции modBox num=5',
                    'modAnnotation'                   => 'подключить модуль АннотацияСтраницы',
                    'modBody'                         => 'подключить модуль ОписаниеСтраницы',
                        'modBody2'                    => '....... синоним функции modBody num=2',
                        'modBody3'                    => '....... синоним функции modBody num=3',
                        'modBody4'                    => '....... синоним функции modBody num=4',
                        'modBody5'                    => '....... синоним функции modBody num=5',
                        'modDemoBody'                 => 'подключить модуль ДЕМО ОписаниеСтраницы',
                            'modDemoBody2'            => '....... синоним функции modDemoBody num=2',
                            'modDemoBody3'            => '....... синоним функции modDemoBody num=3',
                            'modDemoBody4'            => '....... синоним функции modDemoBody num=4',
                            'modDemoBody5'            => '....... синоним функции modDemoBody num=5',
                            'modDemoBody6'            => '....... синоним функции modDemoBody num=6',
                            'modDemoBody7'            => '....... синоним функции modDemoBody num=7',
                            'modDemoBody8'            => '....... синоним функции modDemoBody num=8',
                            'modDemoBody9'            => '....... синоним функции modDemoBody num=9',
                            'modDemoBody10'           => '....... синоним функции modDemoBody num=10',
                    'modSeo'                          => 'подключить модуль SeoТекстСтраницы',
                        'modSeo2'                     => '....... синоним функции modSeo num=2',
                        'modSeo3'                     => '....... синоним функции modSeo num=3',
                        'modSeo4'                     => '....... синоним функции modSeo num=4',
                        'modSeo5'                     => '....... синоним функции modSeo num=5',
                    'modSocials'                      => 'подключить модуль СоциальныеСсылки',
                        'modSocials2'                 => '....... синоним функции modSocials num=2',
                        'modSocials3'                 => '....... синоним функции modSocials num=3',
                        'modSocials4'                 => '....... синоним функции modSocials num=4',
                        'modSocials5'                 => '....... синоним функции modSocials num=5',
                    'modCopyright'                    => 'подключить модуль Копирайт',
                        'modCopyright2'               => '....... синоним функции modCopyright num=2',
                        'modCopyright3'               => '....... синоним функции modCopyright num=3',
                        'modCopyright4'               => '....... синоним функции modCopyright num=4',
                        'modCopyright5'               => '....... синоним функции modCopyright num=5',
                    'modFAQ'                          => 'подключить модуль ЧастыеВопросыОтветы',
                        'modFAQ2'                     => '....... синоним функции modFAQ num=2',
                        'modFAQ3'                     => '....... синоним функции modFAQ num=3',
                        'modFAQ4'                     => '....... синоним функции modFAQ num=4',
                        'modFAQ5'                     => '....... синоним функции modFAQ num=5',
                    'modInform'                       => 'подключить модуль Информер',
                        'modInformer'                 => 'подключить модуль Информер',
                        'modTracker'                  => 'подключить модуль Трекер',
                    'modInfo'                         => 'подключить модуль Информация',
                        'modInfo2'                    => '....... синоним функции modInfo num=2',
                        'modInfo3'                    => '....... синоним функции modInfo num=3',
                        'modInfo4'                    => '....... синоним функции modInfo num=4',
                        'modInfo5'                    => '....... синоним функции modInfo num=5',
                    'modTaxonomy'                     => 'подключить модуль Таксономия',
                    'modNone'                         => 'подключить модуль НетЗаписей',
                        'modNone2'                    => '....... синоним функции modNone num=2',
                        'modNone3'                    => '....... синоним функции modNone num=3',
                        'modNone4'                    => '....... синоним функции modNone num=4',
                        'modNone5'                    => '....... синоним функции modNone num=5',
                    'modForm'                         => 'подключить модуль ФормаВвода',
                        'modForm2'                    => '....... синоним функции modForm num=2',
                        'modForm3'                    => '....... синоним функции modForm num=3',
                        'modForm4'                    => '....... синоним функции modForm num=4',
                        'modForm5'                    => '....... синоним функции modForm num=5',
                    'modSignpost'                     => 'подключить модуль РазводкаПоСтраницамСайта',
                        'modSignpost2'                => '....... синоним функции modSignpost num=2',
                        'modSignpost3'                => '....... синоним функции modSignpost num=3',
                        'modSignpost4'                => '....... синоним функции modSignpost num=4',
                        'modSignpost5'                => '....... синоним функции modSignpost num=5',
                    'modLanguages'                    => 'подключить модуль ЯзыкиСайта',
                    'modSettings'                     => 'подключить модуль КакиеТоНастройки',
                        'modSettings2'                => '....... синоним функции modSettings num=2',
                        'modSettings3'                => '....... синоним функции modSettings num=3',
                        'modSettings4'                => '....... синоним функции modSettings num=4',
                        'modSettings5'                => '....... синоним функции modSettings num=5',
                    'modPolyglot'                     => 'подключить модуль МногоязычныйСловарь',
                        'modPolyglot2'                => '....... синоним функции modPolyglot num=2',
                        'modPolyglot3'                => '....... синоним функции modPolyglot num=3',
                        'modPolyglot4'                => '....... синоним функции modPolyglot num=4',
                        'modPolyglot5'                => '....... синоним функции modPolyglot num=5',
                        'modPolyglotHeader'           => 'подключить модуль СловарьДляШапки',
                        'modPolyglotFooter'           => 'подключить модуль СловарьДляПодвала',
                        'modPolyglotError404'         => 'подключить модуль СловарьДляОшибка404',
                        'modPolyglotHome'             => 'подключить модуль СловарьДляГлавной',
                        'modPolyglotCallme'           => 'подключить модуль СловарьДляОбратногоЗвонка',
                        'modPolyglotFeedback'         => 'подключить модуль СловарьДляОбратнойСвязи',
                        'modPolyglotLogin'            => 'подключить модуль СловарьДляАвторизации',
                        'modPolyglotRegistration'     => 'подключить модуль СловарьДляРегистрации',
                        'modPolyglotCart'             => 'подключить модуль СловарьДляКорзины',
                        'modPolyglotAccount'          => 'подключить модуль СловарьДляКабинета',
                        'modPolyglotSections'         => 'подключить модуль СловарьДляСтраниц',
                        'modPolyglotTopics'           => 'подключить модуль СловарьДляРазделов',
                        'modPolyglotCategories'       => 'подключить модуль СловарьДляКатегорий',
                        'modPolyglotBrands'           => 'подключить модуль СловарьДляБрендов',
                        'modPolyglotProducts'         => 'подключить модуль СловарьДляТоваров',
                        'modPolyglotNews'             => 'подключить модуль СловарьДляНовостей',
                        'modPolyglotBlog'             => 'подключить модуль СловарьДляБлога',
                        'modPolyglotArticles'         => 'подключить модуль СловарьДляСтатей',
                        'modPolyglotReviews'          => 'подключить модуль СловарьДляОбзоров',
                        'modPolyglotStocks'           => 'подключить модуль СловарьДляСкладов',
                        'modPolyglotCountries'        => 'подключить модуль СловарьДляСтран',
                        'modPolyglotRegions'          => 'подключить модуль СловарьДляОбластей',
                        'modPolyglotTowns'            => 'подключить модуль СловарьДляГородов',
                    'modAntispam'                     => 'подключить модуль Антиспам',
                    'modFixes'                        => 'подключить модуль КакиеТоИсправленияБагов',
                        'modFixes2'                   => '....... синоним функции modFixes num=2',
                        'modFixes3'                   => '....... синоним функции modFixes num=3',
                        'modFixes4'                   => '....... синоним функции modFixes num=4',
                        'modFixes5'                   => '....... синоним функции modFixes num=5',
                    'modAdminTools'                   => 'подключить модуль ИнструментыАдмина',
                        'modAdminTools2'              => '....... синоним функции modAdminTools num=2',
                        'modAdminTools3'              => '....... синоним функции modAdminTools num=3',
                        'modAdminTools4'              => '....... синоним функции modAdminTools num=4',
                        'modAdminTools5'              => '....... синоним функции modAdminTools num=5',
                        'modAdminUpload'              => 'подключить модуль ИнструментыАдмина-ЗагрузитьФайл',
                            'modAdminUpload2'         => '....... синоним функции modAdminUpload num=2',
                            'modAdminUpload3'         => '....... синоним функции modAdminUpload num=3',
                            'modAdminUpload4'         => '....... синоним функции modAdminUpload num=4',
                            'modAdminUpload5'         => '....... синоним функции modAdminUpload num=5',
                        'modAdminEdit'                => 'подключить модуль ИнструментыАдмина-Редактировать',
                            'modAdminEdit2'           => '....... синоним функции modAdminEdit num=2',
                            'modAdminEdit3'           => '....... синоним функции modAdminEdit num=3',
                            'modAdminEdit4'           => '....... синоним функции modAdminEdit num=4',
                            'modAdminEdit5'           => '....... синоним функции modAdminEdit num=5',
                    'modStarter'                      => 'подключить модуль Стартер',
                        'modInit'                     => 'подключить модуль Инициализация',
                        'modLayout'                   => 'подключить модуль МакетБлока',
                            'modLayout2'              => '....... синоним функции modLayout num=2',
                            'modLayout3'              => '....... синоним функции modLayout num=3',
                            'modLayout4'              => '....... синоним функции modLayout num=4',
                            'modLayout5'              => '....... синоним функции modLayout num=5',
                    'modTools'                        => 'подключить модуль ИнструментыСтраницы',
                        'modTools2'                   => '....... синоним функции modTools num=2',
                        'modTools3'                   => '....... синоним функции modTools num=3',
                        'modTools4'                   => '....... синоним функции modTools num=4',
                        'modTools5'                   => '....... синоним функции modTools num=5',
                    'modEmail'                        => 'подключить модуль МакетПисьма',
                        'modEmail2'                   => '....... синоним функции modEmail num=2',
                        'modEmail3'                   => '....... синоним функции modEmail num=3',
                        'modEmail4'                   => '....... синоним функции modEmail num=4',
                        'modEmail5'                   => '....... синоним функции modEmail num=5',
                    'modSms'                          => 'подключить модуль МакетСМС',
                        'modSms2'                     => '....... синоним функции modSms num=2',
                        'modSms3'                     => '....... синоним функции modSms num=3',
                        'modSms4'                     => '....... синоним функции modSms num=4',
                        'modSms5'                     => '....... синоним функции modSms num=5',
                    'modModal'                        => 'подключить модуль МодальноеОкно',
                        'modModal2'                   => '....... синоним функции modModal num=2',
                        'modModal3'                   => '....... синоним функции modModal num=3',
                        'modModal4'                   => '....... синоним функции modModal num=4',
                        'modModal5'                   => '....... синоним функции modModal num=5',
                    'modStyles'                       => 'подключить модуль ДизайнерскиеСтили',
                    'modScripts'                      => 'подключить модуль КлиентскиеСкрипты',
                    'modSubdomain'                    => 'подключить модуль ТекстСубдоменаСтраницы',
                    'modUseVariable'                  => 'подключить модуль ОпределитьРабочуюПеременную',
                    'modDemo'                         => 'подключить модуль Демо',
                        'modDemo2'                    => '....... синоним функции modDemo num=2',
                        'modDemo3'                    => '....... синоним функции modDemo num=3',
                        'modDemo4'                    => '....... синоним функции modDemo num=4',
                        'modDemo5'                    => '....... синоним функции modDemo num=5',
                            'modDemoHome'             => 'подключить модуль ДемоДляГлавной',
                            'modDemoSection'          => 'подключить модуль ДемоДляСтатической',
                            'modDemoCategory'         => 'подключить модуль ДемоДляКатегории',
                            'modDemoBrand'            => 'подключить модуль ДемоДляБренда',
                            'modDemoProduct'          => 'подключить модуль ДемоДляТовара',
                            'modDemoNewsItem'         => 'подключить модуль ДемоДляНовости',
                            'modDemoArticle'          => 'подключить модуль ДемоДляСтатьи',
                            'modDemoStock'            => 'подключить модуль ДемоДляСклада',
                            'modDemoCountry'          => 'подключить модуль ДемоДляСтраны',
                            'modDemoRegion'           => 'подключить модуль ДемоДляРегиона',
                            'modDemoTown'             => 'подключить модуль ДемоДляГорода',
                            'modDemoMedia'            => 'подключить модуль ДемоДляМедиафайла',
                        'modDemoRemark'               => 'подключить модуль ДЕМО РемаркаОшаблоне',
                        'modDemoThemes'               => 'подключить модуль ДЕМО ДругиеШаблоны',
                '----- ЭТО БЛОЧНЫЕ ФУНКЦИИ -----'     => 'они открываются и закрываются таким же тегом, действуя на весь блок между тегами'
            ));

            /* регистрируем блочные теги библиотеки ShortCodes */
            $this->expandSmarty(array(
                'enum'                                 => 'перечислить элементы массива',
                    'enumSections'                     => 'перечислить элементы массива специальных страниц',
                    'enumCategories'                   => 'перечислить элементы массива категорий',
                        'enumPath'                     => 'перечислить элементы массива хлебных крошек',
                        'enumSubcats'                  => 'перечислить элементы массива подкатегорий текущей категории',
                        'enumProperties'               => 'перечислить элементы массива свойств',
                            'enumOptions'              => 'перечислить элементы массива значений свойства',
                    'enumBrands'                       => 'перечислить элементы массива брендов',
                    'enumCatalog'                      => 'перечислить элементы массива каталога',
                    'enumProducts'                     => 'перечислить элементы массива товаров',
                        'enumAssortment'               => 'перечислить элементы массива прикрепленных товаров',
                            'enumVariants'             => 'перечислить элементы массива вариантов товара',
                            'enumPhotos'               => 'перечислить элементы массива фотографий товара',
                                'enumImages'           => 'перечислить элементы массива фотографий записи',
                            'enumSpecifications'       => 'перечислить элементы массива свойств товара',
                                'enumSpecies'          => 'перечислить элементы массива значений свойства товара',
                        'enumRelateds'                 => 'перечислить элементы массива связанных товаров',
                        'enumAccessories'              => 'перечислить элементы массива аксессуаров',
                        'enumFeatureds'                => 'перечислить элементы массивов особых товаров',
                            'enumNewests'              => 'перечислить элементы массива новинок',
                            'enumHits'                 => 'перечислить элементы массива хитов продаж',
                            'enumActions'              => 'перечислить элементы массива акционных товаров',
                            'enumCommenteds'           => 'перечислить элементы массива недавно обсуждавшихся товаров',
                            'enumOrdereds'             => 'перечислить элементы массива недавно покупавшихся товаров',
                            'enumAwaiteds'             => 'перечислить элементы массива ожидаемых товаров',
                        'enumAnnounces'                => 'перечислить элементы массивов анонсовых публикаций',
                        'enumRecents'                  => 'перечислить элементы массива недавно просмотренных страниц',
                        'enumSearch'                   => 'перечислить элементы массива поиска',
                    'enumNews'                         => 'перечислить элементы массива новостей',
                    'enumArticles'                     => 'перечислить элементы массива статей',
                    'enumStocks'                       => 'перечислить элементы массива складов',
                    'enumCountries'                    => 'перечислить элементы массива стран',
                        'enumRegions'                  => 'перечислить элементы массива областей',
                        'enumTowns'                    => 'перечислить элементы массива городов',
                    'enumDeliveries'                   => 'перечислить элементы массива способов доставки',
                    'enumPayments'                     => 'перечислить элементы массива способов оплаты',
                    'enumComments'                     => 'перечислить элементы массива комментариев',
                    'enumCurrencies'                   => 'перечислить элементы массива валют',
                    'enumOrders'                       => 'перечислить элементы массива заказов',
                    'enumCart'                         => 'перечислить элементы массива корзины',
                        'enumDefer'                    => 'перечислить элементы массива отложенных товаров',
                        'enumOrder'                    => 'перечислить элементы массива товаров заказа',
                    'enumPages'                        => 'перечислить элементы массива пагинации',
                        'enumSorts'                    => 'перечислить элементы массива сортировок',
                        'enumSizes'                    => 'перечислить элементы массива размеров страницы',
                    'enumLanguages'                    => 'перечислить элементы массива языков',
                    'enumThemes'                       => 'перечислить элементы массива дизайнов',
                    'enumFunctions'                    => 'перечислить имена функций хелпера',
                'ifExistsLogin'                        => 'если уже существует такой аккаунт',
                    'ifNotExistsLogin'                 => '....... синоним функции ifExistsLogin not=TRUE',
                    'ifExistsNick'                     => 'если уже существует аккаунт с таким ником',
                        'ifNotExistsNick'              => '....... синоним функции ifExistsNick not=TRUE',
                    'ifExistsCard'                     => 'если уже существует аккаунт с такой картой клиента (ником)',
                        'ifNotExistsCard'              => '....... синоним функции ifExistsCard not=TRUE',
                        'ifSolventCard'                => 'если аккаунт с такой картой клиента (ником) платежеспособный',
                            'ifNotSolventCard'         => '....... синоним функции ifSolventCard not=TRUE',
                'ifDemo'                               => 'если находимся в демо режиме',
                    'ifNotDemo'                        => '....... синоним функции ifDemo not=TRUE',
                    'ifDemoOrAdmin'                    => 'если находимся в демо режиме или на сайте админ',
                        'ifNotDemoOrAdmin'             => '....... синоним функции ifDemoOrAdmin not=TRUE',
                    'ifAjax'                           => 'если страница вызвана в режиме динамической подгрузки',
                        'ifNotAjax'                    => '....... синоним функции ifAjax not=TRUE',
                    'ifAdmin'                          => 'если на сайте админ',
                        'ifNotAdmin'                   => '....... синоним функции ifAdmin not=TRUE',
                        'ifUser'                       => 'если на сайте авторизованный клиент',
                            'ifNotUser'                => '....... синоним функции ifUser not=TRUE',
                            'ifAdminOrUser'            => 'если на сайте админ или авторизованный клиент',
                                'ifNotAdminOrUser'     => '....... синоним функции ifAdminOrUser not=TRUE',
                        'ifOwner'                      => 'если на сайте клиент управляющий записью',
                            'ifNotOwner'               => '....... синоним функции ifOwner not=TRUE',
                            'ifAdminOrOwner'           => 'если на сайте админ или клиент управляющий записью',
                                'ifNotAdminOrOwner'    => '....... синоним функции ifAdminOrOwner not=TRUE',
                            'ifDemoOrAdminOrOwner'     => 'если находимся в демо режиме или на сайте админ или клиент управляющий записью',
                                'ifNotDemoOrAdminOrOwner' => '....... синоним функции ifDemoOrAdminOrOwner not=TRUE',
                        'ifVisitorIp'                  => 'если это посетитель с известным IP-адресом',
                            'ifNotVisitorIp'           => '....... синоним функции ifVisitorIp not=TRUE',
                        'ifVisitorHost'                => 'если это посетитель с известным хостом',
                            'ifNotVisitorHost'         => '....... синоним функции ifVisitorHost not=TRUE',
                        'ifViaProxy'                   => 'если это посетитель через прокси',
                            'ifNotViaProxy'            => '....... синоним функции ifViaProxy not=TRUE',
                        'ifReferer'                    => 'если это посетитель известно откуда',
                            'ifNotReferer'             => '....... синоним функции ifReferer not=TRUE',
                            'ifRefererQuery'           => 'если есть GET-апарметры среди данных откуда посетитель',
                                'ifNotRefererQuery'    => '....... синоним функции ifRefererQuery not=TRUE',
                        'ifRefererDomain'              => 'если это посетитель известно с какого домена',
                            'ifNotRefererDomain'       => '....... синоним функции ifRefererDomain not=TRUE',
                        'ifRefererSERP'                => 'если это посетитель с какого-то поиска',
                            'ifNotRefererSERP'         => '....... синоним функции ifRefererSERP not=TRUE',
                        'ifRefererGoogle'              => 'если это посетитель из поиска Google',
                            'ifNotRefererGoogle'       => '....... синоним функции ifRefererGoogle not=TRUE',
                        'ifRefererYandex'              => 'если это посетитель из поиска Яндекса',
                            'ifNotRefererYandex'       => '....... синоним функции ifRefererYandex not=TRUE',
                        'ifQueryRequest'               => 'если в запросе страницы есть REQUEST-параметры',
                            'ifNotQueryRequest'        => '....... синоним функции ifQueryRequest not=TRUE',
                        'ifQueryGet'                   => 'если в запросе страницы есть GET-параметры',
                            'ifNotQueryGet'            => '....... синоним функции ifQueryGet not=TRUE',
                        'ifQueryPost'                  => 'если в запросе страницы есть POST-параметры',
                            'ifNotQueryPost'           => '....... синоним функции ifQueryPost not=TRUE',
                        'ifQueryFiles'                 => 'если в запросе страницы есть FILES-параметры',
                            'ifNotQueryFiles'          => '....... синоним функции ifQueryFiles not=TRUE',
                        'ifCookies'                    => 'если в запросе страницы есть COOKIE-параметры',
                            'ifNotCookies'             => '....... синоним функции ifCookies not=TRUE',
                        'ifSession'                    => 'если есть SESSION-параметры',
                            'ifNotSession'             => '....... синоним функции ifSession not=TRUE',
                        'ifServer'                     => 'если есть SERVER-параметры',
                            'ifNotServer'              => '....... синоним функции ifServer not=TRUE',
                    'ifEnumFirst'                      => 'если это первая итерация enum-функции',
                        'ifEnumAfter'                  => 'если это последующая итерация enum-функции',
                        'ifEnumEven'                   => 'если это четная (2-кратная) итерация enum-функции',
                        'ifEnumOdd'                    => 'если это нечетная итерация enum-функции',
                        'ifEnumTriple'                 => 'если это 3-кратная итерация enum-функции',
                        'ifEnumQuart'                  => 'если это 4-кратная итерация enum-функции',
                        'ifEnumQuint'                  => 'если это 5-кратная итерация enum-функции',
                    'ifOwnBlock'                       => 'если категория имеет свой блок на главной',
                        'ifNotOwnBlock'                => '....... синоним функции ifOwnBlock not=TRUE',
                    'ifActual'                         => 'если категорию/бренд или свойство/значение можно показывать',
                        'ifNotActual'                  => '....... синоним функции ifActual not=TRUE',
                    'ifCommented'                      => 'если запись разрешена к обсуждению',
                        'ifNotCommented'               => '....... синоним функции ifCommented not=TRUE',
                    'ifComments'                       => 'если доступны комментарии записи',
                        'ifNotComments'                => '....... синоним функции ifComments not=TRUE',
                    'ifItems'                          => 'если доступны записи',
                        'ifNotItems'                   => '....... синоним функции ifItems not=TRUE',
                        'ifItem'                       => 'если доступна запись',
                            'ifNotItem'                => '....... синоним функции ifItem not=TRUE',
                        'ifValue'                      => 'если доступна переменная value',
                            'ifNotValue'               => '....... синоним функции ifValue not=TRUE',
                        'ifHtmlClass'                  => 'если доступен класс тега HTML',
                            'ifNotHtmlClass'           => '....... синоним функции ifHtmlClass not=TRUE',
                            'ifChildClass'             => 'если доступно название дочернего класса',
                                'ifNotChildClass'      => '....... синоним функции ifChildClass not=TRUE',
                            'ifHtmlUrl'                => 'если доступен URL для некоторого HTML',
                                'ifNotHtmlUrl'         => '....... синоним функции ifHtmlUrl not=TRUE',
                            'ifOpenGraphImage'         => 'если доступен URL скриншота для OpenGraph разметки',
                                'ifNotOpenGraphImage'  => '....... синоним функции ifOpenGraphImage not=TRUE',
                        'ifType'                       => 'если доступна переменная type',
                            'ifNotType'                => '....... синоним функции ifType not=TRUE',
                        'ifSum'                        => 'если переменная sum больше нуля',
                            'ifNotSum'                 => '....... синоним функции ifSum not=TRUE',
                        'ifSection'                    => 'если доступна специальная страница',
                            'ifNotSection'             => '....... синоним функции ifSection not=TRUE',
                        'ifCategory'                   => 'если доступна категория',
                            'ifNotCategory'            => '....... синоним функции ifCategory not=TRUE',
                        'ifBrand'                      => 'если доступен бренд',
                            'ifNotBrand'               => '....... синоним функции ifBrand not=TRUE',
                            'ifBrandOrCategory'        => 'если доступен бренд или категория',
                                'ifNotBrandOrCategory' => '....... синоним функции ifBrandOrCategory not=TRUE',
                            'ifInformative'            => 'если категория или бренд является информативной страницей',
                                'ifNotInformative'     => '....... синоним функции ifInformative not=TRUE',
                        'ifProduct'                    => 'если доступен товар',
                            'ifNotProduct'             => '....... синоним функции ifProduct not=TRUE',
                            'ifSpecifications'         => 'если доступен массив свойств товара',
                                'ifNotSpecifications'  => '....... синоним функции ifSpecifications not=TRUE',
                        'ifNewsItem'                   => 'если доступна новость',
                            'ifNotNewsItem'            => '....... синоним функции ifNewsItem not=TRUE',
                        'ifArticle'                    => 'если доступна статья',
                            'ifNotArticle'             => '....... синоним функции ifArticle not=TRUE',
                        'ifStock'                      => 'если доступен склад',
                            'ifNotStock'               => '....... синоним функции ifStock not=TRUE',
                        'ifCountry'                    => 'если доступна страна',
                            'ifNotCountry'             => '....... синоним функции ifCountry not=TRUE',
                            'ifRegion'                 => 'если доступна область',
                                'ifNotRegion'          => '....... синоним функции ifRegion not=TRUE',
                            'ifTown'                   => 'если доступен город',
                                'ifNotTown'            => '....... синоним функции ifTown not=TRUE',
                        'ifOrders'                     => 'если доступен список заказов',
                            'ifNotOrders'              => '....... синоним функции ifOrders not=TRUE',
                            'ifOrder'                  => 'если доступен заказ',
                                'ifNotOrder'           => '....... синоним функции ifOrder not=TRUE',
                            'ifPaid'                   => 'если заказ оплачен',
                                'ifNotPaid'            => '....... синоним функции ifPaid not=TRUE',
                        'ifPost'                       => 'если доступна запись в переменной post',
                            'ifNotPost'                => '....... синоним функции ifPost not=TRUE',
                    'ifSections'                       => 'если доступны специальные страницы',
                        'ifNotSections'                => '....... синоним функции ifSections not=TRUE',
                    'ifCategories'                     => 'если доступны категории',
                        'ifNotCategories'              => '....... синоним функции ifCategories not=TRUE',
                    'ifBrands'                         => 'если доступны бренды',
                        'ifNotBrands'                  => '....... синоним функции ifBrands not=TRUE',
                    'ifProducts'                       => 'если доступны товары',
                        'ifNotProducts'                => '....... синоним функции ifProducts not=TRUE',
                        'ifProductsOrAnnounces'        => 'если доступны товары или анонсовые публикации',
                            'ifNotProductsOrAnnounces' => '....... синоним функции ifProductsOrAnnounces not=TRUE',
                        'ifFeatureds'                  => 'если доступны особые товары',
                            'ifNotFeatureds'           => '....... синоним функции ifFeatureds not=TRUE',
                        'ifSearch'                     => 'если доступны искомые товары',
                            'ifNotSearch'              => '....... синоним функции ifSearch not=TRUE',
                        'ifPhoto'                      => 'если доступна хотя бы 1 фотография товара',
                            'ifNotPhoto'               => '....... синоним функции ifPhoto not=TRUE',
                        'ifPhotos'                     => 'если доступно более 1 фотографии товара',
                            'ifNotPhotos'              => '....... синоним функции ifPhotos not=TRUE',
                        'ifImage'                      => 'если доступна хотя бы 1 фотография записи',
                            'ifNotImage'               => '....... синоним функции ifImage not=TRUE',
                        'ifImages'                     => 'если доступно более 1 фотографии записи',
                            'ifNotImages'              => '....... синоним функции ifImages not=TRUE',
                        'ifAttachment'                 => 'если доступен хотя бы 1 прикрепленный файл записи',
                            'ifNotAttachment'          => '....... синоним функции ifAttachment not=TRUE',
                        'ifAttachments'                => 'если доступно более 1 прикрепленного файла записи',
                            'ifNotAttachments'         => '....... синоним функции ifAttachments not=TRUE',
                        'ifVariants'                   => 'если доступно более 1 варианта товара',
                            'ifNotVariants'            => '....... синоним функции ifVariants not=TRUE',
                            'ifPrice'                  => 'если указана не нулевая цена варианта товара',
                                'ifNotPrice'           => '....... синоним функции ifPrice not=TRUE',
                    'ifNews'                           => 'если доступны новости',
                        'ifNotNews'                    => '....... синоним функции ifNews not=TRUE',
                    'ifArticles'                       => 'если доступны статьи',
                        'ifNotArticles'                => '....... синоним функции ifArticles not=TRUE',
                        'ifNewsOrArticles'             => 'если доступны новости или статьи',
                            'ifNotNewsOrArticles'      => '....... синоним функции ifNewsOrArticles not=TRUE',
                    'ifStocks'                         => 'если доступны склады',
                        'ifNotStocks'                  => '....... синоним функции ifStocks not=TRUE',
                    'ifCountries'                      => 'если доступны страны',
                        'ifNotCountries'               => '....... синоним функции ifCountries not=TRUE',
                        'ifRegions'                    => 'если доступны области',
                            'ifNotRegions'             => '....... синоним функции ifRegions not=TRUE',
                        'ifTowns'                      => 'если доступны города',
                            'ifNotTowns'               => '....... синоним функции ifTowns not=TRUE',
                    'ifProperties'                     => 'если доступны свойства фильтра',
                        'ifNotProperties'              => '....... синоним функции ifProperties not=TRUE',
                    'ifDeliveries'                     => 'если доступны способы доставки',
                        'ifNotDeliveries'              => '....... синоним функции ifDeliveries not=TRUE',
                        'ifActiveDelivery'             => 'если способ доставки активный',
                            'ifNotActiveDelivery'      => '....... синоним функции ifActiveDelivery not=TRUE',
                    'ifPayments'                       => 'если доступны способы оплаты',
                        'ifNotPayments'                => '....... синоним функции ifPayments not=TRUE',
                        'ifActivePayment'              => 'если способ оплаты активный',
                            'ifNotActivePayment'       => '....... синоним функции ifActivePayment not=TRUE',
                    'ifCurrencies'                     => 'если доступно более 1 валюты',
                        'ifNotCurrencies'              => '....... синоним функции ifCurrencies not=TRUE',
                    'ifCompare'                        => 'если доступны товары сравнения',
                        'ifNotCompare'                 => '....... синоним функции ifCompare not=TRUE',
                    'ifCart'                           => 'если доступны товары корзины',
                        'ifNotCart'                    => '....... синоним функции ifCart not=TRUE',
                        'ifCartDiscount'               => 'если в корзине есть скидка',
                            'ifNotCartDiscount'        => '....... синоним функции ifCartDiscount not=TRUE',
                    'ifDefer'                          => 'если доступны отложенные товары',
                        'ifNotDefer'                   => '....... синоним функции ifDefer not=TRUE',
                    'ifHasPolyglot'                    => 'если определен многоязычный словарь',
                        'ifHasNotPolyglot'             => '....... синоним функции ifHasPolyglot not=TRUE',
                    'ifHasBranch'                      => 'если категория/бренд имеет вложенные',
                        'ifHasNotBranch'               => '....... синоним функции ifHasBranch not=TRUE',
                        'ifHasAnnotation'              => 'если запись имеет непустой текст краткого описания',
                            'ifHasNotAnnotation'       => '....... синоним функции ifHasAnnotation not=TRUE',
                        'ifHasFullText'                => 'если запись имеет непустой текст полного описания',
                            'ifHasNotFullText'         => '....... синоним функции ifHasFullText not=TRUE',
                        'ifHasBody'                    => 'если запись имеет непустой текст описания',
                            'ifHasNotBody'             => '....... синоним функции ifHasBody not=TRUE',
                        'ifHasSeoText'                 => 'если запись имеет непустой SEO-текст',
                            'ifHasNotSeoText'          => '....... синоним функции ifHasSeoText not=TRUE',
                        'ifHasSubdomain'               => 'если запись имеет непустой текст субдомена',
                            'ifHasNotSubdomain'        => '....... синоним функции ifHasSubdomain not=TRUE',
                        'ifHasDiscussion'              => 'если комментарий имеет вложенные',
                            'ifHasNotDiscussion'       => '....... синоним функции ifHasDiscussion not=TRUE',
                    'ifPages'                          => 'если есть листаемые страницы',
                        'ifNotPages'                   => '....... синоним функции ifPages not=TRUE',
                        'ifPrevPage'                   => 'если есть предыдущая страница',
                            'ifNotPrevPage'            => '....... синоним функции ifPrevPage not=TRUE',
                        'ifNextPage'                   => 'если есть следующая страница',
                            'ifNotNextPage'            => '....... синоним функции ifNextPage not=TRUE',
                        'ifAllPage'                    => 'если есть ссылка на список ВСЕ',
                            'ifNotAllPage'             => '....... синоним функции ifAllPage not=TRUE',
                        'ifShowAll'                    => 'если выбран показ всех товаров списка',
                            'ifNotShowAll'             => '....... синоним функции ifShowAll not=TRUE',
                        'ifCanonical'                  => 'если находимся на каноническом адресе',
                            'ifNotCanonical'           => '....... синоним функции ifCanonical not=TRUE',
                    'ifPage404'                        => 'если на странице 404',
                        'ifNotPage404'                 => '....... синоним функции ifPage404 not=TRUE',
                        'ifPageAbstract'               => 'если на абстрактной странице',
                            'ifNotPageAbstract'        => '....... синоним функции ifPageAbstract not=TRUE',
                        'ifPageOff'                    => 'если на отключенной странице',
                            'ifNotPageOff'             => '....... синоним функции ifPageOff not=TRUE',
                        'ifOnCategories'               => 'если на странице Категории',
                            'ifNotOnCategories'        => '....... синоним функции ifOnCategories not=TRUE',
                        'ifOnBrands'                   => 'если на странице Бренды',
                            'ifNotOnBrands'            => '....... синоним функции ifOnBrands not=TRUE',
                        'ifOnProducts'                 => 'если на странице Товары',
                            'ifNotOnProducts'          => '....... синоним функции ifOnProducts not=TRUE',
                        'ifOnNews'                     => 'если на странице Новости',
                            'ifNotOnNews'              => '....... синоним функции ifOnNews not=TRUE',
                    'ifAccepted'                       => 'если форма ввода принята',
                        'ifNotAccepted'                => '....... синоним функции ifAccepted not=TRUE',
                        'ifAgree'                      => 'если включен флажок СОГЛАСЕН в форме ввода',
                            'ifNotAgree'               => '....... синоним функции ifAgree not=TRUE',
                        'ifError'                      => 'если есть сообщение об ошибке',
                            'ifNotError'               => '....... синоним функции ifError not=TRUE',
                        'ifNewPassword'                => 'если есть сообщение о новом пароле',
                            'ifNotNewPassword'         => '....... синоним функции ifNewPassword not=TRUE',
                        'ifMessage'                    => 'если есть сообщение об успехе',
                            'ifNotMessage'             => '....... синоним функции ifMessage not=TRUE',
                            'ifMessageOrNewPassword'   => 'если есть сообщение об успехе или новом пароле',
                                'ifNotMessageOrNewPassword' => '....... синоним функции ifMessageOrNewPassword not=TRUE',
                    'ifCaptcha'                        => 'если капча включена',
                        'ifNotCaptcha'                 => '....... синоним функции ifCaptcha not=TRUE',
                    'ifReferredToMe'                   => 'если запись прикреплена к текущему макету',
                        'ifNotReferredToMe'            => '....... синоним функции ifReferredToMe not=TRUE',
                        'ifDemoOrReferredToMe'         => 'если находимся в демо или запись прикреплена к текущему макету',
                            'ifNotDemoOrReferredToMe'  => '....... синоним функции ifDemoOrReferredToMe not=TRUE',
                    'ifTemplateFlag'                   => 'если установлен флаг в настройках шаблона',
                        'ifNotTemplateFlag'            => '....... синоним функции ifTemplateFlag not=TRUE',
                    'ifFirstRendering'                 => 'если это первый шаг рендеринга страницы',
                        'ifNotFirstRendering'          => '....... синоним функции ifFirstRendering not=TRUE',
                    'ifLanguage'                       => 'если доступен хотя бы 1 язык сайта',
                        'ifNotLanguage'                => '....... синоним функции ifLanguage not=TRUE',
                    'ifLanguages'                      => 'если доступно более 1 языка сайта',
                        'ifNotLanguages'               => '....... синоним функции ifLanguages not=TRUE',
                'splitBody'                            => 'разложить HTML-текст по H-заголовкам в массив HTML-блоков',
                'setModSuffix'                         => 'установить новый суффикс модуля',
                'setFound'                             => 'установить количество найденных записей',
                'setBody'                              => 'поместить в финальное тело страницы',
                    'setContent'                       => 'поместить в контент страницы',
                    'setTitle'                         => 'поместить в заголовок страницы',
                    'setMetaDescription'               => 'поместить в мета описание страницы',
                    'setMetaKeywords'                  => 'поместить в мета ключевые слова страницы',
                    'setSeoText'                       => 'поместить в SEO-текст страницы',
                    'setEndTag'                        => 'поместить в завершающий тег блока',
                    'setAsideHeader'                   => 'поместить в разметку боковой панели Шапка',
                        'setAsideTop'                  => 'поместить в разметку боковой панели Верхняя',
                        'setAsideRight'                => 'поместить в разметку боковой панели Правая',
                        'setAsideBottom'               => 'поместить в разметку боковой панели Нижняя',
                        'setAsideLeft'                 => 'поместить в разметку боковой панели Левая',
                        'setAsideCenter'               => 'поместить в разметку боковой панели Центральная',
                        'setAsideBanner'               => 'поместить в разметку боковой панели Баннер',
                        'setAsideSlider'               => 'поместить в разметку боковой панели Слайдер',
                        'setAsideGallery'              => 'поместить в разметку боковой панели Галерея',
                        'setAsideMenu'                 => 'поместить в разметку боковой панели Меню',
                        'setAsideFooter'               => 'поместить в разметку боковой панели Подвал',
                    'setHtmlClass'                     => 'поместить в класс тега HTML',
                        'setChildClass'                => 'поместить в название дочернего класса',
                        'setHtmlLang'                  => 'поместить в язык тега HTML',
                            'setHtmlLangMarker'        => 'поместить в URL-параметр языка',
                            'setLangDefault'           => 'поместить в язык по умолчанию',
                        'setHtmlUrl'                   => 'поместить в URL для некоторого HTML',
                        'setOpenGraphImage'            => 'поместить в URL скриншота для OpenGraph разметки',
                    'setItem'                          => 'поместить в переменную item',
                        'setItemTitle'                 => 'поместить в мета тайтл записи',
                        'setItemMeta'                  => 'поместить в мета описание записи',
                        'setItemKeywords'              => 'поместить в мета ключевые слова записи',
                        'setItemName'                  => 'поместить в название записи',
                            'setItemUrl'               => 'поместить в URL записи',
                        'setItemAnnotation'            => 'поместить в краткое описание записи',
                        'setItemBody'                  => 'поместить в полное описание записи',
                        'setItemSeo'                   => 'поместить в SEO-текст записи',
                        'setItemSubdomain'             => 'поместить в текст субдомена записи',
                        'setItemCategory'              => 'поместить в имя прикрепленной категории',
                        'setItemBrand'                 => 'поместить в имя прикрепленного бренда',
                        'setItemImages'                => 'поместить в изображения записи',
                    'setValue'                         => 'поместить в переменную value',
                        'setType'                      => 'поместить в переменную type',
                        'setHash'                      => 'поместить как хеш в переменную hash',
                    'setError'                         => 'поместить в переменную error',
                    'setLanguages'                     => 'установить языки сайта',
                    'setPolyglot'                      => 'загрузить фразы в многоязычный словарь',
                        'polyglot'                     => 'перевести фразу через многоязычный словарь',
                'putBody'                              => 'добавить в финальное тело страницы',
                    'putContent'                       => 'добавить в контент страницы',
                    'putOtherMeta'                     => 'добавить в остальные мета',
                    'putOtherJS'                       => 'добавить в остальные скрипты',
                    'putSeoText'                       => 'добавить в SEO-текст страницы',
                    'putAsideHeader'                   => 'добавить в разметку боковой панели Шапка',
                        'putAsideTop'                  => 'добавить в разметку боковой панели Верхняя',
                        'putAsideRight'                => 'добавить в разметку боковой панели Правая',
                        'putAsideBottom'               => 'добавить в разметку боковой панели Нижняя',
                        'putAsideLeft'                 => 'добавить в разметку боковой панели Левая',
                        'putAsideCenter'               => 'добавить в разметку боковой панели Центральная',
                        'putAsideBanner'               => 'добавить в разметку боковой панели Баннер',
                        'putAsideSlider'               => 'добавить в разметку боковой панели Слайдер',
                        'putAsideGallery'              => 'добавить в разметку боковой панели Галерея',
                        'putAsideMenu'                 => 'добавить в разметку боковой панели Меню',
                        'putAsideFooter'               => 'добавить в разметку боковой панели Подвал',
                'viaScript'                            => 'вывести посредством javascript',
                'implantCSS'                           => 'имплантировать CSS-файлы в тело страницы',
                    'implantJS'                        => 'имплантировать JS-файлы в тело страницы',
                'insertTextFileLine'                   => 'вставить строку вначале текстового файла',
                'sendSms'                              => 'отправить СМС',
                'sendEmail'                            => 'отправить емейл',
                'bbCodes'                              => 'обработать BB-коды блока',
                'censor'                               => 'цензурировать блок'
            ), 'block');

            /* регистрируем теги хелперного расширения */
            if ($this->isActualArray($subList)) {
                $this->templateFunctions['----- ЭТО ФУНКЦИИ ИЗ MOD-HELPER -----'] = 'они вызываются одиночным тегом, как функции выше перед блочными';
                $this->expandSmarty($subList);
            }

            /* сам хелпер будет доступен в шаблоне вот так */
            $cms->smarty->assignByRef('emulator', $this);
        }

        /* ========================================================================
        |                                                                         |
        |    Подготовка к отрисовке конкретного файла шаблона.                    |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Используется в случае, когда хотим перехватить начало отрисовки      |
        |    какого-то файла в шаблоне. Входная переменная $template содержит:    |
        |                                                                         |
        |        ->name = реальное имя файла, который предполагаем рисовать       |
        |        ->impera_name = его имя (без расширения) имперское               |
        |        ->standard_name = его имя (без расширения) типичное              |
        |                                                                         |
        |    На выход требуется вернуть имя файла шаблона, который просим         |
        |    рисовать для такой ситуации (как правило, это имя равно входному     |
        |    имени файла, то есть не изменяется).                                 |
        |                                                                         |
        |    Или во входную переменную $body требуется вернуть сгенерированную    |
        |    html-разметку, которую просим рисовать вместо рендеринга такого      |
        |    файла шаблона (null = просим рендерить указанный файл).              |
        |                                                                         |
        |    -------------------------------------------------------------------  |
        |    Обратите внимание: метод вызывается дважды - когда рисуется          |
        |    центральная часть страницы и когда в конце рисуется общий макет      |
        |    страницы (в ajax-запросах этого второго вызова нет).                 |
        |                                                                         |
        |    -------------------------------------------------------------------  |
        |    Обратите внимание: метод действует только для основных макетов, то   |
        |    есть модульные include внутри файлов шаблона не видны методу.        |
        |                                                                         |
        ======================================================================== */

        public function prepare ( $template, & $cms, & $body = null ) {
            $varname = 'categoriesTree';
            if (!isset($this->preassignChecker[$varname])) {
                if ($this->isNotPreassignable($varname)) {
                    $params = array();
                    $this->noCategories($params, $this->cms->smarty);
                    if (!isset($this->cms->category) || empty($this->cms->category)) {
                        $item = $this->getVar('category', $this->cms->smarty);
                        if (!empty($item)) $this->cms->category = $item;
                    }
                    $this->preassignChecker[$varname] = TRUE;
                }
            }

            $varname = 'brandsTree';
            if (!isset($this->preassignChecker[$varname])) {
                if ($this->isNotPreassignable($varname)) {
                    $params = array();
                    $this->noBrands($params, $this->cms->smarty);
                    if (!isset($this->cms->brand) || empty($this->cms->brand)) {
                        $item = $this->getVar('brand', $this->cms->smarty);
                        if (!empty($item)) $this->cms->brand = $item;
                    }
                    $this->preassignChecker[$varname] = TRUE;
                }
            }

            $body = null;
            $tpl = $this->removeFileExt($template->name);
            $tpl = $this->cms->text->lowerCase($tpl);
            switch ($tpl) {

                // если рисуем общий макет страницы
                case 'index':

                    // на каком урле сейчас стоим?
                    $url = $this->abstractUri();
                    switch ($url) {

                        // TODO: Скорее всего место следующей операции в каком-то
                        //       из центральных макетов. Иначе отсюда нам недоступен
                        //       центр, например чтобы вывести там уведомление.
                        //       Если же рисуем центр сами и назначим в переменную
                        //       $content шаблона - тогда нормально.

                        // если в настройках шаблона и это админ запостил форму отредактированных настроек
                        case 'setup':
                            if ($this->cms->request->isPosted()) {
                                if ($this->cms->security->inDemoMode()) {
                                    $this->pushError('В демо версии отклоняются попытки сохранить изменения в настройках шаблона.');
                                } elseif (!$this->existsAdmin()) {
                                    $this->pushError('Такая операция доступна только администратору сайта.');
                                } else {
                                    $msg = $this->saveTemplateSetupParams();
                                }
                            }
                            break;
                    }
                    break;

                // если рисуем центральную часть страницы
                case 'account':                      // личный кабинет
                    break;
                case 'article':                      // статья
                    break;
                case 'articles':                     // список статей
                    break;
                case 'callme':                       // заказать звонок
                    break;
                case 'cart':                         // корзина
                    break;
                case 'catalog':                      // главная страница
                    break;
                case 'compare':                      // сравнение товаров
                    break;
                case 'email_notifyme_confirm':       // письмо "Подтвердите подключение к уведомлениям"
                    break;
                case 'email_order_payment_to_admin': // письмо админу "Заказ оплачен"
                    break;
                case 'email_order_payment_to_user':  // письмо клиенту "Заказ оплачен"
                    break;
                case 'email_order_to_admin':         // письмо админу "Заказ принят"
                    break;
                case 'email_order_to_user':          // письмо клиенту "Заказ принят"
                    break;
                case 'email_password_remind':        // письмо "Подтвердите начало операции смены пароля"
                    break;
                case 'feedback':                     // обратная связь
                    break;
                case 'fulminant.order':              // заказ в один клик
                    break;
                case 'informer':                     // информер для внешних сайтов
                    break;
                case 'login':                        // авторизация
                    break;
                case 'missing_template':             // ошибка 404
                    break;
                case 'navigation':                   // блок "пагинация страниц"
                    break;
                case 'news':                         // список новостей
                    break;
                case 'news_item':                    // новость
                    break;
                case 'order':                        // заказ
                    break;
                case 'password_remind':              // страница "забыли пароль?"
                    break;
                case 'price':                        // файл Pricelist.xls
                    break;
                case 'product':                      // товар
                    break;
                case 'products':                     // список товаров категории/бренда
                    break;
                case 'registration':                 // регистрация
                    break;
                case 'search':                       // результаты поиска
                    break;
                case 'sitemap':                      // карта сайта
                    break;
                case 'sms_order_change_to_user':     // СМС клиенту "Изменения в заказе"
                    break;
                case 'sms_order_payment_to_admin':   // СМС админу "Заказ оплачен"
                    break;
                case 'sms_order_payment_to_user':    // СМС клиенту "Заказ оплачен"
                    break;
                case 'sms_order_to_admin':           // СМС админу "Заказ принят"
                    break;
                case 'sms_order_to_user':            // СМС клиенту "Заказ принят"
                    break;
                case 'static_page':                  // статическая страница
                    break;

                // ТУТ ЕЩЕ МОЖЕТ БЫТЬ МНОЖЕСТВО ИМЕН МАКЕТОВ,
                // ЭТО ЗАВИСИТ ОТ НАБОРА МОДУЛЕЙ В СИСТЕМЕ,
                // ТАК КАК ПОЧТИ КАЖДЫЙ МОДУЛЬ ИМЕЕТ СВОЙ МАКЕТ(Ы)

                // иначе рисуем неизвестный файл шаблона
                default:
            }
            return $template->name;
        }

        /* ========================================================================
        |                                                                         |
        |    Обработчики наших функций.                                           |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Каждая из этих функций может быть вызвана в шаблоне с помощью        |
        |    инструкции такого вида:                                              |
        |                                                                         |
        |        {имяФункции параметр1 параметр2 ... параметрN}                   |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Вывести версию этой библиотеки ShortCodes.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {shortCodesVersion                            |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function shortCodesVersion ( $params = null, & $smarty = null ) {
            return $this->echoValue($this->shortCodesVersion, $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести версию движка.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {version                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function version ( $params = null, & $smarty = null ) {
            return $this->echoConstant('IMPERA_CMS_CURRENT_MMVERSION', $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести номер версии движка.                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {versionYMD                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function versionYMD ( $params = null, & $smarty = null ) {
                return $this->echoConstant('IMPERA_CMS_CURRENT_VERSION', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверный HTTP-заголовок OK.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {header200}                                   |
                    |                                                             |
                    ============================================================ */

        public function header200 ( $params = null, & $smarty = null ) {
            @ header('HTTP/1.1 200 OK', TRUE, 200);
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверный HTTP-заголовок Last-Modified.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerLastModified                           |
                    |                   [plus = HoursCount]  сдвинуть на ? часов  |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function headerLastModified ( $params = null, & $smarty = null ) {
                $hours = $this->paramAsInteger('plus', $params);
                $time = time() + $hours * 3600;
                @ header('Last-Modified: ' . date('r', $time), TRUE);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки Expires.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerExpires                                |
                    |                   [plus = HoursCount]  сдвинуть на ? часов  |
                    |                                        (по умолчанию 1)     |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function headerExpires ( $params = null, & $smarty = null ) {
                $hours = $this->paramAsNaturalExtended('plus', $params, 1);
                $time = time() + $hours * 3600;
                @ header('Expires: ' . date('r', $time), TRUE);
                @ header('Cache-Control: public', TRUE);
                @ header('Pragma: public', TRUE);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки JSON-данных.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerJSON}                                  |
                    |                                                             |
                    ============================================================ */

            public function headerJSON ( $params = null, & $smarty = null ) {
                $this->headerDynamicContent('application/json');
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки текстового контента.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerTEXT}                                  |
                    |                                                             |
                    ============================================================ */

                public function headerTEXT ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('text/plain');
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести контент текстового файла.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {printTextFile                                |
                    |                   name = FileName       имя файла           |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function printTextFile ( $params = null, & $smarty = null ) {
                        $value = '';
                        $file = $this->paramAsSentence('name', $params);
                            $uri = '';
                        if ($this->existsFile($file, array('txt'), 'db', $uri)) {
                            $handle = @ fopen($uri, 'rb');
                            if ($handle !== FALSE) {
                                @ flock($handle, LOCK_EX);
                                while (!feof($handle)) {
                                    $data = @ fread($handle, 8192);
                                    if (!is_string($data) || $data === '') break;
                                    $value .= $data;
                                }
                                @ fclose($handle);
                            }
                        }
                        return $this->echoValue($value, $params, $smarty, FALSE);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки HTML-контента.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerHTML}                                  |
                    |                                                             |
                    ============================================================ */

                public function headerHTML ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('text/html');
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки JS-контента.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerJS}                                    |
                    |                                                             |
                    ============================================================ */

                public function headerJS ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('application/javascript');
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки CSS-контента.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerCSS}                                   |
                    |                                                             |
                    ============================================================ */

                public function headerCSS ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('text/css');
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки XML-данных.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerXML}                                   |
                    |                                                             |
                    ============================================================ */

                public function headerXML ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('text/xml');
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки CSV-данных.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerCSV}                                   |
                    |                                                             |
                    ============================================================ */

                public function headerCSV ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('text/csv');
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки EXCEL-данных.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerEXCEL}                                 |
                    |                                                             |
                    ============================================================ */

                public function headerEXCEL ( $params = null, & $smarty = null ) {
                    $this->headerDynamicContent('application/vnd.ms-excel', FALSE);
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки данных конкретного типа.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerANY                                    |
                    |                   type = TypeName  тип контента (по MIME)   |
                    |                   [utf8 = TRUE]    если важно указание кодир|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function headerANY ( $params = null, & $smarty = null ) {
                    $type = $this->paramAsString('type', $params);
                    $utf8 = $this->paramAsBoolean('utf8', $params);
                    $this->headerDynamicContent($type, $utf8);
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Отправить серверные HTTP-заголовки "передача файла".       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {headerFile                                   |
                    |                   url = FileName   имя сохраняемого файла   |
                    |                   [inline = TRUE]  если не как прикрепление |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function headerFile ( $params = null, & $smarty = null ) {
                    $file = $this->paramAsSentence('url', $params);
                    $file = $this->cms->hdd->safeFilename($file);
                    if ($this->isActualString($file)) {
                        $file = $this->asAttribute($file);
                        $type = $this->paramAsBoolean('inline', $params) ? 'inline' : 'attachment';
                        @ header('Content-Description: File Transfer', TRUE);
                        @ header('Content-Disposition: ' . $type . '; filename="' . $file . '"', TRUE);
                        @ header('Content-Transfer-Encoding: binary', TRUE);
                    }
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести заголовок страницы поиска.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {searchTitle                                  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {titleSearch                                  |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function searchTitle ( $params = null, & $smarty = null ) {
            return $this->echoVariable('search_title', $params, $smarty);
        }

            public function titleSearch ( $params = null, & $smarty = null ) {
                return $this->searchTitle($params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести искомую фразу поиска.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {searchKeyword                                |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {searchTag ... }                              |
                    |                                                             |
                    ============================================================ */

            public function searchKeyword ( $params = null, & $smarty = null ) {
                return $this->echoVariable('keyword', $params, $smarty);
            }

                public function searchTag ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('tag', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести заголовок страницы особых товаров.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {titleFeatured                                |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {titleContent                                 |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function titleFeatured ( $params = null, & $smarty = null ) {
            return $this->echoVariable(array('content_title', 'title'), $params, $smarty);
        }

            public function titleContent ( $params = null, & $smarty = null ) {
                return $this->titleFeatured($params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести мета описание страницы.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {metaDescription                              |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function metaDescription ( $params = null, & $smarty = null ) {
            return $this->echoVariable('description', $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести мета ключевые слова страницы.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {metaKeywords                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function metaKeywords ( $params = null, & $smarty = null ) {
                return $this->echoVariable('keywords', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить мета ключи + теги записи как массив уникальных    |
                    |  строк.                                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {keys                                         |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [except = keyList]    массив исключаемых  |
                    |                                         строк (ключей)      |
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         переменную $items)  |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function keys ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, 'items');
                    $value = $this->getItemStringArray($params, $smarty, array('meta_keywords', 'tags'));
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить мета ключи записи как массив уникальных строк.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {keywords                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [except = keyList]    массив исключаемых  |
                    |                                         строк (ключей)      |
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         переменную $items)  |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function keywords ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('assign', $params, 'items');
                        $value = $this->getItemStringArray($params, $smarty, 'meta_keywords');
                        return $this->assignVar($params, $value, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Получить теги этой записи как массив уникальных строк.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {tags                                         |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [except = keyList]    массив исключаемых  |
                    |                                         строк (тегов)       |
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         переменную $items)  |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function tags ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('assign', $params, 'items');
                        $value = $this->getItemStringArray($params, $smarty, 'tags');
                        return $this->assignVar($params, $value, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Получить мета ключи + теги товаров как массив уникальных   |
                    |  строк.                                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {keysCatalog                                  |
                    |                   [except = keyList]    массив исключаемых  |
                    |                                         строк (ключей)      |
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         переменную $items)  |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function keysCatalog ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, 'items');
                    $authorized = $this->existsUser();
                    $query = 'SELECT `products`.`meta_keywords` AS `keys`, '
                                  . '`products`.`tags` '
                           . 'FROM `products` '
                           . 'LEFT JOIN `categories` ON `products`.`category_id` = `categories`.`category_id` '
                           . 'LEFT JOIN `brands` ON `products`.`brand_id` = `brands`.`brand_id` '
                           . 'WHERE `products`.`enabled` = 1 AND '
                                  . ($authorized ? '' : '`products`.`hidden` = 0 AND '
                                                      . '`categories`.`hidden` = 0 AND '
                                                      . '(`brands`.`hidden` = 0 OR `brands`.`hidden` IS NULL) AND ')
                                 . '`categories`.`enabled` = 1 AND '
                                 . '(`brands`.`enabled` = 1 OR `brands`.`enabled` IS NULL) AND '
                                 . '(`products`.`meta_keywords` != "" OR `products`.`tags` != "");';
                    $result = $this->cms->db->query($query);
                    if ($result !== FALSE) {
                        $except = $this->paramAsArray('except', $params);
                        $value = '';
                        while ($item = $this->cms->db->fetch_assoc($result)) {
                            foreach ($item as & $v) {
                                $v = preg_replace('~[^' . $this->regexpDomainChars(FALSE, FALSE) . ',]+~ui', ' ', $v);
                                $v = trim($v, ' ,');
                                if ($this->isActualString($v)) $value .= ',' . $v;
                            }
                        }
                        $value = ltrim($value, ',');
                        $value = $this->asUniqueStringArray($value, FALSE, $except);
                        if (empty($value)) $value = $this->paramAsArray('def', $params);
                    }
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести финальное тело страницы.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {echoBody                                     |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function echoBody ( $params = null, & $smarty = null ) {
                return $this->echoVariable('body', $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести класс тега HTML.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {htmlClass                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function htmlClass ( $params = null, & $smarty = null ) {
                return $this->echoVariable('htmlClass', $params, $smarty);
            }

                public function childClass ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('childClass', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести язык тега HTML.                                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {htmlLang                                     |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {htmlLangMarker ...}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {langDefault ...}                             |
                    |                                                             |
                    ============================================================ */

                public function htmlLang ( $params = null, & $smarty = null ) {
                    $params2 = array();
                    $def = $this->langDefault($params2, $smarty);
                    $value = $this->getVar('htmlLang', $smarty, $def);
                    return $this->echoValue($value, $params, $smarty);
                }

                    public function htmlLangMarker ( $params = null, & $smarty = null ) {
                        $params2 = array();
                        $value = $this->htmlLang($params2, $smarty);
                        $value = $this->getVar('htmlLangMarker', $smarty, '?lang=' . $value);
                        return $this->echoValue($value, $params, $smarty);
                    }

                    public function langDefault ( $params = null, & $smarty = null ) {
                        $value = $this->getVar('langDefault', $smarty, 'ru');
                        return $this->echoValue($value, $params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL для некоторого HTML.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {htmlUrl                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function htmlUrl ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('htmlUrl', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL скриншота для OpenGraph разметки.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {openGraphImage                               |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function openGraphImage ( $params = null, & $smarty = null ) {
                    $def = $this->getThemeUrl() . 'images/favicon.png';
                    $value = $this->getVar('openGraphImage', $smarty, $def);
                    return $this->echoValue($value, $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести класс demo если мы в деморежиме.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {demoClass                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function demoClass ( $params = null, & $smarty = null ) {
                $value = $this->cms->security->inDemoMode() ? 'demo' : '';
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести остальные мета.                                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {otherMeta                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function otherMeta ( $params = null, & $smarty = null ) {
                return $this->echoVariable('meta', $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести остальные скрипты.                                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {otherJS                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function otherJS ( $params = null, & $smarty = null ) {
                return $this->echoVariable('otherJS', $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести SEO-текст страницы.                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {seoText                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function seoText ( $params = null, & $smarty = null ) {
                return $this->echoVariable('seo_description', $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести SEO-текст этой записи.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {seoBody                                      |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function seoBody ( $params = null, & $smarty = null ) {
                    $def = $this->paramAsString('def', $params);
                    return $this->echoField('seo_description', $params, $smarty, $def, FALSE);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести завершающий тег блока.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {endTag                                       |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function endTag ( $params = null, & $smarty = null ) {
                return $this->echoVariable('endTag', $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести разметку боковой панели страницы.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {asideHeader                                  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function asideHeader ( $params = null, & $smarty = null ) {
                return $this->echoVariable('asideHeader', $params, $smarty, FALSE);
            }

                public function asideTop ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideTop', $params, $smarty, FALSE);
                }

                public function asideRight ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideRight', $params, $smarty, FALSE);
                }

                public function asideBottom ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideBottom', $params, $smarty, FALSE);
                }

                public function asideLeft ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideLeft', $params, $smarty, FALSE);
                }

                public function asideCenter ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideCenter', $params, $smarty, FALSE);
                }

                public function asideBanner ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideBanner', $params, $smarty, FALSE);
                }

                public function asideSlider ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideSlider', $params, $smarty, FALSE);
                }

                public function asideGallery ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideGallery', $params, $smarty, FALSE);
                }

                public function asideMenu ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideMenu', $params, $smarty, FALSE);
                }

                public function asideFooter ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('asideFooter', $params, $smarty, FALSE);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести контент субдомена той записи.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {subdomainHtml                                |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function subdomainHtml ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField('subdomain_html', $params, $smarty, $def, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести относительный канонический адрес текущей страницы. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {canonicalUri                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function canonicalUri ( $params = null, & $smarty = null ) {
                $value = $this->getCanonicalUri();
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести абсолютный канонический адрес текущей страницы.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {canonicalUrl                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function canonicalUrl ( $params = null, & $smarty = null ) {
                $value = $this->getSiteUrl(FALSE) .
                         $this->getCanonicalUri();
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название сайта, указанное в настройках сайта.      |
                    |                                                             |
                    ============================================================ */

        public function siteName ( $params = null, & $smarty = null ) {
            return $this->echoSetting('site_name', $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести имя компании, указанное в настройках сайта.        |
                    |                                                             |
                    ============================================================ */

            public function company ( $params = null, & $smarty = null ) {
                return $this->echoSetting('company_name', $params, $smarty);
            }

                public function companyName ( $params = null, & $smarty = null ) {
                    return $this->company($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название текущего дизайна.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {themeName                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function themeName ( $params = null, & $smarty = null ) {
                return $this->echoVariable('current_theme', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести домен сайта.                                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {siteDomain                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function siteDomain ( $params = null, & $smarty = null ) {
                $value = $this->getSiteDomain();
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Установить значение конфиг параметра (на время сеанса).    |
                    |                                                             |
                    ============================================================ */

        public function setConfigParam ( $params = null, & $smarty = null ) {
            $param = $this->paramAsSentence('name', $params);
            if (!empty($param)) {
                $value = $this->paramAsIs('value', $params);
                $this->setConfig($param, $value);
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Установить значение параметра настроек сайта.              |
                    |                                                             |
                    ============================================================ */

            public function setSettingsParam ( $params = null, & $smarty = null ) {
                $param = $this->paramAsSentence('name', $params);
                if (!empty($param)) {
                    $value = $this->paramAsIs('value', $params);
                    $this->setSettings($param, $value);
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Установить параметр сеанса.                                |
                    |                                                             |
                    ============================================================ */

            public function setSessionParam ( $params = null, & $smarty = null ) {
                $param = $this->paramAsSentence('name', $params);
                if (!empty($param)) {
                    $value = $this->paramAsIs('value', $params);
                    $this->setSession($param, $value);
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести телефон, указанный в настройках сайта.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {phone                                        |
                    |                   [num = phoneNumber]   порядковый номер    |
                    |                                         в настройках        |
                    |                                         (по умолчанию 1)    |
                    |                   [asUrl = TRUE]        если для ссылки     |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function phone ( $params = null, & $smarty = null ) {
            $value = $this->getConfigNumerated('phone', $params);
            $asUrl = $this->paramAsBoolean('asUrl', $params);
            if ($asUrl) {
                $value = preg_replace('/[^0-9]/u', '', $value);
                if ($this->isActualString($value)) $value = 'tel:' . $value;
            }
            return $this->echoValue($value, $params, $smarty);
        }

            public function phone2 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 2);
                return $this->phone($params, $smarty);
            }

            public function phone3 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 3);
                return $this->phone($params, $smarty);
            }

            public function phone4 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 4);
                return $this->phone($params, $smarty);
            }

            public function phone5 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 5);
                return $this->phone($params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести емейл, указанный в настройках сайта.               |
                    |                                                             |
                    ============================================================ */

            public function email ( $params = null, & $smarty = null ) {
                return $this->echoConfigNumerated('email', $params, $smarty);
            }

                public function email2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->email($params, $smarty);
                }

                public function email3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->email($params, $smarty);
                }

                public function email4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->email($params, $smarty);
                }

                public function email5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->email($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести Telegram-номер, указанный в настройках сайта.      |
                    |                                                             |
                    ============================================================ */

            public function telegram ( $params = null, & $smarty = null ) {
                return $this->echoConfigNumerated('telegram', $params, $smarty);
            }

                public function telegram2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->telegram($params, $smarty);
                }

                public function telegram3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->telegram($params, $smarty);
                }

                public function telegram4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->telegram($params, $smarty);
                }

                public function telegram5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->telegram($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести скайп, указанный в настройках сайта.               |
                    |                                                             |
                    ============================================================ */

            public function skype ( $params = null, & $smarty = null ) {
                return $this->echoConfigNumerated('skype', $params, $smarty);
            }

                public function skype2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->skype($params, $smarty);
                }

                public function skype3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->skype($params, $smarty);
                }

                public function skype4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->skype($params, $smarty);
                }

                public function skype5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->skype($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес, указанный в настройках сайта.               |
                    |                                                             |
                    ============================================================ */

            public function address ( $params = null, & $smarty = null ) {
                return $this->echoConfigNumerated('address', $params, $smarty);
            }

                public function address2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->address($params, $smarty);
                }

                public function address3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->address($params, $smarty);
                }

                public function address4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->address($params, $smarty);
                }

                public function address5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->address($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Facebook, указанный в настройках сайта.      |
                    |                                                             |
                    ============================================================ */

        public function facebook ( $params = null, & $smarty = null ) {
            return $this->echoConfig('facebook', $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес ВКонтакте, указанный в настройках сайта.     |
                    |                                                             |
                    ============================================================ */

            public function vkontakte ( $params = null, & $smarty = null ) {
                return $this->echoConfig('vkontakte', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Twitter, указанный в настройках сайта.       |
                    |                                                             |
                    ============================================================ */

            public function twitter ( $params = null, & $smarty = null ) {
                return $this->echoConfig('twitter', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Google+, указанный в настройках сайта.       |
                    |                                                             |
                    ============================================================ */

            public function google ( $params = null, & $smarty = null ) {
                return $this->echoConfig('google', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес YouTube, указанный в настройках сайта.       |
                    |                                                             |
                    ============================================================ */

            public function youtube ( $params = null, & $smarty = null ) {
                return $this->echoConfig('youtube', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Instagram, указанный в настройках сайта.     |
                    |                                                             |
                    ============================================================ */

            public function instagram ( $params = null, & $smarty = null ) {
                return $this->echoConfig('instagram', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес LinkedIn, указанный в настройках сайта.      |
                    |                                                             |
                    ============================================================ */

            public function linkedin ( $params = null, & $smarty = null ) {
                return $this->echoConfig('linkedin', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Pinterest, указанный в настройках сайта.     |
                    |                                                             |
                    ============================================================ */

            public function pinterest ( $params = null, & $smarty = null ) {
                return $this->echoConfig('pinterest', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Dribbble, указанный в настройках сайта.      |
                    |                                                             |
                    ============================================================ */

            public function dribbble ( $params = null, & $smarty = null ) {
                return $this->echoConfig('dribbble', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести адрес Behance, указанный в настройках сайта.       |
                    |                                                             |
                    ============================================================ */

            public function behance ( $params = null, & $smarty = null ) {
                return $this->echoConfig('behance', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название девелопера, указанное в настройках сайта. |
                    |                                                             |
                    ============================================================ */

        public function developer ( $params = null, & $smarty = null ) {
            $value = $this->getConfigNumerated('developer', $params);
            $value = explode(',', $value);
            $value = array_shift($value);
            $value = trim($value);
            return $this->echoValue($value, $params, $smarty);
        }

            public function developer2 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 2);
                return $this->developer($params, $smarty);
            }

            public function developer3 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 3);
                return $this->developer($params, $smarty);
            }

            public function developer4 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 4);
                return $this->developer($params, $smarty);
            }

            public function developer5 ( $params = null, & $smarty = null ) {
                $this->setParam('num', $params, 5);
                return $this->developer($params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести сайт девелопера, указанный в настройках сайта.     |
                    |                                                             |
                    ============================================================ */

            public function developerSite ( $params = null, & $smarty = null ) {
                $value = $this->getConfigNumerated('developer', $params);
                $value = explode(',', $value);
                array_shift($value);
                $value = array_shift($value);
                $value = is_string($value) ? trim($value) : '';
                return $this->echoValue($value, $params, $smarty);
            }

                public function developerSite2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->developerSite($params, $smarty);
                }

                public function developerSite3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->developerSite($params, $smarty);
                }

                public function developerSite4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->developerSite($params, $smarty);
                }

                public function developerSite5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->developerSite($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести сообщение об ошибке.                               |
                    |                                                             |
                    ============================================================ */

        public function error ( $params = null, & $smarty = null ) {
            $name = $this->getErrorVarname($smarty);
            return $this->echoVariable($name, $params, $smarty, FALSE);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести сообщение об успехе.                               |
                    |                                                             |
                    ============================================================ */

            public function message ( $params = null, & $smarty = null ) {
                $name = $this->getMessageVarname($smarty);
                return $this->echoVariable($name, $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести текущий год.                                       |
                    |                                                             |
                    ============================================================ */

        public function year ( $params = null, & $smarty = null ) {
            $value = date('Y', time());
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести коды счетчиков, указанных в настройках сайта.      |
                    |                                                             |
                    ============================================================ */

        public function counters ( $params = null, & $smarty = null ) {
            return $this->echoSetting('counters', $params, $smarty, FALSE);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести html-контент пагинации.                            |
                    |                                                             |
                    ============================================================ */

        public function pagination ( $params = null, & $smarty = null ) {
            return $this->echoVariable('PagesNavigation', $params, $smarty, FALSE);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL пагинационной ссылки Все.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {urlAllOnPage                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function urlAllOnPage ( $params = null, & $smarty = null ) {
                return $this->echoVariable('AllOnPageUrl', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL пагинационной ссылки Назад.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {urlPrevPage                                  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function urlPrevPage ( $params = null, & $smarty = null ) {
                return $this->echoVariable('PrevPageUrl', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL пагинационной ссылки Вперед.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {urlNextPage                                  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function urlNextPage ( $params = null, & $smarty = null ) {
                return $this->echoVariable('NextPageUrl', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести номер текущей страницы.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pageNum                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function pageNum ( $params = null, & $smarty = null ) {
                $value = $this->getNumber('CurrentPage', $smarty) + 1;
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести номер первой позиции на текущей странице.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pageMeter                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function pageMeter ( $params = null, & $smarty = null ) {
                    $value = $this->getNumber('CurrentPage', $smarty);
                    $size = $this->getNumber('CurrentPageMaxsize', $smarty, 1);
                    return $this->echoValue($value * $size + 1, $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество позиций на листаемой странице.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pageSize                                     |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function pageSize ( $params = null, & $smarty = null ) {
                    $value = $this->getNumber('CurrentPageMaxsize', $smarty, 1);
                    return $this->echoValue($value, $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество листаемых страниц.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pageCount                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function pageCount ( $params = null, & $smarty = null ) {
                $item = $this->getVar('Pages', $smarty);
                $value = !empty($item) && is_array($item) ? count($item) : 1;
                $value = max(1, $value);
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Признать файл шаблона последним в цепи рендеринга.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {lastTemplate}                                |
                    |                                                             |
                    ============================================================ */

        public function lastTemplate ( $params = null, & $smarty = null ) {
            $this->cms->quick_content = TRUE;
            $this->cms->smarty->assign('quick_content', TRUE);
            $this->cms->smarty->assign('ajax', TRUE);
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Признать файл шаблона промежуточным в цепи рендеринга.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {centralTemplate}                             |
                    |                                                             |
                    ============================================================ */

        public function centralTemplate ( $params = null, & $smarty = null ) {
            $this->cms->quick_content = FALSE;
            $this->cms->smarty->assign('quick_content', FALSE);
            $this->cms->smarty->assign('ajax', FALSE);
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить HTML-кеш движка.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {cacheClear}                                  |
                    |                                                             |
                    ============================================================ */

        public function cacheClear ( $params = null, & $smarty = null ) {
            $this->cms->htmcache->clear();
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Удалить статику (STATIC-кеш шаблонов) текущей страницы.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {staticClear                                  |
                    |                   [personal = TRUE]  если кешируем в шаблон |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function staticClear ( $params = null, & $smarty = null ) {
                $uri = $this->requestUri();
                $file = $this->staticPageFile($uri, $params);
                $this->cms->hdd->deleteFile($file);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Удалить статику (STATIC-кеш шаблонов) главной страницы.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {staticClearMainpage                          |
                    |                   [personal = TRUE]  если кешируем в шаблон |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function staticClearMainpage ( $params = null, & $smarty = null ) {
                $file = $this->staticPageFile(FALSE, $params);
                $this->cms->hdd->deleteFile($file);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Сохранить тело страницы в статику (STATIC-кеш шаблонов).   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {staticSave                                   |
                    |                   [from = VarName]   где хранится тело      |
                    |                                      (по умолчанию $body)   |
                    |                   [personal = TRUE]  если кешируем в шаблон |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function staticSave ( $params = null, & $smarty = null ) {
                /* TODO */
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Сохранить тело главной в статику (STATIC-кеш шаблонов).    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {staticSaveMainpage                           |
                    |                   [from = VarName]   где хранится тело      |
                    |                                      (по умолчанию $body)   |
                    |                   [personal = TRUE]  если кешируем в шаблон |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function staticSaveMainpage ( $params = null, & $smarty = null ) {
                /* TODO */
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести читабельную дату этой записи.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {date                                         |
                    |                   [item = $record]  из какой записи         |
                    |                                     (по умолчанию из $item) |
                    |                   [time = TRUE]     если вывести и время    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function date ( $params = null, & $smarty = null ) {
            $item = $this->paramAsIs('item', $params);
            if (is_null($item)) $item = $this->getVar('item', $smarty);
            if (!empty($item->date_date)) $item = $item->date_date;
            elseif (!empty($item->date)) $item = $item->date;
            elseif (!empty($item->created)) $item = $item->created;
            elseif (!is_string($item)) $item = '1970-01-01 00:00';
            $names = $this->getMonths();
            $value = $this->getDay($item) . ' ' .
                     $names[$this->getMonth($item) - 1] . ' ' .
                     $this->getYear($item);
            if ($this->paramAsIs('time', $params)) {
                $value .= ' в ' .
                          $this->getHour($item, TRUE) . ':' .
                          $this->getMinute($item, TRUE);
            }
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой записи.                                    |
                    |                                                             |
                    ============================================================ */

        public function id ( $params = null, & $smarty = null ) {
            return $this->echoField(array('currency_id',
                                          'comment_id',
                                          'section_id',
                                          'variant_id', 'product_id', 'kit_id', 'property_id',
                                          'article_id', 'news_id',
                                          'file_id',
                                          'stock_id',
                                          'brand_id',
                                          'event_id',
                                          'country_id', 'region_id', 'town_id',
                                          'category_id',
                                          'id',
                                          'menu_id',
                                          'order_id',
                                          'user_id'), $params, $smarty, 0);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой специальной страницы.                      |
                    |                                                             |
                    ============================================================ */

            public function idSection ( $params = null, & $smarty = null ) {
                return $this->echoField('section_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой категории.                                 |
                    |                                                             |
                    ============================================================ */

            public function idCategory ( $params = null, & $smarty = null ) {
                return $this->echoField('category_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого бренда.                                   |
                    |                                                             |
                    ============================================================ */

            public function idBrand ( $params = null, & $smarty = null ) {
                return $this->echoField('brand_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого товара.                                   |
                    |                                                             |
                    ============================================================ */

            public function idProduct ( $params = null, & $smarty = null ) {
                return $this->echoField('product_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого варианта товара.                          |
                    |                                                             |
                    ============================================================ */

            public function idVariant ( $params = null, & $smarty = null ) {
                return $this->echoField('variant_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого свойства.                                 |
                    |                                                             |
                    ============================================================ */

            public function idProperty ( $params = null, & $smarty = null ) {
                return $this->echoField('property_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой новости.                                   |
                    |                                                             |
                    ============================================================ */

            public function idNews ( $params = null, & $smarty = null ) {
                return $this->echoField('news_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой статьи.                                    |
                    |                                                             |
                    ============================================================ */

            public function idArticle ( $params = null, & $smarty = null ) {
                return $this->echoField('article_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого склада.                                   |
                    |                                                             |
                    ============================================================ */

            public function idStock ( $params = null, & $smarty = null ) {
                return $this->echoField('stock_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой страны.                                    |
                    |                                                             |
                    ============================================================ */

            public function idCountry ( $params = null, & $smarty = null ) {
                return $this->echoField('country_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой области.                                   |
                    |                                                             |
                    ============================================================ */

                public function idRegion ( $params = null, & $smarty = null ) {
                    return $this->echoField('region_id', $params, $smarty, 0);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого города.                                   |
                    |                                                             |
                    ============================================================ */

                public function idTown ( $params = null, & $smarty = null ) {
                    return $this->echoField('town_id', $params, $smarty, 0);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этой валюты.                                    |
                    |                                                             |
                    ============================================================ */

            public function idCurrency ( $params = null, & $smarty = null ) {
                return $this->echoField('currency_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого комментария.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {idComment                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function idComment ( $params = null, & $smarty = null ) {
                return $this->echoField('comment_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД-хеш этого комментария.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {hashComment                                  |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function hashComment ( $params = null, & $smarty = null ) {
                    $value = $this->echoField('comment_id', $params, $smarty, 0);
                    $value = $this->asHash($value);
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL комментируемой записи (не самого комментария,  |
                    |  то есть чистый URL публикации, без хеша комментария).      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {urlComment                                   |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function urlComment ( $params = null, & $smarty = null ) {
                    $value = '';
                    $item = $this->retrieveItem($params, $smarty);
                    if (isset($item->product_url)) {
                        $value = (empty($item->product_url_special) ? 'products/' : '')
                               . $item->product_url;
                    } elseif (isset($item->article_url)) {
                        $value = (empty($item->article_url_special) ? 'articles/' : '')
                               . $item->article_url;
                    } elseif (isset($item->news_url)) {
                        $value = (empty($item->news_url_special) ? 'news/' : '')
                               . $item->news_url;
                    }
                    $value = $this->asAttribute($value);
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести тип комментария как теговый data-атрибут.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {typeComment                                  |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function typeComment ( $params = null, & $smarty = null ) {
                    $value = '';
                    $item = $this->retrieveItem($params, $smarty);
                    if (isset($item->product_url)) $value = 'data-product';
                    elseif (isset($item->article_url)) $value = 'data-article';
                    elseif (isset($item->news_url)) $value = 'data-news';
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого заказа.                                   |
                    |                                                             |
                    ============================================================ */

            public function idOrder ( $params = null, & $smarty = null ) {
                return $this->echoField('order_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого способа доставки.                         |
                    |                                                             |
                    ============================================================ */

            public function idDelivery ( $params = null, & $smarty = null ) {
                return $this->echoField('delivery_method_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ИД этого способа оплаты.                           |
                    |                                                             |
                    ============================================================ */

            public function idPayment ( $params = null, & $smarty = null ) {
                return $this->echoField('payment_method_id', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести тип идентификатора этой записи.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {typeId                                       |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {typePath ...}                                |
                    |                                                             |
                    ============================================================ */

            public function typeId ( $params = null, & $smarty = null ) {
                $item = $this->retrieveItem($params, $smarty);
                if (isset($item->section_id))      $value = 'section';
                elseif (isset($item->product_id))  $value = 'product';
                elseif (isset($item->kit_id))      $value = 'kit';
                elseif (isset($item->article_id))  $value = 'article';
                elseif (isset($item->news_id))     $value = 'news';
                elseif (isset($item->file_id))     $value = 'file';
                elseif (isset($item->stock_id))    $value = 'stock';
                elseif (isset($item->brand_id))    $value = 'brand';
                elseif (isset($item->event_id))    $value = 'event';
                elseif (isset($item->country_id))  $value = 'country';
                elseif (isset($item->region_id))   $value = 'region';
                elseif (isset($item->town_id))     $value = 'town';
                elseif (isset($item->category_id)) $value = 'category';
                else $value = '';
                return $this->assignVar($params, $value, $smarty);
            }

                public function typePath ( $params = null, & $smarty = null ) {
                    $item = $this->retrieveItem($params, $smarty);
                    if (isset($item->section_id))      $value = 'sections/';
                    elseif (isset($item->product_id))  $value = 'products/';
                    elseif (isset($item->kit_id))      $value = 'kits/';
                    elseif (isset($item->article_id))  $value = 'articles/';
                    elseif (isset($item->news_id))     $value = 'news/';
                    elseif (isset($item->file_id))     $value = 'media/';
                    elseif (isset($item->stock_id))    $value = 'stocks/';
                    elseif (isset($item->brand_id))    $value = 'brands/';
                    elseif (isset($item->event_id))    $value = 'events/';
                    elseif (isset($item->country_id))  $value = 'countries/';
                    elseif (isset($item->region_id))   $value = 'regions/';
                    elseif (isset($item->town_id))     $value = 'towns/';
                    elseif (isset($item->category_id)) $value = 'catalog/';
                    else $value = '';
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта специальная страница активная.   |
                    |                                                             |
                    ============================================================ */

        public function activeSection ( $params = null, & $smarty = null ) {
            $mark = $this->getMarkParam($params);
            $field = 'section_id';
            $value = $this->getItemField($params, $smarty, $field, 0);
            $item = $this->getVar('section', $smarty);
            return !empty($item->$field) && $item->$field == $value ? $mark : '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта категория активная.              |
                    |                                                             |
                    ============================================================ */

            public function activeCategory ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'category_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('category', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот бренд активный.                 |
                    |                                                             |
                    ============================================================ */

            public function activeBrand ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'brand_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('brand', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот товар активный.                 |
                    |                                                             |
                    ============================================================ */

            public function activeProduct ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'product_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('product', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта новость активная.                |
                    |                                                             |
                    ============================================================ */

            public function activeNews ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'news_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('news_item', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта статья активная.                 |
                    |                                                             |
                    ============================================================ */

            public function activeArticle ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'article_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('article', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот склад активный.                 |
                    |                                                             |
                    ============================================================ */

            public function activeStock ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'stock_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('stock', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта страна активная.                 |
                    |                                                             |
                    ============================================================ */

            public function activeCountry ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->active);
                if (!$active) {
                    $field = 'country_id';
                    $value = !empty($item->$field) ? $item->$field : 0;
                    $item = $this->getVar('country', $smarty);
                    $active = !empty($item->$field) && $item->$field == $value;
                }
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта область активная.                |
                    |                                                             |
                    ============================================================ */

                public function activeRegion ( $params = null, & $smarty = null ) {
                    $mark = $this->getMarkParam($params);
                    $item = $this->retrieveItem($params, $smarty);
                    $active = !empty($item->active);
                    if (!$active) {
                        $field = 'region_id';
                        $value = !empty($item->$field) ? $item->$field : 0;
                        $item = $this->getVar('region', $smarty);
                        $active = !empty($item->$field) && $item->$field == $value;
                    }
                    return $active ? $mark : '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот город активный.                 |
                    |                                                             |
                    ============================================================ */

                public function activeTown ( $params = null, & $smarty = null ) {
                    $mark = $this->getMarkParam($params);
                    $item = $this->retrieveItem($params, $smarty);
                    $active = !empty($item->active);
                    if (!$active) {
                        $field = 'town_id';
                        $value = !empty($item->$field) ? $item->$field : 0;
                        $item = $this->getVar('town', $smarty);
                        $active = !empty($item->$field) && $item->$field == $value;
                    }
                    return $active ? $mark : '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта сортировка активная.             |
                    |                                                             |
                    ============================================================ */

            public function activeSort ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $idx = $this->getEnumIdx();
                $active = $this->getVar('sort_method', $smarty, 0);
                return $active == $idx ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если это сортировка наоборот.             |
                    |                                                             |
                    ============================================================ */

                public function activeSortDesc ( $params = null, & $smarty = null ) {
                    $mark = $this->getMarkParamAsChecked($params);
                    $active = $this->getVar('sort_descending', $smarty, 0);
                    return $active ? $mark : '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если это значение свойства активное.      |
                    |                                                             |
                    ============================================================ */

            public function activeOption ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParamAsChecked($params);
                $params = $this->cms->request->getGPrequest('filter_property');
                $id = $this->idProperty(null, $smarty);
                $value = $this->option(null, $smarty);
                return isset($params[$id][$value]) ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если это значение свойства действующее.   |
                    |                                                             |
                    ============================================================ */

                public function actualOption ( $params = null, & $smarty = null ) {
                    $mark = $this->paramAsSentence('mark', $params);
                    if ($mark == '') $mark = 'data-actual';
                    $value = $this->getVar('option', $smarty);
                    return !empty($value->actual) ? $mark : '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот размер страницы активный.       |
                    |                                                             |
                    ============================================================ */

            public function activeSize ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $max = $this->getVar('CurrentPageMaxsize', $smarty, 1);
                $sizes = $this->mySizes();
                $sizes = array_reverse($sizes);
                foreach ($sizes as $min) if ($min <= $max) break;
                $size = $this->getVar('item', $smarty, 1);
                return $size == $min ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот номер страницы активный.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {activePage                                   |
                    |                   [mark = Attribute]  нужный атрибут тега   |
                    |                                       (по умолч. selected)  |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function activePage ( $params = null, & $smarty = null ) {
                    $mark = $this->getMarkParam($params);
                    $active = $this->getVar('CurrentPage', $smarty, 0);
                    $idx = $this->getEnumIdx();
                    return $active == $idx ? $mark : '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот номер страницы под многоточием. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ellipsisPage                                 |
                    |                   [mark = Attribute]  нужный атрибут тега   |
                    |                                       (по умолчанию         |
                    |                                       data-ellipsis-left и  |
                    |                                       data-ellipsis-start и |
                    |                                       data-ellipsis-right)  |
                    |                   [size = pageCount]  размер пагинационного |
                    |                                       фрейма (по умолчанию  |
                    |                                       5 страниц)            |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function ellipsisPage ( $params = null, & $smarty = null ) {
                        $markStart = ' data-ellipsis-start';
                        $markLeft = $this->paramAsSentence('mark', $params);
                        if ($markLeft == '') {
                            $markLeft = 'data-ellipsis-left';
                            $markRight = 'data-ellipsis-right';
                        } else $markRight = $markLeft;
                        $size = $this->paramAsNatural('size', $params, 5);
                        $active = $this->getVar('CurrentPage', $smarty, 0);
                        $left = $active - $size;
                        $right = $active + $size;
                        $idx = $this->getEnumIdx();
                        return $idx <= $left ? ($markLeft . ($idx == $left ? $markStart
                                                                           : ''))
                                             : ($idx >= $right ? ($markRight . ($idx == $right ? $markStart
                                                                                               : ''))
                                                               : '');
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если эта валюта активная.                 |
                    |                                                             |
                    ============================================================ */

            public function activeCurrency ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $item = $this->retrieveItem($params, $smarty);
                $field = 'currency_id';
                $value = !empty($item->$field) ? $item->$field : 0;
                $item = $this->getVar('currency', $smarty);
                $active = !empty($item->$field) && $item->$field == $value;
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот способ доставки активный.       |
                    |                                                             |
                    ============================================================ */

            public function activeDelivery ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $varname = 'delivery_method_id';
                $id = $this->getInteger($varname, $smarty);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->$varname)
                          && ($item->$varname == $id || empty($id) && $this->getEnumNum() == 1);
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот способ оплаты активный.         |
                    |                                                             |
                    ============================================================ */

            public function activePayment ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $varname = 'payment_method_id';
                $id = $this->getInteger($varname, $smarty);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->$varname)
                          && ($item->$varname == $id || empty($id) && $this->getEnumNum() == 1);
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот комментарий активный.           |
                    |                                                             |
                    ============================================================ */

            public function activeComment ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParam($params);
                $id = $this->getInteger('post_parent_id', $smarty);
                $item = $this->retrieveItem($params, $smarty);
                $active = !empty($item->comment_id) && $item->comment_id == $id;
                return $active ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если этот язык активный.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {activeLanguage                               |
                    |                   [mark = Attribute]  нужный атрибут тега   |
                    |                                       (по умолчан. checked) |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function activeLanguage ( $params = null, & $smarty = null ) {
                $mark = '';
                $active = $this->htmlLang(null, $smarty);
                if ($this->isActualString($active)) {
                    $active = $this->cms->text->lowerCase($active);
                    $current = $this->language(null, $smarty);
                        $current = $this->cms->text->lowerCase($current);
                    if ($current == $active) {
                        $mark = $this->getMarkParamAsChecked($params);
                    }
                }
                return $mark;
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут, если это первый элемент перечисления.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {activeFirst                                  |
                    |                   [mark = Attribute]  нужный атрибут тега   |
                    |                                       (по умолчан. checked) |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function activeFirst ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParamAsChecked($params);
                return $this->getEnumNum() == 1 ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага Выделен, если эта запись такая.      |
                    |                                                             |
                    ============================================================ */

        public function highlighted ( $params = null, & $smarty = null ) {
            $mark = $this->paramAsSentence('mark', $params);
            if ($mark == '') $mark = 'data-highlighted';
            $value = $this->getItemFlag($params, $smarty, 'highlighted');
            return $value ? $mark : '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага Хит, если этот товар такой.          |
                    |                                                             |
                    ============================================================ */

            public function hit ( $params = null, & $smarty = null ) {
                $mark = $this->paramAsSentence('mark', $params);
                if ($mark == '') $mark = 'data-hit';
                $value = $this->getItemFlag($params, $smarty, 'hit');
                return $value ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага Новинка, если этот товар такой.      |
                    |                                                             |
                    ============================================================ */

            public function newest ( $params = null, & $smarty = null ) {
                $mark = $this->paramAsSentence('mark', $params);
                if ($mark == '') $mark = 'data-newest';
                $value = $this->getItemFlag($params, $smarty, 'newest');
                return $value ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага Акционный, если этот товар такой.    |
                    |                                                             |
                    ============================================================ */

            public function actional ( $params = null, & $smarty = null ) {
                $mark = $this->paramAsSentence('mark', $params);
                if ($mark == '') $mark = 'data-actional';
                $value = $this->getItemFlag($params, $smarty, 'actional');
                return $value ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага СкороВпродаже, если этот товар такой.|
                    |                                                             |
                    ============================================================ */

            public function awaited ( $params = null, & $smarty = null ) {
                $mark = $this->paramAsSentence('mark', $params);
                if ($mark == '') $mark = 'data-awaited';
                $value = $this->getItemFlag($params, $smarty, 'awaited');
                return $value ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага Обсуждаемый, если этот товар такой.  |
                    |                                                             |
                    ============================================================ */

            public function commented ( $params = null, & $smarty = null ) {
                $mark = $this->paramAsSentence('mark', $params);
                if ($mark == '') $mark = 'data-commented';
                $value = $this->getItemFlag($params, $smarty, 'commented');
                return $value ? $mark : '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага НеМожетБытьПродан, если этот вариант |
                    |  товара такой.                                              |
                    |                                                             |
                    ============================================================ */

            public function withhold ( $params = null, & $smarty = null ) {
                $this->mustbeParam('from', $params, 'variant');
                $mark = $this->paramAsSentence('mark', $params);
                if ($mark == '') $mark = 'data-withhold';
                $value = $this->cms->any->settings->getAsBoolean('cart_enable_reservation')
                      || $this->getItemFlag($params, $smarty, 'stock') > 0;
                return $value ? '' : $mark;
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести атрибут флага Экспонат, если этот товар такой.     |
                    |                                                             |
                    ============================================================ */

                public function exhibit ( $params = null, & $smarty = null ) {
                    $mark = $this->paramAsSentence('mark', $params);
                    if ($mark == '') $mark = 'data-exhibit';
                    $value = $this->getItemFlag($params, $smarty, 'non_usable');
                    return $value ? $mark : '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести мета тайтл этой записи.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemTitle                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [strict = TRUE]       если не брать из    |
                    |                                         названия, когда     |
                    |                                         тайтл пустой        |
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function itemTitle ( $params = null, & $smarty = null ) {
            $def = $this->paramAsString('def', $params);
            $field = 'meta_title';
            if (!$this->paramAsBoolean('strict', $params)) {
                $field = array(
                    $field,
                    'model',
                    'header',
                    'name'
                );
            }
            return $this->echoField($field, $params, $smarty, $def);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести мета описание этой записи.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemMeta                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemMeta ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField('meta_description', $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести мета ключевые слова этой записи.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemKeywords                                 |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemKeywords ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField('meta_keywords', $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести теги этой записи.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemKeywords                                 |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemTags ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField('tags', $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название этой записи.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemName                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemName ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField(array('model', 'header', 'name'), $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести относительный URL этой записи.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemUrl                                      |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemUrl ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField('url', $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести категорию этой записи.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemCategory                                 |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemCategoryUrl ... }                        |
                    |                                                             |
                    ============================================================ */

            public function itemCategory ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField(array('category_plural', 'category'), $params, $smarty, $def);
            }

                public function itemCategoryUrl ( $params = null, & $smarty = null ) {
                    $params2 = $this->takeFromParams($params);
                    $url = $this->echoField('category_url', $params2, $smarty);
                    if ($this->isActualString($url)) {
                        $site = $this->getSiteUrl();
                        $path = $this->echoField('category_url_path', $params2, $smarty);
                    } else {
                        $site = '';
                        $path = '';
                        $url = $this->paramAsString('def', $params);
                    }
                    return $site . $path . $url;
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести бренд этой записи.                                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemBrand                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemBrandUrl ... }                           |
                    |                                                             |
                    ============================================================ */

            public function itemBrand ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField('brand', $params, $smarty, $def);
            }

                public function itemBrandUrl ( $params = null, & $smarty = null ) {
                    $params2 = $this->takeFromParams($params);
                    $url = $this->echoField('brand_url', $params2, $smarty);
                    if ($this->isActualString($url)) {
                        $site = $this->getSiteUrl();
                        $path = $this->echoField('brand_url_path', $params2, $smarty);
                    } else {
                        $site = '';
                        $path = '';
                        $url = $this->paramAsString('def', $params);
                    }
                    return $site . $path . $url;
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести бренд или категорию этой записи.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemBrandOrCategory                          |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function itemBrandOrCategory ( $params = null, & $smarty = null ) {
                    $def = $this->paramAsString('def', $params);
                    return $this->echoField(array('brand', 'category_plural', 'category'), $params, $smarty, $def);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести краткий текст этой записи.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemAnnotation                               |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemAnnotation ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField(array('annotation', 'description'), $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести полный текст этой записи.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemBody                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemBody ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField(array('body', 'description'), $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести SEO-текст этой записи.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemSeo                                      |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {seo                                          |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemSeo ( $params = null, & $smarty = null ) {
                return $this->seoBody($params, $smarty);
            }

                public function seo ( $params = null, & $smarty = null ) {
                    return $this->itemSeo($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести текст субдомена этой записи.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemSubdomain                                |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {subdomain                                    |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemSubdomain ( $params = null, & $smarty = null ) {
                return $this->subdomainHtml($params, $smarty);
            }

                public function subdomain ( $params = null, & $smarty = null ) {
                    return $this->itemSubdomain($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести составное имя пользователя из этой записи.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemUser                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {user                                         |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemUser ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField(array('compound_name', 'user_name'), $params, $smarty, $def);
            }

                public function user ( $params = null, & $smarty = null ) {
                    return $this->itemUser($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести ник пользователя из его записи.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {userNick                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $user)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function userNick ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('from', $params, 'user');
                        $def = $this->paramAsString('def', $params);
                        return $this->echoField('nickname', $params, $smarty, $def);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести из записи пользователя баланс его внутреннего      |
                    |  счета.                                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {userCash                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $user)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                                         (по умолчанию 0)    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function userCash ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('from', $params, 'user');
                        $params2 = $this->takeFromParams($params);
                        $value = $this->echoField('cash', $params2, $smarty, null, FALSE);
                        if (is_null($value))
                            $value = $this->paramAsString('def', $params, 0);
                        else
                            $value = $this->getValueAsPrice($value);
                        return $this->echoValue($value, $params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести из записи пользователя количество юнитов на его    |
                    |  внутреннем счете.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {userUnits                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $user)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                                         (по умолчанию 0)    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function userUnits ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('from', $params, 'user');
                        $def = $this->paramAsString('def', $params, 0);
                        return $this->echoField('units', $params, $smarty, $def);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести из записи пользователя его составное имя.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {userName                                     |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $user)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function userName ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('from', $params, 'user');
                        return $this->itemUser($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести из записи пользователя емейл 1 или 2.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {userEmail                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $user)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function userEmail ( $params = null, & $smarty = null ) {
                        $this->mustbeParam('from', $params, 'user');
                        return $this->itemEmail($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести из записи пользователя телефон 1 или 2.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {userPhone                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $user)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                        public function userPhone ( $params = null, & $smarty = null ) {
                            $this->mustbeParam('from', $params, 'user');
                            return $this->itemPhone($params, $smarty);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести имя автора этого комментария.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {commentator                                  |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function commentator ( $params = null, & $smarty = null ) {
                    $def = $this->paramAsString('def', $params);
                    return $this->echoField('name', $params, $smarty, $def);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести текст этого комментария.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {comment                                      |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [size = maxSize]      предельная длина    |
                    |                                         выводимого ТЕКСТА...|
                    |                                         (по умолчанию весь) |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                    public function comment ( $params = null, & $smarty = null ) {
                        $value = $this->getItemField($params, $smarty, 'comment');
                        $value = trim($value);
                        $value = $this->asAttribute($value);
                        $size = $this->paramAsNatural('size', $params, FALSE);
                        if ($size === FALSE) $value = nl2br($value);
                        else {
                            $method = 'plainText';
                            $helper = $this->getVar('helper');
                            if ($this->cms->any->hasMethod($helper, $method)) {
                                $params2 = array('from' => '',
                                                 'def'  => $value,
                                                 'size' => $size);
                                $value = $helper->$method($params2, $smarty);
                            }
                        }
                        return $this->assignVar($params, $value, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести краткий текст этой записи.                         |
                    |                                                             |
                    ============================================================ */

                    public function annotation ( $params = null, & $smarty = null ) {
                        $value = '';
                        $method = 'annotation';
                        $helper = $this->getVar('helper');
                        if ($this->cms->any->hasMethod($helper, $method)) {
                            $params2 = array();
                            $this->copyParam('from', $params, $params2);
                            $this->copyParam('item', $params, $params2);
                            $this->copyParam('def', $params, $params2);
                            $value = $helper->$method($params2, $smarty);
                            $size = $this->paramAsNatural('size', $params, FALSE);
                            if ($size !== FALSE) {
                                $method = 'plainText';
                                $helper = $this->getVar('helper');
                                if ($this->cms->any->hasMethod($helper, $method)) {
                                    $params2 = array('from' => '',
                                                     'def'  => $value,
                                                     'size' => $size);
                                    $value = $helper->$method($params2, $smarty);
                                }
                                $value = $this->asAttribute($value);
                            }
                        }
                        return $this->assignVar($params, $value, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести IP-адрес посетителя.                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {visitorIp                                    |
                    |                   [full = TRUE]         если всю цепочку IP |
                    |                                         для проксируемых    |
                    |                                         адресов, а не только|
                    |                                         первый IP в цепи    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function visitorIp ( $params = null, & $smarty = null ) {
                    $value = $this->cms->security->getVisitorIp();
                    $value = $this->cms->text->lowerCase($value);
                    $items = $this->asUniqueStringArray($value);
                    if (count($items) > 1) {
                        $mustbeFull = $this->paramAsBoolean('full', $params);
                        if (!$mustbeFull) $value = array_shift($items);
                        $value = preg_replace('/,/u', ', ', $value);
                    }
                    return $this->echoValue($value, $params, $smarty);
                }

                    public function visitorHost ( $params = null, & $smarty = null ) {
                        $value = $this->cms->security->getVisitorHost();
                        return $this->echoValue($value, $params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести откуда пришел посетитель.                          |
                    |                                                             |
                    ============================================================ */

                    public function visitorReferer ( $params = null, & $smarty = null ) {
                        $value = $this->getHttpUserReferer(TRUE);
                        return $this->echoValue($value, $params, $smarty);
                    }

                    public function visitorRefererDomain ( $params = null, & $smarty = null ) {
                        $value = $this->getHttpUserReferer(TRUE);
                        $value = $this->getUrlDomain($value);
                        return $this->echoValue($value, $params, $smarty);
                    }

                    public function visitorRefererQuery ( $params = null, & $smarty = null ) {
                        $value = $this->getHttpUserReferer(TRUE, FALSE, TRUE);
                        return $this->echoValue($value, $params, $smarty);
                    }

                    public function visitorQueryRequest ( $params = null, & $smarty = null ) {
                        /* TODO */
                        return '';
                    }

                    public function visitorQueryGet ( $params = null, & $smarty = null ) {
                        /* TODO */
                        return '';
                    }

                    public function visitorQueryPost ( $params = null, & $smarty = null ) {
                        /* TODO */
                        return '';
                    }

                    public function visitorQueryFiles ( $params = null, & $smarty = null ) {
                        /* TODO */
                        return '';
                    }

                    public function visitorCookies ( $params = null, & $smarty = null ) {
                        /* TODO */
                        return '';
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести емейл 1 или 2 этой записи.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemEmail                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemEmail ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                $params2 = $this->takeFromParams($params);
                $def = $this->echoField('email2', $params2, $smarty, $def, FALSE);
                return $this->echoField('email', $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести телефон 1 или 2 этой записи.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemPhone                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function itemPhone ( $params = null, & $smarty = null ) {
                    $def = $this->paramAsString('def', $params);
                    $params2 = $this->takeFromParams($params);
                    $def = $this->echoField('phone2', $params2, $smarty, $def, FALSE);
                    return $this->echoField('phone', $params, $smarty, $def);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести код этого товара.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemPCode                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                                         (по умолчанию --/--)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pcode                                        |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemPCode ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params, '--/--');
                return $this->echoField('pcode', $params, $smarty, $def);
            }

                public function pcode ( $params = null, & $smarty = null ) {
                    return $this->itemPCode($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести артикул этого варианта товара.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemSku                                      |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию в     |
                    |                                         переменной $variant)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                                         (по умолчанию --/--)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {sku                                          |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemSku ( $params = null, & $smarty = null ) {
                $this->mustbeParam('from', $params, 'variant');
                $def = $this->paramAsString('def', $params, '--/--');
                return $this->echoField('sku', $params, $smarty, $def);
            }

                public function sku ( $params = null, & $smarty = null ) {
                    return $this->itemSku($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество этого варианта товара на складе.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemStock                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию в     |
                    |                                         переменной $variant)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemStock ( $params = null, & $smarty = null ) {
                $this->mustbeParam('from', $params, 'variant');
                return $this->echoField('stock', $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название этого варианта товара.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemVariant                                  |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemVariant ( $params = null, & $smarty = null ) {
                $def = $this->paramAsString('def', $params);
                return $this->echoField(array('variant_name', 'name'), $params, $smarty, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести отобранные свойства этого товара.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemProps                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function itemProps ( $params = null, & $smarty = null ) {
                    $def = $this->paramAsString('def', $params);
                    return $this->echoField('name_properties', $params, $smarty, $def);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество товаров в этой записи.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemCount                                    |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemCount ( $params = null, & $smarty = null ) {
                return $this->echoField(array('products_count', 'my_products_count', 'count'), $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество просмотров в этой записи.               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {browsed                                      |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function browsed ( $params = null, & $smarty = null ) {
                    return $this->echoField('browsed', $params, $smarty, 0);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество комментариев в этой записи.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {itemCommentsCount                            |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function itemCommentsCount ( $params = null, & $smarty = null ) {
                return $this->echoField(array('comments_count', 'product_comments_count'), $params, $smarty, 0);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название специальной страницы.                     |
                    |                                                             |
                    ============================================================ */

        public function nameSection ( $params = null, & $smarty = null ) {
            return $this->runName($params, $smarty, 'section', 'title');
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название категории.                                |
                    |                                                             |
                    ============================================================ */

            public function nameCategory ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'category');
            }

                public function category ( $params = null, & $smarty = null ) {
                    return $this->nameCategory($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название бренда.                                   |
                    |                                                             |
                    ============================================================ */

            public function nameBrand ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'brand');
            }

                public function brand ( $params = null, & $smarty = null ) {
                    return $this->nameBrand($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название товара.                                   |
                    |                                                             |
                    ============================================================ */

            public function nameProduct ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'product');
            }

                public function product ( $params = null, & $smarty = null ) {
                    return $this->nameProduct($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название варианта товара.                          |
                    |                                                             |
                    ============================================================ */

                public function nameVariant ( $params = null, & $smarty = null ) {
                    return $this->runName($params, $smarty, 'variant');
                }

                    public function variant ( $params = null, & $smarty = null ) {
                        return $this->nameVariant($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название новости.                                  |
                    |                                                             |
                    ============================================================ */

            public function nameNews ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'news_item');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название статьи.                                   |
                    |                                                             |
                    ============================================================ */

            public function nameArticle ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'article');
            }

                public function article ( $params = null, & $smarty = null ) {
                    return $this->nameArticle($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название склада.                                   |
                    |                                                             |
                    ============================================================ */

            public function nameStock ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'stock');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название страны.                                   |
                    |                                                             |
                    ============================================================ */

            public function nameCountry ( $params = null, & $smarty = null ) {
                return $this->runName($params, $smarty, 'country');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название области.                                  |
                    |                                                             |
                    ============================================================ */

                public function nameRegion ( $params = null, & $smarty = null ) {
                    return $this->runName($params, $smarty, 'region');
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название города.                                   |
                    |                                                             |
                    ============================================================ */

                public function nameTown ( $params = null, & $smarty = null ) {
                    return $this->runName($params, $smarty, 'town');
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести название свойства товара.                          |
                    |                                                             |
                    ============================================================ */

            public function specification ( $params = null, & $smarty = null ) {
                $from = $this->paramAsSentence('from', $params, 'specification');
                $value = $this->getVar($from, $smarty);
                if (is_array($value)) $value = reset($value);
                $value = isset($value->name) ? trim($value->name) : '';
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение свойства товара.                          |
                    |                                                             |
                    ============================================================ */

                public function specie ( $params = null, & $smarty = null ) {
                    $from = $this->paramAsSentence('from', $params, 'specie');
                    $value = $this->getVar($from, $smarty);
                    $value = isset($value->value) ? trim($value->value) : '';
                    if ($value == '') $value = '--/--';
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение свойства фильтра.                         |
                    |                                                             |
                    ============================================================ */

                public function option ( $params = null, & $smarty = null ) {
                    $from = $this->paramAsSentence('from', $params, 'option');
                    $value = $this->getVar($from, $smarty);
                    $value = isset($value->value) ? trim($value->value) : '';
                    if ($value == '') $value = '--/--';
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести текст переменной item.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {item                                         |
                    |                   [safe = TRUE]         если безопасно для  |
                    |                                         применения в теге   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {value ...}                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {type ...}                                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {hash ...}                                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {language ...}                                |
                    |                                                             |
                    ============================================================ */

            public function item ( $params = null, & $smarty = null ) {
                $value = $this->getVar('item', $smarty);
                $escaped = $this->paramAsBoolean('safe', $params);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                public function value ( $params = null, & $smarty = null ) {
                    $value = $this->getVar('value', $smarty);
                    $escaped = $this->paramAsBoolean('safe', $params);
                    return $this->echoValue($value, $params, $smarty, $escaped);
                }

                public function type ( $params = null, & $smarty = null ) {
                    $value = $this->getVar('type', $smarty);
                    $escaped = $this->paramAsBoolean('safe', $params);
                    return $this->echoValue($value, $params, $smarty, $escaped);
                }

                public function hash ( $params = null, & $smarty = null ) {
                    $value = $this->getVar('hash', $smarty);
                    $escaped = $this->paramAsBoolean('safe', $params);
                    return $this->echoValue($value, $params, $smarty, $escaped);
                }

                public function language ( $params = null, & $smarty = null ) {
                    $value = $this->getVar('language', $smarty);
                    $escaped = $this->paramAsBoolean('safe', $params);
                    return $this->echoValue($value, $params, $smarty, $escaped);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание специальной страницы.                     |
                    |                                                             |
                    ============================================================ */

        public function bodySection ( $params = null, & $smarty = null ) {
            return $this->runBody($params, $smarty, 'section');
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание категории.                                |
                    |                                                             |
                    ============================================================ */

            public function bodyCategory ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'category');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание бренда.                                   |
                    |                                                             |
                    ============================================================ */

            public function bodyBrand ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'brand');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание товара.                                   |
                    |                                                             |
                    ============================================================ */

            public function bodyProduct ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'product');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание новости.                                  |
                    |                                                             |
                    ============================================================ */

            public function bodyNews ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'news_item');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание статьи.                                   |
                    |                                                             |
                    ============================================================ */

            public function bodyArticle ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'article');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание склада.                                   |
                    |                                                             |
                    ============================================================ */

            public function bodyStock ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'stock');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание страны.                                   |
                    |                                                             |
                    ============================================================ */

            public function bodyCountry ( $params = null, & $smarty = null ) {
                return $this->runBody($params, $smarty, 'country');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание области.                                  |
                    |                                                             |
                    ============================================================ */

                public function bodyRegion ( $params = null, & $smarty = null ) {
                    return $this->runBody($params, $smarty, 'region');
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести описание города.                                   |
                    |                                                             |
                    ============================================================ */

                public function bodyTown ( $params = null, & $smarty = null ) {
                    return $this->runBody($params, $smarty, 'town');
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этой записи.                         |
                    |                                                             |
                    ============================================================ */

        public function thumb ( $params = null, & $smarty = null ) {
            $this->destroyParam('from', $params);
            $item = $this->retrieveItem($params, $smarty);
            if (isset($item->section_id))      return $this->thumbSection($params, $smarty);
            elseif (isset($item->product_id))  return $this->thumbProduct($params, $smarty);
            elseif (isset($item->kit_id))      $value = '';    // TODO
            elseif (isset($item->article_id))  return $this->thumbArticle($params, $smarty);
            elseif (isset($item->news_id))     return $this->thumbNews($params, $smarty);
            elseif (isset($item->file_id))     $value = '';    // TODO
            elseif (isset($item->stock_id))    return $this->thumbStock($params, $smarty);
            elseif (isset($item->brand_id))    return $this->thumbBrand($params, $smarty);
            elseif (isset($item->event_id))    $value = '';    // TODO
            elseif (isset($item->country_id))  return $this->thumbCountry($params, $smarty);
            elseif (isset($item->region_id))   return $this->thumbRegion($params, $smarty);
            elseif (isset($item->town_id))     return $this->thumbTown($params, $smarty);
            elseif (isset($item->category_id)) return $this->thumbCategory($params, $smarty);
            $value = $this->getThemeUrl() . 'images/no-photo.png';
            return $this->assignVar($params, $value, $smarty);
        }

                        /* ============================================================
                        |                                                             |
                        |  Вывести URL миниатюры этой специальной страницы.           |
                        |                                                             |
                        ============================================================ */

            public function thumbSection ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'section');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этой категории.                      |
                    |                                                             |
                    ============================================================ */

            public function thumbCategory ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'category');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этого бренда.                        |
                    |                                                             |
                    ============================================================ */

            public function thumbBrand ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'brand');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этого товара.                        |
                    |                                                             |
                    ============================================================ */

            public function thumbProduct ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'product');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этой новости.                        |
                    |                                                             |
                    ============================================================ */

            public function thumbNews ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'news');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этой статьи.                         |
                    |                                                             |
                    ============================================================ */

            public function thumbArticle ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'article');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этого склада.                        |
                    |                                                             |
                    ============================================================ */

            public function thumbStock ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'stock');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этой страны.                         |
                    |                                                             |
                    ============================================================ */

            public function thumbCountry ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'country');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этой области.                        |
                    |                                                             |
                    ============================================================ */

                public function thumbRegion ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('view', $params, '*');
                    return $this->runFindImage($params, $smarty, 'region');
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этого города.                        |
                    |                                                             |
                    ============================================================ */

                public function thumbTown ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('view', $params, '*');
                    return $this->runFindImage($params, $smarty, 'town');
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этой записи.                        |
                    |                                                             |
                    ============================================================ */

        public function photo ( $params = null, & $smarty = null ) {
            $this->destroyParam('from', $params);
            $item = $this->retrieveItem($params, $smarty);
            if (isset($item->section_id))      return $this->photoSection($params, $smarty);
            elseif (isset($item->product_id))  return $this->photoProduct($params, $smarty);
            elseif (isset($item->kit_id))      $value = '';    // TODO
            elseif (isset($item->article_id))  return $this->photoArticle($params, $smarty);
            elseif (isset($item->news_id))     return $this->photoNews($params, $smarty);
            elseif (isset($item->file_id))     $value = '';    // TODO
            elseif (isset($item->stock_id))    return $this->photoStock($params, $smarty);
            elseif (isset($item->brand_id))    return $this->photoBrand($params, $smarty);
            elseif (isset($item->event_id))    $value = '';    // TODO
            elseif (isset($item->country_id))  return $this->photoCountry($params, $smarty);
            elseif (isset($item->region_id))   return $this->photoRegion($params, $smarty);
            elseif (isset($item->town_id))     return $this->photoTown($params, $smarty);
            elseif (isset($item->category_id)) return $this->photoCategory($params, $smarty);
            $value = $this->getThemeUrl() . 'images/no-photo.png';
            return $this->assignVar($params, $value, $smarty);
        }

                        /* ============================================================
                        |                                                             |
                        |  Вывести URL фотографии этой специальной страницы.          |
                        |                                                             |
                        ============================================================ */

            public function photoSection ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'section', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этой категории.                     |
                    |                                                             |
                    ============================================================ */

            public function photoCategory ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'category', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этого бренда.                       |
                    |                                                             |
                    ============================================================ */

            public function photoBrand ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'brand', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этого товара.                       |
                    |                                                             |
                    ============================================================ */

            public function photoProduct ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'product', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этой новости.                       |
                    |                                                             |
                    ============================================================ */

            public function photoNews ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'news', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этой статьи.                        |
                    |                                                             |
                    ============================================================ */

            public function photoArticle ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'article', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этого склада.                       |
                    |                                                             |
                    ============================================================ */

            public function photoStock ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'stock', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этой страны.                        |
                    |                                                             |
                    ============================================================ */

            public function photoCountry ( $params = null, & $smarty = null ) {
                $this->mustbeParam('view', $params, '*');
                return $this->runFindImage($params, $smarty, 'country', FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этой области.                       |
                    |                                                             |
                    ============================================================ */

                public function photoRegion ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('view', $params, '*');
                    return $this->runFindImage($params, $smarty, 'region', FALSE);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL фотографии этого города.                       |
                    |                                                             |
                    ============================================================ */

                public function photoTown ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('view', $params, '*');
                    return $this->runFindImage($params, $smarty, 'town', FALSE);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести цену (с учетом скидки) текущего варианта товара.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {price                                        |
                    |                   [full = TRUE]         если цену без скидки|
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолч. $variant)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function price ( $params = null, & $smarty = null ) {
            $from = $this->paramAsSentence('from', $params, 'variant');
            $fields = $this->paramAsBoolean('full', $params)
                      ? $from . '->price'
                      : array($from . '->final_price',
                              $from . '->discount_price',
                              $from . '->price');
            $value = $this->getPrice($fields, $smarty);
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество товарной позиции в корзине / заказе     |
                    |  (функция amount допускает отрицательную величину, функция  |
                    |  quantity всегда положительная).                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {amount                                       |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {quantity ... }                               |
                    |                                                             |
                    ============================================================ */

            public function amount ( $params = null, & $smarty = null ) {
                $from = $this->paramAsSentence('from', $params, 'item');
                $value = $this->getInteger(array($from . '->amount',
                                                 $from . '->quantity'), $smarty);
                return $this->assignVar($params, $value, $smarty);
            }

                public function quantity ( $params = null, & $smarty = null ) {
                    $params2 = $this->takeFromParams($params);
                    $value = $this->amount($params2, $smarty);
                    if ($value < 0) $value = $value * -1;
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести стоимость товарной позиции в корзине / заказе.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {sum                                          |
                    |                   [full = TRUE]         если стоимость без  |
                    |                                         учета скидки        |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function sum ( $params = null, & $smarty = null ) {
                $params2 = $this->takeFromParams($params);
                $value = $this->amount($params2, $smarty);
                $from = $params2['from'];
                $fields = $this->paramAsBoolean('full', $params)
                          ? $from . '->price'
                          : array($from . '->final_price',
                                  $from . '->discount_price',
                                  $from . '->price');
                $value = $this->getPrice($fields, $smarty, $value);
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести итоговую стоимость заказа.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {total                                        |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчанию $item)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function total ( $params = null, & $smarty = null ) {
                $from = $this->paramAsSentence('from', $params, 'item');
                $value = $this->getPrice($from . '->total_amount', $smarty);
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести итоговую стоимость доставки в заказе.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {totalDelivery                                |
                    |                   [from = VarName]      где хранится запись |
                    |                                         (по умолчан. $order)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {totalDiscount ... }                          |
                    |                                                             |
                    ============================================================ */

                public function totalDelivery ( $params = null, & $smarty = null ) {
                    $from = $this->paramAsSentence('from', $params, 'order');
                    $value = $this->getPrice($from . '->delivery_price', $smarty);
                    return $this->assignVar($params, $value, $smarty);
                }

                public function totalDiscount ( $params = null, & $smarty = null ) {
                    $from = $this->paramAsSentence('from', $params, 'order');
                    $value = $this->getPrice($from . '->discount_sum', $smarty);
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество найденных записей.                      |
                    |                                                             |
                    ============================================================ */

        public function found ( $params = null, & $smarty = null ) {
            $value = $this->getNumber('items_count', $smarty);
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести индекс текущего элемента в массиве, перечисляемом  |
                    |  в этот момент с помощью функции enum***.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {idx                                          |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {idxAsTag                                     |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {idxAsHashtag                                 |
                    |                   такие же параметры как перечислены выше   |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function idx ( $params = null, & $smarty = null ) {
            $value = $this->getEnumIdx();
            return $this->assignVar($params, $value, $smarty);
        }

            public function idxAsTag ( $params = null, & $smarty = null ) {
                $value = $this->getEnumIdx();
                $value = preg_replace('~[^' . $this->regexpDomainChars(FALSE, FALSE) . ']+~ui', '-', $value);
                return $this->assignVar($params, $value, $smarty);
            }

            public function idxAsHashtag ( $params = null, & $smarty = null ) {
                $value = $this->getEnumIdx();
                $value = preg_replace('~[^' . $this->regexpDomainChars(FALSE, FALSE) . ']+~ui', ' ', $value);
                $value = function_exists('mb_convert_case') ? mb_convert_case($value, MB_CASE_TITLE, 'UTF-8')
                                                            : ucwords($this->cms->text->lowerCase($value));
                $value = preg_replace('/ /u', '', $value);
                if ($this->isActualString($value)) $value = '#' . $value;
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести номер итерации функции enum***.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {num                                          |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {numPrev ... }                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {numNext ... }                                |
                    |                                                             |
                    ============================================================ */

            public function num ( $params = null, & $smarty = null ) {
                $value = $this->getEnumNum();
                return $this->assignVar($params, $value, $smarty);
            }

                public function numPrev ( $params = null, & $smarty = null ) {
                    $prev = 0;
                    $this->getEnumNum($prev);
                    return $this->assignVar($params, $prev, $smarty);
                }

                public function numNext ( $params = null, & $smarty = null ) {
                    $prev = 0;
                    $next = 0;
                    $this->getEnumNum($prev, $next);
                    return $this->assignVar($params, $next, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Прокрутить за конец итераций функции enum***.              |
                    |                                                             |
                    ============================================================ */

            public function out ( $params = null, & $smarty = null ) {
                if (!empty($this->enumStack) && is_array($this->enumStack)) {
                    $index = count($this->enumStack) - 1;
                    while (next($this->enumStack[$index]) !== FALSE) $this->enumStackNums[$index]++;
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести идентификатор текущего способа сортировки.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {sortMethod                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function sortMethod ( $params = null, & $smarty = null ) {
            return $this->echoVariable('sort_method', $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести итоговую сумму корзины.                            |
                    |                                                             |
                    ============================================================ */

        public function cartTotal ( $params = null, & $smarty = null ) {
            $value = $this->getPrice('cart_total_price', $smarty);
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести итоговую скидку корзины.                           |
                    |                                                             |
                    ============================================================ */

            public function cartDiscount ( $params = null, & $smarty = null ) {
                $value = $this->getPrice('cart_total_discount_sum', $smarty);
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество товаров в корзине.                      |
                    |                                                             |
                    ============================================================ */

            public function cartQuantity ( $params = null, & $smarty = null ) {
                $value = $this->getNumber('cart_products_num', $smarty);
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода ФАМИЛИЯ, ИМЯ, ОТЧЕСТВО.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueSurname                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueName ...}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valuePatronymic ...}                         |
                    |                                                             |
                    ============================================================ */

        public function valueSurname ( $params = null, & $smarty = null ) {
            $tpl = $this->getModuleOrTemplateName($smarty);
            switch ($tpl) {
                case 'registration': $name = 'registration_name'; break;
                case 'cart':         $name = 'name'; break;
                case 'account':      $name = 'name'; break;
                case 'feedback':     // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                case 'login':        // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                case 'callme':       // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                default:             $name = '';
            }
            return $this->echoVariable($name, $params, $smarty);
        }

            public function valueName ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':     $name = 'feedback_name'; break;
                    case 'registration': $name = 'registration_name2'; break;
                    case 'callme':       $name = 'callme_name'; break;
                    case 'cart':         $name = 'name3'; break;
                    case 'account':      $name = 'name3'; break;
                    case 'comments':
                    case 'article':
                    case 'articles':
                    case 'newsitem':
                    case 'news':
                    case 'product':      $name = 'post_name'; break;
                    case 'login':        // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    default:             $name = 'name';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

            public function valuePatronymic ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'registration': $name = 'registration_name3'; break;
                    case 'cart':         $name = 'name2'; break;
                    case 'account':      $name = 'name2'; break;
                    case 'feedback':     // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'login':        // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'callme':       // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    default:             $name = '';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода НИК.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueNick                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueNick ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'registration': $name = 'registration_nickname'; break;
                    case 'account':      $name = 'nickname'; break;
                    default:             $name = 'nickname';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода ЕМЕЙЛ.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueEmail                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueEmail2 ...}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueLogin ...}                              |
                    |                                                             |
                    ============================================================ */

            public function valueEmail ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':        $name = 'feedback_email'; break;
                    case 'login':           $name = 'login_login'; break;
                    case 'loginremind':
                    case 'password_remind': $name = 'item->email'; break;
                    case 'registration':    $name = 'registration_email'; break;
                    case 'callme':          $name = 'callme_email'; break;
                    case 'comments':
                    case 'article':
                    case 'articles':
                    case 'newsitem':
                    case 'news':
                    case 'product':         $name = 'post_email'; break;
                    case 'cart':
                    default:                $name = 'email';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                public function valueEmail2 ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('email2', $params, $smarty);
                }

                public function valueLogin ( $params = null, & $smarty = null ) {
                    return $this->valueEmail($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода ТЕЛЕФОН.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valuePhone                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valuePhone2 ...}                             |
                    |                                                             |
                    ============================================================ */

            public function valuePhone ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':     $name = 'feedback_phone'; break;
                    case 'registration': $name = 'registration_phone'; break;
                    case 'callme':       $name = 'callme_phone'; break;
                    case 'login':        // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'cart':
                    default:             $name = 'phone';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                public function valuePhone2 ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('phone2', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода АДРЕС (страна, область, город, |
                    |  улица, дом, квартира, индекс).                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueAddress                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueCountry ...}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRegion ...}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueTown ...}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueStreet ...}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueHouse ...}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRoom ...}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueZipcode ...}                            |
                    |                                                             |
                    ============================================================ */

            public function valueAddress ( $params = null, & $smarty = null ) {
                return $this->echoVariable('address', $params, $smarty);
            }

                public function valueCountry ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address', $params, $smarty);
                }

                public function valueRegion ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address_2', $params, $smarty);
                }

                public function valueTown ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address_3', $params, $smarty);
                }

                public function valueStreet ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address_4', $params, $smarty);
                }

                public function valueHouse ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address_5', $params, $smarty);
                }

                public function valueRoom ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address_9', $params, $smarty);
                }

                public function valueZipcode ( $params = null, & $smarty = null ) {
                    return $this->echoVariable('address_10', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода ОТДЕЛ.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueDepartment                              |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueDepartment ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback': $name = 'feedback_department'; break;
                    default:         $name = 'department';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода ТЕМА.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueSubject                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueSubject ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':     $name = 'feedback_subject'; break;
                    case 'callme':       // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'login':        // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'registration': // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'cart':
                    default:             $name = 'subject';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода СООБЩЕНИЕ.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueMessage                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueMessage ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':     $name = 'feedback_message'; break;
                    case 'callme':       $name = 'callme_reason'; break;
                    case 'comments':
                    case 'article':
                    case 'articles':
                    case 'newsitem':
                    case 'news':
                    case 'product':      $name = 'post_message'; break;
                    case 'login':        // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'registration': // ТУТ НЕТ ТАКОГО ПОЛЯ, ПОЭТОМУ ПРИРАВНЯЛИ DEFAULT
                    case 'cart':
                    default:             $name = 'message';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода КОММЕНТАРИЙ.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueComment                                 |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueComment ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'cart':
                    default:             $name = 'comment';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода ИД РОДИТЕЛЯ.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueParentId                                |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueParentId ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'comments':
                    case 'article':
                    case 'articles':
                    case 'newsitem':
                    case 'news':
                    case 'product':  $name = 'post_parent_id'; break;
                    default:         $name = 'parent_id';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение поля ввода КОНТРОЛЬ РЕПОСТИНГА.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueCopystop                                |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function valueCopystop ( $params = null, & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':     $name = 'feedback_copystop'; break;
                    case 'login':        $name = 'login_copystop'; break;
                    case 'registration': $name = 'registration_copystop'; break;
                    case 'callme':       $name = 'callme_copystop'; break;
                    case 'comments':
                    case 'article':
                    case 'articles':
                    case 'newsitem':
                    case 'news':
                    case 'product':      $name = 'post_copystop'; break;
                    case 'cart':
                    default:             $name = 'copystop';
                }
                return $this->echoVariable($name, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество этих записей.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {count                                        |
                    |                   [from = VarName]      где хранятся записи |
                    |                                         (по умолчанию       |
                    |                                         в переменной $items)|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function count ( $params = null, & $smarty = null ) {
            $this->mustbeParam('from', $params, 'items');
            $item = $this->retrieveItem($params, $smarty);
            $item = $this->isActualArray($item) ? count($item) : 0;
            return $this->echoValue($item, $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести количество записей в переменной items.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countItems                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countSections ... }                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countCategories ... }                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countBrands ... }                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countProducts ... }                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countNews ... }                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {countArticles ... }                          |
                    |                                                             |
                    ============================================================ */

            public function countItems ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'items');
                return $this->count($params, $smarty);
            }

            public function countSections ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'sections');
                return $this->count($params, $smarty);
            }

            public function countCategories ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'categories');
                return $this->count($params, $smarty);
            }

            public function countBrands ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'all_brands');
                return $this->count($params, $smarty);
            }

            public function countProducts ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'products');
                return $this->count($params, $smarty);
            }

            public function countNews ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'news');
                return $this->count($params, $smarty);
            }

            public function countArticles ( $params = null, & $smarty = null ) {
                $this->setParam('from', $params, 'articles');
                return $this->count($params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вернуть прежний суффикс модуля.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {undoModSuffix}                               |
                    |                                                             |
                    ============================================================ */

        public function undoModSuffix ( $params = null, & $smarty = null ) {
            $this->moduleSuffix = $this->mustbeArray($this->moduleSuffix);
            array_pop($this->moduleSuffix);
            if (!$this->isActualArray($this->moduleSuffix)) $this->moduleSuffix = array('');
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Забыть запись по умолчанию.                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noItem                                       |
                    |                   [full = TRUE]  если и в глобальной области|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function noItem ( $params = null, & $smarty = null ) {
            $scope = $this->paramAsBoolean('full', $params) ? 'global' : '';
            $item = null;
            return $this->setVariableByRef('item', $item, $smarty, $scope);
        }

                    /* ============================================================
                    |                                                             |
                    |  Забыть список записей по умолчанию.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noItems                                      |
                    |                   [full = TRUE]  если и в глобальной области|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noItems ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('full', $params) ? 'global' : '';
                $item = null;
                return $this->setVariableByRef('items', $item, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть дерево категорий.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noCategories                                 |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noCategories ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $item = null;
                return $this->setVariableByRef('categories', $item, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть дерево брендов.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noBrands                                     |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noBrands ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $item = null;
                return $this->setVariableByRef('all_brands', $item, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть список товаров.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noProducts                                   |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noProducts ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $names = array(
                    'products',
                    'search_products',
                    'featureds',
                    'newest_products',
                    'hit_products',
                    'actional_products',
                    'commented_products',
                    'ordered_products',
                    'awaited_products'
                );
                foreach ($names as $var) {
                    $item = null;
                    $this->setVariableByRef($var, $item, $smarty, $scope);
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть список новостей.                                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noNews                                       |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noNews ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $names = array(
                    'news_for_category',
                    'news_for_product',
                    'news'
                );
                foreach ($names as $var) {
                    $item = null;
                    $this->setVariableByRef($var, $item, $smarty, $scope);
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть список статей.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noArticles                                   |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noArticles ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $names = array(
                    'articles_for_category',
                    'articles_for_product',
                    'articles'
                );
                foreach ($names as $var) {
                    $item = null;
                    $this->setVariableByRef($var, $item, $smarty, $scope);
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть анонсовый список публикаций.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noAnnounces                                  |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function noAnnounces ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $item = null;
                return $this->setVariableByRef('announces', $item, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть контент страницы.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noContent}                                   |
                    |                                                             |
                    ============================================================ */

            public function noContent ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('content', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть мета описание страницы.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noMetaDescription}                           |
                    |                                                             |
                    ============================================================ */

            public function noMetaDescription ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('description', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть мета ключевые слова страницы.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noMetaKeywords}                              |
                    |                                                             |
                    ============================================================ */

            public function noMetaKeywords ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('keywords', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть класс тега HTML.                                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noHtmlClass}                                 |
                    |                                                             |
                    ============================================================ */

            public function noHtmlClass ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('htmlClass', $value, $smarty, 'global');
            }

                public function noChildClass ( $params = null, & $smarty = null ) {
                    $value = '';
                    return $this->setVariableByRef('childClass', $value, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Забыть язык тега HTML.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noHtmlLang}                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noHtmlLangMarker}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noLangDefault}                               |
                    |                                                             |
                    ============================================================ */

                public function noHtmlLang ( $params = null, & $smarty = null ) {
                    $value = '';
                    return $this->setVariableByRef('htmlLang', $value, $smarty, 'global');
                }

                    public function noHtmlLangMarker ( $params = null, & $smarty = null ) {
                        $value = '';
                        return $this->setVariableByRef('htmlLangMarker', $value, $smarty, 'global');
                    }

                    public function noLangDefault ( $params = null, & $smarty = null ) {
                        $value = '';
                        return $this->setVariableByRef('langDefault', $value, $smarty, 'global');
                    }

                    /* ============================================================
                    |                                                             |
                    |  Забыть URL для некоторого HTML.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noHtmlUrl}                                   |
                    |                                                             |
                    ============================================================ */

                public function noHtmlUrl ( $params = null, & $smarty = null ) {
                    $value = '';
                    return $this->setVariableByRef('htmlUrl', $value, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Забыть URL скриншота для OpenGraph разметки.               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noOpenGraphImage}                            |
                    |                                                             |
                    ============================================================ */

                public function noOpenGraphImage ( $params = null, & $smarty = null ) {
                    $value = '';
                    return $this->setVariableByRef('openGraphImage', $value, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Забыть остальные мета.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noOtherMeta}                                 |
                    |                                                             |
                    ============================================================ */

            public function noOtherMeta ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('meta', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть остальные скрипты.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noOtherJS}                                   |
                    |                                                             |
                    ============================================================ */

            public function noOtherJS ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('otherJS', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть SEO-текст страницы.                                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noSeoText}                                   |
                    |                                                             |
                    ============================================================ */

            public function noSeoText ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('seo_description', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть завершающий тег блока.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noEndTag}                                    |
                    |                                                             |
                    ============================================================ */

            public function noEndTag ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('endTag', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Забыть html-контент пагинации страниц.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {noPagination}                                |
                    |                                                             |
                    ============================================================ */

            public function noPagination ( $params = null, & $smarty = null ) {
                $value = '';
                return $this->setVariableByRef('PagesNavigation', $value, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести значение произвольного счетчика.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {counter                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function counter ( $params = null, & $smarty = null ) {
            $value = $this->getInteger('counter', $smarty, 1);
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Сбросить произвольный счетчик.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {resetCounter                                 |
                    |                   [local = TRUE]      если этот счетчик     |
                    |                                       только в локальной    |
                    |                                       видимости шаблона     |
                    |                   [def = startValue]  начальное значение    |
                    |                                       (по умолчанию 1)      |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function resetCounter ( $params = null, & $smarty = null ) {
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $value = $this->paramAsInteger('def', $params, 1);
                return $this->setVariableByRef('counter', $value, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Увеличить произвольный счетчик.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {incrementCounter                             |
                    |                   [local = TRUE]      если этот счетчик     |
                    |                                       только в локальной    |
                    |                                       видимости шаблона     |
                    |                   [def = deltaValue]  на сколько увеличить  |
                    |                                       (по умолчанию 1)      |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function incrementCounter ( $params = null, & $smarty = null ) {
                $varname = 'counter';
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                $value = $this->getInteger($varname, $smarty, 1)
                       + $this->paramAsInteger('def', $params, 1);
                return $this->setVariableByRef($varname, $value, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со специальной страницей.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useSection}                                  |
                    |                                                             |
                    ============================================================ */

        public function useSection ( $params = null, & $smarty = null ) {
            $item = $this->getVar('section', $smarty);
            return $this->setVariableByRef('item', $item, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком категорий.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useCategories}                               |
                    |                                                             |
                    ============================================================ */

            public function useCategories ( $params = null, & $smarty = null ) {
                $item = $this->getVar('categories', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с категорией.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useCategory}                                 |
                    |                                                             |
                    ============================================================ */

                public function useCategory ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('category', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком брендов.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useBrands}                                   |
                    |                                                             |
                    ============================================================ */

            public function useBrands ( $params = null, & $smarty = null ) {
                $item = $this->getVar('all_brands', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с брендом.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useBrand}                                    |
                    |                                                             |
                    ============================================================ */

                public function useBrand ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('brand', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со специальной страницей, брендом    |
                    |  или категорией.                                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useSectionOrBrandOrCategory}                 |
                    |                                                             |
                    ============================================================ */

                public function useSectionOrBrandOrCategory ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('section', $smarty);
                    if (is_null($item)) $item = $this->getVar('brand', $smarty);
                    if (is_null($item)) $item = $this->getVar('category', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с брендом или категорией.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useBrandOrCategory}                          |
                    |                                                             |
                    ============================================================ */

                public function useBrandOrCategory ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('brand', $smarty);
                    if (is_null($item)) $item = $this->getVar('category', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком товаров.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useProducts}                                 |
                    |                                                             |
                    ============================================================ */

            public function useProducts ( $params = null, & $smarty = null ) {
                $item = $this->getVar('products', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с товаром.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useProduct}                                  |
                    |                                                             |
                    ============================================================ */

                public function useProduct ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('product', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком новостей.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useNews}                                     |
                    |                                                             |
                    ============================================================ */

            public function useNews ( $params = null, & $smarty = null ) {
                $item = $this->getVar('news', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с новостью.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useNewsItem}                                 |
                    |                                                             |
                    ============================================================ */

                public function useNewsItem ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('news_item', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком статей.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useArticles}                                 |
                    |                                                             |
                    ============================================================ */

            public function useArticles ( $params = null, & $smarty = null ) {
                $item = $this->getVar('articles', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со статьей.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useArticle}                                  |
                    |                                                             |
                    ============================================================ */

                public function useArticle ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('article', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком складов.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useStocks}                                   |
                    |                                                             |
                    ============================================================ */

            public function useStocks ( $params = null, & $smarty = null ) {
                $item = $this->getVar('stocks', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со складом.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useStock}                                    |
                    |                                                             |
                    ============================================================ */

                public function useStock ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('stock', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком стран.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useCountries}                                |
                    |                                                             |
                    ============================================================ */

            public function useCountries ( $params = null, & $smarty = null ) {
                $item = $this->getVar('countries', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со страной.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useCountry}                                  |
                    |                                                             |
                    ============================================================ */

                public function useCountry ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('country', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком областей.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useRegions}                                  |
                    |                                                             |
                    ============================================================ */

                public function useRegions ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('regions', $smarty);
                    return $this->setVariableByRef('items', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с областью.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useRegion}                                   |
                    |                                                             |
                    ============================================================ */

                    public function useRegion ( $params = null, & $smarty = null ) {
                        $item = $this->getVar('region', $smarty);
                        return $this->setVariableByRef('item', $item, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком городов.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useTowns}                                    |
                    |                                                             |
                    ============================================================ */

                public function useTowns ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('towns', $smarty);
                    return $this->setVariableByRef('items', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с городом.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useTown}                                     |
                    |                                                             |
                    ============================================================ */

                    public function useTown ( $params = null, & $smarty = null ) {
                        $item = $this->getVar('town', $smarty);
                        return $this->setVariableByRef('item', $item, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем со списком заказов.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useOrders}                                   |
                    |                                                             |
                    ============================================================ */

            public function useOrders ( $params = null, & $smarty = null ) {
                $item = $this->getVar('orders', $smarty);
                return $this->setVariableByRef('items', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с заказом.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useOrder}                                    |
                    |                                                             |
                    ============================================================ */

                public function useOrder ( $params = null, & $smarty = null ) {
                    $item = $this->getVar('order', $smarty);
                    return $this->setVariableByRef('item', $item, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с переменной post (например объект   |
                    |  в письме или СМС).                                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {usePost}                                     |
                    |                                                             |
                    ============================================================ */

            public function usePost ( $params = null, & $smarty = null ) {
                $item = $this->getVar('post', $smarty);
                return $this->setVariableByRef('item', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  По умолчанию работаем с доступной записью.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {useAnything}                                 |
                    |                                                             |
                    ============================================================ */

            public function useAnything ( $params = null, & $smarty = null ) {
                $item = $this->getVar('section', $smarty);
                if (is_null($item)) $item = $this->getVar('product', $smarty);
                if (is_null($item)) $item = $this->getVar('news_item', $smarty);
                if (is_null($item)) $item = $this->getVar('file', $smarty);
                if (is_null($item)) $item = $this->getVar('category', $smarty);
                if (is_null($item)) $item = $this->getVar('article', $smarty);
                if (is_null($item)) $item = $this->getVar('stock', $smarty);
                if (is_null($item)) $item = $this->getVar('town', $smarty);
                if (is_null($item)) $item = $this->getVar('region', $smarty);
                if (is_null($item)) $item = $this->getVar('country', $smarty);
                if (is_null($item)) $item = $this->getVar('brand', $smarty);
                if (is_null($item)) $item = $this->getVar('category', $smarty);
                return $this->setVariableByRef('item', $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить умалчиваемый список или переменную в стек.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pushItems}                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {pushItem}                                    |
                    |                                                             |
                    ============================================================ */

        public function pushItems ( $params = null, & $smarty = null ) {
            $items = $this->getVar('items', $smarty);
            $this->pushStack[] = & $items;
            return '';
        }

            public function pushItem ( $params = null, & $smarty = null ) {
                $item = $this->getVar('item', $smarty);
                $this->pushStack[] = & $item;
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь умалчиваемый список или переменную из стека.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {popItems                                     |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {popItem ...}                                 |
                    |                                                             |
                    ============================================================ */

        public function popItems ( $params = null, & $smarty = null ) {
            $item = array_pop($this->pushStack);
            $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
            return $this->setVariableByRef('items', $item, $smarty, $scope);
        }

            public function popItem ( $params = null, & $smarty = null ) {
                $item = array_pop($this->pushStack);
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                return $this->setVariableByRef('item', $item, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Глобализировать умалчиваемую переменную.                   |
                    |                                                             |
                    ============================================================ */

        public function globalItem ( $params = null, & $smarty = null ) {
            $varname = 'item';
            $item = $this->getVar($varname, $smarty);
            return $this->setVariableByRef($varname, $item, $smarty, 'global');
        }

                    /* ============================================================
                    |                                                             |
                    |  Глобализировать умалчиваемую переменную до надстоящего     |
                    |  шаблона.                                                   |
                    |                                                             |
                    ============================================================ */

            public function parentItem ( $params = null, & $smarty = null ) {
                $varname = 'item';
                $item = $this->getVar($varname, $smarty);
                return $this->setVariableByRef($varname, $item, $smarty, 'parent');
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить поле некоторой записи из запостченной формы.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRecordField                               |
                    |                   name = FieldName      имя поля            |
                    |                   [id = recordID]       идентификатор записи|
                    |                                         (по умолчанию из    |
                    |                                         переменной $id или  |
                    |                                         считать равным 0)   |
                    |                   [safe = FALSE]        если отключить      |
                    |                                         безопасность для    |
                    |                                         применения в теге   |
                    |                   [def = defaultValue]  текст по умолчанию  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function getRecordField ( $params = null, & $smarty = null ) {
            $name = $this->paramAsSentence('name', $params);
            $id = $this->getInteger('id', $smarty);
                $id = $this->paramAsInteger('id', $params, $id);
            $def = $this->paramAsString('def', $params);
            $value = $this->requestRecordField($name, $id, $def);
            $escaped = $this->paramAsBoolean('safe', $params, TRUE);
            return $this->echoValue($value, $params, $smarty, $escaped);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить параметр запроса страницы.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequest                                   |
                    |                   name = ParamName      имя параметра       |
                    |                   [safe = TRUE]         если безопасно для  |
                    |                                         применения в теге   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function getRequest ( $params = null, & $smarty = null ) {
            $name = $this->paramAsSentence('name', $params);
            $value = $this->requestParam($name);
            $escaped = $this->paramAsBoolean('safe', $params);
            return $this->echoValue($value, $params, $smarty, $escaped);
        }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр page текущего запроса страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestPage                               |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолч. $pagenum) |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestPage                             |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestPage ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'pagenum');
                $value = $this->requestParam('page');
                $value = max(1, intval($value));
                $escaped = $this->paramAsBoolean('safe', $params);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                public function valueRequestPage ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestPage($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр mode текущего запроса страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestMode                               |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию $mode) |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestMode                             |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestMode ( $params = null, & $smarty = null ) {
                $field = 'mode';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestMode ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestMode($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр lang текущего запроса страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestLang                               |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию $lang) |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestLang                             |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestLang ( $params = null, & $smarty = null ) {
                $field = 'lang';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestLang ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestLang($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр sign текущего запроса страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestSign                               |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию $sign) |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestSign                             |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestSign ( $params = null, & $smarty = null ) {
                $field = 'sign';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestSign ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestSign($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр data текущего запроса страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestData                               |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию $data) |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestData                             |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestData ( $params = null, & $smarty = null ) {
                $field = 'data';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestData ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestData($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр id текущего запроса страницы.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestId                                 |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию $id)   |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestId                               |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestId ( $params = null, & $smarty = null ) {
                $field = 'id';
                $this->mustbeParam('assign', $params, $field);
                $value = $this->requestParam($field);
                $value = $this->cms->text->lowerCase($value);
                $escaped = $this->paramAsBoolean('safe', $params);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                public function valueRequestId ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestId($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр title текущего запроса страницы.          |
                    |                                                             |
                    ============================================================ */

            public function getRequestTitle ( $params = null, & $smarty = null ) {
                $field = 'title';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestTitle ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestTitle($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр meta_keywords текущего запроса страницы.  |
                    |                                                             |
                    ============================================================ */

            public function getRequestMetaKeywords ( $params = null, & $smarty = null ) {
                $field = 'meta_keywords';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestMetaKeywords ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestMetaKeywords($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр meta_description запроса страницы.        |
                    |                                                             |
                    ============================================================ */

            public function getRequestMetaDescription ( $params = null, & $smarty = null ) {
                $field = 'meta_description';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestMetaDescription ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestMetaDescription($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр name текущего запроса страницы.           |
                    |                                                             |
                    ============================================================ */

            public function getRequestName ( $params = null, & $smarty = null ) {
                $field = 'name';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestName ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestName($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр card текущего запроса страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getRequestCard                               |
                    |                   [safe = TRUE]        если безопасно для   |
                    |                                        применения в теге    |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию $card) |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {valueRequestCard                             |
                    |                   [var = VarName]   в какую переменную      |
                    |                                     вывести результат (по   |
                    |                                     умолчан. в точку вызова)|
                    |                   [scope = global]  видимость переменной    |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getRequestCard ( $params = null, & $smarty = null ) {
                $field = 'card';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestCard ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestCard($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр email текущего запроса страницы.          |
                    |                                                             |
                    ============================================================ */

            public function getRequestEmail ( $params = null, & $smarty = null ) {
                $field = 'email';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestEmail ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestEmail($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр phone текущего запроса страницы.          |
                    |                                                             |
                    ============================================================ */

            public function getRequestPhone ( $params = null, & $smarty = null ) {
                $field = 'phone';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestPhone ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestPhone($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр message текущего запроса страницы.        |
                    |                                                             |
                    ============================================================ */

            public function getRequestMessage ( $params = null, & $smarty = null ) {
                $field = 'message';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestMessage ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestMessage($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр annotation текущего запроса страницы.     |
                    |                                                             |
                    ============================================================ */

            public function getRequestAnnotation ( $params = null, & $smarty = null ) {
                $field = 'annotation';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestAnnotation ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestAnnotation($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр body текущего запроса страницы.           |
                    |                                                             |
                    ============================================================ */

            public function getRequestBody ( $params = null, & $smarty = null ) {
                $field = 'body';
                $this->mustbeParam('assign', $params, $field);
                $this->setParam('name', $params, $field);
                return $this->getRequest($params, $smarty);
            }

                public function valueRequestBody ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestBody($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр domain текущего запроса страницы.         |
                    |                                                             |
                    ============================================================ */

            public function getRequestDomain ( $params = null, & $smarty = null ) {
                $field = 'domain';
                $this->mustbeParam('assign', $params, $field);
                $value = $this->requestParam($field);
                $value = $this->getUrlDomain($value, TRUE);
                $escaped = $this->paramAsBoolean('safe', $params);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                public function valueRequestDomain ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, '');
                    $this->mustbeParam('safe', $params, TRUE);
                    return $this->getRequestDomain($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести input-атрибут состояния, если выставлен флажковый  |
                    |  параметр agree в текущем запросе страницы.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {activeRequestAgree                           |
                    |                   [mark = AttrValue]    значение атрибута   |
                    |                                         (по умолчанию это   |
                    |                                         слово checked)      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function activeRequestAgree ( $params = null, & $smarty = null ) {
                $mark = $this->getMarkParamAsChecked($params);
                $value = $this->requestParam('agree');
                $value = !empty($value) ? $mark : '';
                return $this->echoValue($value, $params, $smarty, FALSE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести input-атрибут состояния, если выставлен флажковый  |
                    |  параметр setting-... в текущем запросе страницы.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {activeRequestSetting                         |
                    |                   name = SettingName    название настройки  |
                    |                   [mark = AttrValue]    значение атрибута   |
                    |                                         (по умолчанию это   |
                    |                                         слово checked)      |
                    |                   [def = defaultValue]  значение атрибута   |
                    |                                         если параметра      |
                    |                                         ВООБЩЕ НЕТ в запросе|
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function activeRequestSetting ( $params = null, & $smarty = null ) {
                    $mark = $this->getMarkParamAsChecked($params);
                    $name = 'setting-' . $this->paramAsSentence('name', $params);
                    $def = $this->cms->request->existsGPrequest($name)
                           ? ''
                           : $this->paramAsString('def', $params);
                    $value = $this->requestParam($name);
                    $value = !empty($value) ? $mark : $def;
                    return $this->echoValue($value, $params, $smarty, FALSE);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить массив страниц верхнего меню.                     |
                    |                                                             |
                    ============================================================ */

        public function getTopMenu ( $params = null, & $smarty = null ) {
            $items = array();
            $this->mustbeParam('assign', $params, 'sections');
            $helper = $this->getVar('helper');
            if (is_object($helper)) {
                $params2 = array('name' => 'Верхнее меню',
                                 'attach' => 'sections');
                $menu = $helper->menuByLangTechName($params2, $smarty);
                if (!empty($menu->sections)) $items = & $menu->sections;
            }
            return $this->assignVar($params, $items, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить запись о статической странице.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getSection                                   |
                    |                   id = PageID           ИД страницы         |
                    |                   [simple = TRUE]       если простая        |
                    |                                         проверка страницы,  |
                    |                                         без валидации       |
                    |                                         статусов "видна     |
                    |                                         всем", "разрешена"  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                                         (по умолчанию в     |
                    |                                         переменную $section)|
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getSection ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'section');
                $filter = new stdClass;
                if (!$this->paramAsBoolean('simple', $params)) {
                    $filter->enabled = 1;
                    if (!$this->existsUser()) $filter->hidden = 0;
                }
                $item = null;
                $field = 'id';
                $id = $this->paramAsString($field, $params);
                if (!empty($id)) {
                    $filter->$field = $id;
                    $this->cms->db->sections->one($item, $filter);
                }
                return $this->assignVar($params, $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить массив истории поиска.                            |
                    |                                                             |
                    ============================================================ */

            public function getSearchHistory ( $params = null, & $smarty = null ) {
                $items = array();
                $this->mustbeParam('assign', $params, 'items');
                $helper = $this->getVar('helper');
                if (is_object($helper)) {
                    $params2 = array();
                    $history = $helper->searchHistory($params2, $smarty);
                    if (!empty($history)) {
                        $listed = array();
                        foreach ($history as $item) {
                            if (!empty($item['keyword'])) {
                                $value = $this->asAttribute($item['keyword']);
                                $test = $this->cms->text->lowerCase($value);
                                if (empty($listed[$test])) {
                                    $listed[$test] = TRUE;
                                    $items[] = $value;
                                }
                            }
                        }
                    }
                }
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить дерево категорий.                                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getCategories                                |
                    |                   [active = CategID]    идентификатор       |
                    |                                         активной категории  |
                    |                                         (по умолчанию из    |
                    |                                         данных движка)      |
                    |                   [any = TRUE]          если отбирать и     |
                    |                                         пустые категории    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                                         (по умолчанию в     |
                    |                                         $categories)        |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getCategories ( $params = null, & $smarty = null ) {
                $items = array();
                $this->mustbeParam('assign', $params, 'categories');
                $method = 'categories';
                $helper = $this->getVar('helper');
                if ($this->cms->any->hasMethod($helper, $method)) {
                    $params2 = array('sort' => FALSE,
                                     'any' => $this->paramAsBoolean('any', $params),
                                     'counters' => TRUE);
                    $field = 'active';
                        $id = $this->paramAsInteger($field, $params, FALSE);
                        if ($id !== FALSE) $params2[$field] = $id;
                    $items = $helper->$method($params2, $smarty);
                }
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить дерево брендов.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getBrands                                    |
                    |                   [active = BrandID]    идентификатор       |
                    |                                         активного бренда    |
                    |                                         (по умолчанию из    |
                    |                                         данных движка)      |
                    |                   [any = TRUE]          если отбирать и     |
                    |                                         пустые бренды       |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                                         (по умолчанию в     |
                    |                                         $all_brands)        |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function getBrands ( $params = null, & $smarty = null ) {
                    $items = array();
                    $this->mustbeParam('assign', $params, 'all_brands');
                    $method = 'categories';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $params2 = array('table' => 'brands',
                                         'sort' => FALSE,
                                         'any' => $this->paramAsBoolean('any', $params),
                                         'counters' => TRUE);
                        $field = 'active';
                            $id = $this->paramAsInteger($field, $params, FALSE);
                            if ($id !== FALSE) $params2[$field] = $id;
                        $items = $helper->$method($params2, $smarty);
                    }
                    return $this->assignVar($params, $items, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список товаров.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getProducts                                  |
                    |                   [tag = tagName]      брать только товары  |
                    |                                        с таким тегом        |
                    |                   [sortPrice = TRUE]   есди сортировать по  |
                    |                                        цене (по умолчанию   |
                    |                                        по названию)         |
                    |                   [page = pageNumber]  номер текущей        |
                    |                                        страницы списка      |
                    |                                        (по умолчанию 1)     |
                    |                   [pagesize = Number]  сколько товаров      |
                    |                                        на одном листе (по   |
                    |                                        умолчанию 60 или 240 |
                    |                                        если нумерации нет)  |
                    |                   [count = maxCount]   сколько товаров      |
                    |                                        получить (по         |
                    |                                        умолчанию pagesize)  |
                    |                   [assign = VarName]   в какую переменную   |
                    |                   или [var = VarName]  вывести результат    |
                    |                                        (по умолчанию в      |
                    |                                        $products)           |
                    |                   [scope = global]     видимость переменной |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getProducts ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'products');
                $def = $this->existsParam('page', $params) ? 60 : 240;
                $page = $this->paramAsNatural('page', $params) - 1;
                $pagesize = $this->paramAsNatural('pagesize', $params, $def);
                $tag = $this->paramAsSentence('tag', $params);
                $items = null;
                    $filter = $this->makeFilter(TRUE);
                    $filter->type = 0;    // TYPE_PRODUCTS_ANY
                    if ($this->paramAsBoolean('sortPrice', $params)) {
                        $filter->sort = 1;    // SORT_PRODUCTS_MODE_BY_PRICE
                    } else {
                        $filter->sort = 6;    // SORT_PRODUCTS_MODE_BY_NAME
                    }
                    $filter->start = $page * $pagesize;
                    $filter->maxcount = $this->paramAsNatural('count', $params, $pagesize);
                    if ($this->isActualString($tag)) {
                        $filter->search = 'tag:' . $tag;
                    }
                $this->cms->db->products->get($items, $filter);
                $this->cms->db->products->unpackRecords($items);
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить массив особых товаров.                            |
                    |                                                             |
                    ============================================================ */

                public function getFeatureds ( $params = null, & $smarty = null ) {
                    $def = $this->requestParam('mode');
                    $filter = $this->makeFilter(TRUE);
                    $filter->maxcount = $this->paramAsNatural('count', $params, 240);
                    $this->mustbeParam('assign', $params, 'products');
                    $mode = $this->paramAsSentence('mode', $params, $def);
                    switch ($mode) {
                        case 'hit':       $filter->type = TYPE_PRODUCTS_HIT;          break;
                        case 'new':
                            $dayTime = 24 * 60 * 60;
                            $filter->type = TYPE_PRODUCTS_NEWEST_MIXED;
                            $filter->search_date_from = date('Y-m-d', time() - 15 * $dayTime);
                            break;
                        case 'newest':    $filter->type = TYPE_PRODUCTS_NEWEST;       break;
                        case 'actional':  $filter->type = TYPE_PRODUCTS_ACTIONAL;     break;
                        case 'awaited':   $filter->type = TYPE_PRODUCTS_AWAITED;      break;
                        case 'commented': $filter->type = TYPE_PRODUCTS_COMMENTED;    break;
                        case 'ordered':   $filter->type = TYPE_PRODUCTS_ORDERED;      break;
                        case 'best':
                        case 'discount':  $filter->type = TYPE_PRODUCTS_DISCOUNTED;   break;
                        default:          $filter = FALSE;
                    }
                    $items = empty($filter) ? array() : $this->readProducts($filter);
                    return $this->assignVar($params, $items, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список случайных товаров.                         |
                    |                                                             |
                    ============================================================ */

                public function getRandoms ( $params = null, & $smarty = null ) {
                    $this->mustbeParam('assign', $params, 'items');
                    $items = array();
                    $count = $this->paramAsNaturalExtended('count', $params, 30);
                    if ($count > 0) {
                        $authorized = $this->existsUser();
                        $query = 'SELECT `products`.`product_id` AS `id` '
                               . 'FROM `products` '
                               . 'LEFT JOIN `categories` ON `products`.`category_id` = `categories`.`category_id` '
                               . 'LEFT JOIN `brands` ON `products`.`brand_id` = `brands`.`brand_id` '
                               . 'WHERE `products`.`enabled` = 1 AND '
                                      . ($authorized ? '' : '`products`.`hidden` = 0 AND '
                                                          . '`categories`.`hidden` = 0 AND '
                                                          . '(`brands`.`hidden` = 0 OR `brands`.`hidden` IS NULL) AND ')
                                     . '`categories`.`enabled` = 1 AND '
                                     . '(`brands`.`enabled` = 1 OR `brands`.`enabled` IS NULL) '
                               . 'GROUP BY `products`.`product_id`;';
                        $result = $this->cms->db->query($query);
                        if ($result !== FALSE) {
                            $ids = array();
                            while ($item = $this->cms->db->fetch_assoc($result)) $ids[] = $item['id'];
                            $this->cms->db->free_result($result);
                            shuffle($ids);
                            $ids = array_slice($ids, 0, $count);
                            $ids = implode(',', $ids);
                                $filter = new stdClass;
                                $filter->ids = $ids;
                                $this->cms->db->products->get($items, $filter);
                                $this->cms->db->products->unpackRecords($items, $filter);
                            shuffle($items);
                        }
                    }
                    return $this->assignVar($params, $items, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список последних поступивших товаров.             |
                    |                                                             |
                    ============================================================ */

                public function getLasts ( $params = null, & $smarty = null ) {
                    $items = array();
                    $method = 'products';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $params2 = array('filter' => array('sort' => 9,    // SORT_PRODUCTS_MODE_BY_CREATED
                                                           'sort_direction' => 1));
                        $this->copyParam('count', $params, $params2, 30);
                        $this->copyParam('assign', $params, $params2, 'items');
                        $items = $helper->$method($params2, $smarty);
                    } else $this->mustbeParam('assign', $params, 'items');
                    return $this->assignVar($params, $items, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список бесплатных товаров.                        |
                    |                                                             |
                    ============================================================ */

                public function getFreebies ( $params = null, & $smarty = null ) {
                    $items = array();
                    $method = 'products';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $params2 = array('filter' => array('sort' => 9,    // SORT_PRODUCTS_MODE_BY_CREATED
                                                           'sort_direction' => 1,
                                                           'search_cost_to' => 0.000000001,
                                                           'non_usable' => -1));
                        $this->copyParam('count', $params, $params2, 30);
                        $this->copyParam('assign', $params, $params2, 'items');
                        $items = $helper->$method($params2, $smarty);
                    } else $this->mustbeParam('assign', $params, 'items');
                    return $this->assignVar($params, $items, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список товаров экспонатов.                        |
                    |                                                             |
                    ============================================================ */

                public function getExhibits ( $params = null, & $smarty = null ) {
                    $items = array();
                    $method = 'products';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $params2 = array('filter' => array('sort' => 9,    // SORT_PRODUCTS_MODE_BY_CREATED
                                                           'sort_direction' => 1,
                                                           'non_usable' => 1));
                        $this->copyParam('count', $params, $params2, 30);
                        $this->copyParam('assign', $params, $params2, 'items');
                        $items = $helper->$method($params2, $smarty);
                    } else $this->mustbeParam('assign', $params, 'items');
                    return $this->assignVar($params, $items, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список новостей.                                  |
                    |                                                             |
                    ============================================================ */

            public function getNews ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'news');
                $def = $this->existsParam('page', $params) ? 60 : 240;
                $page = $this->paramAsNatural('page', $params) - 1;
                $pagesize = $this->paramAsNatural('pagesize', $params, $def);
                $items = null;
                    $filter = $this->makeFilter();
                    $filter->sort = 5;    // SORT_NEWS_MODE_BY_DATE
                    $filter->sort_direction = 1;
                    $filter->start = $page * $pagesize;
                    $filter->maxcount = $this->paramAsNatural('count', $params, $pagesize);
                $this->cms->db->news->get($items, $filter);
                $this->cms->db->news->unpackRecords($items);
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список статей.                                    |
                    |                                                             |
                    ============================================================ */

            public function getArticles ( $params = null, & $smarty = null ) {
                /* TODO: заменить на код подобный getNews (с пагинацией) */
                $this->mustbeParam('assign', $params, 'articles');
                $items = array();
                $method = 'articles';
                $helper = $this->getVar('helper');
                if ($this->cms->any->hasMethod($helper, $method)) {
                    $params2 = array();
                    $this->copyParam('count', $params, $params2, 120);
                    $items = $helper->$method($params2, $smarty);
                }
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список складов.                                   |
                    |                                                             |
                    ============================================================ */

            public function getStocks ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'stocks');
                $def = $this->existsParam('page', $params) ? 60 : 240;
                $page = $this->paramAsNatural('page', $params) - 1;
                $pagesize = $this->paramAsNatural('pagesize', $params, $def);
                $items = null;
                    $filter = $this->makeFilter();
                    $filter->visible = 1;
                    $filter->sort = 1;    // SORT_STOCKS_MODE_AS_IS
                    $filter->start = $page * $pagesize;
                    $filter->maxcount = $this->paramAsNatural('count', $params, $pagesize);
                $this->cms->db->stocks->get($items, $filter);
                $this->cms->db->stocks->unpackRecords($items);
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список стран.                                     |
                    |                                                             |
                    ============================================================ */

            public function getCountries ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'countries');
                $def = $this->existsParam('page', $params) ? 60 : 240;
                $page = $this->paramAsNatural('page', $params) - 1;
                $pagesize = $this->paramAsNatural('pagesize', $params, $def);
                $items = null;
                    $filter = $this->makeFilter();
                    $filter->sort = 2;    // SORT_COUNTRIES_MODE_BY_NAME
                    $filter->start = $page * $pagesize;
                    $filter->maxcount = $this->paramAsNatural('count', $params, $pagesize);
                $this->cms->db->countries->get($items, $filter);
                $this->cms->db->countries->unpackRecords($items);
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список областей.                                  |
                    |                                                             |
                    ============================================================ */

                public function getRegions ( $params = null, & $smarty = null ) {
                    /* TODO */
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить список городов.                                   |
                    |                                                             |
                    ============================================================ */

                public function getTowns ( $params = null, & $smarty = null ) {
                    /* TODO */
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить адреса картинок этой папки шаблона.               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getSlides                                    |
                    |                   [folder = FolderName]  имя папки внутри   |
                    |                                          папки images       |
                    |                                          шаблона (или по    |
                    |                                          умолчанию смотреть |
                    |                                          в папке images)    |
                    |                   [assign = VarName]     в какую переменную |
                    |                   или [var = VarName]    вывести результат  |
                    |                   или [result=VarName]   (по умолчанию в    |
                    |                                          переменную $items) |
                    |                   [scope = global]       видимость          |
                    |                                          переменной         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getSlides ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'items');
                $folder = $this->paramAsSentence('folder', $params);
                $items = $this->getImages($folder);
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список последних комментариев.                    |
                    |                                                             |
                    ============================================================ */

            public function getComments ( $params = null, & $smarty = null ) {
                $items = array();
                $method = 'lastComments';
                $helper = $this->getVar('helper');
                if ($this->cms->any->hasMethod($helper, $method)) {
                    $params2 = array();
                    $this->copyParam('count', $params, $params2, 120);
                    $this->copyParam('assign', $params, $params2, 'items');
                    $items = $helper->$method($params2, $smarty);
                } else $this->mustbeParam('assign', $params, 'items');
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список последних заказов.                         |
                    |                                                             |
                    ============================================================ */

            public function getOrders ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'items');
                $page = $this->paramAsNatural('page', $params) - 1;
                $pagesize = $this->paramAsNatural('pagesize', $params, 60);
                $items = null;
                    $filter = new stdClass;
                    $filter->sort = 14;    // SORT_ORDERS_MODE_BY_DATE
                    $filter->sort_direction = 1;
                    $filter->start = $page * $pagesize;
                    $filter->maxcount = $this->paramAsNatural('count', $params, $pagesize);
                    $mode = $this->paramAsSentence('mode', $params, $def);
                    switch ($mode) {
                        case 'any':     break;
                        case 'process': $filter->status = 1; break;   // ORDER_STATUS_PROCESS
                        case 'done':    $filter->status = 2; break;   // ORDER_STATUS_DONE
                        case 'cancel':  $filter->status = 3; break;   // ORDER_STATUS_CANCEL
                        case 'new':                                   // ORDER_STATUS_NEW
                        default:        $filter->status = 0; break;
                    }
                $this->cms->db->orders->get($items, $filter);
                $this->cms->db->orders->unpackRecords($items);
                if ($this->existsAdmin()) {
                    $filter = new stdClass;
                    $filter->token = $this->cms->token;
                    $this->cms->db->orders->operable($items, $params);
                }
                return $this->assignVar($params, $items, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить запись о пользователе.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {getUser                                      |
                    |                   [id = UserID]         ИД пользователя     |
                    |                   [login = UserLogin]   или логин           |
                    |                   [nick = UserNick]     или ник             |
                    |                   [simple = TRUE]       если простая        |
                    |                                         проверка клиента,   |
                    |                                         без валидации       |
                    |                                         статусов "разрешен",|
                    |                                         "удален"            |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                                         (по умолчанию в     |
                    |                                         переменную $user)   |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function getUser ( $params = null, & $smarty = null ) {
                $this->mustbeParam('assign', $params, 'user');
                $filter = new stdClass;
                if (!$this->paramAsBoolean('simple', $params)) {
                    $filter->enabled = 1;
                    $filter->deleted = 0;
                }
                $fields = array(
                    'id' => 'id',
                    'login' => 'email',
                    'nick' => 'nickname'
                );
                $item = null;
                foreach ($fields as $name => $field) {
                    $value = $this->paramAsString($name, $params);
                    if (!empty($value)) {
                        $filter2 = $filter;
                        $filter2->$field = $value;
                        $this->cms->db->users->one($item, $filter2);
                        if (!empty($item)) break;
                    }
                }
                return $this->assignVar($params, $item, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Списать юниты с внутреннего счета пользователя.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {writeOffUnits                                |
                    |                   [id = UserID]         ИД пользователя     |
                    |                   [login = UserLogin]   или логин           |
                    |                   [nick = UserNick]     или ник             |
                    |                   [simple = TRUE]       если простая        |
                    |                                         проверка клиента,   |
                    |                                         без валидации       |
                    |                                         статусов "разрешен",|
                    |                                         "удален"            |
                    |                   [amount = Number]     количество          |
                    |                                         списываемых юнитов  |
                    |                                         (по умолчанию 1)    |
                    |                   [info = Comment]      комментарий         |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function writeOffUnits ( $params = null, & $smarty = null ) {
                    $amount = $this->paramAsInteger('amount', $params);
                    if ($amount > 0) {
                        $filter = new stdClass;
                        if (!$this->paramAsBoolean('simple', $params)) {
                            $filter->enabled = 1;
                            $filter->deleted = 0;
                        }
                        $fields = array(
                            'id' => 'id',
                            'login' => 'email',
                            'nick' => 'nickname'
                        );
                        $user = null;
                        foreach ($fields as $name => $field) {
                            $value = $this->paramAsString($name, $params);
                            if (!empty($value)) {
                                $filter2 = $filter;
                                $filter2->$field = $value;
                                $this->cms->db->users->one($user, $filter2);
                                if (!empty($user)) {
                                    $comment = $this->paramAsSentence('info', $params);
                                    if ($this->cms->registrar->chargeUserUnits($user, -$amount, $comment)) {
                                        $this->cms->db->users->update($user);
                                    }
                                    break;
                                }
                            }
                        }
                    }
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Редирект на начало списка, если находимся не там.          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {redirectToFirst}                             |
                    |                                                             |
                    ============================================================ */

        public function redirectToFirst ( $params = null, & $smarty = null ) {
            $num = $this->getVar('page');
            if (empty($num)) return '';
            $url = $this->getUrlBegin($smarty);
            $this->cms->security->redirectToPage($url);
        }

                    /* ============================================================
                    |                                                             |
                    |  Редирект на начало полного списка, если находимся не там.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {redirectToAll}                               |
                    |                                                             |
                    ============================================================ */

            public function redirectToAll ( $params = null, & $smarty = null ) {
                $command = 'show_all';
                $onAllNow = $this->cms->request->existsGPrequest($command);
                if ($onAllNow || !$this->existsVar('AllOnPageUrl', $smarty)) {
                    $num = $this->getVar('page');
                    if (empty($num)) return '';
                }
                $url = $this->getUrlBegin($smarty, $command);
                $this->cms->security->redirectToPage($url);
            }

                    /* ============================================================
                    |                                                             |
                    |  Генерация кода капчи.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {generateCaptcha}                             |
                    |                                                             |
                    ============================================================ */

        public function generateCaptcha ( $params = null, & $smarty = null ) {
            /* TODO: сделать */
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Авторизация под таким клиентским логином.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {loginWith                                    |
                    |                   login = UserLogin  логин клиента          |
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function loginWith ( $params = null, & $smarty = null ) {
            /* TODO: сделать */
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Check404 текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCheck404                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

        public function modCheck404 ( $params = null, & $smarty = null ) {
            return $this->includeModNumerated('check404', $params, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Error404 текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modError404                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modError404 ( $params = null, & $smarty = null ) {
                return $this->includeModuleNumerated('missing_template', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СообщениеОбОшибке шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modError                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modError ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('error', $params, $smarty);
                }

                    public function modError2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modError($params, $smarty);
                    }

                    public function modError3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modError($params, $smarty);
                    }

                    public function modError4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modError($params, $smarty);
                    }

                    public function modError5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modError($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СообщениеОбУспехе шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSuccess                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modSuccess ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('success', $params, $smarty);
                }

                    public function modSuccess2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modSuccess($params, $smarty);
                    }

                    public function modSuccess3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modSuccess($params, $smarty);
                    }

                    public function modSuccess4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modSuccess($params, $smarty);
                    }

                    public function modSuccess5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modSuccess($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Canonizator текущего шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCanonizator                               |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCanonizator ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('canonizator', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ПоисковыеТранзиты из папки     |
                    |  модулей текущего шаблона.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTransit                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modTransit ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('transit', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ExportOrderToCsv шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modExportOrderToCsv                          |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modExportOrderToCsv ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('export-order-to-csv', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптАналитики шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAnalytics                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAnalytics ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('analytics', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптМетрики текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modMetrics                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modMetrics ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('metrics', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптGoogleTagManager шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modGTM                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modGTM ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('gtm', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптGoogle текущего шаблона. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modGoogle                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modGoogle ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('google', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптYandex текущего шаблона. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modYandex                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modYandex ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('yandex', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптFacebook шаблона.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFacebook                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modFacebook ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('facebook', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптВКонтакте шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modVkontakte                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modVkontakte ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('vkontakte', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкриптTwitter текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTwitter                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modTwitter ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('twitter', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл РазметкаOpenGraph шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modOpenGraph                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modOpenGraph ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('open-graph', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл РазметкаSchemaOrg шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSchemaOrg                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modSchemaOrg ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('schema-org', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаВыбораВалюты шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCurrencies                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCurrencies ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('currencies', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ЗаголовкиСтраницы шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modHead                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modHead ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('head', $params, $smarty);
            }

                public function modHead2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modHead($params, $smarty);
                }

                public function modHead3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modHead($params, $smarty);
                }

                public function modHead4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modHead($params, $smarty);
                }

                public function modHead5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modHead($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Шапка шаблона.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modHeader                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modHeader ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('header', $params, $smarty);
            }

                public function modHeader2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modHeader($params, $smarty);
                }

                public function modHeader3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modHeader($params, $smarty);
                }

                public function modHeader4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modHeader($params, $smarty);
                }

                public function modHeader5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modHeader($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Баннер шаблона.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBanner                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBanner ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('banner', $params, $smarty);
            }

                public function modBanner2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBanner($params, $smarty);
                }

                public function modBanner3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBanner($params, $smarty);
                }

                public function modBanner4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBanner($params, $smarty);
                }

                public function modBanner5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBanner($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Слайдер шаблона.               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSlider                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSlider ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('slider', $params, $smarty);
            }

                public function modSlider2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSlider($params, $smarty);
                }

                public function modSlider3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSlider($params, $smarty);
                }

                public function modSlider4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSlider($params, $smarty);
                }

                public function modSlider5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSlider($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Фотогалерея шаблона.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modGallery                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modGallery ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('gallery', $params, $smarty);
            }

                public function modGallery2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modGallery($params, $smarty);
                }

                public function modGallery3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modGallery($params, $smarty);
                }

                public function modGallery4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modGallery($params, $smarty);
                }

                public function modGallery5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modGallery($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ДомашняяСтраница шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modHome                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modHome ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('home', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Основное шаблона.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modMain                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modMain ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('main', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Приветствие шаблона.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modWelcome                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modWelcome ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('welcome', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Услуги шаблона.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modServices                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modServices ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('services', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Карта шаблона.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modMap                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modMap ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('map', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ОНас шаблона.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAbout                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAbout ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('about', $params, $smarty);
            }

                public function modAbout2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modAbout($params, $smarty);
                }

                public function modAbout3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modAbout($params, $smarty);
                }

                public function modAbout4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modAbout($params, $smarty);
                }

                public function modAbout5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modAbout($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Контакты шаблона.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modContacts                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modContacts ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('contacts', $params, $smarty);
            }

                public function modContacts2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modContacts($params, $smarty);
                }

                public function modContacts3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modContacts($params, $smarty);
                }

                public function modContacts4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modContacts($params, $smarty);
                }

                public function modContacts5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modContacts($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Подвал шаблона.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFooter                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFooter ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('footer', $params, $smarty);
            }

                public function modFooter2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modFooter($params, $smarty);
                }

                public function modFooter3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modFooter($params, $smarty);
                }

                public function modFooter4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modFooter($params, $smarty);
                }

                public function modFooter5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modFooter($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл AcceleratedMobilePages шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAMP                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAMP ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('amp', $params, $smarty);
            }

                public function modAMP2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modAMP($params, $smarty);
                }

                public function modAMP3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modAMP($params, $smarty);
                }

                public function modAMP4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modAMP($params, $smarty);
                }

                public function modAMP5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modAMP($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаПоиска текущего шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSearch                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSearch ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('search', $params, $smarty);
            }

                public function modSearch2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSearch($params, $smarty);
                }

                public function modSearch3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSearch($params, $smarty);
                }

                public function modSearch4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSearch($params, $smarty);
                }

                public function modSearch5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSearch($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ИсторияПоиска текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSearchHistory                             |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modSearchHistory ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('search-history', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаРегистрации шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modRegistration                              |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modRegistration ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('registration', $params, $smarty);
            }

                public function modRegistration2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modRegistration($params, $smarty);
                }

                public function modRegistration3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modRegistration($params, $smarty);
                }

                public function modRegistration4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modRegistration($params, $smarty);
                }

                public function modRegistration5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modRegistration($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаАвторизации шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modLogin                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modLogin ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('login', $params, $smarty);
            }

                public function modLogin2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modLogin($params, $smarty);
                }

                public function modLogin3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modLogin($params, $smarty);
                }

                public function modLogin4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modLogin($params, $smarty);
                }

                public function modLogin5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modLogin($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаВосстановленияПароля из   |
                    |  папки модулей текущего шаблона.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modRemind                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modRemind ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('remind', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаОбратнойСвязи шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFeedback                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFeedback ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('feedback', $params, $smarty);
            }

                public function modFeedback2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modFeedback($params, $smarty);
                }

                public function modFeedback3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modFeedback($params, $smarty);
                }

                public function modFeedback4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modFeedback($params, $smarty);
                }

                public function modFeedback5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modFeedback($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаОбратногоЗвонка шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCallme                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCallme ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('callme', $params, $smarty);
            }

                public function modCallme2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCallme($params, $smarty);
                }

                public function modCallme3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCallme($params, $smarty);
                }

                public function modCallme4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCallme($params, $smarty);
                }

                public function modCallme5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCallme($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаЧата текущего шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modChat                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modChat ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('chat', $params, $smarty);
            }

                public function modChat2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modChat($params, $smarty);
                }

                public function modChat3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modChat($params, $smarty);
                }

                public function modChat4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modChat($params, $smarty);
                }

                public function modChat5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modChat($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКорзина текущего шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCart                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCart ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('cart', $params, $smarty);
            }

                public function modCart2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCart($params, $smarty);
                }

                public function modCart3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCart($params, $smarty);
                }

                public function modCart4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCart($params, $smarty);
                }

                public function modCart5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCart($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниОтложенные шаблона.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modDefer                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modDefer ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('defer', $params, $smarty);
                }

                    public function modDefer2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modDefer($params, $smarty);
                    }

                    public function modDefer3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modDefer($params, $smarty);
                    }

                    public function modDefer4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modDefer($params, $smarty);
                    }

                    public function modDefer5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modDefer($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниСравнение текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCompare                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modCompare ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('compare', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ХлебныеКрошки текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBreadcrumbs                               |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBreadcrumbs ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('breadcrumbs', $params, $smarty);
            }

                public function modBreadcrumbs2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBreadcrumbs($params, $smarty);
                }

                public function modBreadcrumbs3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBreadcrumbs($params, $smarty);
                }

                public function modBreadcrumbs4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBreadcrumbs($params, $smarty);
                }

                public function modBreadcrumbs5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBreadcrumbs($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Сортировка текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSort                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSort ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('sort', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Фильтр текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFilter                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFilter ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('filter', $params, $smarty);
            }

                public function modFilter2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modFilter($params, $smarty);
                }

                public function modFilter3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modFilter($params, $smarty);
                }

                public function modFilter4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modFilter($params, $smarty);
                }

                public function modFilter5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modFilter($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ПагинацияСтраниц шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPagination                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modPagination ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('pagination', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Фотографии текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPhotos                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFotos ... }                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modImages ... }                              |
                    |                                                             |
                    ============================================================ */

            public function modPhotos ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('photos', $params, $smarty);
            }

                public function modFotos ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('fotos', $params, $smarty);
                }

                public function modImages ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('images', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Видео из текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modVideos                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modVideos ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('videos', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ПрикрепленныеФайлы шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFiles                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFiles ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('files', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ЦенаТовара шаблона.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPrice                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modPrice ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('price', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СвойстваТовара шаблона.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modProperties                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modProperties ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('properties', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Теги текущего шаблона.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTags                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modTags ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('tags', $params, $smarty);
            }

                public function modTags2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modTags($params, $smarty);
                }

                public function modTags3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modTags($params, $smarty);
                }

                public function modTags4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modTags($params, $smarty);
                }

                public function modTags5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modTags($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Комментарии текущего шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modComments                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modComments ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('comments', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаКомментария        |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modComment                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modComment ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('comment', $params, $smarty);
                }

                    public function modComment2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modComment($params, $smarty);
                    }

                    public function modComment3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modComment($params, $smarty);
                    }

                    public function modComment4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modComment($params, $smarty);
                    }

                    public function modComment5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modComment($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл КартаСайта шаблона.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSitemap                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSitemap ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('sitemap', $params, $smarty);
            }

                public function modSitemap2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSitemap($params, $smarty);
                }

                public function modSitemap3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSitemap($params, $smarty);
                }

                public function modSitemap4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSitemap($params, $smarty);
                }

                public function modSitemap5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSitemap($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Посты из папки модулей         |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPosts                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modPosts ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('posts', $params, $smarty);
            }

                public function modPosts2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modPosts($params, $smarty);
                }

                public function modPosts3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modPosts($params, $smarty);
                }

                public function modPosts4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modPosts($params, $smarty);
                }

                public function modPosts5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modPosts($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СпециальныеСтраницы из папки   |
                    |  модулей текущего шаблона.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSections                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSections ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('sections', $params, $smarty);
            }

                public function modSections2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSections($params, $smarty);
                }

                public function modSections3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSections($params, $smarty);
                }

                public function modSections4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSections($params, $smarty);
                }

                public function modSections5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSections($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Категории текущего шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCategories                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCategories ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('categories', $params, $smarty);
            }

                public function modCategories2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCategories($params, $smarty);
                }

                public function modCategories3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCategories($params, $smarty);
                }

                public function modCategories4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCategories($params, $smarty);
                }

                public function modCategories5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCategories($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Бренды текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBrands                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBrands ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('brands', $params, $smarty);
            }

                public function modBrands2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBrands($params, $smarty);
                }

                public function modBrands3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBrands($params, $smarty);
                }

                public function modBrands4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBrands($params, $smarty);
                }

                public function modBrands5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBrands($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Товары текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modProducts                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modProducts ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('products', $params, $smarty);
            }

                public function modProducts2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modProducts($params, $smarty);
                }

                public function modProducts3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modProducts($params, $smarty);
                }

                public function modProducts4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modProducts($params, $smarty);
                }

                public function modProducts5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modProducts($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Блог текущего шаблона.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBlog                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBlog ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('blog', $params, $smarty);
            }

                public function modBlog2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBlog($params, $smarty);
                }

                public function modBlog3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBlog($params, $smarty);
                }

                public function modBlog4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBlog($params, $smarty);
                }

                public function modBlog5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBlog($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Новости текущего шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modNews                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modNews ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('news', $params, $smarty);
            }

                public function modNews2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modNews($params, $smarty);
                }

                public function modNews3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modNews($params, $smarty);
                }

                public function modNews4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modNews($params, $smarty);
                }

                public function modNews5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modNews($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Обзоры текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modReviews                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modReviews ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('reviews', $params, $smarty);
            }

                public function modReviews2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modReviews($params, $smarty);
                }

                public function modReviews3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modReviews($params, $smarty);
                }

                public function modReviews4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modReviews($params, $smarty);
                }

                public function modReviews5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modReviews($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Статьи текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modArticles                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modArticles ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('articles', $params, $smarty);
            }

                public function modArticles2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modArticles($params, $smarty);
                }

                public function modArticles3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modArticles($params, $smarty);
                }

                public function modArticles4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modArticles($params, $smarty);
                }

                public function modArticles5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modArticles($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Склады текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modStocks                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modStocks ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('stocks', $params, $smarty);
            }

                public function modStocks2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modStocks($params, $smarty);
                }

                public function modStocks3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modStocks($params, $smarty);
                }

                public function modStocks4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modStocks($params, $smarty);
                }

                public function modStocks5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modStocks($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Страны текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCountries                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCountries ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('countries', $params, $smarty);
            }

                public function modCountries2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCountries($params, $smarty);
                }

                public function modCountries3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCountries($params, $smarty);
                }

                public function modCountries4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCountries($params, $smarty);
                }

                public function modCountries5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCountries($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Области текущего шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modRegions                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modRegions ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('regions', $params, $smarty);
            }

                public function modRegions2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modRegions($params, $smarty);
                }

                public function modRegions3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modRegions($params, $smarty);
                }

                public function modRegions4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modRegions($params, $smarty);
                }

                public function modRegions5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modRegions($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Города текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTowns                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modTowns ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('towns', $params, $smarty);
            }

                public function modTowns2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modTowns($params, $smarty);
                }

                public function modTowns3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modTowns($params, $smarty);
                }

                public function modTowns4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modTowns($params, $smarty);
                }

                public function modTowns5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modTowns($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл КартаКатегорий шаблона.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCategoriesMap                             |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCategoriesMap ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('categories-map', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл КаталогТоваров шаблона.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCatalog                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCatalog ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('catalog', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл АнонсовыйСписокПубликаций из   |
                    |  папки модулей текущего шаблона.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAnnounces                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAnnounces ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('announces', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ОсобыеТовары текущего шаблона. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFeatureds                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFeatureds ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('featureds', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Новинки текущего шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modNewests                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modNewests ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('newests', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ХитыПродаж текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modHits                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modHits ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('hits', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Акционные текущего шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modActions                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modActions ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('actions', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл НедавноОбсуждали шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCommenteds                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modCommenteds ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('commenteds', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл НедавноПокупали шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modOrdereds                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modOrdereds ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('ordereds', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СкороВпродаже текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAwaiteds                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modAwaiteds ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('awaiteds', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Выделены текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modHighlighteds                              |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modHighlighteds ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('highlighteds', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл НедавноПросмотренные шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modRecents                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modRecents ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('recents', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Ссылки текущего шаблона.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modLinks                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modLinks ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('links', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Меню текущего шаблона.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modMenu                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modMenu ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('menu', $params, $smarty);
            }

                public function modMenu2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modMenu($params, $smarty);
                }

                public function modMenu3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modMenu($params, $smarty);
                }

                public function modMenu4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modMenu($params, $smarty);
                }

                public function modMenu5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modMenu($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МенюЯзыковСайта из папки       |
                    |  модулей текущего шаблона.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modMenuLanguages                             |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modMenuLanguages ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('menu-languages', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл БоковаяПанель текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAside                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAside ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('aside', $params, $smarty);
            }

                public function modAside2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modAside($params, $smarty);
                }

                public function modAside3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modAside($params, $smarty);
                }

                public function modAside4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modAside($params, $smarty);
                }

                public function modAside5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modAside($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл БоковаяПанель текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAsideTop                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAsideRight ... }                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAsideBottom ... }                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAsideLeft ... }                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAsideCenter ... }                         |
                    |                                                             |
                    ============================================================ */

                public function modAsideTop ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('aside-top', $params, $smarty);
                }

                    public function modAsideRight ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('aside-right', $params, $smarty);
                    }

                    public function modAsideBottom ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('aside-bottom', $params, $smarty);
                    }

                    public function modAsideLeft ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('aside-left', $params, $smarty);
                    }

                    public function modAsideCenter ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('aside-center', $params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл БоковаяПанель текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSidebar                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSidebar ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('sidebar', $params, $smarty);
            }

                public function modSidebar2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSidebar($params, $smarty);
                }

                public function modSidebar3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSidebar($params, $smarty);
                }

                public function modSidebar4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSidebar($params, $smarty);
                }

                public function modSidebar5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSidebar($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаПоста из папки     |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPost                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modPost ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('post', $params, $smarty);
            }

                public function modPost2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modPost($params, $smarty);
                }

                public function modPost3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modPost($params, $smarty);
                }

                public function modPost4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modPost($params, $smarty);
                }

                public function modPost5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modPost($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаКатегории текущего |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCategory                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCategory ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('category', $params, $smarty);
            }

                public function modCategory2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCategory($params, $smarty);
                }

                public function modCategory3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCategory($params, $smarty);
                }

                public function modCategory4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCategory($params, $smarty);
                }

                public function modCategory5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCategory($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаБренда текущего    |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBrand                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBrand ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('brand', $params, $smarty);
            }

                public function modBrand2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBrand($params, $smarty);
                }

                public function modBrand3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBrand($params, $smarty);
                }

                public function modBrand4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBrand($params, $smarty);
                }

                public function modBrand5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBrand($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаТовара текущего    |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modProduct                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modProduct ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('product', $params, $smarty);
            }

                public function modProduct2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modProduct($params, $smarty);
                }

                public function modProduct3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modProduct($params, $smarty);
                }

                public function modProduct4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modProduct($params, $smarty);
                }

                public function modProduct5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modProduct($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаНовости текущего   |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modNewsItem                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modNewsItem ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('news-item', $params, $smarty);
            }

                public function modNewsItem2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modNewsItem($params, $smarty);
                }

                public function modNewsItem3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modNewsItem($params, $smarty);
                }

                public function modNewsItem4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modNewsItem($params, $smarty);
                }

                public function modNewsItem5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modNewsItem($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаСтатьи шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modArticle                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modArticle ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('article', $params, $smarty);
            }

                public function modArticle2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modArticle($params, $smarty);
                }

                public function modArticle3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modArticle($params, $smarty);
                }

                public function modArticle4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modArticle($params, $smarty);
                }

                public function modArticle5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modArticle($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаСклада шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modStock                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modStock ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('stock', $params, $smarty);
            }

                public function modStock2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modStock($params, $smarty);
                }

                public function modStock3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modStock($params, $smarty);
                }

                public function modStock4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modStock($params, $smarty);
                }

                public function modStock5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modStock($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаСтраны шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCountry                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCountry ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('country', $params, $smarty);
            }

                public function modCountry2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCountry($params, $smarty);
                }

                public function modCountry3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCountry($params, $smarty);
                }

                public function modCountry4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCountry($params, $smarty);
                }

                public function modCountry5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCountry($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаОбласти шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modRegion                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modRegion ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('region', $params, $smarty);
            }

                public function modRegion2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modRegion($params, $smarty);
                }

                public function modRegion3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modRegion($params, $smarty);
                }

                public function modRegion4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modRegion($params, $smarty);
                }

                public function modRegion5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modRegion($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МиниКарточкаГорода шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTown                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modTown ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('town', $params, $smarty);
            }

                public function modTown2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modTown($params, $smarty);
                }

                public function modTown3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modTown($params, $smarty);
                }

                public function modTown4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modTown($params, $smarty);
                }

                public function modTown5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modTown($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ОбщийМакетАдминпанели шаблона. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modDashboard                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modDashboard ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('dashboard', $params, $smarty);
            }

                public function modDashboard2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modDashboard($params, $smarty);
                }

                public function modDashboard3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modDashboard($params, $smarty);
                }

                public function modDashboard4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modDashboard($params, $smarty);
                }

                public function modDashboard5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modDashboard($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ЛичныйКабинет из папки модулей |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAccount                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAccount ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('account', $params, $smarty);
            }

                public function modAccountInfo ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-info', $params, $smarty);
                }

                public function modAccountMenu ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-menu', $params, $smarty);
                }

                public function modAccountProfile ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-profile', $params, $smarty);
                }

                public function modAccountOrders ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-orders', $params, $smarty);
                }

                public function modAccountFavorites ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-favorites', $params, $smarty);
                }

                public function modAccountSections ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-sections', $params, $smarty);
                }

                public function modAccountCategories ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-categories', $params, $smarty);
                }

                public function modAccountBrands ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-brands', $params, $smarty);
                }

                public function modAccountProducts ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-products', $params, $smarty);
                }

                public function modAccountNews ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-news', $params, $smarty);
                }

                public function modAccountArticles ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-articles', $params, $smarty);
                }

                public function modAccountPosts ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-posts', $params, $smarty);
                }

                public function modAccountComments ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-comments', $params, $smarty);
                }

                public function modAccountFeedback ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-feedback', $params, $smarty);
                }

                public function modAccountBalance ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('account-balance', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл АбстрактнаяСтраница текущего   |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAbstract                                  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAbstract ( $params = null, & $smarty = null ) {
                $urlParams = FALSE;
                $file = $this->abstractTpl(TRUE, FALSE, $urlParams);
                $this->setVariableByRef('modAbstractParams', $urlParams, $smarty, 'global');
                return $this->includeModule($file, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл НекийБлок текущего шаблона.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBox                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBox ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('box', $params, $smarty);
            }

                public function modBox2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBox($params, $smarty);
                }

                public function modBox3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBox($params, $smarty);
                }

                public function modBox4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBox($params, $smarty);
                }

                public function modBox5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBox($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл АннотацияСтраницы текущего     |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAnnotation                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAnnotation ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('annotation', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ОписаниеСтраницы текущего      |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modBody                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modBody ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('body', $params, $smarty);
            }

                public function modBody2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modBody($params, $smarty);
                }

                public function modBody3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modBody($params, $smarty);
                }

                public function modBody4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modBody($params, $smarty);
                }

                public function modBody5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modBody($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл DemoBody текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modDemoBody                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modDemoBody ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('demo-body', $params, $smarty);
                }

                    public function modDemoBody2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody6 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 6);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody7 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 7);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody8 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 8);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody9 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 9);
                        return $this->modDemoBody($params, $smarty);
                    }

                    public function modDemoBody10 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 10);
                        return $this->modDemoBody($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл SeoТекстСтраницы текущего      |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSeo                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSeo ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('seo', $params, $smarty);
            }

                public function modSeo2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSeo($params, $smarty);
                }

                public function modSeo3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSeo($params, $smarty);
                }

                public function modSeo4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSeo($params, $smarty);
                }

                public function modSeo5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSeo($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл СоциальныеСсылки шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSocials                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSocials ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('socials', $params, $smarty);
            }

                public function modSocials2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSocials($params, $smarty);
                }

                public function modSocials3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSocials($params, $smarty);
                }

                public function modSocials4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSocials($params, $smarty);
                }

                public function modSocials5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSocials($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Копирайт текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modCopyright                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modCopyright ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('copyright', $params, $smarty);
            }

                public function modCopyright2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modCopyright($params, $smarty);
                }

                public function modCopyright3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modCopyright($params, $smarty);
                }

                public function modCopyright4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modCopyright($params, $smarty);
                }

                public function modCopyright5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modCopyright($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ЧастыеВопросыОтветы шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFAQ                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFAQ ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('faq', $params, $smarty);
            }

                public function modFAQ2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modFAQ($params, $smarty);
                }

                public function modFAQ3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modFAQ($params, $smarty);
                }

                public function modFAQ4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modFAQ($params, $smarty);
                }

                public function modFAQ5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modFAQ($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Информер из папки модулей      |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modInform                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modInformer ...}                             |
                    |                                                             |
                    ============================================================ */

            public function modInform ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('inform', $params, $smarty);
            }

                public function modInformer( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('informer', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Трекер из папки модулей        |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTracker                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modTracker( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('tracker', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Информация текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modInfo                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modInfo ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('info', $params, $smarty);
            }

                public function modInfo2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modInfo($params, $smarty);
                }

                public function modInfo3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modInfo($params, $smarty);
                }

                public function modInfo4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modInfo($params, $smarty);
                }

                public function modInfo5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modInfo($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Таксономия из папки модулей    |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTaxonomy                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modTaxonomy ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('taxonomy', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл НетЗаписей текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modNone                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modNone ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('none', $params, $smarty);
            }

                public function modNone2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modNone($params, $smarty);
                }

                public function modNone3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modNone($params, $smarty);
                }

                public function modNone4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modNone($params, $smarty);
                }

                public function modNone5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modNone($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ФормаВвода текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modForm                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modForm ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('form', $params, $smarty);
            }

                public function modForm2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modForm($params, $smarty);
                }

                public function modForm3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modForm($params, $smarty);
                }

                public function modForm4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modForm($params, $smarty);
                }

                public function modForm5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modForm($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл РазводкаПоСтраницамСайта       |
                    |  из папки модулей текущего шаблона.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSignpost                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSignpost ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('signpost', $params, $smarty);
            }

                public function modSignpost2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSignpost($params, $smarty);
                }

                public function modSignpost3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSignpost($params, $smarty);
                }

                public function modSignpost4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSignpost($params, $smarty);
                }

                public function modSignpost5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSignpost($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ЯзыкиСайта шаблона.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modLanguages                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modLanguages ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('languages', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл КакиеТоНастройки шаблона.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSettings                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSettings ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('settings', $params, $smarty);
            }

                public function modSettings2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSettings($params, $smarty);
                }

                public function modSettings3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSettings($params, $smarty);
                }

                public function modSettings4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSettings($params, $smarty);
                }

                public function modSettings5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSettings($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МногоязычныйСловарь из папки   |
                    |  модулей текущего шаблона.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglot                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotHeader ...}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotFooter ...}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotError404 ...}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotHome ...}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotCallme ...}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotFeedback ...}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotLogin ...}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotRegistration ...}                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotCart ...}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotAccount ...}                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotSections ...}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotTopics ...}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotCategories ...}                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotBrands ...}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotProducts ...}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotNews ...}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotBlog ...}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotArticles ...}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotReviews ...}                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotStocks ...}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotCountries ...}                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotRegions ...}                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modPolyglotTowns ...}                        |
                    |                                                             |
                    ============================================================ */

            public function modPolyglot ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('polyglot', $params, $smarty);
            }

                public function modPolyglot2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modPolyglot($params, $smarty);
                }

                public function modPolyglot3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modPolyglot($params, $smarty);
                }

                public function modPolyglot4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modPolyglot($params, $smarty);
                }

                public function modPolyglot5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modPolyglot($params, $smarty);
                }

                public function modPolyglotHeader ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-header', $params, $smarty);
                }

                public function modPolyglotFooter ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-footer', $params, $smarty);
                }

                public function modPolyglotError404 ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-error404', $params, $smarty);
                }

                public function modPolyglotHome ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-home', $params, $smarty);
                }

                public function modPolyglotCallme ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-callme', $params, $smarty);
                }

                public function modPolyglotFeedback ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-feedback', $params, $smarty);
                }

                public function modPolyglotLogin ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-login', $params, $smarty);
                }

                public function modPolyglotRegistration ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-registration', $params, $smarty);
                }

                public function modPolyglotCart ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-cart', $params, $smarty);
                }

                public function modPolyglotAccount ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-account', $params, $smarty);
                }

                public function modPolyglotSections ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-sections', $params, $smarty);
                }

                public function modPolyglotTopics ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-topics', $params, $smarty);
                }

                public function modPolyglotCategories ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-categories', $params, $smarty);
                }

                public function modPolyglotBrands ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-brands', $params, $smarty);
                }

                public function modPolyglotProducts ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-products', $params, $smarty);
                }

                public function modPolyglotNews ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-news', $params, $smarty);
                }

                public function modPolyglotBlog ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-blog', $params, $smarty);
                }

                public function modPolyglotArticles ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-articles', $params, $smarty);
                }

                public function modPolyglotReviews ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-reviews', $params, $smarty);
                }

                public function modPolyglotStocks ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-stocks', $params, $smarty);
                }

                public function modPolyglotCountries ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-countries', $params, $smarty);
                }

                public function modPolyglotRegions ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-regions', $params, $smarty);
                }

                public function modPolyglotTowns ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('polyglot-towns', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Антиспам текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAntispam                                  |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAntispam ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('antispam', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл КакиеТоИсправленияБагов        |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modFixes                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modFixes ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('fixes', $params, $smarty);
            }

                public function modFixes2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modFixes($params, $smarty);
                }

                public function modFixes3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modFixes($params, $smarty);
                }

                public function modFixes4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modFixes($params, $smarty);
                }

                public function modFixes5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modFixes($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ИнструментыАдмина шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAdminTools                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modAdminTools ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('admin-tools', $params, $smarty);
            }

                public function modAdminTools2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modAdminTools($params, $smarty);
                }

                public function modAdminTools3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modAdminTools($params, $smarty);
                }

                public function modAdminTools4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modAdminTools($params, $smarty);
                }

                public function modAdminTools5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modAdminTools($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл визуального модуля             |
                    |  ИнструментыАдмина-ЗагрузитьФайл из папки модулей текущего  |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAdminUpload                               |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modAdminUpload ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('admin-upload', $params, $smarty);
                }

                    public function modAdminUpload2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modAdminUpload($params, $smarty);
                    }

                    public function modAdminUpload3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modAdminUpload($params, $smarty);
                    }

                    public function modAdminUpload4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modAdminUpload($params, $smarty);
                    }

                    public function modAdminUpload5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modAdminUpload($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл визуального модуля             |
                    |  ИнструментыАдмина-Редактировать из папки модулей текущего  |
                    |  шаблона.                                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modAdminEdit                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modAdminEdit ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('admin-edit', $params, $smarty);
                }

                    public function modAdminEdit2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modAdminEdit($params, $smarty);
                    }

                    public function modAdminEdit3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modAdminEdit($params, $smarty);
                    }

                    public function modAdminEdit4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modAdminEdit($params, $smarty);
                    }

                    public function modAdminEdit5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modAdminEdit($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Стартер из папки модулей       |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modStarter                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modStarter ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('starter', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Инициализация из папки модулей |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modInit                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modInit ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('init', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МакетБлока из папки модулей    |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modLayout                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modLayout ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('layout', $params, $smarty);
                }

                    public function modLayout2 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 2);
                        return $this->modLayout($params, $smarty);
                    }

                    public function modLayout3 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 3);
                        return $this->modLayout($params, $smarty);
                    }

                    public function modLayout4 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 4);
                        return $this->modLayout($params, $smarty);
                    }

                    public function modLayout5 ( $params = null, & $smarty = null ) {
                        $this->setParam('num', $params, 5);
                        return $this->modLayout($params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ИнструментыСтраницы шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modTools                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modTools ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('tools', $params, $smarty);
            }

                public function modTools2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modTools($params, $smarty);
                }

                public function modTools3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modTools($params, $smarty);
                }

                public function modTools4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modTools($params, $smarty);
                }

                public function modTools5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modTools($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МакетПисьма текущего шаблона.  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modEmail                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modEmail ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('email', $params, $smarty);
            }

                public function modEmail2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modEmail($params, $smarty);
                }

                public function modEmail3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modEmail($params, $smarty);
                }

                public function modEmail4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modEmail($params, $smarty);
                }

                public function modEmail5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modEmail($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МакетСМС текущего шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSms                                       |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSms ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('sms', $params, $smarty);
            }

                public function modSms2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modSms($params, $smarty);
                }

                public function modSms3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modSms($params, $smarty);
                }

                public function modSms4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modSms($params, $smarty);
                }

                public function modSms5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modSms($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл МодальноеОкно текущего шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modModal                                     |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modModal ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('modal', $params, $smarty);
            }

                public function modModal2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modModal($params, $smarty);
                }

                public function modModal3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modModal($params, $smarty);
                }

                public function modModal4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modModal($params, $smarty);
                }

                public function modModal5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modModal($params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ДизайнерскиеСтили шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modStyles                                    |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modStyles ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('styles', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл КлиентскиеСкрипты шаблона.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modScripts                                   |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modScripts ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('scripts', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ТекстСубдоменаСтраницы шаблона.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {modSubdomain                                 |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modSubdomain ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('subdomain', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл ОпределитьРабочуюПеременную    |
                    |  текущего шаблона.                                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modUseVariable                               |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modUseVariable ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('use-variable', $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл Демо текущего шаблона.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modDemo                                      |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

            public function modDemo ( $params = null, & $smarty = null ) {
                return $this->includeModNumerated('demo', $params, $smarty);
            }

                public function modDemo2 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 2);
                    return $this->modDemo($params, $smarty);
                }

                public function modDemo3 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 3);
                    return $this->modDemo($params, $smarty);
                }

                public function modDemo4 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 4);
                    return $this->modDemo($params, $smarty);
                }

                public function modDemo5 ( $params = null, & $smarty = null ) {
                    $this->setParam('num', $params, 5);
                    return $this->modDemo($params, $smarty);
                }

                    public function modDemoHome ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-home', $params, $smarty);
                    }

                    public function modDemoSection ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-section', $params, $smarty);
                    }

                    public function modDemoCategory ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-category', $params, $smarty);
                    }

                    public function modDemoBrand ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-brand', $params, $smarty);
                    }

                    public function modDemoProduct ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-product', $params, $smarty);
                    }

                    public function modDemoNewsItem ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-news-item', $params, $smarty);
                    }

                    public function modDemoArticle ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-article', $params, $smarty);
                    }

                    public function modDemoStock ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-stock', $params, $smarty);
                    }

                    public function modDemoCountry ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-country', $params, $smarty);
                    }

                    public function modDemoRegion ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-region', $params, $smarty);
                    }

                    public function modDemoTown ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-town', $params, $smarty);
                    }

                    public function modDemoMedia ( $params = null, & $smarty = null ) {
                        return $this->includeModNumerated('demo-media', $params, $smarty);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл DemoRemark текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modDemoRemark                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modDemoRemark ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('demo-remark', $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Подключить в это место файл DemoThemes текущего шаблона.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {modDemoThemes                                |
                    |                   [num = moduleNumber]  номер вида модуля   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                                                             |
                    ============================================================ */

                public function modDemoThemes ( $params = null, & $smarty = null ) {
                    return $this->includeModNumerated('demo-themes', $params, $smarty);
                }

        /* ========================================================================
        |                                                                         |
        |    Обработчики наших блочных функций.                                   |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Каждая из этих функций может быть вызвана в шаблоне с помощью        |
        |    инструкции такого вида:                                              |
        |                                                                         |
        |        {имяФункции параметр1 параметр2 ... параметрN}                   |
        |            ...                                                          |
        |        {/имяФункции}                                                    |
        |                                                                         |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {enum                                         |
                    |                   [item = $record]  с какой записью работаем|
                    |                   [from = VarName]  или где хранится запись |
                    |                                     (по умолчанию $items)   |
                    |                   [via = VarName]   через какую переменную  |
                    |                                     перечислять элементы    |
                    |                                     (по умолчанию $item)    |
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     перечисляемых элементов |
                    |                                     (по умолчанию все)      |
                    |               } ... {/enum}                                 |
                    |                                                             |
                    ============================================================ */

        public function enum ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            $varname = $this->paramAsSentence('via', $params, 'item');
            $max = $this->paramAsNaturalExtended('max', $params, 2000000000);
            if (is_null($content)) {
                $repeat = FALSE;
                if ($max > 0) {
                    if (isset($params['item'])) $item = $params['item'];
                    else {
                        $this->mustbeParam('from', $params, 'items');
                        $item = $this->retrieveItem($params, $smarty);
                    }
                    $repeat = $this->isActualArray($item);
                    if ($repeat) {
                        $index = count($this->enumStack);
                        $this->enumStack[$index] = & $item;
                        $this->enumStackNums[$index] = 1;
                        $value = reset($this->enumStack[$index]);
                        $count = $this->enumStackNums[$index];
                        $index = key($this->enumStack[$index]);
                        $this->setVariableByRef($varname, $value, $smarty);
                        $this->setVariableByRef($varname . 'Num', $count, $smarty);
                        $this->setVariableByRef($varname . 'Idx', $index, $smarty);
                    }
                }
                return '';
            }
            $repeat = $this->isActualArray($this->enumStack);
            if ($repeat) {
                $index = count($this->enumStack) - 1;
                $count = $this->enumStackNums[$index] + 1;
                $value = $count <= $max ? next($this->enumStack[$index]) : FALSE;
                $repeat = $value !== FALSE;
                if ($repeat) {
                    $this->enumStackNums[$index] = $count;
                    $index = key($this->enumStack[$index]);
                    $this->setVariableByRef($varname, $value, $smarty);
                    $this->setVariableByRef($varname . 'Num', $count, $smarty);
                    $this->setVariableByRef($varname . 'Idx', $index, $smarty);
                } else {
                    array_pop($this->enumStack);
                    array_pop($this->enumStackNums);
                }
            }
            return $content;
        }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива специальных страниц.          |
                    |                                                             |
                    ============================================================ */

            public function enumSections ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'sections');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива категорий.                    |
                    |                                                             |
                    ============================================================ */

            public function enumCategories ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'categories');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива хлебных крошек.               |
                    |                                                             |
                    ============================================================ */

                public function enumPath ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'category->path');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива подкатегорий текущей категор. |
                    |                                                             |
                    ============================================================ */

                public function enumSubcats ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $from = $this->paramAsSentence('from', $params, 'category');
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $from . '->subcategories');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива свойств.                      |
                    |                                                             |
                    ============================================================ */

                public function enumProperties ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'properties');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива значений свойства.            |
                    |                                                             |
                    ============================================================ */

                    public function enumOptions ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $from = $this->paramAsSentence('from', $params, 'item');
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, $from . '->options');
                        $this->mustbeParam('via', $params2, 'option');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива брендов.                      |
                    |                                                             |
                    ============================================================ */

            public function enumBrands ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'all_brands');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива каталога.                     |
                    |                                                             |
                    ============================================================ */

            public function enumCatalog ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'catalog');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива товаров.                      |
                    |                                                             |
                    ============================================================ */

            public function enumProducts ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'products');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива прикрепленных товаров.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {enumAssortment                               |
                    |                   [item = $record]  с какой записью работаем|
                    |                   [from = VarName]  или где хранится запись |
                    |                                     (по умолчанию $item)    |
                    |                   [via = VarName]   через какую переменную  |
                    |                                     перечислять элементы    |
                    |                                     (по умолчанию $product) |
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     перечисляемых элементов |
                    |                                     (по умолчанию все)      |
                    |               } ... {/enumAssortment}                       |
                    |                                                             |
                    ============================================================ */

                public function enumAssortment ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $from = $this->paramAsSentence('from', $params, 'item');
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $from . '->products');
                    $this->mustbeParam('via', $params2, 'product');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива вариантов товара.             |
                    |                                                             |
                    ============================================================ */

                    public function enumVariants ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $from = $this->paramAsSentence('from', $params, 'item');
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, $from . '->variants');
                        $this->mustbeParam('via', $params2, 'variant');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива фотографий товара.            |
                    |                                                             |
                    ============================================================ */

                    public function enumPhotos ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $from = $this->paramAsSentence('from', $params, 'item');
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, $from . '->fotos');
                        $this->mustbeParam('via', $params2, 'photo');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива фотографий записи.            |
                    |                                                             |
                    ============================================================ */

                        public function enumImages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $from = $this->paramAsSentence('from', $params, 'item');
                            $params2 = $this->takeEnumParams($params);
                            $this->setParam('from', $params2, $from . '->images');
                            $this->mustbeParam('via', $params2, 'image');
                            return $this->enum($params2, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива свойств товара.               |
                    |                                                             |
                    ============================================================ */

                    public function enumSpecifications ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->mustbeParam('via', $params2, 'specification');
                        if (is_null($content)) {
                            $field = '->properties_tree';
                            $from = $this->paramAsSentence('from', $params);
                            if (empty($from)) {
                                $from = 'item';
                                if (!$this->existsVar($from . $field, $smarty)) $from = 'product';
                            }
                            $this->setParam('from', $params2, $from . $field);
                        }
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива значений свойства товара.     |
                    |                                                             |
                    ============================================================ */

                        public function enumSpecies ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $from = $this->paramAsSentence('from', $params, 'specification');
                            $params2 = $this->takeEnumParams($params);
                            $this->setParam('from', $params2, $from);
                            $this->mustbeParam('via', $params2, 'specie');
                            return $this->enum($params2, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива связанных товаров.            |
                    |                                                             |
                    ============================================================ */

                public function enumRelateds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $from = $this->paramAsSentence('from', $params, 'item');
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $from . '->related_products');
                    $this->mustbeParam('via', $params2, 'product');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива аксессуаров.                  |
                    |                                                             |
                    ============================================================ */

                public function enumAccessories ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $from = $this->paramAsSentence('from', $params, 'item');
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $from . '->accessory_products');
                    $this->mustbeParam('via', $params2, 'product');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массивов особых товаров.              |
                    |                                                             |
                    ============================================================ */

                public function enumFeatureds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $varname = 'featureds';
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $varname);
                    if (is_null($content)) {
                        if (!$this->existsVar($varname, $smarty)) {
                            $items = array();
                            $names = array('newest_products',
                                           'hit_products',
                                           'actional_products',
                                           'commented_products',
                                           'ordered_products',
                                           'awaited_products');
                            foreach ($names as $item) {
                                $item = $this->getVar($item, $smarty);
                                $this->mergeProducts($item, $items);
                            }
                            $params = array('assign' => $varname,
                                            'scope' => 'global');
                            $this->assignVar($params, $items, $smarty);
                        }
                    }
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива новинок.                      |
                    |                                                             |
                    ============================================================ */

                    public function enumNewests ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, 'newest_products');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива хитов продаж.                 |
                    |                                                             |
                    ============================================================ */

                    public function enumHits ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, 'hit_products');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива акционных товаров.            |
                    |                                                             |
                    ============================================================ */

                    public function enumActions ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, 'actional_products');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива недавно обсуждавшихся товаров.|
                    |                                                             |
                    ============================================================ */

                    public function enumCommenteds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, 'commented_products');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива недавно покупавшихся товаров. |
                    |                                                             |
                    ============================================================ */

                    public function enumOrdereds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, 'ordered_products');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива ожидаемых товаров.            |
                    |                                                             |
                    ============================================================ */

                    public function enumAwaiteds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $params2 = $this->takeEnumParams($params);
                        $this->setParam('from', $params2, 'awaited_products');
                        return $this->enum($params2, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массивов анонсовых публикаций.        |
                    |                                                             |
                    ============================================================ */

                public function enumAnnounces ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $varname = 'announces';
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $varname);
                    if (is_null($content)) {
                        if (!$this->existsVar($varname, $smarty)) {
                            $items = array();
                            $names = array('news',
                                           'news_for_category',
                                           'news_for_product',
                                           'articles',
                                           'articles_for_category',
                                           'articles_for_product');
                            foreach ($names as $item) {
                                $item = $this->getVar($item, $smarty);
                                $this->mergePublications($item, $items);
                            }
                            $params = array('assign' => $varname,
                                            'scope' => 'global');
                            $this->assignVar($params, $items, $smarty);
                        }
                    }
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива недавно просмотренных страниц.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {enumRecents                                  |
                    |                   [item = $record]  с какой записью работаем|
                    |                   [from = VarName]  или где хранится запись |
                    |                                     (по умолчанию $recents) |
                    |                   [via = VarName]   через какую переменную  |
                    |                                     перечислять элементы    |
                    |                                     (по умолчанию $item)    |
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     перечисляемых элементов |
                    |                                     (по умолчанию все)      |
                    |               } ... {/enumRecents}                          |
                    |                                                             |
                    ============================================================ */

                public function enumRecents ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $varname = 'recents';
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, $varname);
                    if (is_null($content)) {
                        if (!$this->existsVar($varname, $smarty)) {
                            $items = array();
                            $names = array(
                                'recent_articles',
                                'recent_news',
                                'recent_categories',
                                'recent_brands',
                                'recent_products'
                            );
                            foreach ($names as $item) {
                                $item = $this->getVar($item, $smarty);
                                $this->mergePublications($item, $items);
                            }
                            $params = array('assign' => $varname,
                                            'scope' => 'global');
                            $this->assignVar($params, $items, $smarty);
                        }
                    }
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива поиска.                       |
                    |                                                             |
                    ============================================================ */

                public function enumSearch ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'search_products');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива новостей.                     |
                    |                                                             |
                    ============================================================ */

            public function enumNews ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'news');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива статей.                       |
                    |                                                             |
                    ============================================================ */

            public function enumArticles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'articles');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива складов.                      |
                    |                                                             |
                    ============================================================ */

            public function enumStocks ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'stocks');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива стран.                        |
                    |                                                             |
                    ============================================================ */

            public function enumCountries ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'countries');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива областей.                     |
                    |                                                             |
                    ============================================================ */

                public function enumRegions ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'regions');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива городов.                      |
                    |                                                             |
                    ============================================================ */

                public function enumTowns ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'towns');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива способов доставки.            |
                    |                                                             |
                    ============================================================ */

            public function enumDeliveries ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'delivery_methods');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива способов оплаты.              |
                    |                                                             |
                    ============================================================ */

            public function enumPayments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'payments');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива комментариев.                 |
                    |                                                             |
                    ============================================================ */

            public function enumComments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $varname = 'comments';
                if (is_null($content)) {
                    $def = $this->existsVar($varname, $smarty) ? '' : 'item';
                    $from = $this->paramAsSentence('from', $params, $def);
                    if (!empty($from)) $varname = $from . '->' . $varname;
                }
                $this->setParam('from', $params2, $varname);
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива валют.                        |
                    |                                                             |
                    ============================================================ */

            public function enumCurrencies ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'currencies');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива заказов.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {enumOrders                                   |
                    |                   [item = $list]    с каким списком работаем|
                    |                                     (по умолчанию список из |
                    |                                     переменной $orders)     |
                    |                   [via = VarName]   через какую переменную  |
                    |                                     перечислять элементы    |
                    |                                     (по умолчанию $item)    |
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     перечисляемых элементов |
                    |                                     (по умолчанию все)      |
                    |               } ... {/enumOrders}                           |
                    |                                                             |
                    ============================================================ */

            public function enumOrders ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'orders');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива корзины.                      |
                    |                                                             |
                    ============================================================ */

            public function enumCart ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'cart_products');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива отложенных товаров.           |
                    |                                                             |
                    ============================================================ */

                public function enumDefer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'defer_products');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива товаров заказа.               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {enumOrder                                    |
                    |                   [item = $record]  с какой записью работаем|
                    |                   [from = VarName]  или где хранится запись |
                    |                                     (по умолчанию $order)   |
                    |                   [via = VarName]   через какую переменную  |
                    |                                     перечислять элементы    |
                    |                                     (по умолчанию $item)    |
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     перечисляемых элементов |
                    |                                     (по умолчанию все)      |
                    |               } ... {/enumOrder}                            |
                    |                                                             |
                    ============================================================ */

                public function enumOrder ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->mustbeParam('from', $params, 'order');
                    $this->mustbeParam('via', $params, 'item');
                    return $this->enumAssortment($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива пагинации.                    |
                    |                                                             |
                    ============================================================ */

            public function enumPages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'Pages');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива способов сортировки.          |
                    |                                                             |
                    ============================================================ */

                public function enumSorts ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $params2 = $this->takeEnumParams($params);
                    $this->setParam('from', $params2, 'sort_modes');
                    return $this->enum($params2, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива размеров страницы.            |
                    |                                                             |
                    ============================================================ */

                public function enumSizes ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    return $this->enum(array('item' => $this->mySizes()), $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  ПРИВАТНЫЙ МЕТОД: Список наших размеров страницы.           |
                    |                                                             |
                    ============================================================ */

                    protected function mySizes () {
                        return array(12, 24, 45, 60, 90, 120);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива языков.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {enumLanguages                                |
                    |                   [item = $list]    с каким списком работаем|
                    |                                     (по умолчанию из        |
                    |                                     переменной $languages)  |
                    |                   [via = VarName]   через какую переменную  |
                    |                                     перечислять элементы    |
                    |                                     (по умолчанию $language)|
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     перечисляемых элементов |
                    |                                     (по умолчанию все)      |
                    |               } ... {/enumLanguages}                        |
                    |                                                             |
                    ============================================================ */

            public function enumLanguages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'languages');
                $this->mustbeParam('via', $params2, 'language');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить элементы массива дизайнов.                     |
                    |                                                             |
                    ============================================================ */

            public function enumThemes ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $params2 = $this->takeEnumParams($params);
                $this->setParam('from', $params2, 'themes');
                return $this->enum($params2, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Перечислить имена функций хелпера шаблона.                 |
                    |                                                             |
                    ============================================================ */

            public function enumFunctions ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                return $this->enum(array('item' => $this->templateFunctions), $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Если уже существует аккаунт с таким логином (емейлом).     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifExistsLogin                                |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $email)   |
                    |                   [simple = TRUE]   если простая проверка,  |
                    |                                     без валидации статусов  |
                    |                                     "разрешен", "удален"    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifExistsLogin}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotExistsLogin} ... {/ifNotExistsLogin}    |
                    |                                                             |
                    ============================================================ */

        public function ifExistsLogin ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $varname = $this->paramAsSentence('from', $params, 'email');
                $value = $this->getVar($varname, $smarty);
                $repeat = $this->isActualString($value);
                if ($repeat) {
                    $filter = new stdClass;
                    $filter->email = $value;
                    if (!$this->paramAsBoolean('simple', $params)) {
                        $filter->enabled = 1;
                        $filter->deleted = 0;
                    }
                    $this->cms->db->users->one($repeat, $filter);
                    $repeat = !empty($repeat);
                }
                if ($this->existsParamNOT($params)) $repeat = !$repeat;
                return '';
            }
            return $content;
        }

            public function ifNotExistsLogin ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $this->setParam('not', $params, TRUE);
                return $this->ifExistsLogin($params, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Если уже существует аккаунт с таким ником.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifExistsNick                                 |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $nick)    |
                    |                   [simple = TRUE]   если простая проверка,  |
                    |                                     без валидации статусов  |
                    |                                     "разрешен", "удален"    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifExistsNick}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotExistsNick} ... {/ifNotExistsNick}      |
                    |                                                             |
                    ============================================================ */

            public function ifExistsNick ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $varname = $this->paramAsSentence('from', $params, 'nick');
                    $value = $this->getVar($varname, $smarty);
                    $repeat = $this->isActualString($value);
                    if ($repeat) {
                        $filter = new stdClass;
                        $filter->nickname = $value;
                        if (!$this->paramAsBoolean('simple', $params)) {
                            $filter->enabled = 1;
                            $filter->deleted = 0;
                        }
                        $this->cms->db->users->one($repeat, $filter);
                        $repeat = !empty($repeat);
                    }
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotExistsNick ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifExistsNick($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если уже существует аккаунт с такой картой клиента (ником).|
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifExistsCard                                 |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $card)    |
                    |                   [simple = TRUE]   если простая проверка,  |
                    |                                     без валидации статусов  |
                    |                                     "разрешен", "удален"    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifExistsCard}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotExistsCard} ... {/ifNotExistsCard}      |
                    |                                                             |
                    ============================================================ */

            public function ifExistsCard ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $this->mustbeParam('from', $params, 'card');
                return $this->ifExistsNick($params, $content, $smarty, $repeat);
            }

                public function ifNotExistsCard ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifExistsCard($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если аккаунт с такой картой клиента (ником) имеет          |
                    |  достаточное количество юнитов или достаточный баланс       |
                    |  (то есть платежеспособный).                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSolventCard                                |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $card)    |
                    |                   [simple = TRUE]   если простая проверка,  |
                    |                                     без валидации статусов  |
                    |                                     "разрешен", "удален"    |
                    |                   [forCash = TRUE]  если проверить баланс   |
                    |                                     (иначе проверит юниты)  |
                    |                   [min = minValue]  минимально необходимое  |
                    |                                     количество юнитов (по   |
                    |                                     умолчанию 1 юнит) или   |
                    |                                     минимально необходимый  |
                    |                                     баланс аккаунта (по     |
                    |                                     умолчанию 1.00 у.е.)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifSolventCard}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSolventCard} ... {/ifNotSolventCard}    |
                    |                                                             |
                    ============================================================ */

                public function ifSolventCard ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $varname = $this->paramAsSentence('from', $params, 'card');
                        $value = $this->getVar($varname, $smarty);
                        $repeat = $this->isActualString($value);
                        if ($repeat) {
                            $filter = new stdClass;
                            $filter->nickname = $value;
                            if (!$this->paramAsBoolean('simple', $params)) {
                                $filter->enabled = 1;
                                $filter->deleted = 0;
                            }
                            $this->cms->db->users->one($repeat, $filter);
                            if ($this->paramAsBoolean('forCash', $params)) {
                                $min = $this->paramAsFloat('min', $params, 1.0);
                                $field = 'cash';
                            } else {
                                $min = $this->paramAsInteger('min', $params, 1);
                                $field = 'units';
                            }
                            $repeat = isset($repeat->$field)
                                      && $repeat->$field >= $min;
                        }
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotSolventCard ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifSolventCard($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если находимся в демо режиме.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifDemo                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifDemo}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotDemo} ... {/ifNotDemo}                  |
                    |                                                             |
                    ============================================================ */

        public function ifDemo ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = $this->cms->security->inDemoMode();
                if ($this->existsParamNOT($params)) $repeat = !$repeat;
                return '';
            }
            return $content;
        }

            public function ifNotDemo ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                $this->setParam('not', $params, TRUE);
                return $this->ifDemo($params, $content, $smarty, $repeat);
            }

                    /* ============================================================
                    |                                                             |
                    |  Если находимся в демо режиме или на сайте находится админ. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifDemoOrAdmin                                |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifDemoOrAdmin}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotDemoOrAdmin} ... {/ifNotDemoOrAdmin}    |
                    |                                                             |
                    ============================================================ */

            public function ifDemoOrAdmin ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->cms->security->inDemoMode()
                           || $this->existsAdmin();
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotDemoOrAdmin ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifDemoOrAdmin($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если страница вызвана в режиме динамической подгрузки.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAjax                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifAjax}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAjax} ... {/ifNotAjax}                  |
                    |                                                             |
                    ============================================================ */

            public function ifAjax ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsAjaxSignal($smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotAjax ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifAjax($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если на сайте находится админ.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAdmin                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifAdmin}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAdmin} ... {/ifNotAdmin}                |
                    |                                                             |
                    ============================================================ */

            public function ifAdmin ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsAdmin();
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotAdmin ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifAdmin($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если на сайте находится авторизованный пользователь.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifUser                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifUser}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotUser} ... {/ifNotUser}                  |
                    |                                                             |
                    ============================================================ */

                public function ifUser ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsUser();
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotUser ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifUser($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на сайте находится админ или авторизованный клиент.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAdminOrUser                                |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifAdminOrUser}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAdminOrUser} ... {/ifNotAdminOrUser}    |
                    |                                                             |
                    ============================================================ */

                    public function ifAdminOrUser ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsAdmin() ||
                                      $this->existsUser();
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotAdminOrUser ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifAdminOrUser($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если на сайте находится авторизованный клиент, который     |
                    |  назначен управляющим этой записью.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOwner                                      |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifOwner}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOwner} ... {/ifNotOwner}                |
                    |                                                             |
                    ============================================================ */

                public function ifOwner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsUser();
                        if ($repeat) {
                            $item = $this->retrieveItem($params, $smarty);
                            $repeat = !empty($item->user_id) &&
                                      $item->user_id == $this->getUserId();
                        }
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotOwner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOwner($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на сайте находится админ или авторизованный клиент,   |
                    |  который назначен управляющим этой записью.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAdminOrOwner                               |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifAdminOrOwner}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAdminOrOwner} ... {/ifNotAdminOrOwner}  |
                    |                                                             |
                    ============================================================ */

                    public function ifAdminOrOwner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsAdmin();
                            if (!$repeat) {
                                $repeat = $this->existsUser();
                                if ($repeat) {
                                    $item = $this->retrieveItem($params, $smarty);
                                    $repeat = !empty($item->user_id) &&
                                              $item->user_id == $this->getUserId();
                                }
                            }
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotAdminOrOwner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifAdminOrOwner($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если находимся в демо режиме или на сайте находится админ  |
                    |  или авторизованный клиент, который назначен управляющим    |
                    |  этой записью.                                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifDemoOrAdminOrOwner                         |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifDemoOrAdminOrOwner}                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotDemoOrAdminOrOwner}                     |
                    |               ... {/ifNotDemoOrAdminOrOwner}                |
                    |                                                             |
                    ============================================================ */

                    public function ifDemoOrAdminOrOwner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->cms->security->inDemoMode()
                                   || $this->existsAdmin();
                            if (!$repeat) {
                                $repeat = $this->existsUser();
                                if ($repeat) {
                                    $item = $this->retrieveItem($params, $smarty);
                                    $repeat = !empty($item->user_id) &&
                                              $item->user_id == $this->getUserId();
                                }
                            }
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotDemoOrAdminOrOwner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifDemoOrAdminOrOwner($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель с известным IP-адресом.                |
                    |                                                             |
                    ============================================================ */

                public function ifVisitorIp ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $value = $this->cms->security->getVisitorIp();
                        $value = $this->cms->text->lowerCase($value);
                        $items = $this->asUniqueStringArray($value);
                        $repeat = $this->isActualString($value);
                        if ($repeat) {
                            $except = $this->paramAsArray('except', $params);
                            foreach ($except as $item) {
                                if ($this->isActualString($item)) {
                                    $item = trim($item);
                                    if (strpos($item, '*') !== FALSE) {
                                        $item = preg_quote($item, '/');
                                        $pattern = '/^' . str_replace('\\*', '[0-9]{1,3}', $item) . '$/';
                                        foreach ($items as $value) {
                                            $repeat = !preg_match($pattern, $value);
                                            if (!$repeat) break;
                                        }
                                    } else {
                                        $repeat = !in_array($item, $items);
                                    }
                                    if (!$repeat) break;
                                }
                            }
                        }
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotVisitorIp ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifVisitorIp($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель с известным хостом.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifVisitorHost                                |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifVisitorHost}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotVisitorHost} ... {/ifNotVisitorHost}    |
                    |                                                             |
                    ============================================================ */

                public function ifVisitorHost ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $value = $this->cms->security->getVisitorHost();
                        $repeat = $this->isActualString($value);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotVisitorHost ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifVisitorHost($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель, зашедший на сайт через прокси-сервер. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifViaProxy                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifViaProxy}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotViaProxy} ... {/ifNotViaProxy}          |
                    |                                                             |
                    ============================================================ */

                public function ifViaProxy ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $value = $this->cms->security->getVisitorIp();
                        $repeat = strpos($value, ',') !== FALSE;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotViaProxy ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifViaProxy($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель с информацией откуда пришел.           |
                    |                                                             |
                    ============================================================ */

                public function ifReferer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $maybeSameSite = $this->paramAsBoolean('full', $params);
                        $value = $this->getHttpUserReferer($maybeSameSite);
                        $repeat = $this->isActualString($value);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotReferer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifReferer($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть GET-параметры в данных откуда пришел посетитель. |
                    |                                                             |
                    ============================================================ */

                    public function ifRefererQuery ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $maybeSameSite = $this->paramAsBoolean('full', $params);
                            $value = $this->getHttpUserReferer($maybeSameSite);
                            $repeat = $this->getHttpUserReferer($maybeSameSite, FALSE);
                            $repeat = $value != $repeat;
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotRefererQuery ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifRefererQuery($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель известно с какого домена пришел.       |
                    |                                                             |
                    ============================================================ */

                public function ifRefererDomain ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $maybeSameSite = $this->paramAsBoolean('full', $params);
                        $value = $this->getHttpUserReferer($maybeSameSite);
                        $value = $this->getUrlDomain($value);
                        $repeat = $this->isActualString($value);
                        if ($repeat) {
                            $among = $this->paramAsArray('among', $params);
                            foreach ($among as $item) {
                                if ($this->isActualString($item)) {
                                    $item = trim($item);
                                    $repeat = FALSE;
                                    $item2 = ltrim($item, '*');
                                    $item3 = rtrim($item2, '.');
                                    if ($item2 == '') $repeat = TRUE;
                                    else if ($item != $item3) {
                                        $item = $this->cms->text->lowerCase($item3);
                                        $item = preg_quote($item, '/');
                                        if ($item2 != $item3) $item .= '\.(ru|by|kz|ua|[a-z][a-z]|com\.ua|com)';
                                        $pattern = '/^(' . $this->regexpDomainChars(FALSE) . '+\.)*' . $item . '$/ui';
                                        $repeat = preg_match($pattern, $value);
                                    } else {
                                        $item = $this->cms->text->lowerCase($item);
                                        $repeat = $item == $value;
                                    }
                                    if ($repeat) break;
                                }
                            }
                        }
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotRefererDomain ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifRefererDomain($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель пришел с какого-то поиска.             |
                    |                                                             |
                    ============================================================ */

                public function ifRefererSERP ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $among = array(
                            '*android.googlequicksearchbox',    /* звездочка в начале = может быть поддомен        */
                            '*google...',                       /* точка(и) в конце = какой-то из языковых доменов */
                            '*yandex...',
                            '*rambler.ru',
                            '*mail.ru',
                            '*nigma.ru',
                            '*sputnik.ru',
                            '*meta.ua',
                            '*kaz.kz',
                            '*yahoo.net',
                            '*msn.com',
                            '*live.com',
                            '*bing.com',
                            '*ask.com',
                            '*aol.com',
                            '*baidu.com',
                            'away.vk.com',
                            '*facebook.com',
                            '*twitter.com',
                            'plus.google.com',
                            '*ok.ru'
                        );
                        $this->setParam('full', $params, FALSE);
                        $this->setParam('among', $params, $among);
                        return $this->ifRefererDomain($params, $content, $smarty, $repeat);
                    }
                    return $content;
                }

                    public function ifNotRefererSERP ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifRefererSERP($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель пришел с поиска Google.                |
                    |                                                             |
                    ============================================================ */

                public function ifRefererGoogle ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $among = array(
                            '*googlequicksearchbox',    /* звездочка в начале = может быть поддомен        */
                            'google...'                 /* точка(и) в конце = какой-то из языковых доменов */
                        );
                        $this->setParam('full', $params, FALSE);
                        $this->setParam('among', $params, $among);
                        return $this->ifRefererDomain($params, $content, $smarty, $repeat);
                    }
                    return $content;
                }

                    public function ifNotRefererGoogle ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifRefererGoogle($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это посетитель пришел с поиска Яндекс.                |
                    |                                                             |
                    ============================================================ */

                public function ifRefererYandex ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $among = array(
                            'yandex...'    /* точка(и) в конце = какой-то из языковых доменов */
                        );
                        $this->setParam('full', $params, FALSE);
                        $this->setParam('among', $params, $among);
                        return $this->ifRefererDomain($params, $content, $smarty, $repeat);
                    }
                    return $content;
                }

                    public function ifNotRefererYandex ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifRefererYandex($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если в запросе страницы присутствуют REQUEST-параметры.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifQueryRequest                               |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifQueryRequest}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotQueryRequest} ... {/ifNotQueryRequest}  |
                    |                                                             |
                    ============================================================ */

                public function ifQueryRequest ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_REQUEST', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotQueryRequest ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifQueryRequest($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если в запросе страницы присутствуют GET-параметры.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifQueryGet                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifQueryGet}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotQueryGet} ... {/ifNotQueryGet}          |
                    |                                                             |
                    ============================================================ */

                public function ifQueryGet ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_GET', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotQueryGet ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifQueryGet($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если в запросе страницы присутствуют POST-параметры.       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifQueryPost                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifQueryPost}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotQueryPost} ... {/ifNotQueryPost}        |
                    |                                                             |
                    ============================================================ */

                public function ifQueryPost ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_POST', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotQueryPost ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifQueryPost($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если в запросе страницы присутствуют FILES-параметры.      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifQueryFiles                                 |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifQueryFiles}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotQueryFiles} ... {/ifNotQueryFiles}      |
                    |                                                             |
                    ============================================================ */

                public function ifQueryFiles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_FILES', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotQueryFiles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifQueryFiles($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если в запросе страницы присутствуют COOKIE-параметры.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCookies                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCookies}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCookies} ... {/ifNotCookies}            |
                    |                                                             |
                    ============================================================ */

                public function ifCookies ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_COOKIE', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotCookies ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifCookies($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если присутствуют SESSION-параметры.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSession                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifSession}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSession} ... {/ifNotSession}            |
                    |                                                             |
                    ============================================================ */

                public function ifSession ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_SESSION', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotSession ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifSession($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если присутствуют SERVER-параметры.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifServer                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifServer}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotServer} ... {/ifNotServer}              |
                    |                                                             |
                    ============================================================ */

                public function ifServer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsQueryParams('_SERVER', $params);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotServer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifServer($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если это N-кратная итерация enum-функции.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumFirst                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifEnumFirst}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumAfter} ... {/ifEnumAfter}              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumEven} ... {/ifEnumEven}                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumOdd} ... {/ifEnumOdd}                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumTriple} ... {/ifEnumTriple}            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumQuart} ... {/ifEnumQuart}              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifEnumQuint} ... {/ifEnumQuint}              |
                    |                                                             |
                    ============================================================ */

            public function ifEnumFirst ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->getEnumNum() == 1;
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifEnumAfter ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifEnumFirst($params, $content, $smarty, $repeat);
                }

                public function ifEnumEven ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getEnumNum();
                        $repeat = $repeat > 0 && $repeat % 2 == 0;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                public function ifEnumOdd ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifEnumEven($params, $content, $smarty, $repeat);
                }

                public function ifEnumTriple ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getEnumNum();
                        $repeat = $repeat > 0 && $repeat % 3 == 0;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                public function ifEnumQuart ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getEnumNum();
                        $repeat = $repeat > 0 && $repeat % 4 == 0;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                public function ifEnumQuint ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getEnumNum();
                        $repeat = $repeat > 0 && $repeat % 5 == 0;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    /* ============================================================
                    |                                                             |
                    |  Если у этой категории включен флаг своего блока на главной.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOwnBlock                                   |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifOwnBlock}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOwnBlock} ... {/ifNotOwnBlock}          |
                    |                                                             |
                    ============================================================ */

            public function ifOwnBlock ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->retrieveItem($params, $smarty);
                    $repeat = !empty($item->own_block);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                    public function ifNotOwnBlock ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOwnBlock($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если эту категорию/бренд или свойство/значение можно       |
                    |  показывать.                                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifActual                                     |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifActual}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotActual} ... {/ifNotActual}              |
                    |                                                             |
                    ============================================================ */

            public function ifActual ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->retrieveItem($params, $smarty);
                    if (!empty($item->actual)) $repeat = TRUE;
                    elseif (isset($item->property_id)) {
                        foreach ($item->options as $option) {
                            $repeat = !empty($option->actual);
                            if ($repeat) break;
                        }
                    } else $repeat = $this->actualCategory($item);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                    public function ifNotActual ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifActual($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если эта запись разрешена к обсуждению.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCommented                                  |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifCommented}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCommented} ... {/ifNotCommented}        |
                    |                                                             |
                    ============================================================ */

            public function ifCommented ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->retrieveItem($params, $smarty);
                    $repeat = !empty($item->commented);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                    public function ifNotCommented ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifCommented($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны комментарии записи.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifComments                                   |
                    |                   [from = VarName]  где ЕЩЕ МОЖЕТ храниться |
                    |                                     запись (по умолч. $item)|
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifComments}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotComments} ... {/ifNotComments}          |
                    |                                                             |
                    ============================================================ */

            public function ifComments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('comments', $smarty);
                    if (!$repeat) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->comments);
                    }
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotComments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifComments($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны записи.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifItems                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifItems}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotItems} ... {/ifNotItems}                |
                    |                                                             |
                    ============================================================ */

            public function ifItems ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('items', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotItems ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifItems($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна запись.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifItem                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifItem}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotItem} ... {/ifNotItem}                  |
                    |                                                             |
                    ============================================================ */

                public function ifItem ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('item', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotItem ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifItem($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна непустая переменная value.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifValue                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifValue}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotValue} ... {/ifNotValue}                |
                    |                                                             |
                    ============================================================ */

                public function ifValue ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('value', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotValue ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifValue($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен класс тега HTML.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHtmlClass                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifHtmlClass}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotHtmlClass} ... {/ifNotHtmlClass}        |
                    |                                                             |
                    ============================================================ */

                public function ifHtmlClass ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('htmlClass', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotHtmlClass ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHtmlClass($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно название дочернего класса.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifChildClass                                 |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifChildClass}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotChildClass} ... {/ifNotChildClass}      |
                    |                                                             |
                    ============================================================ */

                    public function ifChildClass ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsVar('childClass', $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotChildClass ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifChildClass($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен URL для некоторого HTML.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHtmlUrl                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifHtmlUrl}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotHtmlUrl} ... {/ifNotHtmlUrl}            |
                    |                                                             |
                    ============================================================ */

                    public function ifHtmlUrl ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsVar('htmlUrl', $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotHtmlUrl ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifHtmlUrl($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен URL скриншота для OpenGraph разметки.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOpenGraphImage                             |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOpenGraphImage}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOpenGraphImage}                         |
                    |               ... {/ifNotOpenGraphImage}                    |
                    |                                                             |
                    ============================================================ */

                    public function ifOpenGraphImage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsVar('openGraphImage', $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotOpenGraphImage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifOpenGraphImage($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна непустая переменная type.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifType                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifType}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotType} ... {/ifNotType}                  |
                    |                                                             |
                    ============================================================ */

                public function ifType ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('type', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotType ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifType($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если переменная sum больше нуля.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSum                                        |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifSum}                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSum} ... {/ifNotSum}                    |
                    |                                                             |
                    ============================================================ */

                public function ifSum ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getFloat('sum', $smarty) > 0.0;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotSum ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifSum($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна специальная страница.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSection                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifSection}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSection} ... {/ifNotSection}            |
                    |                                                             |
                    ============================================================ */

                public function ifSection ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('section', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotSection ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifSection($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна категория.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCategory                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCategory}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCategory} ... {/ifNotCategory}          |
                    |                                                             |
                    ============================================================ */

                public function ifCategory ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('category', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotCategory ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifCategory($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен бренд.                                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifBrand                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifBrand}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotBrand} ... {/ifNotBrand}                |
                    |                                                             |
                    ============================================================ */

                public function ifBrand ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('brand', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotBrand ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifBrand($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен бренд или категория.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifBrandOrCategory                            |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifBrandOrCategory}                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotBrandOrCategory}                        |
                    |               ... {/ifNotBrandOrCategory}                   |
                    |                                                             |
                    ============================================================ */

                    public function ifBrandOrCategory ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsVar(array('category', 'brand'), $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotBrandOrCategory ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifBrandOrCategory($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если категория/бренд является информативной страницей.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifInformative                                |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifInformative}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotInformative} ... {/ifNotInformative}    |
                    |                                                             |
                    ============================================================ */

                    public function ifInformative ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $item = $this->getVar('category', $smarty);
                            $repeat = !empty($item->informative);
                            if (!$repeat) {
                                $item = $this->getVar('brand', $smarty);
                                $repeat = !empty($item->informative);
                            }
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotInformative ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifInformative($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен товар.                                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifProduct                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifProduct}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotProduct} ... {/ifNotProduct}            |
                    |                                                             |
                    ============================================================ */

                public function ifProduct ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('product', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotProduct ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifProduct($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен непустой массив свойств товара.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSpecifications                             |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item или |
                    |                                     затем $product)         |
                    |                   [not = TRUE]      если проверить на       |
                    |                                     отрицание               |
                    |               } ... {/ifSpecifications}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSpecifications}                         |
                    |               ... {/ifNotSpecifications}                    |
                    |                                                             |
                    ============================================================ */

                    public function ifSpecifications ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = FALSE;
                            $field = '->properties_tree';
                            $from = $this->paramAsSentence('from', $params);
                            if (empty($from)) {
                                $from = 'item';
                                $repeat = $this->existsVar($from . $field, $smarty);
                                if (!$repeat) $from = 'product';
                            }
                            $repeat = $repeat || $this->existsVar($from . $field, $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotSpecifications ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifSpecifications($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна новость.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNewsItem                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifNewsItem}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotNewsItem} ... {/ifNotNewsItem}          |
                    |                                                             |
                    ============================================================ */

                public function ifNewsItem ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('news_item', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotNewsItem ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifNewsItem($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна статья.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifArticle                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifArticle}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotArticle} ... {/ifNotArticle}            |
                    |                                                             |
                    ============================================================ */

                public function ifArticle ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('article', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotArticle ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifArticle($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен склад.                                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifStock                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifStock}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotStock} ... {/ifNotStock}                |
                    |                                                             |
                    ============================================================ */

                public function ifStock ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('stock', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotStock ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifStock($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна страна.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCountry                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCountry}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCountry} ... {/ifNotCountry}            |
                    |                                                             |
                    ============================================================ */

                public function ifCountry ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('country', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotCountry ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifCountry($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна область.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifRegion                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifRegion}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotRegion} ... {/ifNotRegion}              |
                    |                                                             |
                    ============================================================ */

                    public function ifRegion ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsVar('region', $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotRegion ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifRegion($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен город.                                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifTown                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifTown}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotTown} ... {/ifNotTown}                  |
                    |                                                             |
                    ============================================================ */

                    public function ifTown ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->existsVar('town', $smarty);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotTown ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifTown($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен список заказов.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOrders                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOrders}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOrders} ... {/ifNotOrders}              |
                    |                                                             |
                    ============================================================ */

                public function ifOrders ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('orders', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotOrders ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOrders($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен заказ.                                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOrder                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOrder}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOrder} ... {/ifNotOrder}                |
                    |                                                             |
                    ============================================================ */

                    public function ifOrder ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->accessibleOrder();
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotOrder ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifOrder($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если указанный заказ оплачен.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPaid                                       |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $order)   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPaid}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPaid} ... {/ifNotPaid}                  |
                    |                                                             |
                    ============================================================ */

                    public function ifPaid ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $varname = $this->retrieveFrom($params, 'order');
                            $item = $this->getVar($varname, $smarty);
                            $repeat = !empty($item->payment_status);
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotPaid ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifPaid($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна запись в переменной post.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPost                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPost}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPost} ... {/ifNotPost}                  |
                    |                                                             |
                    ============================================================ */

                public function ifPost ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('post', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotPost ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifPost($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны специальные страницы.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSections                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifSections}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSections} ... {/ifNotSections}          |
                    |                                                             |
                    ============================================================ */

            public function ifSections ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('sections', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotSections ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifSections($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны категории.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCategories                                 |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCategories}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCategories} ... {/ifNotCategories}      |
                    |                                                             |
                    ============================================================ */

            public function ifCategories ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('categories', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotCategories ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifCategories($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны бренды.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifBrands                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifBrands}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotBrands} ... {/ifNotBrands}              |
                    |                                                             |
                    ============================================================ */

            public function ifBrands ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('all_brands', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotBrands ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifBrands($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны товары.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifProducts                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifProducts}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotProducts} ... {/ifNotProducts}          |
                    |                                                             |
                    ============================================================ */

            public function ifProducts ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('products', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotProducts ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifProducts($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны товары или анонсовые публикации.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifProductsOrAnnounces                        |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifProductsOrAnnounces}                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotProductsOrAnnounces} ...                |
                    |               {/ifNotProductsOrAnnounces}                   |
                    |                                                             |
                    ============================================================ */

                public function ifProductsOrAnnounces ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar(array('products',
                                                         'announces',
                                                         'news',
                                                         'news_for_category',
                                                         'news_for_product',
                                                         'articles',
                                                         'articles_for_category',
                                                         'articles_for_product'), $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotProductsOrAnnounces ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifProductsOrAnnounces($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны особые товары.                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifFeatureds                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifFeatureds}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotFeatureds} ... {/ifNotFeatureds}        |
                    |                                                             |
                    ============================================================ */

                public function ifFeatureds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = !empty($smarty)
                               && !empty($smarty->tpl_vars['products'])
                               && isset($smarty->tpl_vars['products']->value);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotFeatureds ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifFeatureds($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны искомые товары.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifSearch                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifSearch}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotSearch} ... {/ifNotSearch}              |
                    |                                                             |
                    ============================================================ */

                public function ifSearch ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('search_products', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotSearch ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifSearch($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна хотя бы 1 фотография товара.                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPhoto                                      |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifPhoto}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPhoto} ... {/ifNotPhoto}                |
                    |                                                             |
                    ============================================================ */

                public function ifPhoto ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsFoto($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotPhoto ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifPhoto($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно более 1 фотографии товара.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPhotos                                     |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifPhotos}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPhotos} ... {/ifNotPhotos}              |
                    |                                                             |
                    ============================================================ */

                public function ifPhotos ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsFotos($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotPhotos ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifPhotos($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступна хотя бы 1 фотография этой записи.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifImage                                      |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifImage}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotImage} ... {/ifNotImage}                |
                    |                                                             |
                    ============================================================ */

                public function ifImage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsSnapshot($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotImage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifImage($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно более 1 фотографии этой записи.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifImages                                     |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifImages}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotImages} ... {/ifNotImages}              |
                    |                                                             |
                    ============================================================ */

                public function ifImages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsSnapshots($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotImages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifImages($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен хотя бы 1 прикрепленный файл этой записи.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAttachment                                 |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifAttachment}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAttachment} ... {/ifNotAttachment}      |
                    |                                                             |
                    ============================================================ */

                public function ifAttachment ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsAttachment($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotAttachment ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifAttachment($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно более 1 прикрепленного файла этой записи.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAttachments                                |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifAttachments}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAttachments} ... {/ifNotAttachments}    |
                    |                                                             |
                    ============================================================ */

                public function ifAttachments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsAttachments($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotAttachments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifAttachments($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно более 1 варианта товара.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifVariants                                   |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifVariants}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotVariants} ... {/ifNotVariants}          |
                    |                                                             |
                    ============================================================ */

                public function ifVariants ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = $this->existsVariants($item);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotVariants ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifVariants($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если указана не нулевая цена варианта товара.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPrice                                      |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $variant) |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifPrice}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPrice} ... {/ifNotPrice}                |
                    |                                                             |
                    ============================================================ */

                    public function ifPrice ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $from = $this->paramAsSentence('from', $params, 'variant');
                            $value = $this->getPrice(array($from . '->final_price',
                                                           $from . '->discount_price',
                                                           $from . '->price'), $smarty);
                            $repeat = $value > 0;
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotPrice ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifPrice($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны новости.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNews                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifNews}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotNews} ... {/ifNotNews}                  |
                    |                                                             |
                    ============================================================ */

            public function ifNews ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('news', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotNews ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifNews($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны статьи.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifArticles                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifArticles}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotArticles} ... {/ifNotArticles}          |
                    |                                                             |
                    ============================================================ */

            public function ifArticles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('articles', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotArticles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifArticles($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны новости или статьи.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNewsOrArticles                             |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifNewsOrArticles}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotNewsOrArticles}                         |
                    |               ... {/ifNotNewsOrArticles}                    |
                    |                                                             |
                    ============================================================ */

                public function ifNewsOrArticles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar(array('news', 'articles'), $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotNewsOrArticles ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifNewsOrArticles($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны склады.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifStocks                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifStocks}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotStocks} ... {/ifNotStocks}              |
                    |                                                             |
                    ============================================================ */

            public function ifStocks ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('stocks', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotStocks ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifStocks($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны страны.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCountries                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCountries}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCountries} ... {/ifNotCountries}        |
                    |                                                             |
                    ============================================================ */

            public function ifCountries ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('countries', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotCountries ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifCountries($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны области.                                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifRegions                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifRegions}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotRegions} ... {/ifNotRegions}            |
                    |                                                             |
                    ============================================================ */

                public function ifRegions ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('regions', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotRegions ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifRegions($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны города.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifTowns                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifTowns}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotTowns} ... {/ifNotTowns}                |
                    |                                                             |
                    ============================================================ */

                public function ifTowns ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('towns', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotTowns ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifTowns($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны свойства фильтра.                            |
                    |                                                             |
                    ============================================================ */

            public function ifProperties ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $full = $this->paramAsBoolean('full', $params);
                    $item = $this->getVar('properties');
                    $item2 = $full ? $this->getVar('brands') : TRUE;
                    $item3 = $full ? $this->getVar('variants') : TRUE;
                    $repeat = !empty($item) && !empty($item2) && !empty($item3);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotProperties ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifProperties($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны способы доставки.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifDeliveries                                 |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifDeliveries}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotDeliveries} ... {/ifNotDeliveries}      |
                    |                                                             |
                    ============================================================ */

            public function ifDeliveries ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('delivery_methods', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotDeliveries ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifDeliveries($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если этот способ доставки активный.                        |
                    |                                                             |
                    ============================================================ */

                public function ifActiveDelivery ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $varname = 'delivery_method_id';
                        $id = $this->getInteger($varname, $smarty);
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->$varname)
                                  && ($item->$varname == $id || empty($id) && $this->getEnumNum() == 1);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotActiveDelivery ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifActiveDelivery($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны способы оплаты.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPayments                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPayments}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPayments} ... {/ifNotPayments}          |
                    |                                                             |
                    ============================================================ */

            public function ifPayments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar(array('payments', 'payment_methods'), $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotPayments ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifPayments($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если этот способ оплаты активный.                          |
                    |                                                             |
                    ============================================================ */

                public function ifActivePayment ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $varname = 'payment_method_id';
                        $id = $this->getInteger($varname, $smarty);
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->$varname)
                                  && ($item->$varname == $id || empty($id) && $this->getEnumNum() == 1);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotActivePayment ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifActivePayment($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно более 1 валюты.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCurrencies                                 |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCurrencies}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCurrencies} ... {/ifNotCurrencies}      |
                    |                                                             |
                    ============================================================ */

            public function ifCurrencies ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsCurrencies();
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotCurrencies ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifCurrencies($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны товары сравнения.                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCompare                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCompare}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCompare} ... {/ifNotCompare}            |
                    |                                                             |
                    ============================================================ */

            public function ifCompare ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->getSession('compared_products');
                    $repeat = !empty($repeat) && is_array($repeat);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotCompare ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifCompare($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны товары корзины.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCart                                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCart}                               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCart} ... {/ifNotCart}                  |
                    |                                                             |
                    ============================================================ */

            public function ifCart ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('cart_products', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotCart ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifCart($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если в корзине есть скидка.                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCartDiscount                               |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCartDiscount}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCartDiscount} ... {/ifNotCartDiscount}  |
                    |                                                             |
                    ============================================================ */

                public function ifCartDiscount ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getPrice('cart_total_discount_sum', $smarty);
                        $repeat = $repeat != '0';
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotCartDiscount ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifCartDiscount($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если доступны отложенные товары.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifDefer                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifDefer}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotDefer} ... {/ifNotDefer}                |
                    |                                                             |
                    ============================================================ */

            public function ifDefer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('defer_products', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotDefer ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifDefer($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если определен многоязычный словарь.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasPolyglot                                |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifHasPolyglot}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotPolyglot} ... {/ifHasNotPolyglot}    |
                    |                                                             |
                    ============================================================ */

            public function ifHasPolyglot ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->existsVar('polyglot', $smarty);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifHasNotPolyglot ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifHasPolyglot($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если эта категория/бренд имеет вложенные.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasBranch                                  |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasBranch}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotBranch} ... {/ifHasNotBranch}        |
                    |                                                             |
                    ============================================================ */

            public function ifHasBranch ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->retrieveItem($params, $smarty);
                    $repeat = !empty($item->subcategories) && is_array($item->subcategories)
                              || !empty($item->subbrands) && is_array($item->subbrands);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifHasNotBranch ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifHasBranch($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если эта запись имеет непустой текст краткого описания.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasAnnotation                              |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasAnnotation}                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotAnnotation} ... {/ifHasNotAnnotation}|
                    |                                                             |
                    ============================================================ */

                public function ifHasAnnotation ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->annotation)
                                  || !isset($item->annotation) && (
                                                                       !empty($item->description)
                                                                    || !isset($item->description) && !empty($item->body)
                                                                   );
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifHasNotAnnotation ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHasAnnotation($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если эта запись имеет непустой текст полного описания.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasFullText                                |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasFullText}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotFullText} ... {/ifHasNotFullText}    |
                    |                                                             |
                    ============================================================ */

                public function ifHasFullText ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = isset($item->annotation) && (!empty($item->description) || !empty($item->body))
                                  || !isset($item->annotation) && isset($item->description) && !empty($item->body);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifHasNotFullText ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHasFullText($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если эта запись имеет непустой текст описания.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasBody                                    |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasBody}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotBody} ... {/ifHasNotBody}            |
                    |                                                             |
                    ============================================================ */

                public function ifHasBody ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->body)
                                  || !empty($item->description)
                                  || !empty($item->annotation);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifHasNotBody ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHasBody($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если эта запись имеет непустой SEO-текст.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasSeoText                                 |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasSeoText}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotSeoText} ... {/ifHasNotSeoText}      |
                    |                                                             |
                    ============================================================ */

                public function ifHasSeoText ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->seo_description);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifHasNotSeoText ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHasSeoText($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если эта запись имеет непустой текст субдомена.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasSubdomain                               |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasSubdomain}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotSubdomain} ... {/ifHasNotSubdomain}  |
                    |                                                             |
                    ============================================================ */

                public function ifHasSubdomain ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->subdomain_html);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifHasNotSubdomain ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHasSubdomain($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если этот комментарий имеет вложенные.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasDiscussion                              |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifHasDiscussion}                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifHasNotDiscussion} ... {/ifHasNotDiscussion}|
                    |                                                             |
                    ============================================================ */

                public function ifHasDiscussion ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $item = $this->retrieveItem($params, $smarty);
                        $repeat = !empty($item->discussion) && is_array($item->discussion);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifHasNotDiscussion ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifHasDiscussion($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть более 1 листаемой страницы.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPages                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPages}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPages} ... {/ifNotPages}                |
                    |                                                             |
                    ============================================================ */

            public function ifPages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->getVar('Pages', $smarty);
                    $repeat = !empty($item) && is_array($item) && count($item) > 1;
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotPages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifPages($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если есть предыдущая листаемая страница.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPrevPage                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPrevPage}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPrevPage} ... {/ifNotPrevPage}          |
                    |                                                             |
                    ============================================================ */

                public function ifPrevPage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('PrevPageUrl', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotPrevPage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifPrevPage($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть следующая листаемая страница.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNextPage                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifNextPage}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotNextPage} ... {/ifNotNextPage}          |
                    |                                                             |
                    ============================================================ */

                public function ifNextPage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('NextPageUrl', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotNextPage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifNextPage($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть есть ссылка на список ВСЕ НА ОДНОЙ СТРАНИЦЕ.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAllPage                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifAllPage}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAllPage} ... {/ifNotAllPage}            |
                    |                                                             |
                    ============================================================ */

                public function ifAllPage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->existsVar('AllOnPageUrl', $smarty);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotAllPage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifAllPage($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если выбран показ всех записей списка на одной странице.   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifShowAll                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifShowAll}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotShowAll} ... {/ifNotShowAll}            |
                    |                                                             |
                    ============================================================ */

                public function ifShowAll ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->cms->request->getGPrequest('show_all') !== FALSE;
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotShowAll ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifShowAll($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если находимся на каноническом адресе.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCanonical                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCanonical}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCanonical} ... {/ifNotCanonical}        |
                    |                                                             |
                    ============================================================ */

                public function ifCanonical ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getCanonicalUri() == $this->originalUri();
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotCanonical ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifCanonical($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на странице 404.                                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPage404                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPage404}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPage404} ... {/ifNotPage404}            |
                    |                                                             |
                    ============================================================ */

            public function ifPage404 ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = $this->checkNoPage();
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotPage404 ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifPage404($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если на абстрактной странице.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPageAbstract                               |
                    |                   [local = TRUE]  если это локальный тест   |
                    |                                   (внутримодульный) для     |
                    |                                   проверки что модуль был   |
                    |                                   вызван с абстрактной      |
                    |                                   страницы                  |
                    |                   [not = TRUE]    если проверить на         |
                    |                                   отрицание                 |
                    |               } ... {/ifPageAbstract}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPageAbstract} ... {/ifNotPageAbstract}  |
                    |                                                             |
                    ============================================================ */

                public function ifPageAbstract ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $file = $this->abstractTpl();
                        $repeat = $this->paramAsBoolean('local', $params);
                        $repeat = ($repeat || $this->checkNoPage())
                               && $this->existsModule($file);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotPageAbstract ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifPageAbstract($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на отключенной странице.                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifPageOff                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifPageOff}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotPageOff} ... {/ifNotPageOff}            |
                    |                                                             |
                    ============================================================ */

                public function ifPageOff ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->checkNoModule() || $this->checkNoService();
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotPageOff ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifPageOff($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на странице Категории.                                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOnCategories                               |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOnCategories}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOnCategories} ... {/ifNotOnCategories}  |
                    |                                                             |
                    ============================================================ */

                public function ifOnCategories ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getRootUri() == '/categories';
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotOnCategories ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOnCategories($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на странице Бренды.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOnBrands                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOnBrands}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOnBrands} ... {/ifNotOnBrands}          |
                    |                                                             |
                    ============================================================ */

                public function ifOnBrands ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getRootUri() == '/brands';
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotOnBrands ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOnBrands($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на странице Товары.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOnProducts                                 |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOnProducts}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOnProducts} ... {/ifNotOnProducts}      |
                    |                                                             |
                    ============================================================ */

                public function ifOnProducts ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getModuleName() == 'products';
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotOnProducts ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOnProducts($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если на странице Новости.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifOnNews                                     |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifOnNews}                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotOnNews} ... {/ifNotOnNews}              |
                    |                                                             |
                    ============================================================ */

                public function ifOnNews ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getModuleName() == 'news';
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotOnNews ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifOnNews($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если форма ввода принята успешно.                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAccepted                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifAccepted}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAccepted} ... {/ifNotAccepted}          |
                    |                                                             |
                    ============================================================ */

            public function ifAccepted ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $tpl = $this->getModuleOrTemplateName($smarty);
                    switch ($tpl) {
                        case 'feedback':     $name = 'feedback_accepted'; break;
                        case 'login':        $name = 'login_accepted'; break;
                        case 'registration': $name = 'registration_accepted'; break;
                        case 'callme':       $name = 'callme_accepted'; break;
                        default:             $name = 'message';
                    }
                    $repeat = $this->getVar($name, $smarty);
                    $repeat = !empty($repeat);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotAccepted ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifAccepted($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если включен флажок СОГЛАСЕН в форме ввода.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifAgree                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifAgree}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotAgree} ... {/ifNotAgree}                |
                    |                                                             |
                    ============================================================ */

                public function ifAgree ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->requestParam('agree');
                        $repeat = !empty($repeat);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotAgree ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifAgree($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть сообщение об ошибке.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifError                                      |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifError}                              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotError} ... {/ifNotError}                |
                    |                                                             |
                    ============================================================ */

                public function ifError ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $tpl = $this->getModuleOrTemplateName($smarty);
                        switch ($tpl) {
                            case 'feedback':     $name = 'feedback_error'; break;
                            case 'login':        $name = 'login_error'; break;
                            case 'registration': $name = 'registration_error'; break;
                            case 'callme':       $name = 'callme_error'; break;
                            case 'cart':
                            default:             $name = 'error';
                        }
                        $repeat = $this->getVar($name, $smarty);
                        $repeat = !empty($repeat);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotError ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifError($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть сообщение о новом пароле.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNewPassword                                |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifNewPassword}                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotNewPassword} ... {/ifNotNewPassword}    |
                    |                                                             |
                    ============================================================ */

                public function ifNewPassword ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->getVar('new_password', $smarty);
                        $repeat = !empty($repeat);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotNewPassword ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifNewPassword($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть сообщение об успехе.                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifMessage                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifMessage}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotMessage} ... {/ifNotMessage}            |
                    |                                                             |
                    ============================================================ */

                public function ifMessage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $tpl = $this->getModuleOrTemplateName($smarty);
                        switch ($tpl) {
                            case 'feedback':     $name = 'feedback_accepted'; break;
                            case 'login':        $name = 'login_message'; break;
                            case 'registration': $name = 'registration_message'; break;
                            case 'callme':       $name = 'callme_message'; break;
                            case 'cart':
                            default:             $name = 'message';
                        }
                        $repeat = $this->getVar($name, $smarty);
                        $repeat = !empty($repeat);
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotMessage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifMessage($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если есть сообщение об успехе или новом пароле.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifMessageOrNewPassword                       |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifMessageOrNewPassword}               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotMessageOrNewPassword}                   |
                    |               ... {/ifNotMessageOrNewPassword}              |
                    |                                                             |
                    ============================================================ */

                    public function ifMessageOrNewPassword ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = $this->getVar('new_password', $smarty);
                            $repeat = $repeat !== null;    // здесь именно такая проверка,
                                                           // а не проверка !empty($repeat)
                                                           // как сделана в ifNewPassword
                            if (!$repeat) {
                                $tpl = $this->getModuleOrTemplateName($smarty);
                                switch ($tpl) {
                                    case 'feedback':     $name = 'feedback_accepted'; break;
                                    case 'login':        $name = 'login_message'; break;
                                    case 'registration': $name = 'registration_message'; break;
                                    case 'callme':       $name = 'callme_message'; break;
                                    case 'cart':
                                    default:             $name = 'message';
                                }
                                $repeat = $this->getVar($name, $smarty);
                                $repeat = !empty($repeat);
                            }
                            if ($this->existsParamNOT($params)) $repeat = !$repeat;
                            return '';
                        }
                        return $content;
                    }

                        public function ifNotMessageOrNewPassword ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                            $this->setParam('not', $params, TRUE);
                            return $this->ifMessageOrNewPassword($params, $content, $smarty, $repeat);
                        }

                    /* ============================================================
                    |                                                             |
                    |  Если капча для этой формы включена.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifCaptcha                                    |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifCaptcha}                            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotCaptcha} ... {/ifNotCaptcha}            |
                    |                                                             |
                    ============================================================ */

            public function ifCaptcha ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $inverse = TRUE;
                    $tpl = $this->getModuleOrTemplateName($smarty);
                    switch ($tpl) {
                        case 'feedback':     $name = 'feedback_captcha_disabled'; break;
                        case 'login':        $name = 'login_captcha_disabled'; break;
                        case 'registration': $name = 'registration_captcha_disabled'; break;
                        case 'callme':       $name = 'callme_captcha_disabled'; break;
                        case 'cart':         $name = 'cart_captcha_protecting'; $inverse = FALSE; break;
                        default:             $name = '';
                    }
                    $repeat = $this->cms->any->settings->getAsBoolean($name);
                    $repeat = $inverse ? !$repeat : $repeat;
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotCaptcha ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifCaptcha($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если запись прикреплена к текущему макету шаблона.         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifReferredToMe                               |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifReferredToMe}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotReferredToMe} ... {/ifNotReferredToMe}  |
                    |                                                             |
                    ============================================================ */

            public function ifReferredToMe ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->retrieveItem($params, $smarty);
                    $repeat = !empty($item->template)
                           && !empty($smarty->template_resource)
                           && $item->template == $smarty->template_resource;
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotReferredToMe ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifReferredToMe($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если находимся в демо режиме или запись прикреплена к      |
                    |  текущему макету шаблона.                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifDemoOrReferredToMe                         |
                    |                   [from = VarName]  где хранится запись     |
                    |                                     (по умолчанию $item)    |
                    |                   [not = TRUE]      если проверить          |
                    |                                     на отрицание            |
                    |               } ... {/ifDemoOrReferredToMe}                 |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotDemoOrReferredToMe}                     |
                    |               ... {/ifNotDemoOrReferredToMe}                |
                    |                                                             |
                    ============================================================ */

                public function ifDemoOrReferredToMe ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = $this->cms->security->inDemoMode();
                        if (!$repeat) {
                            $item = $this->retrieveItem($params, $smarty);
                            $repeat = !empty($item->template)
                                   && !empty($smarty->template_resource)
                                   && $item->template == $smarty->template_resource;
                        }
                        if ($this->existsParamNOT($params)) $repeat = !$repeat;
                        return '';
                    }
                    return $content;
                }

                    public function ifNotDemoOrReferredToMe ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        $this->setParam('not', $params, TRUE);
                        return $this->ifDemoOrReferredToMe($params, $content, $smarty, $repeat);
                    }

                    /* ============================================================
                    |                                                             |
                    |  Если установлен такой флаг в настройках шаблона.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifTemplateFlag                               |
                    |                   [name = FlagName]  имя параметра настроек |
                    |                   [not = TRUE]       если проверить на      |
                    |                                      отрицание              |
                    |               } ... {/ifTemplateFlag}                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotTemplateFlag name=...}                  |
                    |               ... {/ifNotTemplateFlag}                      |
                    |                                                             |
                    ============================================================ */

            public function ifTemplateFlag ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $field = $this->paramAsSentence('name', $params);
                    $item = $this->getVar('tplSettings', $smarty);
                    $repeat = !empty($field) && !empty($item[$field]);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotTemplateFlag ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifTemplateFlag($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если установлен такой флаг в настройках шаблона.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifFirstRendering                             |
                    |                   [not = TRUE]       если проверить на      |
                    |                                      отрицание              |
                    |               } ... {/ifFirstRendering}                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotFirstRendering}                         |
                    |               ... {/ifNotFirstRendering}                    |
                    |                                                             |
                    ============================================================ */

            public function ifFirstRendering ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    $ptr = & $smarty;
                    while ($repeat && isset($ptr->template_resource)) {
                        if (!$this->isActualString($ptr->template_resource)) break;
                        $file = trim($ptr->template_resource);
                        $file = $this->cms->text->lowerCase($file);
                        $repeat = $file != 'index.tpl' &&
                                  $file != 'index.htm';
                        if ($repeat) {
                            if (!isset($ptr->parent)) break;
                            $ptr = & $ptr->parent;
                        }
                    }
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotFirstRendering ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifFirstRendering($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступен хотя бы 1 язык сайта.                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifLanguage                                   |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifLanguage}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotLanguage} ... {/ifNotLanguage}          |
                    |                                                             |
                    ============================================================ */

            public function ifLanguage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->getVar('languages', $smarty);
                    $repeat = !empty($item) && is_array($item);
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotLanguage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifLanguage($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Если доступно более 1 языка сайта.                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifLanguages                                  |
                    |                   [not = TRUE]  если проверить на отрицание |
                    |               } ... {/ifLanguages}                          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {ifNotLanguages} ... {/ifNotLanguages}        |
                    |                                                             |
                    ============================================================ */

            public function ifLanguages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $item = $this->getVar('languages', $smarty);
                    $repeat = !empty($item) && is_array($item) && count($item) > 1;
                    if ($this->existsParamNOT($params)) $repeat = !$repeat;
                    return '';
                }
                return $content;
            }

                public function ifNotLanguages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    $this->setParam('not', $params, TRUE);
                    return $this->ifLanguages($params, $content, $smarty, $repeat);
                }

                    /* ============================================================
                    |                                                             |
                    |  Разложить HTML-текст по H-заголовкам в массив HTML-блоков. |
                    |  ---------------------------------------------------------  |
                    |  применение:  {splitBody                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         $items)             |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                   <p>Block 1 begins here</p>                |
                    |                       <p>...</p>                            |
                    |                       ...                                   |
                    |                       <div>Block 1 ends here</div>          |
                    |                   <h5>Block 2 begins here</h5>              |
                    |                       <div>...</div>                        |
                    |                       ...                                   |
                    |                       <center>Block 2 ends here</center>    |
                    |                   <h2>Block 3 begins here</h2>              |
                    |                       <figure>...</figure>                  |
                    |                       ...                                   |
                    |                       <p>Block 3 ends here</p>              |
                    |               {/splitBody}                                  |
                    |                                                             |
                    ============================================================ */

        public function splitBody ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) $repeat = TRUE;
            else {
                $this->mustbeParam('assign', $params, 'items');
                $pattern = '<h([1-6])(>|\s[^>]*>).+?';
                $content = $this->mustbeString($content);
                $content = preg_split('~\s*(' . $pattern . '</h\2>)\s*~uis', $content, -1, PREG_SPLIT_DELIM_CAPTURE);
                $items = array();
                $block = '';
                $skipNext = 0;
                foreach ($content as & $item) {
                    if ($skipNext > 0) {
                        $skipNext--;
                        continue;
                    }
                    if ($this->isActualString($item)) {
                        if (preg_match('~^' . $pattern . '</h\1>$~uis', $item)) {
                            if ($this->isActualString($block)) $items[] = $block;
                            $block = $item;
                            $skipNext = 2;
                        } else {
                            $item = preg_replace('~^\s*([^\s<][^<]*)~ui', '<p>$1</p>', $item);
                            $block .= $item;
                        }
                    }
                }
                if ($this->isActualString($block)) $items[] = $block;
                $this->assignVar($params, $items, $smarty);
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Установить новый суффикс модуля.                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setModSuffix} ... {/setModSuffix}            |
                    |                                                             |
                    ============================================================ */

        public function setModSuffix ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) $repeat = TRUE;
            else {
                $content = $this->mustbeString($content);
                $content = $this->cms->hdd->safeFilename($content, FALSE, 128);
                if ($this->isActualString($content)) {
                    $content = preg_replace('/[^a-z0-9]+/', '-', '-' . $content);
                    $content = $this->cms->text->lowerCase($content);
                    $content = rtrim($content, '-');
                }
                $this->moduleSuffix = $this->mustbeArray($this->moduleSuffix);
                $this->moduleSuffix[] = $content;
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в количестов найденных записей.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setFound} ... {/setFound}                    |
                    |                                                             |
                    ============================================================ */

        public function setFound ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            return $this->setVariableByRef('items_count', $content, $smarty, 'global');
        }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в финальное тело страницы.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setBody} ... {/setBody}                      |
                    |                                                             |
                    ============================================================ */

        public function setBody ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            return $this->setVariableByRef('body', $content, $smarty, 'global');
        }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в контент страницы.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setContent} ... {/setContent}                |
                    |                                                             |
                    ============================================================ */

            public function setContent ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('content', $content, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в заголовок страницы.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setTitle} ... {/setTitle}                    |
                    |                                                             |
                    ============================================================ */

            public function setTitle ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('title', $content, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в мета описание страницы.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setMetaDescription} ... {/setMetaDescription}|
                    |                                                             |
                    ============================================================ */

            public function setMetaDescription ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('description', $content, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в мета ключевые слова страницы.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setMetaKeywords} ... {/setMetaKeywords}      |
                    |                                                             |
                    ============================================================ */

            public function setMetaKeywords ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('keywords', $content, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в SEO-текст страницы.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setSeoText} ... {/setSeoText}                |
                    |                                                             |
                    ============================================================ */

            public function setSeoText ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('seo_description', $content, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот фрагмент в завершающий тег блока.           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setEndTag} ... {/setEndTag}                  |
                    |                                                             |
                    ============================================================ */

            public function setEndTag ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('endTag', $content, $smarty, 'global');
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в разметку боковой панели страницы.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideHeader} ... {/setAsideHeader}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideTop} ... {/setAsideTop}              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideRight} ... {/setAsideRight}          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideBottom} ... {/setAsideBottom}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideLeft} ... {/setAsideLeft}            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideCenter} ... {/setAsideCenter}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideBanner} ... {/setAsideBanner}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideSlider} ... {/setAsideSlider}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideGallery} ... {/setAsideGallery}      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideMenu} ... {/setAsideMenu}            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setAsideFooter} ... {/setAsideFooter}        |
                    |                                                             |
                    ============================================================ */

            public function setAsideHeader ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('asideHeader', $content, $smarty, 'global');
            }

                public function setAsideTop ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideTop', $content, $smarty, 'global');
                }

                public function setAsideRight ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideRight', $content, $smarty, 'global');
                }

                public function setAsideBottom ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideBottom', $content, $smarty, 'global');
                }

                public function setAsideLeft ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideLeft', $content, $smarty, 'global');
                }

                public function setAsideCenter ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideCenter', $content, $smarty, 'global');
                }

                public function setAsideBanner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideBanner', $content, $smarty, 'global');
                }

                public function setAsideSlider ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideSlider', $content, $smarty, 'global');
                }

                public function setAsideGallery ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideGallery', $content, $smarty, 'global');
                }

                public function setAsideMenu ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideMenu', $content, $smarty, 'global');
                }

                public function setAsideFooter ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('asideFooter', $content, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в класс тега HTML.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setHtmlClass} ... {/setHtmlClass}            |
                    |                                                             |
                    ============================================================ */

            public function setHtmlClass ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                return $this->setVariableByRef('htmlClass', $content, $smarty, 'global');
            }

                public function setChildClass ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('childClass', $content, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в язык тега HTML.                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setHtmlLang} ... {/setHtmlLang}              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setHtmlLangMarker} ... {/setHtmlLangMarker}  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setLangDefault} ... {/setLangDefault}        |
                    |                                                             |
                    ============================================================ */

                public function setHtmlLang ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('htmlLang', $content, $smarty, 'global');
                }

                    public function setHtmlLangMarker ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = TRUE;
                            return '';
                        }
                        return $this->setVariableByRef('htmlLangMarker', $content, $smarty, 'global');
                    }

                    public function setLangDefault ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = TRUE;
                            return '';
                        }
                        return $this->setVariableByRef('langDefault', $content, $smarty, 'global');
                    }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в URL для некоторого HTML.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setHtmlUrl} ... {/setHtmlUrl}                |
                    |                                                             |
                    ============================================================ */

                public function setHtmlUrl ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('htmlUrl', $content, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в URL скриншота для OpenGraph разметки.|
                    |  ---------------------------------------------------------  |
                    |  применение:  {setOpenGraphImage} ... {/setOpenGraphImage}  |
                    |                                                             |
                    ============================================================ */

                public function setOpenGraphImage ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    return $this->setVariableByRef('openGraphImage', $content, $smarty, 'global');
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в переменную item.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItem                                      |
                    |                   [full = TRUE]  если и в глобальной области|
                    |               } ... {/setItem}                              |
                    |                                                             |
                    ============================================================ */

            public function setItem ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $scope = $this->paramAsBoolean('full', $params) ? 'global' : '';
                return $this->setVariableByRef('item', $content, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в поле записи, находящейся сейчас      |
                    |  в переменной item.                                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemTitle                                 |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               } ... {/setItemTitle}                         |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemMeta ...} ... {/setItemMeta}          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemMetaKeywords ...}                     |
                    |               ... {/setItemMetaKeywords}                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemName ...} ... {/setItemName}          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemUrl                                   |
                    |                   [full = TRUE]   если это не "особый URL", |
                    |                                   то есть с неким           |
                    |                                   префиксным сегментом в    |
                    |                                   начале составленного URL  |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               } ... {/setItemUrl}                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemAnnotation ...}                       |
                    |               ... {/setItemAnnotation}                      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemBody ...} ... {/setItemBody}          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemSeo ...} ... {/setItemSeo}            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemSubdomain ...} ... {/setItemSubdomain}|
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemCategory ...} ... {/setItemCategory}  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemBrand ...} ... {/setItemBrand}        |
                    |                                                             |
                    ============================================================ */

                public function setItemTitle ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $item->meta_title = $this->mustbeString($content);
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemMeta ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $item->meta_description = $this->mustbeString($content);
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemKeywords ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $item->meta_keywords = $this->mustbeString($content);
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemName ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $content = $this->mustbeString($content);
                    if ($this->hasProperty($item, 'model')) $item->model = $content;
                        else if ($this->hasProperty($item, 'header')) $item->header = $content;
                        else $item->name = $content;
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                    public function setItemUrl ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                        if (is_null($content)) {
                            $repeat = TRUE;
                            return '';
                        }
                        $params2 = array();
                        $item = $this->getVar('item', $smarty);
                        $this->mustbeObject($item);
                        $content = $this->mustbeString($content);
                            $content = $this->cms->text->lowerCase(
                                           $this->cms->hdd->safeFilename(
                                               $content, FALSE
                                           )
                                       );
                            if (!$this->isActualString($content)) {
                                $content = $this->id($params2, $smarty);
                            }
                        $item->url = $content;
                            if ($this->paramAsBoolean('full', $params)) {
                                $item->url_special = 0;
                                $item->url_path = $this->typePath($params2, $smarty);
                            } else {
                                $item->url_special = 1;
                                $item->url_path = '';
                            }
                            $item->url_full = $item->url_path . $item->url;
                        $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                        return $this->setVariableByRef('item', $item, $smarty, $scope);
                    }

                public function setItemAnnotation ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $content = $this->mustbeString($content);
                    if ($this->hasProperty($item, 'annotation')) $item->annotation = $content;
                        else if ($this->hasProperty($item, 'body')) $item->description = $content;
                        else $item->description = $content;
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemBody ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $content = $this->mustbeString($content);
                    if ($this->hasProperty($item, 'body')) $item->body = $content;
                        else if ($this->hasProperty($item, 'description')) $item->description = $content;
                        else $item->body = $content;
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemSeo ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $item->seo_description = $this->mustbeString($content);
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemSubdomain ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $item->subdomain_html = $this->mustbeString($content);
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemCategory ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $value = $this->mustbeString($content);
                        $item->category = $value;
                        $item->category_url = $this->cms->hdd->safeFilename(
                                                  $this->cms->text->lowerCase(
                                                      $value
                                                  )
                                              );
                        $item->category_url_path = 'catalog/';
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                public function setItemBrand ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $value = $this->mustbeString($content);
                        $item->brand = $value;
                        $item->brand_url = $this->cms->hdd->safeFilename(
                                               $this->cms->text->lowerCase(
                                                   $value
                                               )
                                           );
                        $item->brand_url_path = 'brands/';
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                    /* ============================================================
                    |                                                             |
                    |  Распарсить и поместить этот блок в поля для изображений    |
                    |  записи, которая находится сейчас в переменной item.        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setItemImages                                |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                   [url 1 | thumb | alt | text | link | view]|
                    |                   [url 2 | thumb | alt | text | link | view]|
                    |                   ...                                       |
                    |                   [url N | thumb | alt | text | link | view]|
                    |               {/setItemImages}                              |
                    |                                                             |
                    ============================================================ */

                public function setItemImages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $item = $this->getVar('item', $smarty);
                    $this->mustbeObject($item);
                    $content = $this->mustbeString($content);
                        $content = preg_replace('~^[ \t\r\n\s]*\[[ \t\r\n\s]*~u', '', $content);
                        $content = preg_replace('~[ \t\r\n\s]*\][ \t\r\n\s]*$~u', '', $content);
                    $urls = array();
                    $thumbs = array();
                    $alts = array();
                    $texts = array();
                    $views = array();
                    if ($this->isActualString($content)) {
                        $content = preg_split('~[ \t\r\n\s]*\][ \t\r\n\s]*\[[ \t\r\n\s]*~u', $content, -1, PREG_SPLIT_NO_EMPTY);
                        if ($this->isActualArray($content)) {
                            foreach ($content as $line) {
                                $line = preg_split('~[ \t\r\n\s]*\|[ \t\r\n\s]*~u', $line);
                                if ($this->isActualArray($line)) {
                                    if ($this->isActualString($line[0])) {
                                        $urls[] = $line[0];
                                        $thumbs[] = isset($line[1]) && $this->isActualString($line[1])
                                                    ? $line[1]
                                                    : $line[0];
                                        $alts[] = isset($line[2])
                                                  ? $line[2]
                                                  : '';
                                        $texts[] = trim(
                                                       (empty($line[4]) ? '' : ($line[4] . ' ')) .
                                                       (empty($line[3]) ? '' : $line[3])
                                                   );
                                        $views[] = isset($line[5])
                                                   ? (empty($line[5]) ? 0 : 1)
                                                   : 1;
                                    }
                                }
                            }
                        }
                    }
                    $item->images = $urls;
                    $item->images_thumbs = $thumbs;
                    $item->images_alts = $alts;
                    $item->images_texts = $texts;
                    $item->images_view = $views;
                    $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                    return $this->setVariableByRef('item', $item, $smarty, $scope);
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в переменную value.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setValue                                     |
                    |                   [full = TRUE]  если и в глобальной области|
                    |               } ... {/setValue}                             |
                    |                                                             |
                    ============================================================ */

            public function setValue ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $scope = $this->paramAsBoolean('full', $params) ? 'global' : '';
                return $this->setVariableByRef('value', $content, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в переменную type.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setType                                      |
                    |                   [full = TRUE]  если и в глобальной области|
                    |               } ... {/setType}                              |
                    |                                                             |
                    ============================================================ */

                public function setType ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $scope = $this->paramAsBoolean('full', $params) ? 'global' : '';
                    return $this->setVariableByRef('type', $content, $smarty, $scope);
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить хеш этого блока в переменную hash.               |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setHash                                      |
                    |                   [full = TRUE]  если и в глобальной области|
                    |               } ... {/setHash}                              |
                    |                                                             |
                    ============================================================ */

                public function setHash ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $scope = $this->paramAsBoolean('full', $params) ? 'global' : '';
                    $content = $this->asHash($content);
                    return $this->setVariableByRef('hash', $content, $smarty, $scope);
                }

                    /* ============================================================
                    |                                                             |
                    |  Поместить этот блок в переменную error.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setError                                     |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               } ... {/setError}                             |
                    |                                                             |
                    ============================================================ */

            public function setError ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                return $this->setVariableByRef('error', $content, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Распарсить и поместить этот блок в массив языков сайта.    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setLanguages                                 |
                    |                   [local = TRUE]  если только в локальной   |
                    |                                   видимости шаблона         |
                    |               }                                             |
                    |                   lang-1 | lang-2 | ... | lang-N            |
                    |               {/setLanguages}                               |
                    |                                                             |
                    ============================================================ */

            public function setLanguages ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $items = array();
                $content = $this->mustbeString($content);
                    $content = preg_replace('~[^a-z0-9_\-]+~ui', ' ', $content);
                if ($this->isActualString($content)) {
                    $content = explode(' ', $content);
                    foreach ($content as $item) {
                        $item = trim($item, '_-');
                        if ($this->isActualString($item)) {
                            $idx = $this->cms->text->lowerCase($item);
                            $items[$idx] = $item;
                        }
                    }
                }
                $scope = $this->paramAsBoolean('local', $params) ? '' : 'global';
                return $this->setVariableByRef('languages', $items, $smarty, $scope);
            }

                    /* ============================================================
                    |                                                             |
                    |  Загрузить фразы в многоязычный словарь.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {setPolyglot}                                 |
                    |                   <dfn>                                     |
                    |                       Phrase 1 in mnemonic Language         |
                    |                       <hr class="en">                       |
                    |                           Phrase 1 in English               |
                    |                           class may bе "en", "en-US" etc    |
                    |                       <hr class="de"> Phrase 1 in German    |
                    |                       <hr class="es"> Phrase 1 in Spanish   |
                    |                       <hr class="ru"> Phrase 1 in Russian   |
                    |                       <hr class="uk"> Phrase 1 in Ukrainian |
                    |                       ...                                   |
                    |                       <hr class="##"> Phrase 1 in ##        |
                    |                   </dfn>                                    |
                    |                   ...                                       |
                    |                   <dfn>                                     |
                    |                       Phrase 2 in mnemonic Language         |
                    |                       ...                                   |
                    |                       <hr class="##"> Phrase 2 in ##        |
                    |                   </dfn>                                    |
                    |               {/setPolyglot}                                |
                    |                                                             |
                    ============================================================ */

            public function setPolyglot ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) $repeat = TRUE;
                else {
                    $items = array();
                    $getDefinitionPattern  = '~^.*?<dfn>(.+?)</dfn>(.*)$~usi';
                    while (($dfn = preg_replace($getDefinitionPattern, '$1', $content)) != $content) {
                        $content = preg_replace($getDefinitionPattern, '$2', $content);
                        $lang = FALSE;
                        $this->parsePolyglotDFN($dfn, $lang, $items);
                    }
                    $this->setVariableByRef('polyglot', $items, $smarty, 'global');
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Перевести фразу через многоязычный словарь.                |
                    |  ---------------------------------------------------------  |
                    |  применение:  {polyglot                                     |
                    |                   [asAttribute = TRUE]  если это текст для  |
                    |                                         тегового атрибута   |
                    |                   [noAdminPin = TRUE]   если без редакторс. |
                    |                                         кнопки админа       |
                    |                   [lang = langMarker]   маркер желаемого    |
                    |                                         языка, например ru  |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                   Phrase in mnemonic Language               |
                    |               {/polyglot}                                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {polyglot ...}                                |
                    |                   Phrase in mnemonic Language               |
                    |                   <hr class="en">                           |
                    |                       Phrase in English                     |
                    |                       class may bе "en", "en-US" etc        |
                    |                   <hr class="de"> Phrase in German          |
                    |                   <hr class="es"> Phrase in Spanish         |
                    |                   <hr class="ru"> Phrase in Russian         |
                    |                   <hr class="uk"> Phrase in Ukrainian       |
                    |                   ...                                       |
                    |                   <hr class="##"> Phrase in ##              |
                    |               {/polyglot}                                   |
                    |                                                             |
                    ============================================================ */

                public function polyglot ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $lang = $this->paramAsString('lang', $params);
                    $asAttr = $this->paramAsBoolean('asAttribute', $params);
                    if ($asAttr) {
                        $value = $this->parsePolyglotDFN($content, $lang);
                        $value = $this->asAttribute($value);
                    } else {
                        $special = $this->cms->security->inDemoMode()
                                || $this->existsAdmin();
                        if ($special) {
                            $noPin = $this->paramAsBoolean('noAdminPin', $params);
                            if ($noPin) $value = $this->parsePolyglotDFN($content, $lang);
                            else {
                                $id = null;
                                $value = $this->parsePolyglotDFN($content, $lang, $id, TRUE);
                                    $id = array_shift($id);
                                    $id = isset($id['marker']) ? $this->asHash($id['marker'])
                                                               : '';
                                if ($this->isActualString($id)) $value .= '<kbd class="admin-pin" data-id="' . $this->asAttribute($id) . '"></kbd>';
                            }
                        } else $value = $this->parsePolyglotDFN($content, $lang);
                    }
                    return $this->assignVar($params, $value, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Добавить этот блок в финальное тело страницы.              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putBody} ... {/putBody}                      |
                    |                                                             |
                    ============================================================ */

        public function putBody ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            $this->appendString('body', $content, $smarty);
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Добавить этот блок в контент страницы.                     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putContent} ... {/putContent}                |
                    |                                                             |
                    ============================================================ */

            public function putContent ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $this->appendString('content', $content, $smarty);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить этот блок в остальные мета.                       |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putOtherMeta} ... {/putOtherMeta}            |
                    |                                                             |
                    ============================================================ */

            public function putOtherMeta ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $this->appendString('meta', $content, $smarty);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить этот блок в остальные скрипты.                    |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putOtherJS} ... {/putOtherJS}                |
                    |                                                             |
                    ============================================================ */

            public function putOtherJS ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $this->appendString('otherJS', $content, $smarty);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить этот блок в SEO-текст страницы.                   |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putSeoText} ... {/putSeoText}                |
                    |                                                             |
                    ============================================================ */

            public function putSeoText ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $this->appendString('seo_description', $content, $smarty);
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить этот блок в разметку боковой панели страницы.     |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideHeader} ... {/putAsideHeader}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideTop} ... {/putAsideTop}              |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideRight} ... {/putAsideRight}          |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideBottom} ... {/putAsideBottom}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideLeft} ... {/putAsideLeft}            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideCenter} ... {/putAsideCenter}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideBanner} ... {/putAsideBanner}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideSlider} ... {/putAsideSlider}        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideGallery} ... {/putAsideGallery}      |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideMenu} ... {/putAsideMenu}            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {putAsideFooter} ... {/putAsideFooter}        |
                    |                                                             |
                    ============================================================ */

            public function putAsideHeader ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $this->appendString('asideHeader', $content, $smarty);
                return '';
            }

                public function putAsideTop ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideTop', $content, $smarty);
                    return '';
                }

                public function putAsideRight ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideRight', $content, $smarty);
                    return '';
                }

                public function putAsideBottom ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideBottom', $content, $smarty);
                    return '';
                }

                public function putAsideLeft ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideLeft', $content, $smarty);
                    return '';
                }

                public function putAsideCenter ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideCenter', $content, $smarty);
                    return '';
                }

                public function putAsideBanner ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideBanner', $content, $smarty);
                    return '';
                }

                public function putAsideSlider ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideSlider', $content, $smarty);
                    return '';
                }

                public function putAsideGallery ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideGallery', $content, $smarty);
                    return '';
                }

                public function putAsideMenu ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideMenu', $content, $smarty);
                    return '';
                }

                public function putAsideFooter ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                    if (is_null($content)) {
                        $repeat = TRUE;
                        return '';
                    }
                    $this->appendString('asideFooter', $content, $smarty);
                    return '';
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести этот блок посредством javascript.                  |
                    |                                                             |
                    ============================================================ */

        public function viaScript ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            $plain = $this->paramAsBoolean('plain', $params);
                $opencmd = $plain ? '' : 'document.writeln(';
                $closecmd = $plain ? '' : ');';
            $notags = $plain ||
                      $this->paramAsBoolean('notags', $params);
                $opentag = $notags ? '' : '<script class="via-script">';
                $closetag = $notags ? '' : '</script>';
            $content = preg_replace('~[\\\\]~u', '\\\\\\', $content);
            $content = preg_replace("~'~u", "\\'", $content);
            $content = preg_replace('~<~u', "<'+'", $content);
            $content = preg_replace('~>~u', "'+'>", $content);
            $content = $opentag .
                           $opencmd .
                               "'" . $content . "'" .
                           $closecmd .
                       $closetag;
            return $this->assignVar($params, $content, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Имплантировать в тело страницы стили из этого блока мета   |
                    |  тегов подключения локальных CSS-файлов шаблона.            |
                    |  ---------------------------------------------------------  |
                    |  применение:  {implantCSS                                   |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                   <link href="{theme}css/example-1.css" ...>|
                    |                   <link href="{theme}css/example/2.css" ...>|
                    |               {/implantCSS}                                 |
                    |                                                             |
                    ============================================================ */

        public function implantCSS ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            $value = '';
            $method = 'includeHtml';
            $helper = $this->getVar('helper');
            if ($this->cms->any->hasMethod($helper, $method)) {
                $theme = $this->getThemeUrl();
                $theme2 = $this->getThemeUrl(TRUE, FALSE);
                $size = $this->cms->text->length($theme2);
                $pattern = '~^.*?<link\s[^>]*?href=("|\')([^>]+?)\1(.*)$~usi';
                while (($url = preg_replace($pattern, '$2', $content)) != $content) {
                    $content = preg_replace($pattern, '$3', $content);
                    if ($this->cms->text->substr($url, 0, $size) == $theme2) {
                        $url = $this->cms->text->substr($url, $size);
                        if ($this->isActualString($url)) {
                            $params2 = array('url' => $url);
                            $data = $helper->$method($params2, $this->cms->smarty);
                            $data = $this->removeComments($data);
                            $ifSpaceAfter = '!([;:,\{\}\(~>])\s!u';
                                $data = preg_replace($ifSpaceAfter, '$1', $data);
                            $ifSpaceBefore = '!\s([\{\}\)~>])!u';
                                $data = preg_replace($ifSpaceBefore, '$1', $data);
                            $ifSemicolonLast = '!;+\}!u';
                                $data = preg_replace($ifSemicolonLast, '}', $data);
                            $ifUrlLocal = '!(url ?\( ?["\']?)../!u';
                                $data = preg_replace($ifUrlLocal, '$1' . $theme, $data);
                            $ifSelectorUseless = '![^\{\}]+\{\}!u';
                                $data = preg_replace($ifSelectorUseless, '', $data);    /* для правил         */
                                $data = preg_replace($ifSelectorUseless, '', $data);    /* для медиа-запросов */
                            $value .= trim($data);
                        }
                    }
                }
                if ($this->isActualString($value)) $value = '<style>' . $value . '</style>';
            }
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Имплантировать в тело страницы скрипты из этого блока      |
                    |  тегов подключения локальных JS-файлов шаблона.             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {implantJS                                    |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                   <script src="{theme}js/example-1.js" ...> |
                    |                   <script src="{theme}js/example/2.js" ...> |
                    |               {/implantJS}                                  |
                    |                                                             |
                    ============================================================ */

            public function implantJS ( $params = null, $content = null, & $smarty = null, & $repeat ) {
                if (is_null($content)) {
                    $repeat = TRUE;
                    return '';
                }
                $value = '';
                $method = 'includeHtml';
                $helper = $this->getVar('helper');
                if ($this->cms->any->hasMethod($helper, $method)) {
                    $theme = $this->getThemeUrl();
                    $theme2 = $this->getThemeUrl(TRUE, FALSE);
                    $size = $this->cms->text->length($theme2);
                    $pattern = '~^.*?<script\s[^>]*?src=("|\')([^>]+?)\1(.*)$~usi';
                    while (($url = preg_replace($pattern, '$2', $content)) != $content) {
                        $content = preg_replace($pattern, '$3', $content);
                        if ($this->cms->text->substr($url, 0, $size) == $theme2) {
                            $url = $this->cms->text->substr($url, $size);
                            if ($this->isActualString($url)) {
                                $params2 = array('url' => $url);
                                $data = $helper->$method($params2, $this->cms->smarty);
                                $data = $this->removeComments($data);
                                $value .= trim($data);
                            }
                        }
                    }
                    if ($this->isActualString($value)) $value = '<script>' . $value . '</script>';
                }
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вставить строку вначале текстового файла.                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {insertTextFileLine                           |
                    |                   name = FileName   имя файла               |
                    |                   [max = MaxCount]  предельное количество   |
                    |                                     сохраняемых строк в     |
                    |                                     файле (по умолчанию без |
                    |                                     ограничений)            |
                    |               }                                             |
                    |                   содержимое строки                         |
                    |               {/insertTextFileLine}                         |
                    |                                                             |
                    ============================================================ */

        public function insertTextFileLine ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            $file = $this->paramAsSentence('name', $params);
                $uri = '';
            $exists = $this->existsFile($file, array('txt'), 'db', $uri);
            $handle = @ fopen($uri, $exists ? 'r+b'
                                            : 'wb');
            if ($handle !== FALSE) {
                @ flock($handle, LOCK_EX);
                $value = '';
                if ($exists) {
                    $max = $this->paramAsNaturalExtended('max', $params, 2000000000);
                    while ($max > 1 && !feof($handle)) {
                        $data = @ fgets($handle, 262144);
                        if (!is_string($data) || $data === '') break;
                        $value .= rtrim($data, "\x00..\x20") . "\r\n";
                        $max--;
                    }
                    @ fseek($handle, 0);
                }
                $content = is_string($content) ? preg_replace('~[\r\n]+~u', ' ', $content)
                                               : '';
                $value = rtrim($content, "\x00..\x20") . "\r\n" .
                         $value;
                @ fwrite($handle, $value);
                if ($exists) {
                    $size = @ ftell($handle);
                    if ($size !== FALSE) @ ftruncate($handle, $size);
                }
                @ fclose($handle);
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Отправить СМС.                                             |
                    |  ---------------------------------------------------------  |
                    |  применение:  {sendSms                                      |
                    |                   [for = PhoneList]  список телефонов через |
                    |                                      запятую                |
                    |               }                                             |
                    |                   текст СМСки                               |
                    |               {/sendSms}                                    |
                    |                                                             |
                    ============================================================ */

        public function sendSms ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            if ($this->isActualString($content)) {
                $content = $this->cms->text->stripTags($content, TRUE);
                $to = $this->cms->text->substr($content, 0, 200);
                if ($content != $to) $content = $to . '...';
            }
            if ($this->isActualString($content)) {
                $to = $this->paramAsString('for', $params, null);
                if (is_null($to)) {
                    $to = $this->getConfig('shortcodesSendSmsItems');
                    $to = $this->cms->text->lowerCase($to);
                    $items = $this->asUniqueStringArray($to);
                    if (!$this->isActualString($to)) {
                        $to = 'administrator';
                        $items = $this->asUniqueStringArray($to);
                    }
                } else {
                    $to = $this->cms->text->lowerCase($to);
                    $items = $this->asUniqueStringArray($to);
                }
                if ($this->isActualString($to)) {
                    $ip = $this->cms->security->getVisitorIp();
                    $timer = substr(date('YmdHi', time()), 0, -1);
                    $checker = 'antispamTplSms' . $this->asHash($ip . $to . $content . $timer);
                    if (!$this->cms->request->existsSession($checker)) {
                        $this->cms->session->set($checker, 1);
                        foreach ($items as $to) {
                            $this->cms->send_sms($to, $content);
                        }
                    }
                }
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Отправить емейл.                                           |
                    |  ---------------------------------------------------------  |
                    |  применение:  {sendEmail                                    |
                    |                   [subject = Text]   тема письма            |
                    |                   [for = EmailList]  список емейлов через   |
                    |                                      запятую                |
                    |               }                                             |
                    |                   текст письма                              |
                    |               {/sendEmail}                                  |
                    |                                                             |
                    ============================================================ */

        public function sendEmail ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            if ($this->isActualString($content)) {
                $def = 'Сообщение с сайта ' . $this->getSiteDomain();
                $subject = $this->paramAsSentence('subject', $params, $def);
                $to = $this->paramAsString('for', $params, null);
                if (is_null($to)) {
                    $to = $this->getConfig('shortcodesSendEmailItems');
                    $to = $this->cms->text->lowerCase($to);
                    $items = $this->asUniqueStringArray($to);
                    if (!$this->isActualString($to)) {
                        $to = $this->getSettings('admin_email');
                        $to = $this->cms->text->lowerCase($to);
                        $items = $this->asUniqueStringArray($to);
                    }
                } else {
                    $to = $this->cms->text->lowerCase($to);
                    $items = $this->asUniqueStringArray($to);
                }
                if ($this->isActualString($to)) {
                    $ip = $this->cms->security->getVisitorIp();
                    $timer = substr(date('YmdHi', time()), 0, -1);
                    $checker = 'antispamTplEmail' . $this->asHash($ip . $to . $subject . $content . $timer);
                    if (!$this->cms->request->existsSession($checker)) {
                        $this->cms->session->set($checker, 1);
                        foreach ($items as $to) {
                            $this->cms->email($to, $subject, $content);
                        }
                    }
                }
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Обработать BB-коды блока.                                  |
                    |  ---------------------------------------------------------  |
                    |  применение:  {bbCodes                                      |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                   контент в котором хотим                   |
                    |                   преобразовать BB-коды                     |
                    |                   в актуальную HTML-разметку                |
                    |               {/bbCodes}                                    |
                    |  ---------------------------------------------------------  |
                    |  реализованные коды:                                        |
                    |      [audio   relative-or-absolute-url-of-mp3-file]         |
                    |      [video   relative-or-absolute-url-of-mp4-file]         |
                    |      [youtube absolute-url-of-video-on-youtube]             |
                    |                                                             |
                    ============================================================ */

        public function bbCodes ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            $theme = $this->getThemeUrl();
            $adminSource = $this->cms->security->inDemoMode()
                        || $this->existsAdmin()
                           ? '<template class="admin-source">$1</template>'
                           : '';
            $pOpeningWrapper = '~(<(p|div)[^>]*?>\s*';
            $pClosingWrapper = '\s*</\2>)~uis';
            $this->setParam('change', $params, array(
                $pOpeningWrapper . '\[audio\s+([^\]\s\'"]+)\]' . $pClosingWrapper
                    => '<center class="audio">' .
                           '<audio controls type="audio/mpeg" data-lazy="$3">' .
                           '</audio>' .
                           '<img class="placeholder" src="' . $theme . 'images/loading-audio.gif" alt="" />' .
                           $adminSource .
                       '</center>',
                $pOpeningWrapper . '\[video\s+([^\]\s\'"]+)\]' . $pClosingWrapper
                    => '<center class="video">' .
                           '<video controls type="video/mp4" data-lazy="$3">' .
                           '</video>' .
                           '<img class="placeholder" src="' . $theme . 'images/loading-video.gif" alt="" />' .
                           $adminSource .
                       '</center>',
                $pOpeningWrapper . '\[youtube\s+(?:https?:)?/+(?:www\.)?(?:youtu\.be/|youtube(?:-nocookie)?\.com/(?:watch/?\?v=|embed/?))([a-z0-9_-]+)(?:[&\?][^\]\s\'"]*?)?\]' . $pClosingWrapper
                    => '<center class="youtube">' .
                           '<iframe data-lazy="https://www.youtube.com/embed/$3">' .
                           '</iframe>' .
                           '<img class="placeholder" src="' . $theme . 'images/loading-video.gif" alt="" />' .
                           $adminSource .
                       '</center>'
            ));
            return $this->censor($params, $content, $smarty, $repeat);
        }

                    /* ============================================================
                    |                                                             |
                    |  Цензурировать блок.                                        |
                    |  ---------------------------------------------------------  |
                    |  применение:  {censor                                       |
                    |                   [assign = VarName]    в какую переменную  |
                    |                   или [var = VarName]   вывести результат   |
                    |                   или [result=VarName]  (по умолчанию в     |
                    |                                         точку вызова)       |
                    |                   [scope = global]      видимость переменной|
                    |               }                                             |
                    |                   контент который хотим                     |
                    |                   подвергнуть цензуре                       |
                    |               {/censor}                                     |
                    |                                                             |
                    ============================================================ */

        public function censor ( $params = null, $content = null, & $smarty = null, & $repeat ) {
            if (is_null($content)) {
                $repeat = TRUE;
                return '';
            }
            if ($this->isActualString($content)) {
                $items = $this->paramAsArray('change', $params);
                if ($this->isActualArray($items)) {
                    foreach ($items as $pattern => $replacer) {
                        if ($this->isActualString($pattern)) {
                            if (is_string($replacer)) {
                                try {
                                    $test = @ preg_replace($pattern, $replacer, $content);
                                    if (is_string($test)) $content = $test;
                                } catch (Exception $e) { }
                            }
                        }
                    }
                }
            }
            return $this->assignVar($params, $content, $smarty);
        }

        /* ========================================================================
        |                                                                         |
        |    Функции объекта $emulator.                                           |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Каждая из этих функций может быть вызвана в шаблоне с помощью        |
        |    инструкции такого вида, например:                                    |
        |                                                                         |
        |        {if $emulator->имяФункции(параметры)}                            |
        |            ...                                                          |
        |            ...                                                          |
        |        {/if}                                                            |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Отсортировать этот массив (или объект) по такому полю.     |
                    |                                                             |
                    ============================================================ */

        public function sortBy ( & $items, $fieldName, $descending = FALSE ) {
            if (is_array($items)) {
                $this->sortByFieldName = $fieldName;
                $this->sortByDescending = $descending == TRUE;
                uasort($items, array($this, 'compareBy'));
            }
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что мы на странице "Ошибка 404".                  |
                    |                                                             |
                    ============================================================ */

        public function checkNoPage () {
            return $this->getConst('CONTENT_MESSAGE_NO_PAGE') === $this->getVar('content');
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что мы на странице "Нет такого модуля или выкл".  |
                    |                                                             |
                    ============================================================ */

            public function checkNoModule () {
                $v = $this->getConst('CONTENT_MESSAGE_NO_MODULE');
                    $v = preg_replace('/([~#:\+\[\]\{\}\(\)\?\.\$\^\|\\\\])/u', '\\\\$1', $v);
                    $v = preg_replace('/\*/u', '[a-z0-9_]+', $v);
                return preg_match('~^' . $v . '$~iu', $this->getVar('content')) > 0;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что мы на странице "Не обслуживается этот шаблон".|
                    |                                                             |
                    ============================================================ */

            public function checkNoService () {
                $pattern = '~^<!-- хелпер шаблона указал не обслуживать файл шаблона с именем .+? -->$~iu';
                return preg_match($pattern, $this->getVar('content')) > 0;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в хелпере шаблона есть такая функция.         |
                    |                                                             |
                    ============================================================ */

        public function existsFunction ( $name ) {
            return !empty($this->templateFunctions)
                && is_array($this->templateFunctions)
                && isset($this->templateFunctions[$name])
                && method_exists($this, $name);
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в шаблоне есть такой модуль (файл).           |
                    |                                                             |
                    ============================================================ */

            public function existsModule ( $file ) {
                $types = array('tpl', 'htm', 'html', 'xml', 'css', 'js', 'txt');
                return $this->existsFile($file, $types, 'html');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в шаблоне есть такое изображение (файл).      |
                    |                                                             |
                    ============================================================ */

            public function existsImage ( $file ) {
                return $this->existsPhoto($file, 'images');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть картинка такого баннера (файл). |
                    |                                                             |
                    ============================================================ */

            public function existsBanner ( $file, $folder = null ) {
                $folder = is_string($folder) ? $this->cms->hdd->safeFilename($folder, FALSE)
                                             : '';
                return $this->existsPhoto($file, 'banners' . rtrim('/' . $folder, '/'));
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото товара (файл).       |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoProduct ( $file ) {
                return $this->existsPhoto($file, 'products');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото категории (файл).    |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoCategory ( $file ) {
                return $this->existsPhoto($file, 'categories');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото бренда (файл).       |
                    |                                                             |
                    ============================================================ */

                public function existsPhotoBrand ( $file ) {
                    return $this->existsPhoto($file, 'brands');
                }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото новости (файл).      |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoNews ( $file ) {
                return $this->existsPhoto($file, 'images/news');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото статьи (файл).       |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoArticle ( $file ) {
                return $this->existsPhoto($file, 'images/articles');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото страницы (файл).     |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoSection ( $file ) {
                return $this->existsPhoto($file, 'images/sections');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото склада (файл).       |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoStock ( $file ) {
                return $this->existsPhoto($file, 'stocks');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото страны (файл).       |
                    |                                                             |
                    ============================================================ */

            public function existsPhotoCountry ( $file ) {
                return $this->existsPhoto($file, 'countries');
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото области (файл).      |
                    |                                                             |
                    ============================================================ */

                public function existsPhotoRegion ( $file ) {
                    return $this->existsPhoto($file, 'regions');
                }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть такое фото города (файл).       |
                    |                                                             |
                    ============================================================ */

                public function existsPhotoTown ( $file ) {
                    return $this->existsPhoto($file, 'towns');
                }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте сейчас авторизованный клиент.        |
                    |                                                             |
                    ============================================================ */

            public function existsUser ( $login = '' ) {
                if ($this->getUserId() == 0) return FALSE;
                return empty($login)
                       || $this->getUserEmail() == $login
                       || $this->getUserEmail2() == $login;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте сейчас администратор.                |
                    |                                                             |
                    ============================================================ */

            public function existsAdmin ( $login = '' ) {
                if (!$this->cms->security->existsAdmin()) return FALSE;
                return empty($login)
                       || $login == $this->cms->security->currentAdminLogin();
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что на сайте есть более 1 валюты.                 |
                    |                                                             |
                    ============================================================ */

            public function existsCurrencies () {
                $count = 0;
                $items = $this->getVar('currencies');
                if (!empty($items) && is_array($items))
                    foreach ($items as $item)
                        if (!empty($item->enabled)) $count++;
                return $count > 1;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что этот товар имеет хотя бы 1 вариант.           |
                    |                                                             |
                    ============================================================ */

            public function existsVariant ( & $item = null ) {
                return !empty($item->variants)
                       && is_array($item->variants);
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что этот товар имеет более 1 варианта.            |
                    |                                                             |
                    ============================================================ */

            public function existsVariants ( & $item = null ) {
                return $this->existsVariant($item)
                       && count($item->variants) > 1;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что этот товар имеет хотя бы 1 фотографию.        |
                    |                                                             |
                    ============================================================ */

            public function existsFoto ( & $item = null ) {
                return !empty($item->fotos)
                       && is_array($item->fotos);
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что этот товар имеет более 1 фотографии.          |
                    |                                                             |
                    ============================================================ */

            public function existsFotos ( & $item = null ) {
                return $this->existsFoto($item)
                       && count($item->fotos) > 1;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эта запись имеет хотя бы 1 снимок (фото).     |
                    |                                                             |
                    ============================================================ */

            public function existsSnapshot ( & $item = null ) {
                return !empty($item->images)
                       && is_array($item->images);
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эта запись имеет более 1 снимка (фото).       |
                    |                                                             |
                    ============================================================ */

            public function existsSnapshots ( & $item = null ) {
                return $this->existsSnapshot($item)
                       && count($item->images) > 1;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эта запись имеет хотя бы 1 прикрепленный файл.|
                    |                                                             |
                    ============================================================ */

            public function existsAttachment ( & $item = null ) {
                return !empty($item->files)
                       && is_array($item->files);
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эта запись имеет более 1 прикрепленного файла.|
                    |                                                             |
                    ============================================================ */

            public function existsAttachments ( & $item = null ) {
                return $this->existsAttachment($item)
                       && count($item->files) > 1;
            }

                    /* ============================================================
                    |                                                             |
                    |  Вычислить количество голосов этой записи.                  |
                    |                                                             |
                    ============================================================ */

        public function calculateVotes ( & $item ) {
            $minimum = isset($item->rating) ? ($item->rating > 0 ? 1 : 0) : 0;
            $votes = !empty($item->votes) ? intval($item->votes) : 0;
            return max($minimum, $votes);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вычислить рейтинг этой записи.                             |
                    |                                                             |
                    ============================================================ */

            public function calculateRating ( & $item, $asSumStored = FALSE, $minimal = 0, $maximal = 5 ) {
                $value = 0;
                $votes = $asSumStored ? $this->calculateVotes($item) : 1;
                if (!empty($votes)) {
                    if (isset($item->rating)) $value = $item->rating / $votes;
                }
                if ($value < $minimal) $value = $minimal;
                if ($value > $maximal) $value = $maximal;
                return str_replace(',', '.', sprintf('%1.2f', $value));
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эту категорию (или бренд) можно показывать.   |
                    |                                                             |
                    ============================================================ */

        public function actualCategory ( & $item = null ) {
            return $this->accessibleItem($item)
                && (!empty($item->filled) || !empty($item->products_count) || !empty($item->informative));
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эта категория (или бренд) имеет вложенные.    |
                    |                                                             |
                    ============================================================ */

            public function hasChild ( & $item = null ) {
                return !empty($item->subcategories)
                    || !empty($item->subbrands);
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что эта запись может быть показана.               |
                    |                                                             |
                    ============================================================ */

        public function accessibleItem ( & $item = null ) {
            if (is_null($item)) return FALSE;
            return !empty($item->enabled)
                && (empty($item->hidden) || $this->existsUser());
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что этот заказ может быть показан.                |
                    |                                                             |
                    ============================================================ */

            public function accessibleOrder ( $item = null ) {
                if (is_null($item)) $item = $this->getVar('order');
                if (empty($item->order_id)) return FALSE;
                return empty($item->hidden)
                       || $this->existsAdmin()
                       || $this->existsUser() && $this->getUserId() == $this->getUserId($item);
            }

                    /* ============================================================
                    |                                                             |
                    |  Удалить из этого списка товаров отсутствующие и обрезать   |
                    |  список по размеру.                                         |
                    |                                                             |
                    ============================================================ */

        public function rejectAbsents ( & $products, $maxcount = null ) {
            $items = array();
            $helper = $this->getVar('helper');
            if (is_object($helper)) {
                if (!empty($products) && is_array($products)) {
                    $maxcount = is_numeric($maxcount) ? intval($maxcount) : 2000000000;
                    foreach ($products as $index => $item) {
                        $maxcount--;
                        if ($maxcount < 0) break;
                        if ($helper->maybeSale($item) === TRUE) $items[$index] = $item;
                    }
                }
            }
            $products = $items;
            return '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Удалить из этого списка товаров текущий (на странице       |
                    |  которого сейчас стоим) и обрезать список по размеру.       |
                    |                                                             |
                    ============================================================ */

            public function rejectCurrent ( & $products, $maxcount = null ) {
                $items = array();
                $helper = $this->getVar('helper');
                if (is_object($helper)) {
                    if (!empty($products) && is_array($products)) {
                        $params = array('except' => '*',
                                        'nopages' => TRUE);
                        $url = $helper->requestUri($params, $this->cms->smarty);
                        $url = preg_replace('!^[/\\\\]+!u', '', $url);
                        $params = array('root' => TRUE);
                        $url = $helper->site($params, $this->cms->smarty) . $url;
                        $maxcount = is_numeric($maxcount) ? intval($maxcount) : 2000000000;
                        foreach ($products as $index => $item) {
                            $maxcount--;
                            if ($maxcount < 0) break;
                            $params = array('item' => $item);
                            if ($helper->url($params, $this->cms->smarty) != $url) $items[$index] = $item;
                        }
                    }
                }
                $products = $items;
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Прочесть список свойств для фильтра.                       |
                    |                                                             |
                    ============================================================ */

        public function getFilterProperties () {
            // TODO: а нужен ли такой метод?
            //       пока существует как заглушка
            $items = array();
            return $items;
        }

                    /* ============================================================
                    |                                                             |
                    |  Прочесть список брендов для фильтра.                       |
                    |                                                             |
                    ============================================================ */

        public function getFilterBrands ( $short = TRUE ) {
            $items = array();
            $category = isset($this->cms->category) ? $this->cms->category : null;
            if (empty($category->subcats_ids)) $category = $this->getVar('category');
            if (empty($category->subcats_ids)) return $items;
            $authorized = $this->existsUser();
            $ids = join($category->subcats_ids, ',');
            $query = 'SELECT ' . ($short ? '`brands`.`brand_id`, '
                                         . '`brands`.`name`, '
                                         : '`brands`.*, ')
                          . 'COUNT(DISTINCT `products`.`product_id`) AS `count` '
                   . 'FROM `brands` '
                   . 'LEFT JOIN `products` ON `products`.`brand_id` = `brands`.`brand_id` AND '
                                           . ($authorized ? '' : '`products`.`hidden` = 0 AND ')
                                           . '`products`.`enabled` = 1 '
                   . 'LEFT JOIN `products_categories` ON `products_categories`.`product_id` = `products`.`product_id` AND '
                                                      . '`products_categories`.`category_id` IN (' . $ids . ') '
                   . 'WHERE `brands`.`enabled` = 1 AND '
                         . ($authorized ? '' : '`brands`.`hidden` = 0 AND ')
                         . '(`products_categories`.`category_id` IS NULL AND '
                          . '`products`.`category_id` IN (' . $ids . ') OR '
                         . '`products_categories`.`category_id` IS NOT NULL AND '
                         . '`products`.`category_id` != `products_categories`.`category_id`) '
                   . 'GROUP BY `brands`.`brand_id` '
                   . 'ORDER BY `brands`.`name` ASC;';
            $result = $this->cms->db->query($query);
            if ($result !== FALSE) {
                while ($item = $this->cms->db->fetch_object($result)) {
                    if (empty($short)) $this->cms->db->fix_brands_record($item);
                    $item->actual = !empty($item->count);
                    $item->products_count = $item->count;
                    array_push($items, $item);
                }
                $this->cms->db->free_result($result);
            }
            return $items;
        }

                    /* ============================================================
                    |                                                             |
                    |  Прочесть глоссарий (список всех названий).                 |
                    |                                                             |
                    ============================================================ */

            public function getGlossary ( $short = TRUE ) {
                $items = array();
                $dbtables = array( 'brands'     => array('brand_id',    'name', TRUE, FALSE, FALSE, FALSE, FALSE),
                                   'categories' => array('category_id', 'name', TRUE, FALSE, FALSE, FALSE, FALSE) );
                if (empty($short)) {
                    $dbtables['products'] = array('product_id', 'model',  TRUE, TRUE, TRUE, TRUE, TRUE);
                    $dbtables['news']     = array('news_id',    'header', TRUE, FALSE, FALSE, FALSE, FALSE);
                    $dbtables['articles'] = array('article_id', 'header', TRUE, FALSE, FALSE, FALSE, FALSE);
                }
                $authorized = $this->existsUser();
                foreach ($dbtables as $dbtable => $value) {
                    $name = $value[1];
                    $higlight = !empty($value[2]) ? '`' . $dbtable . '`.`highlighted`, '
                                                  : '';
                    $hit = !empty($value[3]) ? '`' . $dbtable . '`.`hit`, '
                                             : '';
                    $newest = !empty($value[4]) ? '`' . $dbtable . '`.`newest`, '
                                                : '';
                    $actional = !empty($value[5]) ? '`' . $dbtable . '`.`actional`, '
                                                  : '';
                    $awaited = !empty($value[6]) ? '`' . $dbtable . '`.`awaited`, '
                                                 : '';
                    $query = 'SELECT `' . $dbtable . '`.`' . $value[0] . '`, '
                                  . '`' . $dbtable . '`.`' . $name . '`, '
                                  . $hit
                                  . $newest
                                  . $actional
                                  . $highlight
                                  . '`' . $dbtable . '`.`url`, '
                                  . '`' . $dbtable . '`.`url_special`, '
                                  . '`' . $dbtable . '`.`meta_title` '
                           . 'FROM `' . $dbtable . '` '
                           . 'WHERE `' . $dbtable . '`.`enabled` = 1 '
                                 . ($authorized ? '' : 'AND `' . $dbtable . '`.`hidden` = 0 ')
                           . 'GROUP BY `' . $dbtable . '`.`' . $name . '` '
                           . 'ORDER BY `' . $dbtable . '`.`' . $name . '` ASC;';
                    $result = $this->cms->db->query($query);
                    if ($result !== FALSE) {
                        while ($item = $this->cms->db->fetch_object($result)) {
                            $value = $this->cms->text->asPlainSentence($item->$name);
                            $value = preg_replace('/^[^a-zа-яё]+/iu', '', $value);
                            if ($this->isActualString($value)) {
                                $item->original_name = $item->$name;
                                $item->$name = $value;
                                $value = $this->cms->text->lowerCase($value);
                                $char = $this->cms->text->substr($value, 0, 1);
                                if (!isset($items[$char])) $items[$char] = array();
                                $items[$char][$value] = $item;
                            }
                        }
                        $this->cms->db->free_result($result);
                    }
                }
                ksort($items, SORT_STRING);
                foreach ($items as & $item) ksort($item, SORT_STRING);
                return $items;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить курсовой коэффициент этой валюты.                 |
                    |                                                             |
                    ============================================================ */

            public function getPaymentRate ( $item ) {
                return $this->cms->any->currency->rate($item, TRUE);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список баннеров в этой папке сайта.               |
                    |                                                             |
                    ============================================================ */

            public function getBanners ( $folder = null ) {
                $items = array();
                $helper = $this->getVar('helper');
                if (is_object($helper)) {
                    $path = dirname(__FILE__) . '/../../../';
                    $folder = is_string($folder) ? $this->cms->hdd->safeFilename($folder, FALSE)
                                                 : '';
                    $folder = 'files/banners/' . ltrim($folder, '/');
                    $handle = @ opendir($path . $folder);
                    if ($handle !== FALSE) {
                        $folder = rtrim($folder, '/') . '/';
                        $params = array('root' => TRUE);
                        $site = $helper->site($params, $this->cms->smarty);
                        $validTypes = array('png', 'jpg', 'jpeg', 'gif', 'bmp', 'svg');
                        while (($file = readdir($handle)) !== FALSE) {
                            if (trim($file, '.') != '') {
                                $ext = $this->getFileExt($file);
                                if (in_array($ext, $validTypes)) {
                                    if (!$this->isThumbnail($file)) {
                                        if ($this->cms->hdd->isReadableFile($path . $folder . $file)) {
                                            $data = new stdClass;
                                            $this->cms->get_file_record($path . $folder . $file, $data);

                                            // только разрешенные (включенные) слайды
                                            if (!empty($data->enabled)) {
                                                $link = isset($data->link) ? $data->link : '';
                                                $file = $folder . $file;

                                                $item = new stdClass;
                                                $item->name = isset($data->title) ? $data->title : '';
                                                $item->descr = isset($data->description) ? $data->description : '';
                                                $item->price = isset($data->price) ? $data->price : '';
                                                $item->link = $this->asAttribute($link);
                                                $item->image = $site . $this->asAttribute($file);
                                                $items[] = $item;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        @ closedir($handle);
                        if (!empty($items)) $this->sortBy($items, 'image');
                    }
                }
                return $items;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список изображений в этой папке шаблона.          |
                    |                                                             |
                    ============================================================ */

            public function getImages ( $folder = null ) {
                $items = array();
                $path = dirname(__FILE__) . '/../';
                $folder = is_string($folder) ? $this->cms->hdd->safeFilename($folder, FALSE)
                                             : '';
                $folder = 'images/' . ltrim($folder, '/');
                $handle = @ opendir($path . $folder);
                if ($handle !== FALSE) {
                    $folder = rtrim($folder, '/') . '/';
                    $theme = $this->getThemeUrl();
                    $validTypes = array('png', 'jpg', 'jpeg', 'gif', 'bmp', 'svg');
                    while (($file = readdir($handle)) !== FALSE) {
                        if (trim($file, '.') != '') {
                            $ext = $this->getFileExt($file);
                            if (in_array($ext, $validTypes)) {
                                if (!$this->isThumbnail($file)) {
                                    if ($this->cms->hdd->isReadableFile($path . $folder . $file)) {
                                        $file = $folder . $file;
                                        $items[] = $theme . $this->asAttribute($file);
                                    }
                                }
                            }
                        }
                    }
                    @ closedir($handle);
                    if (!empty($items)) sort($items, SORT_STRING);
                }
                return $items;
            }

                    /* ============================================================
                    |                                                             |
                    |  Подготовить запись способа доставки для корзины, которая   |
                    |  оперирует связью со способами оплаты.                      |
                    |                                                             |
                    ============================================================ */

        public function fixDelivery ( & $item ) {
            if (!isset($item->types_ids) || !is_string($item->types_ids)) return $item;
            $item2 = $item;
            $this->cms->db->fix_deliveries_record($item2);
            return $item2;
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести цену целым числом с отступом тысяч.                |
                    |                                                             |
                    ============================================================ */

        public function easyPrice ( $price ) {
            $price = '' . intval($price);
            $price = preg_replace('/^([0-9]+)([0-9][0-9][0-9])$/', '$1 $2', $price);
            return preg_replace('/^(-)(.+)$/', '$1 $2', $price);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести сумму двух величин целым числом с отступом тысяч.  |
                    |                                                             |
                    ============================================================ */

            public function easySum ( $v1, $v2, $negative = FALSE ) {
                $v1 = intval(is_string($v1) ? preg_replace('/\s+/', '', $v1) : $v1);
                $v2 = intval(is_string($v2) ? preg_replace('/\s+/', '', $v2) : $v2);
                $v1 = $negative ? ($v1 - $v2) : ($v1 + $v2);
                return $this->easyPrice($v1);
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить к первой величине вторую целым числом с отступом  |
                    |  тысяч.                                                     |
                    |                                                             |
                    ============================================================ */

            public function easyPlus ( & $v1, $v2, $negative = FALSE ) {
                $v1 = $this->easySum($v1, $v2, $negative);
                return '';
            }

        /* ========================================================================
        |                                                                         |
        |    Личные методы публичных функций.                                     |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Используются только для внутренних целей хелпера. Намеренно сделаны  |
        |    приватными или защищенными, чтобы не оказались доступными в шаблоне, |
        |    так как служат лишь выполнению технических операций низкого уровня.  |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Обработчик операции сравнения.                             |
                    |                                                             |
                    ============================================================ */

        protected function compareBy ( $left, $right ) {
            $field = $this->sortByFieldName;
            $v1 = empty($this->sortByDescending) ? -1 : 1;
                $left = isset($left->$field) ? $left->$field
                                             : (isset($left[$field]) ? $left[$field] : null);
                if (is_null($left)) return $v1;
            $v2 = empty($this->sortByDescending) ? 1 : -1;
                $right = isset($right->$field) ? $right->$field
                                               : (isset($right[$field]) ? $right[$field] : null);
                if (is_null($right)) return $v2;
            if ($left == $right) return 0;
            return $left < $right ? $v1 : $v2;
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в папке сайта (или шаблона, если папка не     |
                    |  указана или равна images, img, js, css, fonts, html, xml,  |
                    |  db) есть такой файл. И еще во входной параметр $fileUri    |
                    |  вернуть абсолютный путь файла.                             |
                    |                                                             |
                    ============================================================ */

        protected function existsFile ( $file, $validTypes = array(), $folder = null, & $fileUri = '' ) {
            $file = $this->cms->hdd->safeFilename($file, FALSE);    // позволяем и путь вглубь папки
                $fileUri = '';
            if (empty($file)) return FALSE;
            $ext = $this->getFileExt($file);
            if (!in_array($ext, $validTypes)) return FALSE;
            $folder = is_string($folder) ? $this->cms->hdd->safeFilename($folder, FALSE)
                                         : 'html';
            $ext = $this->cms->text->lowerCase($folder);
            switch ($ext) {
                case '':
                    $ext = 'html';
                case 'css':
                case 'fonts':
                case 'html':
                case 'xml':
                case 'images':
                case 'img':
                case 'js':
                case 'db':
                    $folder = '/../' . $ext;
                    break;
                default:
                    $folder = rtrim('/../../../files/' . $folder, '/. ');
            }
            $fileUri = dirname(__FILE__) . $folder . '/' . $file;
            return $this->cms->hdd->isReadableFile($fileUri);
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в папке сайта (или шаблона, если папка не     |
                    |  указана) есть такое изображение (файл).                    |
                    |                                                             |
                    ============================================================ */

            protected function existsPhoto ( $file, $folder = null ) {
                $types = array('png', 'jpg', 'jpeg', 'gif', 'bmp', 'svg');
                return $this->existsFile($file, $types, $folder);
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить файл шаблона.                                   |
                    |                                                             |
                    ============================================================ */

        protected function includeModule ( $file, & $params, & $smarty ) {
            $value = '';
            if ($this->existsModule($file)) $value = $smarty->fetch($file);
            return $this->assignVar($params, $value, $smarty);
        }

            protected function includeModuleNumerated ( $templateName, & $params, & $smarty ) {
                $file = $this->buildModuleFilename($templateName, $params);
                return $this->includeModule($file, $params, $smarty);
            }

            protected function buildModuleFilename ( $templateName, & $params ) {
                $num = $this->paramAsNatural('num', $params);
                $num = $num > 1 ? ('-' . $num) : '';
                $this->moduleSuffix = $this->mustbeArray($this->moduleSuffix);
                $suffix = end($this->moduleSuffix);
                $suffix = $this->mustbeString($suffix);
                return $templateName . $suffix . $num . '.htm';
            }

                    /* ============================================================
                    |                                                             |
                    |  Подключить модуль шаблона (файл mod-... в корне папки      |
                    |  макетов шаблона или файл modules/... шаблона).             |
                    |                                                             |
                    ============================================================ */

            protected function includeMod ( $file, & $params, & $smarty ) {
                $path = 'mod-';
                if (!$this->existsModule($path . $file)) $path = 'modules/';
                return $this->includeModule($path . $file, $params, $smarty);
            }

                protected function includeModNumerated ( $templateName, & $params, & $smarty ) {
                    $file = $this->buildModuleFilename($templateName, $params);
                    return $this->includeMod($file, $params, $smarty);
                }

                    /* ============================================================
                    |                                                             |
                    |  Вывести переменную.                                        |
                    |                                                             |
                    ============================================================ */

        protected function echoValue ( $value, & $params, & $smarty, $escaped = TRUE ) {
            $value = $this->mustbeString($value);
            if ($escaped) $value = $this->asAttribute($value);
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести константу.                                         |
                    |                                                             |
                    ============================================================ */

            protected function echoConstant ( $name, & $params, & $smarty ) {
                $value = $this->getConst($name);
                return $this->echoValue($value, $params, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести переменную.                                        |
                    |                                                             |
                    ============================================================ */

            protected function echoVariable ( $name, & $params, & $smarty, $escaped = TRUE ) {
                $value = $this->getVar($name, $smarty);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести настройку.                                         |
                    |                                                             |
                    ============================================================ */

            protected function echoSetting ( $name, & $params, & $smarty, $escaped = TRUE ) {
                $method = $escaped ? 'getAsSentence' : 'get';
                $value = $this->cms->any->settings->$method($name);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести конфигурационный параметр.                         |
                    |                                                             |
                    ============================================================ */

            protected function echoConfig ( $name, & $params, & $smarty, $escaped = TRUE ) {
                $value = $this->getConfig($name);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести конфигурационный параметр (нумерованный).          |
                    |                                                             |
                    ============================================================ */

            protected function echoConfigNumerated ( $name, & $params, & $smarty, $escaped = TRUE ) {
                $value = $this->getConfigNumerated($name, $params);
                return $this->echoValue($value, $params, $smarty, $escaped);
            }

                    /* ============================================================
                    |                                                             |
                    |  Вывести поле записи.                                       |
                    |  ---------------------------------------------------------  |
                    |  Анализируемые параметры (по порядку обхода):               |
                    |      from                                                   |
                    |          var                                                |
                    |          assign                                             |
                    |          result                                             |
                    |          scope                                              |
                    |                                                             |
                    ============================================================ */

            protected function echoField ( $name, & $params, & $smarty, $def = '', $escaped = TRUE ) {
                $value = $this->getItemField($params, $smarty, $name, $def);
                if ($escaped) $value = $this->asAttribute($value);
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |    Сохранение измененных настроек шаблона.                  |
                    |    -------------------------------------------------------  |
                    |                                                             |
                    |    Каждый параметр поступает через POST с помощью полей,    |
                    |    например:                                                |
                    |                                                             |
                    |        <input name="tplSetupParams[param_name]" value="">   |
                    |                                                             |
                    ============================================================ */

        protected function saveTemplateSetupParams () {
            $name = 'tplSetupParams';
            if ($this->cms->request->isPostArray($name)) {
                $items = $this->cms->request->getPost($name);
                $changed = FALSE;
                foreach ($items as $name => $value) {
                    if ($this->validTemplateSetupField($name)) {
                        $old = $this->getConfig($name, null);
                        if (is_null($old) || $old != $value) {
                            $this->setConfig($name, $value);
                            $changed = TRUE;
                        }
                    }
                }
                if ($changed) $this->saveConfig();
            }
        }

            protected function validTemplateSetupField ( $name ) {
                if ($this->isActualString($name)) {
                    if (isset($this->tplSetupParams)) {
                        if ($this->isActualArray($this->tplSetupParams)) {
                            if (isset($this->tplSetupParams[$name])) return TRUE;
                            foreach ($this->tplSetupParams as $idx => $value) {
                                if (is_int($idx) && $name === $value) return TRUE;
                            }
                        }
                    }
                }
                return FALSE;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение входного параметра MARK.                 |
                    |                                                             |
                    ============================================================ */

        protected function getMarkParam ( & $params, $def = 'selected' ) {
            $mark = $this->paramAsSentence('mark', $params);
            return $mark != '' ? $mark : $def;
        }

            protected function getMarkParamAsChecked ( & $params ) {
                return $this->getMarkParam($params, 'checked');
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение переменной как сумму.                    |
                    |                                                             |
                    ============================================================ */

        protected function getPrice ( $varname, & $smarty, $quantity = 1 ) {
            $value = $this->getVar($varname, $smarty, 0);
            $value = $this->getValueAsPrice($value, $quantity);
            return $this->easyPrice($value);
        }

            protected function getValueAsPrice ( $value, $quantity = 1 ) {
                $value = $value * abs($quantity);
                $method = 'priceForScreen';
                $helper = $this->getVar('helper');
                if ($this->cms->any->hasMethod($helper, $method)) {
                    $value = $helper->$method($value);
                }
                return $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение переменной как целое число.              |
                    |                                                             |
                    ============================================================ */

        protected function getInteger ( $varname, & $smarty, $def = 0 ) {
            $value = $this->getVar($varname, $smarty, $def);
            return intval($value);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение переменной как натуральное число.        |
                    |                                                             |
                    ============================================================ */

        protected function getNumber ( $varname, & $smarty, $def = 0 ) {
            $value = $this->getVar($varname, $smarty, $def);
            return max(0, $value);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение переменной как действительное число.     |
                    |                                                             |
                    ============================================================ */

        protected function getFloat ( $varname, & $smarty, $def = 0.0 ) {
            $value = $this->getVar($varname, $smarty, $def);
            return $this->cms->number->floatValue($value);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить индекс текущего элемента в массиве, перечисляемом |
                    |  в этот момент с помощью функции enum***.                   |
                    |                                                             |
                    ============================================================ */

        protected function getEnumIdx () {
            if (empty($this->enumStack) || !is_array($this->enumStack)) return 0;
            $index = count($this->enumStack) - 1;
            return key($this->enumStack[$index]);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить номер итерации функции enum***.                   |
                    |                                                             |
                    ============================================================ */

        protected function getEnumNum ( & $prev = 0, & $next = 0 ) {
            $prev = 0;
            $next = 0;
            if (empty($this->enumStack) || !is_array($this->enumStack)
            || empty($this->enumStackNums) || !is_array($this->enumStackNums)
            || count($this->enumStack) != count($this->enumStackNums)) return 0;
            $index = count($this->enumStackNums) - 1;
            $value = intval($this->enumStackNums[$index]);
            $value = max(1, $value);
                $index = count($this->enumStack[$index]);
                $prev = $value - 1;
                    if ($prev < 1) $prev = $index;
                $next = $value + 1;
                    if ($next > $index) $next = 1;
            return $value;
        }

                    /* ============================================================
                    |                                                             |
                    |  Взять базовые параметры функции enum***.                   |
                    |                                                             |
                    ============================================================ */

        protected function takeEnumParams ( & $params ) {
            $params2 = array();
            $this->copyParam('from', $params, $params2);
            $this->copyParam('via', $params, $params2);
            $this->copyParam('max', $params, $params2);
            return $params2;
        }

                    /* ============================================================
                    |                                                             |
                    |  Взять параметр from некоторой функции.                     |
                    |  ---------------------------------------------------------  |
                    |  Анализируемые параметры (по порядку обхода):               |
                    |      from                                                   |
                    |                                                             |
                    ============================================================ */

        protected function takeFromParams ( & $params, $def = 'item' ) {
            $def = $this->mustbeString($def);
            $from = $this->paramAsSentence('from', $params, $def);
            return array('from' => $from);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение поля этой записи.                        |
                    |  ---------------------------------------------------------  |
                    |  Анализируемые параметры (по порядку обхода):               |
                    |      from                                                   |
                    |                                                             |
                    ============================================================ */

        protected function getItemField ( $params = null, & $smarty = null, $fields = '', $def = '' ) {
            if (is_string($fields) && !empty($fields)) $fields = array($fields);
            if (!empty($fields) && is_array($fields)) {
                $item = $this->retrieveItem($params, $smarty);
                foreach ($fields as $field) {
                    if (isset($item->$field)) {
                        $test = trim($item->$field);
                        if (!empty($test)) return $item->$field;
                    }
                }
            }
            return $def;
        }

            protected function getField ( & $item, $field, $def = '' ) {
                if (is_object($item)) return isset($item->$field) ? $item->$field : $def;
                return is_array($item) && isset($item[$field]) ? $item[$field] : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение флага этой записи.                       |
                    |                                                             |
                    ============================================================ */

            protected function getItemFlag ( $params, & $smarty, $field ) {
                $item = $this->retrieveItem($params, $smarty);
                return !empty($item->$field) ? 1 : 0;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить массив уникальных строк поля этой записи.         |
                    |                                                             |
                    ============================================================ */

            protected function getItemStringArray ( $params, & $smarty, $fields ) {
                $value = array();
                if (is_string($fields) && !empty($fields)) $fields = array($fields);
                if (!empty($fields) && is_array($fields)) {
                    $except = $this->paramAsArray('except', $params);
                    $varname = $this->retrieveFrom($params, 'item');
                    $item = $this->getVar($varname, $smarty);
                    $value = '';
                    foreach ($fields as $field) {
                        if ($value != '') $value .= ',';
                        $value .= $this->getField($item, $field);
                    }
                    $value = $this->asUniqueStringArray($value, FALSE, $except);
                }
                if (empty($value)) $value = $this->paramAsArray('def', $params);
                return $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь имя переменной из параметра вызова from.           |
                    |  ---------------------------------------------------------  |
                    |  Анализируемые параметры (по порядку обхода):               |
                    |      from                                                   |
                    |                                                             |
                    ============================================================ */

            protected function retrieveFrom ( & $params, $def = '' ) {
                return $this->paramAsSentence('from', $params, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь запись из параметров вызова from или item.         |
                    |  ---------------------------------------------------------  |
                    |  Анализируемые параметры (по порядку обхода):               |
                    |      from                                                   |
                    |                                                             |
                    ============================================================ */

            protected function retrieveItem ( & $params, & $smarty ) {
                $varname = $this->retrieveFrom($params, 'item');
                return $this->getVar($varname, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Выполнить дефолтную функцию N с неким from (может быть     |
                    |  указан массивом, тогда первым подходящим from).            |
                    |                                                             |
                    ============================================================ */

        protected function runFunction ( $name, & $params, & $smarty, $from, $defFrom = '' ) {
            $value = !empty($defFrom) && is_string($defFrom) ? $this->getVar($defFrom, $smarty, '')
                                                             : '';
            if (!is_array($from)) $from = array($from);
            foreach ($from as $varname) {
                if ($this->existsVar($varname, $smarty)) {
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $name)) {
                        $params2 = array('from' => $varname,
                                         'def' => $value);
                        $value = $helper->$name($params2, $smarty);
                    }
                    break;
                }
            }
            return $this->assignVar($params, $value, $smarty);
        }

                    /* ============================================================
                    |                                                             |
                    |  Выполнить дефолтную функцию NAME с неким from(-ами).       |
                    |                                                             |
                    ============================================================ */

            protected function runName ( & $params, & $smarty, $from, $defFrom = '' ) {
                return $this->runFunction('name', $params, $smarty, $from, $defFrom);
            }

                    /* ============================================================
                    |                                                             |
                    |  Выполнить дефолтную функцию BODY с неким from(-ами).       |
                    |                                                             |
                    ============================================================ */

            protected function runBody ( & $params, & $smarty, $from ) {
                return $this->runFunction('body', $params, $smarty, $from);
            }

                    /* ============================================================
                    |                                                             |
                    |  Выполнить дефолтную функцию FINDIMAGE, но вернуть URL.     |
                    |                                                             |
                    ============================================================ */

            protected function runFindImage ( & $params, & $smarty, $type, $forThumb = TRUE ) {
                $value = '';
                $name = 'findImage';
                $helper = $this->getVar('helper');
                if ($this->cms->any->hasMethod($helper, $name)) {
                    $type = $this->cms->text->lowerCase($type);
                    $params2 = array('type' => $type,
                                     'view' => TRUE);
                    $this->copyParam('from', $params, $params2);
                    $this->copyParam('num', $params, $params2);
                    $this->copyParam('view', $params, $params2);
                    $this->copyParam('def', $params, $params2);
                    $data = $helper->$name($params2, $smarty);
                    $this->supposeDemoImage($data, ucfirst($type));
                    if ($forThumb && !empty($data['thumb'])) $value = $data['thumb'];
                    elseif (!empty($data['url'])) $value = $data['url'];
                }
                return $this->assignVar($params, $value, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить базовые установки фильтра товаров.                |
                    |                                                             |
                    ============================================================ */

        protected function makeFilter ( $forProducts = FALSE, $strong = TRUE ) {
            $filter = new stdClass;
            $filter->sort = 0;
            $filter->sort_direction = 0;
            $filter->sort_laconical = 1;
            $filter->deleted = 0;
            $filter->enabled = 1;
            if (!$this->existsUser()) $filter->hidden = 0;
            if ($forProducts) {
                $filter->discount = $this->getUserDiscount();
                $filter->price_id = $this->getUserPriceId();
                if ($strong) {
                    $filter->search_cost_from = 0.01;
                    $filter->search_amount_from = 1;
                    $filter->non_usable = -1;
                }
            }
            return $filter;
        }

                    /* ============================================================
                    |                                                             |
                    |  Прочесть товары согласно указанному фильтру.               |
                    |                                                             |
                    ============================================================ */

        protected function readProducts ( & $filter ) {
            $items = null;
            $this->cms->db->products->get($items, $filter);
            $this->cms->db->products->unpackRecords($items, $filter);
            return $items;
        }

                    /* ============================================================
                    |                                                             |
                    |  Добавить один список товаров в другой, кроме повторов.     |
                    |                                                             |
                    ============================================================ */

            protected function mergeProducts ( & $src, & $dest ) {
                $changed = FALSE;
                if (!is_array($dest)) $dest = array();
                $known = array();
                foreach ($dest as & $item) {
                    if (!empty($item->product_id)) {
                        $id = $item->product_id;
                        $known[$id] = TRUE;
                    }
                }
                if (!empty($src->product_id)) $src = array($src);
                if (is_array($src)) {
                    foreach ($src as & $item) {
                        if (!empty($item->product_id)) {
                            $id = $item->product_id;
                            if (empty($known[$id])) {
                                $dest[$id] = $item;
                                $known[$id] = TRUE;
                                $changed = TRUE;
                            }
                        }
                    }
                }
                return $changed;
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить один список публикаций в другой, кроме повторов.  |
                    |                                                             |
                    ============================================================ */

            protected function mergePublications ( & $src, & $dest,
                                                   $andArticles = TRUE,
                                                   $andNews = TRUE,
                                                   $andProducts = TRUE,
                                                   $andBrands = TRUE,
                                                   $andCategories = TRUE ) {
                $changed = FALSE;
                if (!is_array($dest)) $dest = array();
                $known = array();
                foreach ($dest as & $item) {
                    if ($andArticles && !empty($item->article_id)) $id = $item->article_id . 'a';
                    elseif ($andNews && !empty($item->news_id)) $id = $item->news_id . 'n';
                    elseif ($andProducts && !empty($item->product_id)) $id = $item->product_id . 'p';
                    elseif ($andBrands && !empty($item->brand_id)) $id = $item->brand_id . 'b';
                    elseif ($andCategories && !empty($item->category_id)) $id = $item->category_id . 'c';
                    else continue;
                    $known[$id] = TRUE;
                }
                if (!empty($src->article_id)
                || !empty($src->news_id)
                || !empty($src->product_id)
                || !empty($src->brand_id)
                || !empty($src->category_id)) $src = array($src);
                if (is_array($src)) {
                    foreach ($src as & $item) {
                        if ($andArticles && !empty($item->article_id)) $id = $item->article_id . 'a';
                        elseif ($andNews && !empty($item->news_id)) $id = $item->news_id . 'n';
                        elseif ($andProducts && !empty($item->product_id)) $id = $item->product_id . 'p';
                        elseif ($andBrands && !empty($item->brand_id)) $id = $item->brand_id . 'b';
                        elseif ($andCategories && !empty($item->category_id)) $id = $item->category_id . 'c';
                        else continue;
                        if (empty($known[$id])) {
                            $dest[$id] = $item;
                            $known[$id] = TRUE;
                            $changed = TRUE;
                        }
                    }
                }
                return $changed;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить URL начала текущей страницы.                      |
                    |                                                             |
                    ============================================================ */

        protected function getUrlBegin ( & $smarty, $plusQuery = '' ) {
            $tpl = $this->getTemplateName($smarty);
            if ($tpl == 'catalog') {
                $mode = $this->requestParam('mode');
                $mode = $this->asAttribute($mode);
                $url = $this->cms->security->siteUrl();
                $url .= $tpl . '?mode=' . $mode;
            } else $url = $this->getCanonicalUri(FALSE);
            $separator = preg_match('/\?/u', $url) ? '&' : '?';
            return $url . ($this->isActualString($plusQuery) ? $separator . $plusQuery : '');
        }

        /* ========================================================================
        |                                                                         |
        |    Системные методы.                                                    |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Используются только для внутренних целей хелпера. Намеренно сделаны  |
        |    приватными или защищенными, чтобы не оказались доступными в шаблоне, |
        |    так как служат лишь выполнению технических операций низкого уровня.  |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Конструктор хелпера.                                       |
                    |                                                             |
                    ============================================================ */

        public function __construct ( & $cms ) {
            $this->cms = & $cms;
            $this->antispam($cms);
            $this->setSmartyPlugins($cms);
            $cms->smarty->registerDefaultPluginHandler(array($this, 'handlerUndefinedTag'));
                $this->resetCounter();
            $this->doAdminActions();
        }

                    /* ============================================================
                    |                                                             |
                    |  Расширить новыми командами список команд PHP, которые      |
                    |  допускаем использовать в шаблоне.                          |
                    |                                                             |
                    ============================================================ */

            protected function expandPhp ( $commands ) {
                if ($this->isActualArray($commands)) {
                    $list = & $this->cms->smarty->security_policy->php_functions;
                    foreach ($commands as $name) {
                        if ($this->isActualString($name)) {
                            if (!in_array($name, $list)) {
                                $list[] = $name;
                            }
                        }
                    }
                }
            }

                    /* ============================================================
                    |                                                             |
                    |  Расширить новыми тегами список шаблонизационных тегов.     |
                    |                                                             |
                    ============================================================ */

            protected function expandSmarty ( $tags, $type = 'function' ) {
                if (!$this->isActualArray($this->templateFunctions)) $this->templateFunctions = array();
                if ($this->isActualArray($tags)) {
                    foreach ($tags as $name => $descr) {
                        if (is_integer($name)) {
                            $name = $descr;
                            $descr = '....... нет описания';
                        }
                        if ($this->isActualString($name)) {
                            if ($this->cms->any->hasMethod($this, $name)) {
                                $this->templateFunctions[$name] = $descr;
                                $this->cms->smarty->registerPlugin($type, $name, array($this, $name));
                            }
                        }
                    }
                }
            }

                    /* ============================================================
                    |                                                             |
                    |  Антиспам постинговых каналов.                              |
                    |                                                             |
                    ============================================================ */

            protected function antispam ( & $cms ) {
                $url = $this->getModuleName();
                switch ($url) {
                    case 'feedback':
                    case 'registration':
                    case 'callme':
                        break;
                    default:
                        $url = $this->existsAjaxSignal() ? 'analytic'
                                                         : $this->abstractUri();
                }
                switch ($url) {
                    case 'analytic':
                    case 'informer':
                    case 'setup':
                        break;
                    default:
                        $this->antispamChannel($cms, 'Feedback', 'feedback_message');
                        $this->antispamChannel($cms, 'Registration', 'registration_email');
                        $this->antispamChannel($cms, 'Callme', 'callme_phone');
                        $this->antispamChannel($cms, 'Comments', 'post_message');
                }
            }

                protected function antispamChannel ( & $cms, $channelName, $fieldName ) {
                    $param = 'antispam' . $channelName . 'Name';
                    if ($this->getConfigAsBoolean('antispamEnabled')) {
                        if ($cms->request->existsPost($fieldName)) {
                            $value = $this->getSession($param, FALSE);
                            if ($value === FALSE || !$cms->request->existsPost($value)) {
                                $_POST['captcha'] = '';
                                $_REQUEST['captcha'] = '';
                            }
                        }
                    }
                    $value = $this->asHash(uniqid());
                    $cms->session->set($param, $value);
                    $cms->smarty->assignByRef($param, $value);
                }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что существует признак ajax-вызова.               |
                    |                                                             |
                    ============================================================ */

                protected function existsAjaxSignal ( & $smarty = null ) {
                    return !empty($this->cms->quick_content)
                           || $this->cms->request->getGPrequestAsBoolean('ajax')
                           || $this->cms->request->getGPrequestAsBoolean('pseudoAjax')
                           || $this->cms->request->getGPrequestAsBoolean('dynamic')
                           || $this->getVar(array('quick_content', 'ajax'), $smarty) == TRUE;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить исходный адрес текущей страницы.                  |
                    |                                                             |
                    ============================================================ */

            protected function originalUri () {
                return $this->cms->request->getServerAsSentence('REQUEST_URI');
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить относительный адрес текущей страницы (без         |
                    |  GET-параметров и хеша).                                    |
                    |                                                             |
                    ============================================================ */

            protected function requestUri () {
                $uri = $this->originalUri();
                $uri = @ parse_url($uri);
                $uri = isset($uri['path']) ? trim($uri['path']) : '';
                $uri = preg_replace('![ \t\r\n\./\\\\]+$!u', '', $uri);
                $uri = preg_replace('![/\\\\][ \t\r\n]+!u', '/', $uri);
                $uri = preg_replace('![ \t\r\n\.]+[/\\\\]!u', '/', $uri);
                return preg_replace('![/\\\\]+!u', '/', $uri);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить из адреса текущей страницы путевое имя в шаблоне. |
                    |                                                             |
                    ============================================================ */

            protected function abstractUri () {
                $uri = $this->requestUri();
                $uri = preg_replace('!^/+!u', '', $uri);
                $uri = explode('/', $uri);
                    $root = $this->cms->root_dir;
                    $root = preg_replace('![/\\\\]+!u', '/', $root);
                    $root = preg_replace('!^/+!u', '', $root);
                    $root = preg_replace('!/+$!u', '', $root);
                    if ($this->isActualString($root)) {
                        $root = explode('/', $root);
                        $root = count($root);
                    } else $root = 0;
                while ($root > 0) {
                    array_shift($uri);
                    $root--;
                }
                $uri = is_array($uri) ? implode('/', $uri) : '';
                    $patterns = array(
                        '/town[_\-][^/]*',
                        '/filter[_\-][^/]*',
                        '/page[_\-][0-9]*'
                    );
                    $uri = preg_replace('!(' . implode('|', $patterns) . ')+$!ui', '', $uri);
                return $this->cms->text->lowerCase($uri);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить из адреса текущей страницы имя файла в шаблоне.   |
                    |                                                             |
                    ============================================================ */

                protected function abstractTpl ( $nearest = TRUE, $asUrlPart = FALSE, & $urlParams = FALSE ) {
                    $urlParams = array();
                    $path = 'abstract/';
                    $file = $this->abstractUri();
                    $ext = '.htm';
                    $mod = $path . $file . $ext;
                    if ($nearest) {
                        while ($this->isActualString($file) && !$this->existsModule($mod)) {
                            $file = explode('/', $file);
                            array_unshift(
                                $urlParams,
                                array_pop($file)
                            );
                            $file = implode('/', $file);
                            $mod = $path . $file . $ext;
                        }
                    }
                    return $asUrlPart ? $file
                                      : $mod;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить канонический адрес текущей страницы от корневой   |
                    |  папки сайта.                                               |
                    |                                                             |
                    ============================================================ */

                protected function getRootUri () {
                    $uri = $this->getCanonicalUri();
                        $uri = $this->cms->text->lowerCase($uri);
                        $uri = preg_replace('![/\\\\]+!u', '/', $uri);
                    $root = $this->cms->root_dir;
                        $root = preg_replace('![/\\\\]+!u', '/', $root);
                        $root = preg_replace('!^/+!u', '', $root);
                        $root = preg_replace('!/+$!u', '', $root);
                    if ($this->isActualString($root)) {
                        $root = '/' . $this->cms->text->lowerCase($root) . '/';
                        $size = $this->cms->text->length($root);
                        if ($this->cms->text->substr($uri, 0, $size) == $root) {
                            $uri = '/' . $this->cms->text->substr($uri, $size);
                        }
                    }
                    return $uri;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить канонический адрес текущей страницы.              |
                    |                                                             |
                    ============================================================ */

                protected function getCanonicalUri ( $absolute = TRUE ) {
                    $value = '';
                    $method = 'requestUri';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $trimmer = '~[/\\\\]+(\?.*)?$~us';
                        $changing = '$1';
                        $params = array('nopages' => TRUE);
                        if ($absolute) {
                            $params['except'] = '*';
                            $trimmer = '~[/\\\\]+$~u';
                            $changing = '';
                        }
                        $value = $helper->$method($params, $this->cms->smarty);
                        $value = preg_replace($trimmer, $changing, $value);
                        if ($value == '') $value = '/';
                        elseif (substr($value, 0, 1) == '?') $value = '/' . $value;
                    }
                    return $value;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить RegExp-шаблон символов, используемых в доменном   |
                    |  имени.                                                     |
                    |                                                             |
                    ============================================================ */

                protected function regexpDomainChars ( $mustbeDot = TRUE, $mustbeDash = TRUE ) {
                    $value = 'a-z0-9а-яѓђљњќћџўіґёєјѕї';
                    if ($mustbeDot) $value .= '\.';
                    if ($mustbeDash) $value .= '\-';
                    return $value;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя домена из этого URL.                          |
                    |                                                             |
                    ============================================================ */

                protected function getUrlDomain ( $url, $maybeBad = FALSE, $inLowercase = TRUE ) {
                    $pattern = '~^(([a-z\-]+:[/\\\\]*|[/\\\\][/\\\\]+)(www\.)*|(www\.)+)~ui';
                    $value = preg_replace($pattern, '', $url);
                    if ($maybeBad || $value != $url) {
                        $pattern = '~^([^\?\s#:/\\\\]+).*?$~uis';
                        if (preg_match($pattern, $value)) {
                            $value = preg_replace($pattern, '$1', $value);
                            $value = preg_replace('~^[^' . $this->regexpDomainChars(FALSE, FALSE) . ']+~ui', '', $value);
                            $value = preg_replace('~[^' . $this->regexpDomainChars(FALSE, FALSE) . ']+$~ui', '', $value);
                            if ($inLowercase) $value = $this->cms->text->lowerCase($value);
                        } else $value = '';
                    } else $value = '';
                    return $value;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя домена нашего сайта.                          |
                    |                                                             |
                    ============================================================ */

                protected function getSiteDomain () {
                    $value = $this->getSiteUrl();
                    return $this->getUrlDomain($value);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить URL корневой папки сайта.                         |
                    |                                                             |
                    ============================================================ */

                protected function getSiteUrl ( $closed = TRUE ) {
                    $value = '';
                    $method = 'site';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $params = array();
                        $value = $helper->$method($params, $this->cms->smarty);
                        if (!$closed) $value = preg_replace('~[/\\\\]+$~u', '', $value);
                        $this->mustbeAbsoluteUrl($value);
                        $this->maybeHttps($value);
                    }
                    return $value;
                }

                    /* ============================================================
                    |                                                             |
                    |  Убедиться что URL содержит и протокол.                     |
                    |                                                             |
                    ============================================================ */

                    private function mustbeAbsoluteUrl ( & $url ) {
                        $char = substr($url, 0, 1);
                        if ($char == '/' || $char == '\\') {
                            $protocol = 'http';
                                $char = $this->cms->request->getServerAsSentence('HTTPS');
                                if ($char && $char != 'off') $protocol = 'https';
                            $url = $protocol . ':' . $url;
                        }
                    }

                    /* ============================================================
                    |                                                             |
                    |  Скорректировать протокол URL, если смотрим сайт по HTTPS.  |
                    |                                                             |
                    ============================================================ */

                    private function maybeHttps ( & $url ) {
                        $protocol = substr($url, 0, 5);
                        if ($this->cms->text->lowerCase($protocol) == 'http:') {
                            $char = $this->cms->request->getServerAsSentence('HTTPS');
                            if ($char && $char != 'off') $url = 'https:' . substr($url, 5);
                        }
                    }

                    /* ============================================================
                    |                                                             |
                    |  Получить URL корневой папки текущего дизайна.              |
                    |                                                             |
                    ============================================================ */

                protected function getThemeUrl ( $closed = TRUE, $checkAbsolute = TRUE ) {
                    $value = '';
                    $method = 'theme';
                    $helper = $this->getVar('helper');
                    if ($this->cms->any->hasMethod($helper, $method)) {
                        $params = array();
                        $value = $helper->$method($params, $this->cms->smarty);
                        if (!$closed) $value = preg_replace('~[/\\\\]+$~u', '', $value);
                        if ($checkAbsolute) {
                            $this->mustbeAbsoluteUrl($value);
                            $this->maybeHttps($value);
                        }
                    }
                    return $value;
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя папки текущего шаблона.                       |
                    |                                                             |
                    ============================================================ */

                protected function getTemplateFoldername () {
                    $name = preg_replace('![/\\\\]!u', '/', dirname(__FILE__));
                    $name = preg_replace('![ \t\r\n\./]+$!u', '', $name);
                    $name = explode('/', $name);
                    array_pop($name);
                    $name = array_pop($name);
                    return $this->cms->hdd->safeFilename($name);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя файла текущего шаблона.                       |
                    |                                                             |
                    ============================================================ */

                protected function getTemplateName ( & $smarty, $withoutExt = TRUE ) {
                    $field = 'template_resource';
                    if (!isset($smarty->$field) || !is_string($smarty->$field)) return '';
                    $field = $smarty->$field;
                    if ($withoutExt) $field = $this->removeFileExt($field);
                    $field = trim($field);
                    return $this->cms->text->lowerCase($field);
                }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя текущего модуля.                              |
                    |                                                             |
                    ============================================================ */

                protected function getModuleName () {
                    $value = $this->cms->request->getGPrequest('module', '');
                    $value = $this->isActualString($value)
                             ? trim($value)
                             : '';
                    return $this->cms->text->lowerCase($value);
                }

                    protected function getModuleOrTemplateName ( & $smarty ) {
                        $value = $this->getModuleName();
                        if ($value === '') {
                            $value = $this->getTemplateName($smarty);
                            $value = $this->onlyFileName($value);
                        }
                        return $value;
                    }

                    /* ============================================================
                    |                                                             |
                    |  Извлечение расширения файла (в нижнем регистре).           |
                    |                                                             |
                    ============================================================ */

            protected function getFileExt ( $file ) {
                $file = $this->onlyFileName($file);
                $ext = explode('.', $file);
                if (count($ext) < 2) return '';
                $ext = array_pop($ext);
                return $this->cms->text->lowerCase($ext);
            }

                    /* ============================================================
                    |                                                             |
                    |  Удаление расширения из имени файла.                        |
                    |                                                             |
                    ============================================================ */

                protected function removeFileExt ( $file ) {
                    $ext = $this->getFileExt($file);
                    $ext = $this->cms->text->length($ext) + 1;
                    return $ext > 1 ? $this->cms->text->substr($file, 0, -$ext) : $file;
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечение только имени файла (без пути).                  |
                    |                                                             |
                    ============================================================ */

                protected function onlyFileName ( $file ) {
                    $file = preg_replace('![/\\\\]!u', '/', $file);
                    $file = explode('/', $file);
                    return array_pop($file);
                }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что это имя файла миниатюры.                      |
                    |                                                             |
                    ============================================================ */

            protected function isThumbnail ( $file ) {
                $ext = $this->getFileExt($file);
                $ext = 'thumb.' . $ext;
                $file = $this->cms->text->substr($file, -strlen($ext));
                return $this->cms->text->lowerCase($file) == $ext;
            }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что объект имеет такое свойство.            |
                    |                                                             |
                    ============================================================ */

        protected function hasProperty ( & $who, $property ) {
            return is_object($who) && isset($who->$property);
        }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что результат является строкой.             |
                    |                                                             |
                    ============================================================ */

        protected function mustbeString ( & $value, $def = '' ) {
            if (is_string($value)) return $value;
            $test = '' . $value;
            if ($value == $test) return $test;
            return $def;
        }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что результат является массивом.            |
                    |                                                             |
                    ============================================================ */

        protected function mustbeArray ( & $value, $def = array() ) {
            return is_array($value) ? $value : $def;
        }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что результат является объектом.            |
                    |                                                             |
                    ============================================================ */

        protected function mustbeObject ( & $value ) {
            if (!is_object($value)) $value = new stdClass;
        }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что в параметрах вызова есть такой, а если  |
                    |  нет - добавить.                                            |
                    |                                                             |
                    ============================================================ */

        protected function mustbeParam ( $name, & $params, $def = '' ) {
            if (!isset($params[$name])) {
                $this->setParam($name, $params, $def);
            }
        }

                    /* ============================================================
                    |                                                             |
                    |  Установить этот параметр вызова.                           |
                    |                                                             |
                    ============================================================ */

            protected function setParam ( $name, & $params, $value ) {
                if (!is_array($params)) $params = array();
                $params[$name] = $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Уничтожить этот параметр вызова.                           |
                    |                                                             |
                    ============================================================ */

            protected function destroyParam ( $name, & $params ) {
                if (!is_array($params)) $params = array();
                if (isset($params[$name])) unset($params[$name]);
            }

                    /* ============================================================
                    |                                                             |
                    |  Установить этот параметр вызова.                           |
                    |                                                             |
                    ============================================================ */

            protected function copyParam ( $name, & $fromParams, & $toParams, $def = null ) {
                $exists = isset($fromParams[$name]);
                if ($exists || !is_null($def)) {
                    if (!is_array($toParams)) $toParams = array();
                    $toParams[$name] = $exists ? $fromParams[$name] : $def;
                }
            }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что папка существует.                       |
                    |                                                             |
                    ============================================================ */

        protected function mustbeFolder ( $folder, $guardIndex = TRUE, $guardHtaccess = TRUE ) {
            return $this->cms->hdd->createFolder($folder, $guardIndex, $guardHtaccess);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя папки для хранения статики (STATIC-кеш).      |
                    |                                                             |
                    ============================================================ */

            protected function staticFoldername ( $personal = FALSE ) {
                return 'cache-statics' . ($personal ? ''
                                                    : '-' . $this->getTemplateFoldername());
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя папки для хранения статики такого URL-а.      |
                    |                                                             |
                    ============================================================ */

            protected function staticPageFolder ( $uri, & $params, & $personal = null ) {
                if (is_null($personal)) $personal = $this->paramAsBoolean('personal', $params);
                $name = $this->staticFoldername($personal);
                return dirname(__FILE__) . ($personal ? '/../' . $name
                                                      : '/../../..' . (is_string($uri) ? '/' . $name
                                                                                       : ''));
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя файла для хранения статики такого URL-а.      |
                    |                                                             |
                    ============================================================ */

            protected function staticPageFile ( $uri, & $params, & $personal = null ) {
                $folder = $this->staticPageFolder($uri, $params, $personal);
                if (is_string($uri)) return $folder . '/' . $this->asUrlHash($uri) . '.html';
                return $folder . ($personal ? '/mainpage.html'
                                            : '/index.html');
            }

                    /* ============================================================
                    |                                                             |
                    |  Удостовериться что спецсимволы результата экранированы.    |
                    |                                                             |
                    ============================================================ */

        protected function asAttribute ( & $value ) {
            return htmlspecialchars($value, ENT_QUOTES, 'UTF-8');
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в запросе страницы есть NAME-параметры.       |
                    |                                                             |
                    ============================================================ */

        protected function existsQueryParams ( $name, & $params ) {
            switch ($name) {
                case '_REQUEST': $var = & $_REQUEST; break;
                case '_GET': $var = & $_GET; break;
                case '_POST': $var = & $_POST; break;
                case '_FILES': $var = & $_FILES; break;
                case '_COOKIE': $var = & $_COOKIE; break;
                case '_SESSION': $var = & $_SESSION; break;
                case '_SERVER': $var = & $_SERVER; break;
                default:
                    try {
                        $var = & ${$name};
                    } catch (Exception $e) {
                        $var = null;
                    }
            }
            $ok = $this->isActualArray($var);
            if ($ok) {
                $items = array();
                foreach ($var as $idx => $item) $items[] = $idx;
                $items = $this->uniqueStringArray($items);
                $ok = !empty($items);
                if ($ok) {
                    $except = $this->paramAsArray('except', $params);
                    $except = $this->uniqueStringArray($except);
                    if ($this->isActualArray($except)) {
                        $except = $this->uniqueStringArray($items, $except);
                        $ok = !empty($except);
                    }
                    if ($ok) {
                        $among = $this->paramAsArray('among', $params);
                        $among = $this->uniqueStringArray($among);
                        if ($this->isActualArray($among)) {
                            $except = count($among);
                            $among = $this->uniqueStringArray($items, $among);
                            $among = count($items) - count($among);
                            $ok = $except == $among;
                        }
                    }
                }
            }
            return $ok;
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в параметрах вызова есть такой.               |
                    |                                                             |
                    ============================================================ */

        protected function existsParam ( $name, & $params ) {
            return isset($params[$name]);
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в параметрах вызова есть такой и он - строка. |
                    |                                                             |
                    ============================================================ */

            protected function existsParamAsString ( $name, & $params, $mustbe = '' ) {
                $state = !empty($params[$name]) && is_string($params[$name]);
                if ($state && $mustbe != '') $state = $params[$name] == $mustbe;
                return $state;
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что в параметрах вызова присутствует not=TRUE.    |
                    |                                                             |
                    ============================================================ */

            protected function existsParamNOT ( & $params ) {
                return !empty($params['not']);
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как есть.                     |
                    |                                                             |
                    ============================================================ */

        protected function paramAsIs ( $name, & $params, $def = null ) {
            return isset($params[$name]) ? $params[$name] : $def;
        }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как флаг.                     |
                    |                                                             |
                    ============================================================ */

            protected function paramAsBoolean ( $name, & $params, $def = FALSE ) {
                return isset($params[$name]) ? $params[$name] == TRUE : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как строку.                   |
                    |                                                             |
                    ============================================================ */

            protected function paramAsString ( $name, & $params, $def = '' ) {
                return isset($params[$name]) ? trim($params[$name]) : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь параметр вызова как печатный текст (антитрейлинг,  |
                    |  оптимизация пробелов).                                     |
                    |                                                             |
                    ============================================================ */

                protected function paramAsSentence ( $name, & $params, $def = '' ) {
                    $def = $this->paramAsString($name, $params, $def);
                    return $this->cms->request->getAsSentence($def);
                }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как целое число.              |
                    |                                                             |
                    ============================================================ */

            protected function paramAsInteger ( $name, & $params, $def = 0 ) {
                return isset($params[$name]) ? intval($params[$name]) : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как целое число.              |
                    |                                                             |
                    ============================================================ */

            protected function paramAsFloat ( $name, & $params, $def = 0.00 ) {
                return isset($params[$name]) ? $this->cms->number->floatValue($params[$name]) : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как натуральное число.        |
                    |                                                             |
                    ============================================================ */

            protected function paramAsNatural ( $name, & $params, $def = 1 ) {
                return isset($params[$name]) ? max(1, intval($params[$name])) : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как число расширенного        |
                    |  натурального ряда.                                         |
                    |                                                             |
                    ============================================================ */

            protected function paramAsNaturalExtended ( $name, & $params, $def = 0 ) {
                return isset($params[$name]) ? max(0, intval($params[$name])) : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь этот параметр вызова как массив.                   |
                    |                                                             |
                    ============================================================ */

            protected function paramAsArray ( $name, & $params, $def = array() ) {
                return isset($params[$name]) && is_array($params[$name]) ? $params[$name] : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение этой константы.                          |
                    |                                                             |
                    ============================================================ */

        protected function getConst ( $name, $def = '' ) {
            return defined($name) ? constant($name) : $def;
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение этого параметра сеанса.                  |
                    |                                                             |
                    ============================================================ */

        protected function getSession ( $name, $def = '' ) {
            return $this->cms->session->get($name, $def);
        }

                    /* ============================================================
                    |                                                             |
                    |  Установить значение параметра сеанса.                      |
                    |                                                             |
                    ============================================================ */

            protected function setSession ( $name, $value ) {
                if (!is_null($value)) $this->cms->session->set($name, $value);
                else $this->cms->session->delete($name);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение этого параметра настроек сайта.          |
                    |                                                             |
                    ============================================================ */

        protected function getSettings ( $name, $def = '' ) {
            return $this->cms->any->settings->get($name, $def);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение параметра настроек с неким номером.      |
                    |                                                             |
                    ============================================================ */

            protected function getSettingsNumerated ( $name, & $params, $def = '' ) {
                $num = $this->paramAsNatural('num', $params);
                $value = $this->getSettings($name . $num, $def);
                if ($value === $def && $num == 1) $value = $this->getSettings($name);
                return $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Установить значение параметра настроек (на время сеанса).  |
                    |                                                             |
                    ============================================================ */

            protected function setSettings ( $name, $value ) {
                if (!isset($this->cms->settings)) $this->cms->settings = new stdClass;
                if (!is_null($value)) $this->cms->any->settings->set($name, $value);
                elseif (!isset($this->cms->settings->$name)) return;
                else unset($this->cms->settings->$name);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение этого конфиг параметра сайта.            |
                    |                                                             |
                    ============================================================ */

        protected function getConfig ( $name, $def = '' ) {
            return isset($this->cms->config->$name) ? trim($this->cms->config->$name) : $def;
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение конфиг параметра как флаг.               |
                    |                                                             |
                    ============================================================ */

            protected function getConfigAsBoolean ( $name, $def = FALSE ) {
                return isset($this->cms->config->$name) ? !empty($this->cms->config->$name) : $def;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение этого параметра из конфига или настроек. |
                    |                                                             |
                    ============================================================ */

            protected function getConfigOrSettings ( $name, $def = '' ) {
                $value = $this->getSettings($name, $def);
                return $this->getConfig($name, $value);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение этого параметра из входных параметров    |
                    |  вызова, или из конфига, или из настроек сайта.             |
                    |                                                             |
                    ============================================================ */

            protected function getParamOrConfigOrSettings ( $name, $configName, & $params,  $def = '' ) {
                $value = $this->paramAsIs($name, $params);
                if (!is_null($value)) return $value;
                return $this->getConfigOrSettings($configName, $def);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение конфиг параметра с неким номером.        |
                    |                                                             |
                    ============================================================ */

            protected function getConfigNumerated ( $name, & $params, $def = '' ) {
                $num = $this->paramAsNatural('num', $params);
                $value = $this->getConfig($name . $num, $def);
                if ($value === $def && $num == 1) $value = $this->getConfig($name);
                return $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Установить значение конфиг параметра (на время сеанса).    |
                    |                                                             |
                    ============================================================ */

            protected function setConfig ( $name, $value ) {
                if (!isset($this->cms->config)) $this->cms->config = new stdClass;
                if (!is_null($value)) $this->cms->config->$name = $value;
                elseif (!isset($this->cms->config->$name)) return;
                else unset($this->cms->config->$name);
            }

                    /* ============================================================
                    |                                                             |
                    |  Сохранить текущие значения конфиг параметров.              |
                    |                                                             |
                    ============================================================ */

            protected function saveConfig () {
                // TODO: закодить сохранение полей объекта $this->cms->config
                // в файле конфигурации сайта Config.class.php по аналогии,
                // как это делает модуль objects/configfile/ConfigFile.admin.php
            }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что такая переменная объявлена в списке           |
                    |  отклоняемых для упреждающего наполнения.                   |
                    |                                                             |
                    ============================================================ */

        protected function isNotPreassignable ( $varname ) {
            return $this->isActualString($varname) &&
                   isset($this->not_preassignable) &&
                   $this->isActualArray($this->not_preassignable) &&
                   in_array($varname, $this->not_preassignable);
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить значение переменной с таким именем в шаблоне      |
                    |  (может быть указан массив имен переменных, тогда получить  |
                    |  значение первой доступной переменной).                     |
                    |                                                             |
                    ============================================================ */

        protected function getVar ( $names, & $smarty = null, $def = null ) {
            $value = null;
            if (!is_array($names)) $names = array($names);
            foreach ($names as $name) {
                if (is_string($name) && !empty($name)) {
                    $found = FALSE;
                    $nodes = explode('->', $name);
                    foreach ($nodes as $name) {
                        $name = trim($name);
                        if ($this->isActualString($name)) {
                            if (!$found) {
                                if (!is_null($smarty)) $value = $smarty->getTemplateVars($name);
                                if (is_null($value)) $value = $this->cms->smarty->getTemplateVars($name);
                                if (is_null($value)) break;
                                $found = TRUE;
                            } else {
                                $index = intval($name);
                                if (is_object($value) && isset($value->$name)) $value = & $value->$name;
                                elseif (is_array($value) && isset($value[$name])) $value = & $value[$name];
                                elseif (is_array($value) && strval($index) === $name && isset($value[$index])) $value = & $value[$index];
                                else {
                                    $value = null;
                                    break;
                                }
                            }
                        }
                    }
                    if (!is_null($value)) break;
                }
            }
            return is_null($value) ? $def : $value;
        }

                    /* ============================================================
                    |                                                             |
                    |  Признак что переменная с таким именем существует в шаблоне |
                    |  (может быть указан массив имен переменных, тогда признак   |
                    |  что хотя бы одна из них существует).                       |
                    |                                                             |
                    ============================================================ */

            protected function existsVar ( $names, & $smarty = null ) {
                return $this->getVar($names, $smarty) != null
                    || is_null($smarty)
                    || $this->getVar($names) != null;
            }

                    /* ============================================================
                    |                                                             |
                    |  Отправить HTTP-заголовки для контента некоторого типа.     |
                    |                                                             |
                    ============================================================ */

        protected function headerDynamicContent ( $type, $charset = TRUE ) {
            if (is_string($type)) {
                $type = $this->cms->request->getAsSentence($type);
                if (!empty($type)) {
                    $charset = $charset ? '; charset=UTF-8' : '';
                    $time = time() - 24 * 3600;
                    @ header('Content-Type: ' . $type . $charset, TRUE);
                    @ header('Expires: ' . date('r', $time), TRUE);
                    @ header('Cache-Control: must-revalidate', TRUE);
                    @ header('Pragma: no-cache', TRUE);
                }
            }
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить поле текущего авторизованного пользователя.       |
                    |                                                             |
                    ============================================================ */

        protected function getUserField ( $name, $def = '', & $item = null ) {
            if (is_object($item)) return isset($item->$name) ? $item->$name : $def;
            return isset($this->cms->user) && isset($this->cms->user->$name) ? $this->cms->user->$name : $def;
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить ИД текущего авторизованного пользователя.         |
                    |                                                             |
                    ============================================================ */

            protected function getUserId ( & $item = null ) {
                return $this->getUserField('user_id', 0, $item);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить емейл текущего пользователя.                      |
                    |                                                             |
                    ============================================================ */

            protected function getUserEmail ( & $item = null ) {
                return $this->getUserField('email', '', $item);
            }

            protected function getUserEmail2 ( & $item = null ) {
                return $this->getUserField('email2', '', $item);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить процент скидки текущего пользователя.             |
                    |                                                             |
                    ============================================================ */

            protected function getUserDiscount ( & $item = null ) {
                return $this->getUserField('discount', 0, $item);
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить ИД группы цен текущего пользователя.              |
                    |                                                             |
                    ============================================================ */

            protected function getUserPriceId ( & $item = null ) {
                return $this->getUserField('price_id', 0, $item);
            }

                    /* ============================================================
                    |                                                             |
                    |  Определение откуда пришел посетитель.                      |
                    |                                                             |
                    ============================================================ */

            protected function getHttpUserReferer ( $maybeSameSite = FALSE, $clipQuery = TRUE, $clipUrl = FALSE ) {
                $value = $this->cms->request->getServerAsSentence('HTTP_REFERER');
                $value = $this->cms->text->asPlainSentence($value);
                if ($this->isActualString($value)) {
                    if ($clipQuery) {
                        $value = preg_replace('/^([^\?\#]*)[\?\#].*$/us', '$1', $value);
                        $value = $this->cms->text->asSentence($value);
                    }
                    if ($clipUrl) $value = preg_replace('/^[^\?\#]+/u', '', $value);
                    if ($this->isActualString($value)) {
                        if (empty($maybeSameSite)) {
                            $test = $this->getUrlDomain($value);
                            $test2 = $this->getSiteDomain();
                            if ($test == $test2) $value = '';
                        }
                    }
                }
                return $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить список месяцев (на русском языке).                |
                    |                                                             |
                    ============================================================ */

        protected function getMonths ( $short = TRUE ) {
            $items = array(
                'января',
                'февраля',
                'марта',
                'апреля',
                'мая',
                'июня',
                'июля',
                'августа',
                'сентября',
                'октября',
                'ноября',
                'декабря'
            );
            if ($short) {
                foreach ($items as & $item) {
                    $item = $this->cms->text->substr($item, 0, 3);
                }
            }
            return $items;
        }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь день из строки вида ГГГГ.ММ.ДД.........            |
                    |                                                             |
                    ============================================================ */

            protected function getDay ( $yyyy_mm_dd, $forClock = FALSE ) {
                $offset = preg_match('/^[0-9][0-9][^0-9]/', $yyyy_mm_dd) ? 0 : 8;
                $value = intval(max(1, min(substr($yyyy_mm_dd, $offset, 2), 31)));
                return $forClock ? sprintf('%02d', $value) : $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь номер месяца из строки вида ГГГГ.ММ.ДД.........    |
                    |                                                             |
                    ============================================================ */

            protected function getMonth ( $yyyy_mm_dd, $forClock = FALSE ) {
                $offset = preg_match('/^[0-9][0-9][^0-9]/', $yyyy_mm_dd) ? 3 : 5;
                $value = intval(max(1, min(substr($yyyy_mm_dd, $offset, 2), 12)));
                return $forClock ? sprintf('%02d', $value) : $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь год из строки вида ГГГГ.ММ.ДД.........             |
                    |                                                             |
                    ============================================================ */

            protected function getYear ( $yyyy_mm_dd ) {
                $offset = preg_match('/^[0-9][0-9][^0-9]/', $yyyy_mm_dd) ? 6 : 0;
                $now = date('Y');
                return intval(max(1970, min(substr($yyyy_mm_dd, $offset, 4), $now)));
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь час из строки вида ГГГГ.ММ.ДД.ЧЧ.мм...             |
                    |                                                             |
                    ============================================================ */

            protected function getHour ( $yyyy_mm_dd_hh_mm, $forClock = FALSE ) {
                $value = intval(max(0, min(substr($yyyy_mm_dd_hh_mm, 11, 2), 23)));
                return $forClock ? sprintf('%02d', $value) : $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь минуту из строки вида ГГГГ.ММ.ДД.ЧЧ.мм...          |
                    |                                                             |
                    ============================================================ */

            protected function getMinute ( $yyyy_mm_dd_hh_mm, $forClock = FALSE ) {
                $value = intval(max(0, min(substr($yyyy_mm_dd_hh_mm, 14, 2), 59)));
                return $forClock ? sprintf('%02d', $value) : $value;
            }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь такой параметр текущего запроса страницы.          |
                    |                                                             |
                    ============================================================ */

        protected function requestParam ( $param, $def = '' ) {
            $value = $this->cms->request->getGPrequest($param, $def);
            return $value !== $def ? trim($value) : $value;
        }

                    /* ============================================================
                    |                                                             |
                    |  Извлечь такое поле запостченной формы для некоторой записи.|
                    |                                                             |
                    ============================================================ */

        protected function requestRecordField ( $field, $id, $def = '' ) {
            return $this->cms->request->getPostRecordField($field, $id, $def);
        }

                    /* ============================================================
                    |                                                             |
                    |  Удалить комментарии и избыточные пробелы.                  |
                    |                                                             |
                    ============================================================ */

        protected function removeComments ( $src ) {
            $src = preg_replace('~/\*.*?\*/~su', ' ', $src);
            return preg_replace('~[ \t\r\n]+~u', ' ', $src);
        }

                    /* ============================================================
                    |                                                             |
                    |  Отдать значение в точку вызова или указанную переменную.   |
                    |  ---------------------------------------------------------  |
                    |  Анализируемые параметры (по порядку обхода):               |
                    |      var                                                    |
                    |      assign                                                 |
                    |      result                                                 |
                    |      scope                                                  |
                    |                                                             |
                    ============================================================ */

        protected function assignVar ( & $params, & $value, & $smarty, $defaultName = '' ) {
            if (!is_object($smarty)) return $value;
            $varname = $this->existsParamAsString('var', $params)
                           ? trim($params['var'])
                           : ($this->existsParamAsString('assign', $params)
                               ? trim($params['assign'])
                               : ($this->existsParamAsString('result', $params)
                                   ? trim($params['result'])
                                   : trim($defaultName)));
            if (empty($varname)) return $value;
            $maybe = 'global';
            $scope = $this->existsParamAsString('scope', $params, $maybe) ? $maybe : '';
            return $this->setVariableByRef($varname, $value, $smarty, $scope);
        }

                    /* ============================================================
                    |                                                             |
                    |  Установить переменную шаблонизатора по ссылке.             |
                    |                                                             |
                    ============================================================ */

            protected function setVariableByRef ( $varname, & $value, & $smarty, $scope = '' ) {
                $asGlobal = $scope == 'root' || $scope == 'global';
                $ok = is_object($smarty);
                if ($ok) {
                    $smarty->clearAssign($varname);
                    $smarty->assignByRef($varname, $value);
                    if ($scope == 'parent' || $asGlobal) {
                        if (isset($smarty->parent)) {
                            $parent = & $smarty->parent;
                            while (is_object($parent) && $smarty !== $parent) {
                                $parent->clearAssign($varname);
                                $parent->assignByRef($varname, $value);
                                if ($scope == 'parent') break;
                                if (!isset($parent->parent)) break;
                                $parent = & $parent->parent;
                            }
                        }
                    }
                }
                if (!$ok || $asGlobal) {
                    if ($smarty !== $this->cms->smarty) {
                        $this->cms->smarty->clearAssign($varname);
                        $this->cms->smarty->assignByRef($varname, $value);
                    }
                }
                return '';
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить фрагмент текста к текстовой переменной.           |
                    |                                                             |
                    ============================================================ */

            protected function appendString ( $varname, & $text, & $smarty ) {
                $prev = $this->getVar($varname, $smarty);
                $prev = $this->mustbeString($prev);
                $text = $this->mustbeString($text);
                $text = $prev . $text;
                $params = array('assign' => $varname,
                                'scope' => 'global');
                $this->assignVar($params, $text, $smarty);
            }

                    /* ============================================================
                    |                                                             |
                    |  Преобразовать значение в хеш.                              |
                    |                                                             |
                    ============================================================ */

        protected function asHash ( $value ) {
            return strtolower(md5($value));
        }

                    /* ============================================================
                    |                                                             |
                    |  Преобразовать URL в хеш.                                   |
                    |                                                             |
                    ============================================================ */

        protected function asUrlHash ( $uri, $intelligible = TRUE, $sensitive = FALSE ) {
            $name = '';
            $value = $this->cms->text->lowerCase($uri);
            if ($intelligible) {
                $name = $this->cms->hdd->safeFilename($value, FALSE, 210);
                if ($name != '') $name = preg_replace('/[^a-z0-9]+/', '-', $name . '-');
            }
            return $name . $this->asHash($sensitive ? $uri
                                                    : $value);
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что переменная является непустым массивом.        |
                    |                                                             |
                    ============================================================ */

        protected function isActualArray ( & $var ) {
            return is_array($var) && !empty($var);
        }

                    /* ============================================================
                    |                                                             |
                    |  Проверка что переменная является непустой строкой.         |
                    |                                                             |
                    ============================================================ */

        protected function isActualString ( & $var ) {
            return is_string($var) && trim($var) != '';
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить из входного массив уникальных строк.              |
                    |                                                             |
                    ============================================================ */

        protected function uniqueStringArray ( & $items, $excludes = array() ) {
            $result = array();
            if ($this->isActualArray($items)) {
                $minus = array();
                if ($this->isActualArray($excludes)) {
                    foreach ($excludes as $item) {
                        if ($this->isActualString($item)) {
                            $item = $this->cms->request->getAsSentence($item);
                            if ($this->isActualString($item)) {
                                $index = $this->cms->text->lowerCase($item);
                                $minus[$index] = TRUE;
                            }
                        }
                    }
                }
                foreach ($items as $item) {
                    if ($this->isActualString($item)) {
                        $item = $this->cms->request->getAsSentence($item);
                        if ($this->isActualString($item)) {
                            $index = $this->cms->text->lowerCase($item);
                            if (empty($minus[$index])) $result[$index] = $item;
                        }
                    }
                }
            }
            return $result;
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить из входной строки массив уникальных строк.        |
                    |                                                             |
                    ============================================================ */

            protected function asUniqueStringArray ( & $str, $updateStr = TRUE, $excludes = array() ) {
                $items = array();
                if ($this->isActualString($str)) {
                    $items = explode(',', $str);
                    $items = $this->uniqueStringArray($items, $excludes);
                }
                if ($updateStr) $str = implode(',', $items);
                return $items;
            }

                    /* ============================================================
                    |                                                             |
                    |  Получить RegExp-шаблон делителя секций многоязычного       |
                    |  словаря.                                                   |
                    |                                                             |
                    ============================================================ */

        protected function regexpPolyglotSeparator () {
            return '[<«]hr[ \t\r\n\s]+class=["\']?([a-z][a-z](?:-[a-z]+)*)["\']?[/ \t\r\n\s]*[»>]';
        }

                    /* ============================================================
                    |                                                             |
                    |  Распарсить HTML-запись статьи многоязычного словаря и      |
                    |  добавить в список распарсенных статей, а на выход вернуть  |
                    |  перевод согласно указанному или текущему htmlLang.         |
                    |                                                             |
                    ============================================================ */

        protected function parsePolyglotDFN ( $dfn, $lang = FALSE, & $items = array(), $withMarkers = FALSE ) {
            if (!is_array($items)) $items = array();
            $separator                 = $this->regexpPolyglotSeparator();
                $getPhrasePattern      = '~^(.+?)(' . $separator . '.*)$~usi';
                $getTranslationPattern = '~^.*?' . $separator . '(.+?)(' . $separator . '.+)?$~usi';
                $trimLeftPattern       = '~^[ \t\r\n\s\xA0]+~u';
                $trimRightPattern      = '~[ \t\r\n\s\xA0]+$~u';
                $reduceSpacesPattern   = '~[ \t\r\n\s\xA0]+~u';
            $phrase = preg_replace($getPhrasePattern, '$1', $dfn);
                $existsVariants = $phrase != $dfn;
                $phrase = preg_replace($trimLeftPattern, '', $phrase);
                $phrase = preg_replace($trimRightPattern, '', $phrase);
                $phrase = preg_replace($reduceSpacesPattern, ' ', $phrase);
                $phraseIdx = $this->cms->text->lowerCase($phrase);
                    if ($withMarkers) {
                        if (!isset($items[$phraseIdx])) $items[$phraseIdx] = array();
                        if (!isset($items[$phraseIdx]['marker']))
                            $items[$phraseIdx]['marker'] = $phrase;
                    }
                $translated = $phrase;
            $langIdx = $this->isActualString($lang)
                       ? $lang
                       : $this->htmlLang();
                $langIdx = preg_replace($trimLeftPattern, '', $langIdx);
                $langIdx = preg_replace($reduceSpacesPattern, ' ', $langIdx);
                $langIdx = $this->cms->text->substr($langIdx, 0, 2);
                $langIdx = $this->cms->text->lowerCase($langIdx);
            if ($existsVariants) {
                if ($this->isActualString($phrase)) {
                    $dfn = preg_replace($getPhrasePattern, '$2', $dfn);
                    while (($item = preg_replace($getTranslationPattern, '$2', $dfn)) != $dfn) {
                        $item = preg_replace($trimLeftPattern, '', $item);
                        if ($this->isActualString($item)) {
                            $item = preg_replace($trimRightPattern, '', $item);
                                $item = preg_replace($reduceSpacesPattern, ' ', $item);
                            $lang = preg_replace($getTranslationPattern, '$1', $dfn);
                                $lang = $this->cms->text->lowerCase($lang);
                                if ($lang == $langIdx) $translated = $item;
                            if (!isset($items[$phraseIdx])) $items[$phraseIdx] = array();
                            $items[$phraseIdx][$lang] = $item;
                        }
                        $dfn = preg_replace($getTranslationPattern, '$3', $dfn);
                    }
                }
            } else {
                $dictionary = $this->getVar('polyglot');
                if (isset($dictionary[$phraseIdx][$langIdx])) $translated = $dictionary[$phraseIdx][$langIdx];
            }
            return $translated;
        }

                    /* ============================================================
                    |                                                             |
                    |  Вывести URL миниатюры этого товара.                        |
                    |                                                             |
                    ============================================================ */

        protected function supposeDemoImage ( & $image, $type = '' ) {
            if (!$this->cms->security->inDemoMode()
            || !is_array($image)
            || !isset($image['url'])) return;
            $restart = FALSE;
            $field = 'demo' . $type . 'Images';
            if (!isset($this->$field) || !is_array($this->$field)) {
                $this->$field = array();
                $url = $this->getThemeUrl() . 'images/';
                $path = 'demo/' . $this->cms->text->lowerCase($type) . '-';
                for ($i = 1; $i <= 100; $i++) {
                    $file = $path . $i . '.jpg';
                    if (!$this->existsImage($file)) {
                        $file = $path . $i . '.png';
                        if (!$this->existsImage($file)) break;
                    }
                    array_push($this->$field, $url . $file);
                }
                $restart = TRUE;
            }
            if (empty($this->$field)) return;
            if (!$restart) $url = next($this->$field);
            if ($restart || !is_string($url)) {
                shuffle($this->$field);
                $url = reset($this->$field);
            }
            if (is_string($url)) {
                $image['url'] = $url;
                $image['thumb'] = $url;
            }
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя переменной сообщения об ошибке.               |
                    |                                                             |
                    ============================================================ */

        protected function getErrorVarname ( & $smarty = null ) {
            $tpl = $this->getModuleOrTemplateName($smarty);
            switch ($tpl) {
                case 'feedback':     return 'feedback_error';
                case 'login':        return 'login_error';
                case 'registration': return 'registration_error';
                case 'callme':       return 'callme_error';
                case 'cart':
                default:             return 'error';
            }
        }

                    /* ============================================================
                    |                                                             |
                    |  Получить имя переменной информационного сообщения.         |
                    |                                                             |
                    ============================================================ */

            public function getMessageVarname ( & $smarty = null ) {
                $tpl = $this->getModuleOrTemplateName($smarty);
                switch ($tpl) {
                    case 'feedback':     return 'feedback_message';
                    case 'login':        return 'login_message';
                    case 'registration': return 'registration_message';
                    case 'callme':       return 'callme_message';
                    case 'cart':
                    default:             return 'message';
                }
            }

                    /* ============================================================
                    |                                                             |
                    |  Добавить фразу в сообщение клиенту об ошибке.              |
                    |                                                             |
                    ============================================================ */

        protected function pushError ( $text, $divider = '<br><br>' ) {
            if ($this->cms->error_msg != '') $this->cms->error_msg .= $divider;
            $this->cms->error_msg .= trim($text);
            $name = $this->getErrorVarname($this->cms->smarty);
            $this->cms->smarty->assignByRef($name, $this->cms->error_msg);
            return TRUE;
        }

                    /* ============================================================
                    |                                                             |
                    |  Добавить фразу в информационное сообщение клиенту.         |
                    |                                                             |
                    ============================================================ */

            protected function pushInfo ( $text, $divider = '<br><br>' ) {
                if ($this->cms->info_msg != '') $this->cms->info_msg .= $divider;
                $this->cms->info_msg .= trim($text);
                $name = $this->getMessageVarname($this->cms->smarty);
                $this->cms->smarty->assignByRef($name, $this->cms->info_msg);
                return TRUE;
            }

        /* ========================================================================
        |                                                                         |
        |    Системные методы / Админпанель на фронтенде.                         |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Используются только для внутренних целей хелпера. Намеренно сделаны  |
        |    приватными или защищенными, чтобы не оказались доступными в шаблоне, |
        |    так как служат лишь выполнению технических операций низкого уровня.  |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Выполнить админские команды.                               |
                    |                                                             |
                    ============================================================ */

            protected function doAdminActions () {
                $enabled = $this->cms->security->inDemoMode()
                        || $this->existsAdmin();
                if ($enabled) {
                    $field = 'adminAction';
                    if ($this->cms->request->isPostArray($field)) {
                        foreach ($_POST[$field] as $action => $dummy) {
                            unset($_POST[$field][$action]);
                            unset($_REQUEST[$field][$action]);
                            switch ($action) {
                                case 'uploadImage':
                                    $this->doAdminActions_uploadImage();
                                    break;
                                case 'uploadPhoto':
                                    $this->doAdminActions_uploadPhoto();
                                    break;
                                case 'editAnnotation':
                                    $this->doAdminActions_editHtml('annotation');
                                    break;
                                case 'editDescription':
                                    $this->doAdminActions_editHtml('description');
                                    break;
                                case 'editSeo':
                                    $this->doAdminActions_editHtml('seo');
                                    break;
                            }
                        }
                        unset($_POST[$field]);
                        unset($_REQUEST[$field]);
                    }
                }
            }

                    /* ============================================================
                    |                                                             |
                    |  Обработчик админ команды "Загрузить картинку в шаблон".    |
                    |                                                             |
                    ============================================================ */

                protected function doAdminActions_uploadImage () {
                    $field = 'image';
                    if ($this->cms->request->isPostedFile($field)) {
                        $param = 'imageName';
                        if ($this->cms->request->isPostArray($param)) {
                            $files = $this->cms->request->getFiles($field);
                            if ($this->isActualArray($files['tmp_name'])) {
                                $names = $this->cms->request->getPost($param);
                                foreach ($names as $index => $filename) {
                                    if ($this->isActualString($filename)) {
                                        if (preg_replace('/\.(jpe?g|png|gif|bmp|svg)\s*$/ui', '', $filename) != $filename) {
                                            if (!empty($files['tmp_name'][$index])) {
                                                if (empty($files['error'][$index])) {
                                                    if ($this->cms->security->inDemoMode()) {
                                                        $this->pushError('Этот сайт работает сейчас в режиме демо площадки. Процесс загрузки файла был показан почти до конца. Но чтобы не испортить демонстрационные данные отвлеченными файлами, которые пользователь может загрузить сюда в ходе своих экспериментов с демо площадкой, финальная часть загрузки файла была пропущена специально. На вашем реальном сайте загрузка будет работать как положено до конца.');
                                                    } else {
                                                        $folder = $this->cms->hdd->safeFilename($filename, FALSE);
                                                            $folder = $this->cms->hdd->getDirname($folder);
                                                            $filename = $this->cms->hdd->safeFilename($filename);
                                                        $dir = getcwd() . '/design/' . $this->getTemplateFoldername() . '/images/' . $folder;
                                                        $this->cms->hdd->createFolder($dir, TRUE, FALSE);
                                                        $this->cms->hdd->deleteFile($dir . $filename);
                                                        $this->cms->hdd->renameFile($files['tmp_name'][$index], $dir . $filename);
                                                            @ chmod($dir . $filename, 0644);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                    /* ============================================================
                    |                                                             |
                    |  Обработчик админ команды "Загрузить фото в запись".        |
                    |                                                             |
                    ============================================================ */

                protected function doAdminActions_uploadPhoto () {
                    $field = 'photo';
                    if ($this->cms->request->isPostedFile($field)) {
                        $param = 'photoName';
                        if ($this->cms->request->isPostArray($param)) {
                            $files = $this->cms->request->getFiles($field);
                            if ($this->isActualArray($files['tmp_name'])) {
                                $names = $this->cms->request->getPost($param);
                                foreach ($names as $type => & $ids) {
                                    $path = '';
                                    switch ($type) {
                                        case 'section': $path = 'images/sections/'; break;
                                        case 'product': $path = 'products/'; break;
                                        case 'kit': $path = 'kits/'; break;
                                        case 'article': $path = 'images/articles/'; break;
                                        case 'news': $path = 'images/news/'; break;
                                        case 'file': $path = 'media/'; break;
                                        case 'stock': $path = 'stocks/'; break;
                                        case 'brand': $path = 'brands/'; break;
                                        case 'event': $path = 'events/'; break;
                                        case 'comment': $path = 'comments/'; break;
                                        case 'country': $path = 'countries/'; break;
                                        case 'region': $path = 'regions/'; break;
                                        case 'town': $path = 'towns/'; break;
                                        case 'category': $path = 'categories/'; break;
                                        case 'banner': $path = 'banners/'; break;
                                    }
                                    if ($this->isActualString($path)) {
                                        if ($this->isActualArray($ids)) {
                                            if (!empty($files['tmp_name'][$type])) {
                                                foreach ($ids as $id => & $images) {
                                                    if ($this->isActualArray($images)) {
                                                        if (!empty($files['tmp_name'][$type][$id])) {
                                                            foreach ($images as $index => $filename) {
                                                                if ($this->isActualString($filename)) {
                                                                    if (preg_replace('/\.(jpe?g|png|gif|bmp)\s*$/ui', '', $filename) != $filename) {
                                                                        if (!empty($files['tmp_name'][$type][$id][$index])) {
                                                                            if (empty($files['error'][$type][$id][$index])) {
                                                                                if ($this->cms->security->inDemoMode()) {
                                                                                    $this->pushError('Этот сайт работает сейчас в режиме демо площадки. Процесс загрузки файла был показан почти до конца. Но чтобы не испортить демонстрационные данные отвлеченными файлами, которые пользователь может загрузить сюда в ходе своих экспериментов с демо площадкой, финальная часть загрузки файла была пропущена специально. На вашем реальном сайте загрузка будет работать как положено до конца.');
                                                                                } else {
                                                                                    $folder = $this->cms->hdd->safeFilename($filename, FALSE);
                                                                                        $folder = $this->cms->hdd->getDirname($folder);
                                                                                        $filename = $this->cms->hdd->safeFilename($filename);
                                                                                            $filename = preg_replace('/(\.thumb)+(\.[a-z]+)$/ui', '$2', $filename);
                                                                                            $thumb = preg_replace('/(\.[a-z]+)$/ui', '.thumb$1', $filename);
                                                                                    $dir = getcwd() . '/files/' . $path . $folder;
                                                                                    $this->cms->hdd->createFolder($dir, TRUE, FALSE);
                                                                                    $this->cms->hdd->deleteFile($dir . $filename);
                                                                                        $this->cms->hdd->deleteFile($dir . $thumb);
                                                                                    $this->cms->hdd->renameFile($files['tmp_name'][$type][$id][$index], $dir . $filename);
                                                                                        @ chmod($dir . $filename, 0644);
                                                                                        // TODO: create ".thumb" file with resize to Settings.Thumb.maxWidthHeight
                                                                                        //       resize big file to Settings.Image.maxWidthHeight
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                    /* ============================================================
                    |                                                             |
                    |  Обработчик админ команды "Сохранить html-поле в запись".   |
                    |                                                             |
                    ============================================================ */

                protected function doAdminActions_editHtml ( $field ) {
                    if ($this->cms->request->isPostArray($field)) {
                        $langDefault = $this->cms->request->getPostAsSentence('langDefault');
                            if (!$this->isActualString($langDefault))
                                $langDefault = $this->langDefault();
                        $htmls = $this->cms->request->getPost($field);
                        foreach ($htmls as $type => & $ids) {
                            $dbtable = '';
                            $idfield = '';
                            $dbfield = '';
                            switch ($type) {
                                case 'section':
                                    $dbtable = 'sections';
                                    $idfield = 'section_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = 'body'; break;
                                            case 'annotation': $dbfield = 'body'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'product':
                                    $dbtable = 'products';
                                    $idfield = 'product_id';
                                    switch ($field) {
                                        case 'annotation':  $dbfield = 'description'; break;
                                        case 'description': $dbfield = 'body'; break;
                                        case 'seo':         $dbfield = 'seo_description'; break;
                                        case 'subdomain':   $dbfield = 'subdomain_html'; break;
                                    }
                                    break;
                                case 'kit':
                                    $dbtable = 'products_kits';
                                    $idfield = 'kit_id';
                                    switch ($field) {
                                        case 'annotation':  $dbfield = 'description'; break;
                                        case 'description': $dbfield = 'body'; break;
                                        case 'seo':         $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'article':
                                    $dbtable = 'articles';
                                    $idfield = 'article_id';
                                    switch ($field) {
                                        case 'annotation':  $dbfield = $field; break;
                                        case 'description': $dbfield = 'body'; break;
                                        case 'seo':         $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'news':
                                    $dbtable = 'news';
                                    $idfield = 'news_id';
                                    switch ($field) {
                                        case 'annotation':  $dbfield = $field; break;
                                        case 'description': $dbfield = 'body'; break;
                                        case 'seo':         $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'file':
                                    $dbtable = 'files';
                                    $idfield = 'file_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = $field; break;
                                            case 'annotation': $dbfield = 'description'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'stock':
                                    $dbtable = 'stocks';
                                    $idfield = 'stock_id';
                                    switch ($field) {
                                        case 'annotation':  $dbfield = $field; break;
                                        case 'description': $dbfield = $field; break;
                                        case 'seo':         $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'brand':
                                    $dbtable = 'brands';
                                    $idfield = 'brand_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = $field; break;
                                            case 'annotation': $dbfield = 'description'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                        case 'subdomain':      $dbfield = 'subdomain_html'; break;
                                    }
                                    break;
                                case 'country':
                                    $dbtable = 'countries';
                                    $idfield = 'country_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = $field; break;
                                            case 'annotation': $dbfield = 'description'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'region':
                                    $dbtable = 'regions';
                                    $idfield = 'region_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = $field; break;
                                            case 'annotation': $dbfield = 'description'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'town':
                                    $dbtable = 'towns';
                                    $idfield = 'town_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = $field; break;
                                            case 'annotation': $dbfield = 'description'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                    }
                                    break;
                                case 'category':
                                    $dbtable = 'categories';
                                    $idfield = 'category_id';
                                    switch ($field) {
                                        case 'description':    $dbfield = $field; break;
                                            case 'annotation': $dbfield = 'description'; break;
                                        case 'seo':            $dbfield = 'seo_description'; break;
                                        case 'subdomain':      $dbfield = 'subdomain_html'; break;
                                    }
                                    break;
                            }
                            if ($this->isActualString($dbtable)) {
                                if ($this->isActualString($dbfield)) {
                                    if ($this->isActualArray($ids)) {
                                        foreach ($ids as $id => & $languages) {
                                            if ($this->isActualArray($languages)) {
                                                $query = 'SELECT `' . $dbtable . '`.`' . $dbfield . '` AS `html` '
                                                       . 'FROM `' . $dbtable . '` '
                                                       . 'WHERE `' . $dbtable . '`.`' . $idfield . '` = "' . $this->cms->db->query_value($id) . '";';
                                                $result = $this->cms->db->query($query);
                                                if ($result !== FALSE) {
                                                    while ($item = $this->cms->db->fetch_assoc($result)) break;
                                                    $this->cms->db->free_result($result);
                                                    if ($this->cms->security->inDemoMode()) {
                                                        $this->pushError('Этот сайт работает сейчас в режиме демо площадки. Процесс редактирования текста был показан почти до конца. Но чтобы не испортить демонстрационные данные отвлеченными текстовками, которые пользователь может ввести сюда в ходе своих экспериментов с демо площадкой, финальная часть редактирования текста была пропущена специально. На вашем реальном сайте редактирование будет работать как положено до конца.');
                                                    } else {
                                                        $lang = FALSE;
                                                        $items = array();
                                                        $this->parsePolyglotDFN($item['html'], $lang, $items, TRUE);
                                                        if ($this->isActualArray($items)) {
                                                            $items = array_shift($items);
                                                            $marker = 'marker';
                                                            foreach ($languages as $lang => & $html) {
                                                                $lang = $this->cms->text->lowerCase($lang);
                                                                if ($this->isActualString($html)) {
                                                                    if (isset($items[$lang]) || $lang != $langDefault)
                                                                        $items[$lang] = $html;
                                                                    else
                                                                        $items[$marker] = $html;
                                                                } else {
                                                                    if (isset($items[$lang]) && $lang != $marker)
                                                                        unset($items[$lang]);
                                                                }
                                                            }
                                                            $item = isset($items[$marker]) ? $items[$marker]
                                                                                           : '';
                                                            foreach ($items as $lang => & $html) {
                                                                if ($lang != $marker) {
                                                                    $item .= '<hr class="' . $this->asAttribute($lang) . '">'
                                                                            . $html;
                                                                }
                                                            }
                                                            $query = 'UPDATE `' . $dbtable . '` '
                                                                   . 'SET `' . $dbtable . '`.`' . $dbfield . '` = "' . $this->cms->db->query_value($item) . '" '
                                                                   . 'WHERE `' . $dbtable . '`.`' . $idfield . '` = "' . $this->cms->db->query_value($id) . '";';
                                                            $this->cms->db->query($query);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                } else {
                                    $this->pushError('Для записей такого типа или не предусмотрено редактирование поля ' . $this->asAttribute($field) . ' или оно вообще отсутствует в объявлении типа.');
                                }
                            }
                        }
                    }
                }

        /* ========================================================================
        |                                                                         |
        |    Перехваты.                                                           |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Используются для обработки неизвестных шаблонизационных тегов        |
        |    в момент компиляции шаблона.                                         |
        |                                                                         |
        ======================================================================== */

        public function handlerUndefinedTag ( $tag, $type, $template, & $callback, & $script, & $cacheable ) {
            switch ($type) {
                case 'function':
                    $callback = array(get_class($this), 'callbackUndefinedTag');
                    if (preg_match('/^mod([A-Z])[A-Za-z9-0]+$/', $tag) > 0) {
                        // TODO: мы сейчас на уровне compile,
                        //       а callback - это уровень runtime,
                        //       надо как-то связать их именем тега
                        $callback = array(get_class($this), 'callbackModTag');
                    }
                    return TRUE;
                case 'block':
                    $callback = array(get_class($this), 'callbackUndefinedBlock');
                    return TRUE;
                case 'compiler':
                case 'modifier':
                case 'modifiercompiler':
                case 'class':
                default:
            }
            return FALSE;
        }

            public function callbackModTag ( $params, $smarty ) {
                // TODO: закодить подключение модуля на основе имени тега (надо додумать, как
                //       грамотно передать имя тега из уровня compile на уровень runtime)
                //           if (preg_match('/^mod([A-Z])[A-Za-z9-0]+$/', $tag) > 0) {
                //               $tag = preg_replace('/([A-Z])/', '-$1', $tag);
                //               $tag = preg_replace('/^mod-/', '', $tag);
                //               $tag = $this->cms->text->lowerCase($tag);
                //               $html = $this->includeMod($tag . '.htm', $params, $smarty);
                //           }
                //       а пока выводим заглушку
                return '<!-- здесь был неизвестный шаблонизационный mod... тег -->';
            }

            public function callbackUndefinedTag ( $params, $smarty ) {
                return '<!-- здесь был неизвестный шаблонизационный тег -->';
            }

            public function callbackUndefinedBlock ( $params, $content, $smarty, & $repeat ) {
                return '<!-- здесь был неизвестный шаблонизационный блочный тег -->';
            }

        /* ========================================================================
        |                                                                         |
        |    Технические методы.                                                  |
        |    -------------------------------------------------------------------  |
        |                                                                         |
        |    Предназначены на случай, когда нужно почистить некоторые записи      |
        |    базы данных согласно задаче.                                         |
        |                                                                         |
        ======================================================================== */

                    /* ============================================================
                    |                                                             |
                    |  Удалить товары, выпавшие из дерева категорий.              |
                    |                                                             |
                    ============================================================ */

        private function cleanFallenProducts () {
            $this->cms->db->query('DELETE FROM `products` '
                                . 'WHERE `category_id` = 0;');
            $this->cms->db->query('DELETE `products_variants` '
                                . 'FROM `products_variants`, '
                                     . '`products` '
                                . 'WHERE `products`.`product_id` IS NULL;');
            $this->cms->db->query('TRUNCATE TABLE `cache_cbproducts_sv`;');
            $this->cms->db->query('TRUNCATE TABLE `cache_pvnames`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить список товаров.                                   |
                    |                                                             |
                    ============================================================ */

        private function cleanProducts ( $andComments = TRUE ) {
            $this->cms->db->query('TRUNCATE TABLE `products`;');
            $this->cms->db->query('TRUNCATE TABLE `products_variants`;');
            $this->cms->db->query('TRUNCATE TABLE `cache_cbproducts_sv`;');
            $this->cms->db->query('TRUNCATE TABLE `cache_pvnames`;');
            if ($andComments) $this->cms->db->query('TRUNCATE TABLE `products_comments`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить свойства товаров.                                 |
                    |                                                             |
                    ============================================================ */

        private function cleanProperties () {
            $this->cms->db->query('TRUNCATE TABLE `properties`;');
            $this->cms->db->query('TRUNCATE TABLE `properties_values`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить новости.                                          |
                    |                                                             |
                    ============================================================ */

        private function cleanNews ( $andComments = TRUE ) {
            $this->cms->db->query('TRUNCATE TABLE `news`;');
            if ($andComments) $this->cms->db->query('TRUNCATE TABLE `news_comments`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить статьи.                                           |
                    |                                                             |
                    ============================================================ */

        private function cleanArticles ( $andComments = TRUE ) {
            $this->cms->db->query('TRUNCATE TABLE `articles`;');
            if ($andComments) $this->cms->db->query('TRUNCATE TABLE `articles_comments`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить публикации.                                       |
                    |                                                             |
                    ============================================================ */

        private function cleanPublications ( $andComments = TRUE ) {
            $this->cleanNews($andComments);
            $this->cleanArticles($andComments);
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить отзывы и переписку.                               |
                    |                                                             |
                    ============================================================ */

        private function cleanComments () {
            $this->cms->db->query('TRUNCATE TABLE `products_comments`;');
            $this->cms->db->query('TRUNCATE TABLE `articles_comments`;');
            $this->cms->db->query('TRUNCATE TABLE `news_comments`;');
            $this->cms->db->query('TRUNCATE TABLE `feedback`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить запросы обратного звонка и уведомлений.           |
                    |                                                             |
                    ============================================================ */

        private function cleanRequests () {
            $this->cms->db->query('TRUNCATE TABLE `callme`;');
            $this->cms->db->query('TRUNCATE TABLE `notifies`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить заказы.                                           |
                    |                                                             |
                    ============================================================ */

        private function cleanOrders () {
            $this->cms->db->query('TRUNCATE TABLE `orders`;');
            $this->cms->db->query('TRUNCATE TABLE `orders_products`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить список пользователей.                             |
                    |                                                             |
                    ============================================================ */

        private function cleanUsers () {
            $this->cms->db->query('TRUNCATE TABLE `users`;');
            $this->cms->db->query('TRUNCATE TABLE `cache_users_sv`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Очистить записей поискового сопровождения.                 |
                    |                                                             |
                    ============================================================ */

        private function cleanSearches () {
            $this->cms->db->query('TRUNCATE TABLE `searches`;');
        }

                    /* ============================================================
                    |                                                             |
                    |  Урлы в нижний регистр, подчеркивания заменить дефисами.    |
                    |                                                             |
                    ============================================================ */

        private function fairUrls () {
            $this->cms->db->query('UPDATE `sections`   SET `url` = LCASE(REPLACE(`url`, "_", "-"));');
            $this->cms->db->query('UPDATE `articles`   SET `url` = LCASE(REPLACE(`url`, "_", "-"));');
            $this->cms->db->query('UPDATE `news`       SET `url` = LCASE(REPLACE(`url`, "_", "-"));');
            $this->cms->db->query('UPDATE `categories` SET `url` = LCASE(REPLACE(`url`, "_", "-"));');
            $this->cms->db->query('UPDATE `brands`     SET `url` = LCASE(REPLACE(`url`, "_", "-"));');
            $this->cms->db->query('UPDATE `products`   SET `url` = LCASE(REPLACE(`url`, "_", "-"));');
        }

                    /* ============================================================
                    |                                                             |
                    |  Запретить все капчи.                                       |
                    |                                                             |
                    ============================================================ */

        private function disableAllCaptcha () {
            $this->cms->db->settings->save('login_captcha_disabled', 1);
            $this->cms->db->settings->save('callme_captcha_disabled', 1);
            $this->cms->db->settings->save('feedback_captcha_disabled', 1);
            $this->cms->db->settings->save('registration_captcha_disabled', 1);
            $this->cms->db->settings->save('cart_captcha_protecting', 0);
            $this->cms->db->settings->save('quickorder_captcha_protecting', 0);
            $this->cms->db->settings->save('fulminantorder_captcha_disabled', 1);
            $this->cms->db->settings->save('users_captcha_disabled', 1);
        }
    }

    return;
?>